-- ============================================================================
-- Project:	rbench - Logic Programming and Database Benchmarks
-- Filename:	db/create_db.sql
-- Purpose:	Database for Benchmark Runs: Create Tables and Views
-- Last Change:	05.10.2019
-- Language:	SQL (Tested with PostgreSQL)
-- Author:	Stefan Brass
-- EMail:	brass@informatik.uni-halle.de
-- WWW:		http://www.informatik.uni-halle.de/~brass/
-- Address:	Feldschloesschen 15, D-06120 Halle (Saale), GERMANY
-- Copyright:	(c) 2016-2019 by Stefan Brass
-- License:	See file "LICENSE" for copying conditions.
-- Note:	There is no warranty at all - this code may contain bugs.
-- ============================================================================


-- This file contains the CREATE TABLE statements for the benchmark database,
-- and also many CREATE VIEW statements to store interesting queries
-- that can be used to analyze the data and to generate various output formats
-- (HTML, LaTeX, csv, tsv, Markdown, pieces of bash-scripts).

-- One should first delete old versions of the tables with the script
--	drop_db.sql

-- The table data are in various files which should be run after this script.
-- See the load_data script.
--	MACHINE:	machines.sql
--	INPUT_TYPE:	input_types.sql
--	BENCHMARK:	benchmarks.sql
--	SYS_TYPE:	sys_types.sql
--	SYSTEM, SYS_VER, SYS_INSTALL, BENCH_IMPL:
--			One file per system, e.g. xsb.sql
--	INPUT_FILE:	input_files.sql
--	INPUT_SET:	input_sets.sql
--	INPUT_GRAPH:	input_graphs.sql
--	INPUT_JOIN1:	input_join1.sql
--	BENCH_RUN:	results/*.sql
-- These SQL scripts should be run in the given sequence after this file
-- was executed.


-- ============================================================================
-- List of Machines on which Benchmarks are Executed:
-- ============================================================================

CREATE TABLE MACHINE(
	MACHINE     VARCHAR(20)  NOT NULL,
	SEQ_NO      NUMERIC(2)   NOT NULL,
	STATUS      CHAR(1)      NOT NULL,
        MODEL       VARCHAR(60)  NULL,
	YEAR        NUMERIC(4)   NULL,
        MONTH       NUMERIC(2)   NULL,
        PRICE       NUMERIC(7)   NULL,
	NUM_CPUS    NUMERIC(4)   NOT NULL,
	CPU         VARCHAR(60)  NOT NULL,
	CPU_CORES   NUMERIC(4)   NULL, -- Number of cores per CPU
        CPU_THREADS NUMERIC(4)   NULL, -- Number of hardware threads per CPU
	GHZ         NUMERIC(5,2) NULL,
	ARCH        VARCHAR(20)  NULL,
	RAM         NUMERIC(8,3) NOT NULL,
	NUM_DISKS   NUMERIC(4)   NULL,
        DISK        VARCHAR(40)  NULL,
	SSD         CHAR(1)      NULL,
	OS          VARCHAR(40)  NULL,
	OS_CORE     VARCHAR(40)  NULL,
	CONSTRAINT MACHINE_PK
		PRIMARY KEY(MACHINE),
	CONSTRAINT MACHINE_KEY_SEQ
		UNIQUE (SEQ_NO),
	CONSTRAINT MACHINE_SEQ_NO_POSITIVE
		CHECK(SEQ_NO > 0),
        CONSTRAINT MACHINE_YEAR_VALID
		CHECK(YEAR >= 2000),
        CONSTRAINT MACHINE_MONTH_VALID
                CHECK(MONTH >= 1 AND MONTH <= 12),
        CONSTRAINT MACHINE_PRICE_NON_NEGATIVE
                CHECK(PRICE >= 0),
	CONSTRAINT MACHINE_STATUS_S_OR_SPACE
		CHECK(STATUS IN ('S', ' ')),
		-- The machine with STATUS='S' is the standard machine.
		-- Only one row of the table should have STATUS='S'.
		-- Some views select only benchmark results on the standard
		-- machine.
		-- In future, there many be more STATUS values, e.g. for
		-- comparing the standard machine to an alternative machine.
	CONSTRAINT MACHINE_NUM_CPUS_POSITIVE
		CHECK(NUM_CPUS > 0),
	CONSTRAINT MACHINE_CPU_CORES_POSITIVE
		CHECK(CPU_CORES > 0),
	CONSTRAINT MACHINE_CPU_THREADS_POSITIVE
		CHECK(CPU_THREADS > 0),
	CONSTRAINT MACHINE_RAM_POSITIVE
		CHECK(RAM > 0),
	CONSTRAINT MACHINE_NUM_DISKS_POSITIVE
		CHECK(NUM_DISKS > 0),
	CONSTRAINT MACHINE_SSD_YN
		CHECK(SSD IN ('Y','N')));

-- Note that the CORE might not be current. It changes with every update.
-- It was current when we started the benchmarks in May 2018.
-- It gives some impression of the machine and its operating system.

-- The table data are in the file "machines.sql".


-- ============================================================================
-- List of Input Data (Problem Instance) Types:
-- ============================================================================


CREATE TABLE INPUT_TYPE(
	ITYPE       VARCHAR(8)  NOT NULL,
	SEQ_NO      NUMERIC(2)  NOT NULL,
	HEADLINE    VARCHAR(80) NOT NULL,
	DESCRIPTION VARCHAR(80) NOT NULL,
	CONSTRAINT INPUT_TYPE_SEQ_NO_NONNEGATIVE
		CHECK(SEQ_NO >= 0),
	CONSTRAINT INPUT_TYPE_PK
		PRIMARY KEY(ITYPE),
	CONSTRAINT INPUT_TYPE_SEQ_NO_UNIQUE
		UNIQUE(SEQ_NO));

-- The table data are in the file "input_types.sql".


-- ============================================================================
-- List of Benchmarks:
-- ============================================================================


CREATE TABLE BENCHMARK(
	BENCH      VARCHAR(20)  NOT NULL,
	SEQ_NO     NUMERIC(2)   NOT NULL,
	ITYPE      VARCHAR(8)   NULL,
	HEADLINE   VARCHAR(80)  NOT NULL,
	CONSTRAINT BENCHMARK_SEQ_NONNEGATIVE
		CHECK(SEQ_NO >= 0),
	CONSTRAINT BENCHMARK_PK
		PRIMARY KEY (BENCH),
	CONSTRAINT BENCHMARK_KEY_SEQ
		UNIQUE (SEQ_NO),
	CONSTRAINT BENCHMARK_REF_INPUT_TYPE
		FOREIGN KEY (ITYPE) REFERENCES INPUT_TYPE);

-- The table data are in the file "benchmarks.sql".


-- ============================================================================
-- System Types (for Subheadlines in the list of systems to be tested):
-- ============================================================================

CREATE TABLE SYS_TYPE(
	STYPE      VARCHAR(10)  NOT NULL,
	SEQ_NO     NUMERIC(2)   NOT NULL,
	HEADLINE   VARCHAR(80)  NOT NULL,
	CONSTRAINT SYS_TYPE_SEQ_NONNEGATIVE
		CHECK(SEQ_NO >= 0),
	CONSTRAINT SYS_TYPE_PK
		PRIMARY KEY (STYPE),
	CONSTRAINT SYS_TYPE_KEY_SEQ
		UNIQUE (SEQ_NO));

-- The table data are in the file "sys_types.sql".


-- ============================================================================
-- List of Systems to be Tested/Compared:
-- ============================================================================

-- SYSTEM is a reserved word in SQL:2003 and in DB2.

CREATE TABLE TEST_SYS(
	SYS         VARCHAR(20)  NOT NULL,
	SEQ_NO      NUMERIC(2)   NOT NULL,
	FULL_NAME   VARCHAR(40)  NOT NULL,
	STYPE       VARCHAR(10)  NOT NULL,
	URL         VARCHAR(80)  NULL,
	REMARK      VARCHAR(800) NULL,
	CONSTRAINT TEST_SYS_SEQ_NONNEGATIVE
		CHECK(SEQ_NO >= 0),
	CONSTRAINT TEST_SYS_PK
		PRIMARY KEY (SYS),
	CONSTRAINT TEST_SYS_KEY_SEQ
		UNIQUE (SEQ_NO),
	CONSTRAINT TEST_SYS_REF_SYS_TYPE
		FOREIGN KEY(STYPE) REFERENCES SYS_TYPE);

-- The table data are in the files for the single systems, e.g. "xsb.sql".


-- ============================================================================
-- Table for Versions of the Test Systems:
-- ============================================================================

-- This table contains information about program versions.
--
-- VER_ADDTEXT should be concatenated with VER to get the full name
-- of the version.
--
-- This table also permits to store additional information about a version
-- in the column REMARK (e.g., the major changes).
--
-- Note that this table describes a version of the source code.
-- SYS_INSTALL (below) describes a version of the program binary,
-- which also depends on the machine, the compiler, compiler settings,
-- and configuration settings.

CREATE TABLE SYS_VER (
	SYS         VARCHAR(20)  NOT NULL,
	VER         VARCHAR(20)  NOT NULL,
	VER_ADDTEXT VARCHAR(80)  NOT NULL,
	VER_DATE    DATE         NULL,
	REMARK      VARCHAR(800) NULL,
	CONSTRAINT SYS_VER_PK
		PRIMARY KEY (SYS, VER),
	CONSTRAINT SYS_VER_REF_TEST_SYS
		FOREIGN KEY (SYS) REFERENCES TEST_SYS);

-- The table data are in the files for the single systems, e.g. "xsb.sql".


-- ============================================================================
-- Table for Installed Program Versions on Machines:
-- ============================================================================

-- This table lists program versions that are or were installed
-- on a test machine.
-- They are identified here by a number INST_NO that is unique together with
-- the program name and the machine.
-- These three elements (program, installation ID, and machine)
-- are also encoded in the file names with the test results.
--
-- Whereas SYS_VER describes a version of the source code,
-- SYS_INSTALL describes a binary program.
-- For instance, if the same source code is compiled with g++ and with clang,
-- these are two different entries in SYS_INSTALL, both referring to the
-- same entry in SYS_VER.
-- Also different configuration settings may give different binaries,
-- and therefore different entries in SYS_INSTALL.
--
-- I try to make sure that the same version of a program
-- gets the same INST_NO number even on different machines,
-- but the database schema does not require this
-- (and it is impossible when there are really different installations
--  of the same program version).
--
-- Only program installations that are listed here can appear in the test runs.
--
-- This table also define a current version (more precisely, installation)
-- of a program on a machine (by STATUS='C').
-- Some views show only results for the current version.
-- It is also possible to define a previous version (STATUS='P') as reference,
-- if the result of a program update should be evaluated.
-- Finally, one can mark an alternative version (STATUS='A'),
-- e.g. compiled with a different compiler or compiler options,
-- which can also be used for comparison.

CREATE TABLE SYS_INSTALL (
	SYS         VARCHAR(20)  NOT NULL,
	VER         VARCHAR(20)  NOT NULL,
	MACHINE     VARCHAR(20)  NOT NULL,
	INST_NO     NUMERIC(4)   NOT NULL,
	COMPILER    VARCHAR(80)  NOT NULL,
	STATUS      CHAR(1)      NOT NULL,
	REMARK      VARCHAR(800) NULL,
	CONSTRAINT SYS_INSTALL_PK
		PRIMARY KEY (SYS, INST_NO, MACHINE),
	CONSTRAINT SYS_INSTALL_REF_SYS_VER
		FOREIGN KEY (SYS, VER) REFERENCES SYS_VER,
	CONSTRAINT SYS_INSTALL_REF_MACHINE
		FOREIGN KEY (MACHINE) REFERENCES MACHINE,
	CONSTRAINT SYS_INSTALL_STATUS_VALID
		CHECK(STATUS IN ('C','P','A',' ')));
	-- 'C': Current version
	-- 'P': Previous version
	-- 'A': Alternate version
	-- ' ': Any other version
	-- Views may select only benchmark results for the current version,
	-- or help to compare the current with the previous version.

-- The table data are in the files for the single systems, e.g. "xsb.sql".


-- ============================================================================
-- Table for Implementation Variants of Benchmarks:
-- ============================================================================

CREATE TABLE BENCH_IMPL(
	SYS         VARCHAR(20)  NOT NULL,
	BENCH       VARCHAR(20)  NOT NULL,
	IMPL        VARCHAR(20)  NOT NULL,
	STATUS      CHAR(1)      NOT NULL,
	DESCRIPTION VARCHAR(80)  NULL,
	CONSTRAINT BENCH_IMPL_PK
		PRIMARY KEY (SYS, BENCH, IMPL),
	CONSTRAINT BENCH_IMPL_REF_BENCHMARK
		FOREIGN KEY (BENCH) REFERENCES BENCHMARK,
	CONSTRAINT BENCH_IMPL_REF_TEST_SYS
		FOREIGN KEY (SYS) REFERENCES TEST_SYS,
	CONSTRAINT BENCH_IMPL_STATUS_VALID
		CHECK(STATUS IN ('B','A',' ')));

-- STATUS='B' marks the implementation variant (option settings)
-- that is considered best and is used in comparisons between systems.
-- One can use the views BEST_IMPL_REAL and BEST_IMPL_CPU to check
-- whether these implementation variants are really best.
-- Sometimes the best implementation depends on the specific data file,
-- but it would somehow be unfair to adapt the options to the data file.

-- STATUS='A' marks an interesting alternative variants
-- that appear in some result tables.
-- There should be only one "best" version,
-- but there can be several alterative versions.

-- STATUS=' ' means that the variant was tested
-- in order to prove that the best version is really good,
-- but it will not be listed in summary tables.

-- The table data are in the files for the single systems, e.g. "xsb.sql".


-- ============================================================================
-- Table for Data Files (Input Files to Benchmarks):
-- ============================================================================

CREATE TABLE INPUT_FILE(
	FILE_ID     VARCHAR(10)  NOT NULL,
	SEQ_NO      NUMERIC(3)   NOT NULL,
	ITYPE       VARCHAR(8)   NOT NULL,
	LINES       NUMERIC(12)  NOT NULL, -- should be the number of facts
	BYTES       NUMERIC(12)  NOT NULL, -- refers to Datalog facts version
	FILENAME    VARCHAR(40)  NOT NULL, -- without extension
--	DESCRIPTION VARCHAR(80)  NULL,
	CONSTRAINT INPUT_FILE_PK
		PRIMARY KEY (FILE_ID),
	CONSTRAINT INPUT_FILE_KEY2
		UNIQUE (SEQ_NO),
	CONSTRAINT INPUT_FILE_SEQ_NO_NOT_NEGATIVE
		CHECK(SEQ_NO >= 0),
	CONSTRAINT INPUT_FILE_REF_INPUT_TYPE
		FOREIGN KEY (ITYPE) REFERENCES INPUT_TYPE,
	CONSTRAINT INPUT_FILE_LINES_NOT_NEGATIVE
		CHECK(LINES >= 0),
	CONSTRAINT INPUT_FILE_BYTES_NOT_NEGATIVE
		CHECK(BYTES >= 0));

-- wc was used to compute lines and bytes

-- The table data are in file "input_files.sql"


-- ============================================================================
-- Table of Collections of Files Used for Publications:
-- ============================================================================

-- This can be used to put arbitrary tags at input files.

CREATE TABLE INPUT_SET(
	SET_ID      VARCHAR(10)  NOT NULL,
	FILE_ID     VARCHAR(10)  NOT NULL,
	CONSTRAINT INPUT_SET_PK
		PRIMARY KEY (SET_ID, FILE_ID),
	CONSTRAINT INPUT_SET_REFERENCES_INPUT_FILE
		FOREIGN KEY (FILE_ID) REFERENCES INPUT_FILE);

-- The table data are in file "input_sets.sql"


-- ============================================================================
-- Graph Types / Parameterized Graphs (For Input Data Files that are Graphs):
-- ============================================================================

CREATE TABLE GRAPH_TYPE(
	GTYPE      CHAR(1)      NOT NULL,
	SEQ_NO     NUMERIC(2)   NOT NULL,
	NAME       VARCHAR(80)  NOT NULL,
	CONSTRAINT GRAPH_TYPE_SEQ_NONNEGATIVE
		CHECK(SEQ_NO >= 0),
	CONSTRAINT GRAPH_TYPE_PK
		PRIMARY KEY (GTYPE),
	CONSTRAINT GRAPH_TYPE_KEY_SEQ
		UNIQUE (SEQ_NO));

-- The table data are in file "graph_types.sql".


-- ============================================================================
-- More Data on Data Files That are Directed Graphs (Subclass of Input Files):
-- ============================================================================

CREATE TABLE INPUT_GRAPH (
	FILE_ID        VARCHAR(10) NOT NULL,
	GTYPE          CHAR(1)     NOT NULL,
	PARAM_1        NUMERIC(12) NULL,
	PARAM_2        NUMERIC(12) NULL,
	PARAM_3        CHAR(1)     NULL,
	NUM_NODES      NUMERIC(12) NOT NULL,
	NUM_EDGES      NUMERIC(12) NOT NULL,
	DUP_EDGES      NUMERIC(12) NOT NULL,
	LOOPS          CHAR(1)     NOT NULL,
	MIN_IN_DEGREE  NUMERIC(12) NOT NULL,
	MAX_IN_DEGREE  NUMERIC(12) NOT NULL,
	MIN_OUT_DEGREE NUMERIC(12) NOT NULL,
	MAX_OUT_DEGREE NUMERIC(12) NOT NULL,
	CYCLES         CHAR(1)     NOT NULL,
	CONSTRAINT INPUT_GRAPH_PK
		PRIMARY KEY(FILE_ID),
	CONSTRAINT INPUT_GRAPH_REF_INPUT_FILE
		FOREIGN KEY(FILE_ID) REFERENCES INPUT_FILE,
	CONSTRAINT INPUT_GRAPH_REF_GRAPH_TYPE
		FOREIGN KEY (GTYPE) REFERENCES GRAPH_TYPE,
	CONSTRAINT INPUT_GRAPH_PARAM_1_NONNEGATIVE
		CHECK(PARAM_1 >= 0),
	CONSTRAINT INPUT_GRAPH_PARAM_2_NONNEGATIVE
		CHECK(PARAM_2 >= 0),
	CONSTRAINT INPUT_GRAPH_PARAM_3_VALID
		CHECK(PARAM_3 IN ('I')),
	CONSTRAINT INPUT_GRAPH_NUM_NODES_NONNEGATIVE
		CHECK(NUM_NODES >= 0),
	CONSTRAINT INPUT_GRAPH_NUM_EDGES_NONNEGATIVE
		CHECK(NUM_EDGES >= 0),
	CONSTRAINT INPUT_GRAPH_DUP_EDGES_NONNEGATIVE
		CHECK(DUP_EDGES >= 0),
	CONSTRAINT INPUT_GRAPH_LOOPS_YES_NO
		CHECK(LOOPS  in ('Y','N')),
	CONSTRAINT INPUT_GRAPH_MIN_IN_DEGREE_NONNEGATIVE
		CHECK(MIN_IN_DEGREE >= 0),
	CONSTRAINT INPUT_GRAPH_MAX_IN_DEGREE_NONNEGATIVE
		CHECK(MAX_IN_DEGREE >= 0),
	CONSTRAINT INPUT_GRAPH_MIN_OUT_DEGREE_NONNEGATIVE
		CHECK(MIN_OUT_DEGREE >= 0),
	CONSTRAINT INPUT_GRAPH_MAX_OUT_DEGREE_NONNEGATIVE
		CHECK(MAX_OUT_DEGREE >= 0),
	CONSTRAINT INPUT_GRAPH_CYCLES_YES_NO
		CHECK(CYCLES IN ('Y','N')));


-- GTYPE, PARAM_1, PARAM_2, and PARAM_3 are the parameters for the
-- Data Generation Program (currently, PARAM_3 has the only possible value
-- 'I' for inverting the direction of the edges).

-- DUP_EDGES is the number of duplicate edges,
-- i.e. the number of node pairs (x,y) such that there are at least two edges
-- between these nodes.
-- Normally, it should be 0.
-- However, the OpenRuleBench graph generation program generates such
-- duplicate edges for acyclic graphs.
-- Therefore, our graph analysis checks the input files for this case.

-- LOOPS='Y' means that there are edges of the form (x,x).

-- The data is in file input_graphs.sql


-- ============================================================================
-- More Data on Data Files That are Input Data Sets for the Join1 Benchmark:
-- ============================================================================


CREATE TABLE INPUT_JOIN1 (
	FILE_ID        VARCHAR(10) NOT NULL,
	NUM_NODES      NUMERIC(12) NOT NULL,
	C2_SIZE        NUMERIC(12) NOT NULL,
	C3_SIZE        NUMERIC(12) NOT NULL,
	C4_SIZE        NUMERIC(12) NOT NULL,
	D1_SIZE        NUMERIC(12) NOT NULL,
	D2_SIZE        NUMERIC(12) NOT NULL,
	A_SIZE         NUMERIC(12) NOT NULL,
	B1_SIZE        NUMERIC(12) NOT NULL,
	B2_SIZE        NUMERIC(12) NOT NULL,
	C1_SIZE        NUMERIC(12) NOT NULL,
	A_INST         NUMERIC(12) NOT NULL,
	B1_INST        NUMERIC(12) NOT NULL,
	B2_INST        NUMERIC(12) NOT NULL,
	C1_INST        NUMERIC(12) NOT NULL,
	CONSTRAINT INPUT_JOIN1_PK
		PRIMARY KEY(FILE_ID),
	CONSTRAINT INPUT_JOIN1_REF_INPUT_FILE
		FOREIGN KEY(FILE_ID) REFERENCES INPUT_FILE,
	CONSTRAINT INPUT_JOIN1_NUM_NODES_NONNEGATIVE
		CHECK(NUM_NODES >= 0),
	CONSTRAINT INPUT_JOIN1_A_SIZE_NONNEGATIVE
		CHECK(A_SIZE >= 0),
	CONSTRAINT INPUT_JOIN1_B1_SIZE_NONNEGATIVE
		CHECK(B1_SIZE >= 0),
	CONSTRAINT INPUT_JOIN1_B2_SIZE_NONNEGATIVE
		CHECK(B2_SIZE >= 0),
	CONSTRAINT INPUT_JOIN1_C1_SIZE_NONNEGATIVE
		CHECK(C1_SIZE >= 0),
	CONSTRAINT INPUT_JOIN1_C2_SIZE_NONNEGATIVE
		CHECK(C2_SIZE >= 0),
	CONSTRAINT INPUT_JOIN1_C3_SIZE_NONNEGATIVE
		CHECK(C3_SIZE >= 0),
	CONSTRAINT INPUT_JOIN1_C4_SIZE_NONNEGATIVE
		CHECK(C4_SIZE >= 0),
	CONSTRAINT INPUT_JOIN1_D1_SIZE_NONNEGATIVE
		CHECK(D1_SIZE >= 0),
	CONSTRAINT INPUT_JOIN1_D2_SIZE_NONNEGATIVE
		CHECK(D2_SIZE >= 0),
	CONSTRAINT INPUT_JOIN1_A_INST_NONNEGATIVE
		CHECK(A_INST >= 0),
	CONSTRAINT INPUT_JOIN1_B1_INST_NONNEGATIVE
		CHECK(B1_INST >= 0),
	CONSTRAINT INPUT_JOIN1_B2_INST_NONNEGATIVE
		CHECK(B2_INST >= 0),
	CONSTRAINT INPUT_JOIN1_C1_INST_NONNEGATIVE
		CHECK(C1_INST >= 0),
	CONSTRAINT INPUT_JOIN1_A_INST_GE_A_SIZE
		CHECK(A_INST >= A_SIZE),
	CONSTRAINT INPUT_JOIN1_B1_INST_GE_B1_SIZE
		CHECK(B1_INST >= B1_SIZE),
	CONSTRAINT INPUT_JOIN1_B2_INST_GE_B2_SIZE
		CHECK(B2_INST >= B2_SIZE),
	CONSTRAINT INPUT_JOIN1_C1_INST_GE_C1_SIZE
		CHECK(C1_INST >= C1_SIZE));

-- A_SIZE and so on are the number of facts for the predicate.
-- The input file contains the predicates D1, D2, C2, C3, C4.
-- The other predicates are derived by means of the following rules:
--	a(X, Y)  :- b1(X, Z), b2(Z, Y).
--	b1(X, Y) :- c1(X, Z), c2(Z, Y).
--	b2(X, Y) :- c3(X, Z), c4(Z, Y).
--	c1(X, Y) :- d1(X, Z), d2(Z, Y).

-- A_INST and so on are the number of instances of the rule that derives A.

-- The data is in file input_join1.sql

-- ============================================================================
-- Cost Measures for Benchmarks (for Analysis of Runtimes):
-- ============================================================================

CREATE TABLE BENCH_COST (
	BENCH          VARCHAR(20) NOT NULL,
	FILE_ID        VARCHAR(10) NOT NULL,
	SIZE           NUMERIC(18) NOT NULL,
	ITER           NUMERIC(12) NOT NULL,
	INST           NUMERIC(18) NOT NULL,
	CONSTRAINT BENCH_COST_PK
		PRIMARY KEY(BENCH, FILE_ID),
	CONSTRAINT BENCH_COST_REF_BENCHMARK
		FOREIGN KEY (BENCH) REFERENCES BENCHMARK,
	CONSTRAINT BENCH_COST_REF_INPUT_FILE
		FOREIGN KEY(FILE_ID) REFERENCES INPUT_FILE,
	CONSTRAINT BENCH_COST_SIZE_NONNEGATIVE
		CHECK(SIZE >= 0),
	CONSTRAINT BENCH_COST_ITER_NONNEGATIVE
		CHECK(ITER >= 0),
	CONSTRAINT BENCH_COST_INST_NONNEGATIVE
		CHECK(INST >= 0));

-- SIZE is the number of derived tuples of the benchmark (result size).
-- INST is the number of applicable rule instances in the benchmark program.
-- This is also the number of rule instances that semi-naive evaluation would
-- consider.
-- ITER is the number of iterations that the T_P operator needs to compute
-- the entire minimal model (counting only iterations that derive new tuples).
-- The application of the non-recursive rules is the first iteration
-- (if they derive at least one tuple, otherwise it does not count as an
-- iteration).

-- The data is in file bench_cost.sql


-- ============================================================================
-- Table for Results of Benchmark Runs (Test Runs):
-- ============================================================================


CREATE TABLE BENCH_RUN(
	RUN_DATE    DATE          NOT NULL,
	RUN_NO      NUMERIC(4)    NOT NULL,
	SYS         VARCHAR(20)   NOT NULL,
	INST_NO     NUMERIC(4)    NOT NULL,
	MACHINE     VARCHAR(20)   NOT NULL,
	BENCH       VARCHAR(20)   NOT NULL,
	IMPL        VARCHAR(20)   NOT NULL,
	FILE_ID     VARCHAR(10)   NOT NULL,
	WITH_OUTPUT CHAR(1)       NOT NULL,
	LOAD_TIME   NUMERIC(10,3) NULL,
	EXEC_TIME   NUMERIC(10,3) NULL,
	TOTAL_TIME  NUMERIC(10,3) NULL,
	USER_TIME   NUMERIC(10,3) NULL,
	SYS_TIME    NUMERIC(10,3) NULL,
	REAL_TIME   NUMERIC(10,3) NOT NULL,
	MEM         NUMERIC(10,3) NULL,
	STATUS      VARCHAR(80)   NOT NULL,
	REMARK      VARCHAR(800)  NULL,
        CONSTRAINT  BENCH_RUN_RUN_NO_POSITIVE
		CHECK(RUN_NO > 0),
        CONSTRAINT  BENCH_RUN_WITH_OUTPUT_BOOL
		CHECK(WITH_OUTPUT IN ('Y','N')),
        CONSTRAINT  BENCH_RUN_LOAD_TIME_NONNEGATIVE
		CHECK(LOAD_TIME >= 0),
        CONSTRAINT  BENCH_RUN_EXEC_TIME_NONNEGATIVE
		CHECK(EXEC_TIME >= 0),
        CONSTRAINT  BENCH_RUN_TOTAL_TIME_NONNEGATIVE
		CHECK(TOTAL_TIME >= 0),
        CONSTRAINT  BENCH_RUN_USER_TIME_NONNEGATIVE
		CHECK(USER_TIME >= 0),
        CONSTRAINT  BENCH_RUN_SYS_TIME_NONNEGATIVE
		CHECK(SYS_TIME >= 0),
        CONSTRAINT  BENCH_RUN_REAL_TIME_NONNEGATIVE
		CHECK(REAL_TIME >= 0),
        CONSTRAINT  BENCH_RUN_MEM_NONNEGATIVE
		CHECK(MEM >= 0),
        CONSTRAINT  BENCH_RUN_STATUS_VALID
		CHECK(STATUS IN ('OK', 'FAILED', 'OUTL',
				'Warning', 'Warn1', 'Warn2', 'Warn3', 'Warn4')),
        CONSTRAINT  BENCH_RUN_PK
		PRIMARY KEY(RUN_DATE,RUN_NO,MACHINE,SYS,INST_NO),
        CONSTRAINT  BENCH_RUN_REF_BENCH_IMPL
		FOREIGN KEY(SYS, BENCH, IMPL) REFERENCES BENCH_IMPL,
	CONSTRAINT  BENCH_RUN_REF_SYS_INSTALL
		FOREIGN KEY(SYS, INST_NO, MACHINE) REFERENCES SYS_INSTALL,
	CONSTRAINT  BENCH_RUN_REF_INPUT_FILE
		FOREIGN KEY(FILE_ID) REFERENCES INPUT_FILE);

-- All times are stored in seconds.
-- MEM is the maximum resident set size.


-- ============================================================================
-- Experimental Table for Runtime Estimation Parameters:
-- ============================================================================


CREATE TABLE ESTIMATE_PAR(
	SYS         VARCHAR(20)  NOT NULL,
	BENCH       VARCHAR(20)  NOT NULL,
	STARTUP     NUMERIC(10)  NOT NULL,
	LOAD_IDX    NUMERIC(10)  NOT NULL,
	RULE_APP    NUMERIC(10)  NOT NULL,
	RESULT      NUMERIC(10)  NOT NULL,
	HITS        NUMERIC(10)  NOT NULL,
        CONSTRAINT  ESTIMATE_PAR_STARTUP_NONNEGATIVE
		CHECK(STARTUP >= 0),
        CONSTRAINT  ESTIMATE_PAR_LOAD_IDX_NONNEGATIVE
		CHECK(LOAD_IDX >= 0),
        CONSTRAINT  ESTIMATE_PAR_RULE_APP_NONNEGATIVE
		CHECK(RULE_APP >= 0),
        CONSTRAINT  ESTIMATE_PAR_RESULT_NONNEGATIVE
		CHECK(RESULT >= 0),
        CONSTRAINT  ESTIMATE_PAR_HITS_NONNEGATIVE
		CHECK(HITS >= 0),
        CONSTRAINT  ESTIMATE_PAR_PK
		PRIMARY KEY(SYS,BENCH,STARTUP,LOAD_IDX,RULE_APP,RESULT),
        CONSTRAINT  ESTIMATE_PAR_REF_SYS
		FOREIGN KEY(SYS) REFERENCES TEST_SYS,
        CONSTRAINT  ESTIMATE_PAR_REF_BENCH
		FOREIGN KEY(BENCH) REFERENCES BENCHMARK);

-- The data for this table are in file "estimate_par.sql".


-- ============================================================================
-- Dummy Table with Exactly One Row Used in Some Views:
-- ============================================================================


CREATE TABLE DUMMY_TAB(
	ATTR NUMERIC(1)
);

INSERT INTO DUMMY_TAB VALUES(0);


