-- ============================================================================
-- Project:	rbench - Logic Programming and Database Benchmarks
-- Filename:	db/v_basic.sql
-- Purpose:	Benchmark DB: Basic Views (Benchmark Statistics)
-- Last Change:	04.03.2019
-- Language:	SQL (Tested with PostgreSQL)
-- Author:	Stefan Brass
-- EMail:	brass@informatik.uni-halle.de
-- WWW:		http://www.informatik.uni-halle.de/~brass/
-- Address:	Feldschloesschen 15, D-06120 Halle (Saale), GERMANY
-- Copyright:	(c) 2018-2019 by Stefan Brass
-- License:	See file "LICENSE" for copying conditions.
-- Note:	There is no warranty at all - this code may contain bugs.
-- ============================================================================


-- ############################################################################
-- Views to get an Overview of the Existing Measurements:
-- ############################################################################


-- ============================================================================
-- Overview of Existing measurements:
-- ============================================================================

CREATE VIEW OVERVIEW AS
	SELECT SYS, INST_NO, MACHINE, BENCH, IMPL, COUNT(*) AS NUM_RUNS
	FROM BENCH_RUN
	GROUP BY SYS, INST_NO, MACHINE, BENCH, IMPL;

-- ############################################################################
-- Views for Statistical Evaluation of Run Data:
-- ############################################################################


-- ============================================================================
-- Full Statistical Evaluation of Run Data:
-- ============================================================================

CREATE VIEW RUN_STATISTICS AS
SELECT BENCH, IMPL, FILE_ID, MACHINE, SYS, INST_NO, WITH_OUTPUT,
	-- Load Time:
	ROUND(AVG(LOAD_TIME),3) AS AVG_LOAD_TIME,
	MIN(LOAD_TIME) AS MIN_LOAD_TIME,
	MAX(LOAD_TIME) AS MAX_LOAD_TIME,
	MAX(LOAD_TIME)-MIN(LOAD_TIME) AS DIFF_LOAD_TIME,
	STDDEV(LOAD_TIME) AS SDEV_LOAD_TIME,
	-- Exec Time:
	ROUND(AVG(EXEC_TIME),3) AS AVG_EXEC_TIME,
	MIN(EXEC_TIME) AS MIN_EXEC_TIME,
	MAX(EXEC_TIME) AS MAX_EXEC_TIME,
	MAX(EXEC_TIME)-MIN(EXEC_TIME) AS DIFF_EXEC_TIME,
	STDDEV(EXEC_TIME) AS SDEV_EXEC_TIME,
	-- Total Time:
	ROUND(AVG(TOTAL_TIME),3) AS AVG_TOTAL_TIME,
	MIN(TOTAL_TIME) AS MIN_TOTAL_TIME,
	MAX(TOTAL_TIME) AS MAX_TOTAL_TIME,
	MAX(TOTAL_TIME)-MIN(TOTAL_TIME) AS DIFF_TOTAL_TIME,
	STDDEV(TOTAL_TIME) AS SDEV_TOTAL_TIME,
	-- User Mode Time:
	ROUND(AVG(USER_TIME),3) AS AVG_USER_TIME,
	MIN(USER_TIME) AS MIN_USER_TIME,
	MAX(USER_TIME) AS MAX_USER_TIME,
	MAX(USER_TIME)-MIN(USER_TIME) AS DIFF_USER_TIME,
	STDDEV(USER_TIME) AS SDEV_USER_TIME,
	-- System Mode Time:
	ROUND(AVG(SYS_TIME),3) AS AVG_SYS_TIME,
	MIN(SYS_TIME) AS MIN_SYS_TIME,
	MAX(SYS_TIME) AS MAX_SYS_TIME,
	MAX(SYS_TIME)-MIN(SYS_TIME) AS DIFF_SYS_TIME,
	STDDEV(SYS_TIME) AS SDEV_SYS_TIME,
	-- CPU Time:
	ROUND(AVG(SYS_TIME+USER_TIME),3) AS AVG_CPU_TIME,
	MIN(SYS_TIME+USER_TIME) AS MIN_CPU_TIME,
	MAX(SYS_TIME+USER_TIME) AS MAX_CPU_TIME,
	MAX(SYS_TIME+USER_TIME)-MIN(SYS_TIME+USER_TIME) AS DIFF_CPU_TIME,
	STDDEV(SYS_TIME+USER_TIME) AS SDEV_CPU_TIME,
	-- Real Time:
	ROUND(AVG(REAL_TIME),3) AS AVG_REAL_TIME,
	MIN(REAL_TIME) AS MIN_REAL_TIME,
	MAX(REAL_TIME) AS MAX_REAL_TIME,
	MAX(REAL_TIME)-MIN(REAL_TIME) AS DIFF_REAL_TIME,
	STDDEV(REAL_TIME) AS SDEV_REAL_TIME,
	-- Memory:
	ROUND(AVG(MEM),3) AS AVG_MEM,
	MIN(MEM) AS MIN_MEM,
	MAX(MEM) AS MAX_MEM,
	MAX(MEM)-MIN(MEM) AS DIFF_MEM,
	STDDEV(MEM) AS SDEV_MEM,
	-- Number of Runs:
	COUNT(*) AS NUM_BENCH_RUN
FROM BENCH_RUN
WHERE STATUS <> 'FAILED' AND STATUS <> 'OUTL'
GROUP BY MACHINE, SYS, INST_NO, BENCH, IMPL, FILE_ID, WITH_OUTPUT
ORDER BY MACHINE, SYS, INST_NO, BENCH, IMPL, FILE_ID, WITH_OUTPUT;

-- ============================================================================
-- Statistical Evaluation of Run Data, without STDDEV - This works for SQLite:
-- ============================================================================

CREATE VIEW RUN_STAT AS
SELECT BENCH, IMPL, FILE_ID, MACHINE, SYS, INST_NO, WITH_OUTPUT,
	-- Load Time:
	ROUND(AVG(LOAD_TIME),3) AS AVG_LOAD_TIME,
	MIN(LOAD_TIME) AS MIN_LOAD_TIME,
	MAX(LOAD_TIME) AS MAX_LOAD_TIME,
	MAX(LOAD_TIME)-MIN(LOAD_TIME) AS DIFF_LOAD_TIME,
	-- Exec Time:
	ROUND(AVG(EXEC_TIME),3) AS AVG_EXEC_TIME,
	MIN(EXEC_TIME) AS MIN_EXEC_TIME,
	MAX(EXEC_TIME) AS MAX_EXEC_TIME,
	MAX(EXEC_TIME)-MIN(EXEC_TIME) AS DIFF_EXEC_TIME,
	-- Total Time:
	ROUND(AVG(TOTAL_TIME),3) AS AVG_TOTAL_TIME,
	MIN(TOTAL_TIME) AS MIN_TOTAL_TIME,
	MAX(TOTAL_TIME) AS MAX_TOTAL_TIME,
	MAX(TOTAL_TIME)-MIN(TOTAL_TIME) AS DIFF_TOTAL_TIME,
	-- User Mode Time:
	ROUND(AVG(USER_TIME),3) AS AVG_USER_TIME,
	MIN(USER_TIME) AS MIN_USER_TIME,
	MAX(USER_TIME) AS MAX_USER_TIME,
	MAX(USER_TIME)-MIN(USER_TIME) AS DIFF_USER_TIME,
	-- System Mode Time:
	ROUND(AVG(SYS_TIME),3) AS AVG_SYS_TIME,
	MIN(SYS_TIME) AS MIN_SYS_TIME,
	MAX(SYS_TIME) AS MAX_SYS_TIME,
	MAX(SYS_TIME)-MIN(SYS_TIME) AS DIFF_SYS_TIME,
	-- CPU Time:
	ROUND(AVG(SYS_TIME+USER_TIME),3) AS AVG_CPU_TIME,
	MIN(SYS_TIME+USER_TIME) AS MIN_CPU_TIME,
	MAX(SYS_TIME+USER_TIME) AS MAX_CPU_TIME,
	MAX(SYS_TIME+USER_TIME)-MIN(SYS_TIME+USER_TIME) AS DIFF_CPU_TIME,
	-- Real Time:
	ROUND(AVG(REAL_TIME),3) AS AVG_REAL_TIME,
	MIN(REAL_TIME) AS MIN_REAL_TIME,
	MAX(REAL_TIME) AS MAX_REAL_TIME,
	MAX(REAL_TIME)-MIN(REAL_TIME) AS DIFF_REAL_TIME,
	-- Memory:
	ROUND(AVG(MEM),3) AS AVG_MEM,
	MIN(MEM) AS MIN_MEM,
	MAX(MEM) AS MAX_MEM,
	MAX(MEM)-MIN(MEM) AS DIFF_MEM,
	-- Number of Runs:
	COUNT(*) AS NUM_BENCH_RUN
FROM BENCH_RUN
WHERE STATUS <> 'FAILED' AND STATUS <> 'OUTL'
GROUP BY MACHINE, SYS, INST_NO, BENCH, IMPL, FILE_ID, WITH_OUTPUT
ORDER BY MACHINE, SYS, INST_NO, BENCH, IMPL, FILE_ID, WITH_OUTPUT;

-- ============================================================================
-- Benchmark Results for Current Version on the Standard Machine without Output:
-- ============================================================================

CREATE VIEW RESULT AS
SELECT R.BENCH AS BENCH,
	R.IMPL AS IMPL,
	R.FILE_ID AS FILE_ID,
	R.SYS AS SYS,
	R.AVG_LOAD_TIME AS LOAD_T,
	R.AVG_EXEC_TIME AS EXEC_T,
	R.AVG_TOTAL_TIME AS TOTAL_T,
	R.AVG_CPU_TIME AS CPU_T,
	R.AVG_REAL_TIME AS REAL_T,
	R.AVG_MEM AS MEM_MB
FROM   RUN_STAT R, SYS_INSTALL V, MACHINE M, INPUT_FILE F
WHERE  R.MACHINE = M.MACHINE AND V.MACHINE = R.MACHINE AND M.STATUS = 'S'
AND    R.SYS = V.SYS AND R.INST_NO = V.INST_NO AND V.STATUS = 'C'
AND    R.WITH_OUTPUT = 'N'
AND    R.FILE_ID = F.FILE_ID
ORDER BY R.BENCH, R.IMPL, F.SEQ_NO, R.SYS;

-- ============================================================================
-- Benchmark Results (Current Ver., Std. Machine, No Output, Best Settings):
-- ============================================================================

CREATE VIEW BEST_RESULT AS
SELECT R.BENCH AS BENCH,
	R.IMPL AS IMPL,
	R.FILE_ID AS FILE_ID,
	R.SYS AS SYS,
	R.AVG_LOAD_TIME AS LOAD_T,
	R.AVG_EXEC_TIME AS EXEC_T,
	R.AVG_TOTAL_TIME AS TOTAL_T,
	R.AVG_CPU_TIME AS CPU_T,
	R.AVG_REAL_TIME AS REAL_T,
	R.AVG_MEM AS MEM_MB
FROM   RUN_STAT R, SYS_INSTALL V, MACHINE M, BENCH_IMPL I, INPUT_FILE F
WHERE  R.MACHINE = M.MACHINE AND V.MACHINE = R.MACHINE AND M.STATUS = 'S'
AND    R.SYS = V.SYS AND R.INST_NO = V.INST_NO AND V.STATUS = 'C'
AND    R.WITH_OUTPUT = 'N'
AND    I.SYS = R.SYS AND I.BENCH = R.BENCH AND I.IMPL = R.IMPL
AND    I.STATUS = 'B'
AND    R.FILE_ID = F.FILE_ID
ORDER BY R.BENCH, R.IMPL, F.SEQ_NO, R.SYS;


-- ############################################################################
-- Views to Find/Check Best Option Settings for a System and a Benchmark:
-- ############################################################################


-- ============================================================================
-- Best Option Settings (in Real Time) for a Program for a Benchmark:
-- ============================================================================

-- This looks only at the runtimes of the standard program version
-- on the standard machine.

CREATE VIEW BEST_IMPL_REAL AS
SELECT	R.BENCH,
	R.FILE_ID,
	R.SYS,
	R.IMPL,
	R.REAL_T,
	I.STATUS
FROM	RESULT R, BENCHMARK B, INPUT_FILE F, BENCH_IMPL I
WHERE   R.BENCH = B.BENCH AND R.FILE_ID = F.FILE_ID
AND	I.BENCH = R.BENCH AND I.IMPL = R.IMPL AND I.SYS = R.SYS
AND NOT EXISTS(SELECT * FROM RESULT R2
		WHERE R2.BENCH = R.BENCH AND R2.SYS = R.SYS
		AND   R2.FILE_ID = R.FILE_ID
		AND   R2.REAL_T < R.REAL_T)
-- Eliminate trivial tests with run time 0: All options are best.
AND EXISTS(SELECT * FROM RESULT R3
		WHERE R3.BENCH = R.BENCH AND R3.SYS = R.SYS
		AND   R3.FILE_ID = R.FILE_ID
		AND   R3.REAL_T > 0)
ORDER BY B.SEQ_NO, F.SEQ_NO;


-- ============================================================================
-- Best Option Settings (in CPU Time) for a Program for a Benchmark:
-- ============================================================================

-- This looks only at the runtimes of the standard program version
-- on the standard machine.

CREATE VIEW BEST_IMPL_CPU AS
SELECT	R.BENCH,
	R.FILE_ID,
	R.SYS,
	R.IMPL,
	R.CPU_T,
	I.STATUS
FROM	RESULT R, BENCHMARK B, INPUT_FILE F, BENCH_IMPL I
WHERE   R.BENCH = B.BENCH AND R.FILE_ID = F.FILE_ID
AND	I.BENCH = R.BENCH AND I.IMPL = R.IMPL AND I.SYS = R.SYS
AND NOT EXISTS(SELECT * FROM RESULT R2
		WHERE R2.BENCH = R.BENCH AND R2.SYS = R.SYS
		AND   R2.FILE_ID = R.FILE_ID
		AND   R2.CPU_T < R.CPU_T)
-- Eliminate trivial tests with run time 0: All options are best.
AND EXISTS(SELECT * FROM RESULT R3
		WHERE R3.BENCH = R.BENCH AND R3.SYS = R.SYS
		AND   R3.FILE_ID = R.FILE_ID
		AND   R3.CPU_T > 0)
ORDER BY B.SEQ_NO, F.SEQ_NO;

