-- ============================================================================
-- Project:	rbench - Logic Programming and Database Benchmarks
-- Filename:	db/v_check.sql
-- Purpose:	Benchmark DB: Views for Outlier Detection and other Checks
-- Last Change:	10.10.2019
-- Language:	SQL (Tested with PostgreSQL)
-- Author:	Stefan Brass
-- EMail:	brass@informatik.uni-halle.de
-- WWW:		http://www.informatik.uni-halle.de/~brass/
-- Address:	Feldschloesschen 15, D-06120 Halle (Saale), GERMANY
-- Copyright:	(c) 2018-2019 by Stefan Brass
-- License:	See file "LICENSE" for copying conditions.
-- Note:	There is no warranty at all - this code may contain bugs.
-- ============================================================================


-- ############################################################################
-- Outlier Detection, Checking of Results for Plausibility, Integrity Check:
-- ############################################################################


-- ============================================================================
-- Detect Outliers with regard to Real Time:
-- ============================================================================

CREATE VIEW OUTLIER_REAL AS
SELECT R.REAL_TIME, S.AVG_REAL_TIME, R.SYS, R.INST_NO, R.STATUS,
       R.BENCH, R.IMPL, R.FILE_ID, R.MACHINE, R.WITH_OUTPUT,
       R.RUN_DATE, R.RUN_NO
FROM   BENCH_RUN R, RUN_STAT S
WHERE  R.STATUS <> 'FAILED'
AND    R.BENCH       = S.BENCH
AND    R.IMPL        = S.IMPL
AND    R.FILE_ID     = S.FILE_ID
AND    R.MACHINE     = S.MACHINE
AND    R.SYS         = S.SYS
AND    R.INST_NO     = S.INST_NO
AND    R.WITH_OUTPUT = S.WITH_OUTPUT
AND    R.REAL_TIME   > 2.0 * S.AVG_REAL_TIME
AND    R.REAL_TIME   > S.AVG_REAL_TIME + 1
ORDER BY MACHINE, SYS, INST_NO, BENCH, IMPL, FILE_ID, WITH_OUTPUT;

-- ============================================================================
-- Detect Outliers with regard to CPU Time:
-- ============================================================================

CREATE VIEW OUTLIER_CPU AS
SELECT R.SYS_TIME+R.USER_TIME AS CPU_TIME, S.AVG_CPU_TIME,
       R.SYS, R.INST_NO, R.STATUS,
       R.BENCH, R.IMPL, R.FILE_ID, R.MACHINE, R.WITH_OUTPUT,
       R.RUN_DATE, R.RUN_NO
FROM   BENCH_RUN R, RUN_STAT S
WHERE  R.STATUS <> 'FAILED'
AND    R.BENCH       = S.BENCH
AND    R.IMPL        = S.IMPL
AND    R.FILE_ID     = S.FILE_ID
AND    R.MACHINE     = S.MACHINE
AND    R.SYS         = S.SYS
AND    R.INST_NO     = S.INST_NO
AND    R.WITH_OUTPUT = S.WITH_OUTPUT
AND    R.SYS_TIME+R.USER_TIME > 2.0 * S.AVG_CPU_TIME
AND    R.SYS_TIME+R.USER_TIME > S.AVG_CPU_TIME + 1
ORDER BY MACHINE, SYS, INST_NO, BENCH, IMPL, FILE_ID, WITH_OUTPUT;

-- ============================================================================
-- Detect Outliers with regard to Memory:
-- ============================================================================

CREATE VIEW OUTLIER_MEM AS
SELECT R.MEM, S.AVG_MEM, R.SYS, R.INST_NO, R.STATUS,
       R.BENCH, R.IMPL, R.FILE_ID, R.MACHINE, R.WITH_OUTPUT,
       R.RUN_DATE, R.RUN_NO
FROM   BENCH_RUN R, RUN_STAT S
WHERE  R.STATUS <> 'FAILED'
AND    R.BENCH       = S.BENCH
AND    R.IMPL        = S.IMPL
AND    R.FILE_ID     = S.FILE_ID
AND    R.MACHINE     = S.MACHINE
AND    R.SYS         = S.SYS
AND    R.INST_NO     = S.INST_NO
AND    R.WITH_OUTPUT = S.WITH_OUTPUT
AND    R.MEM         > 2.0 * S.AVG_MEM
AND    R.MEM         > S.AVG_MEM + 5
ORDER BY MACHINE, SYS, INST_NO, BENCH, IMPL, FILE_ID, WITH_OUTPUT;

-- ============================================================================
-- Check where Realtime is much more than CPU time (at least double):
-- ============================================================================

CREATE VIEW REAL_TOO_LARGE AS
SELECT R.REAL_TIME, S.AVG_REAL_TIME,
       R.SYS_TIME+R.USER_TIME AS CPU_TIME, S.AVG_CPU_TIME,
       R.SYS, R.INST_NO, R.STATUS,
       R.BENCH, R.IMPL, R.FILE_ID, R.MACHINE, R.WITH_OUTPUT,
       R.RUN_DATE, R.RUN_NO
FROM   BENCH_RUN R, RUN_STAT S
WHERE  R.STATUS <> 'FAILED'
AND    R.BENCH       = S.BENCH
AND    R.IMPL        = S.IMPL
AND    R.FILE_ID     = S.FILE_ID
AND    R.MACHINE     = S.MACHINE
AND    R.SYS         = S.SYS
AND    R.INST_NO     = S.INST_NO
AND    R.WITH_OUTPUT = S.WITH_OUTPUT
AND    R.REAL_TIME   > 2.0 * (R.SYS_TIME+R.USER_TIME)
AND    R.REAL_TIME   > (R.SYS_TIME+R.USER_TIME) + 1
ORDER BY MACHINE, SYS, INST_NO, BENCH, IMPL, FILE_ID, WITH_OUTPUT;

-- ============================================================================
-- Compare Real Time with CPU Time:
-- ============================================================================

CREATE VIEW PCT_CPU AS
SELECT SYS, RUN_DATE, MACHINE, BENCH,
	ROUND(AVG((SYS_TIME+USER_TIME) * 100 / REAL_TIME)) AS AVG_CPU,
	ROUND(MIN((SYS_TIME+USER_TIME) * 100 / REAL_TIME)) AS MIN_CPU,
	ROUND(MAX((SYS_TIME+USER_TIME) * 100 / REAL_TIME)) AS MAX_CPU
FROM   BENCH_RUN
WHERE  SYS_TIME IS NOT NULL AND USER_TIME IS NOT NULL
AND    REAL_TIME > 0
AND    STATUS <> 'FAILED' AND STATUS <> 'OUTL'
AND    WITH_OUTPUT = 'N'
GROUP  BY SYS, RUN_DATE, MACHINE, BENCH
ORDER  BY SYS, RUN_DATE, MACHINE, BENCH;


-- ============================================================================
-- Compare Real Time with CPU Time:
-- ============================================================================

CREATE VIEW PCT_CPU_INPUT AS
SELECT SYS, MACHINE, BENCH, FILE_ID,
	ROUND(AVG((SYS_TIME+USER_TIME) * 100 / REAL_TIME)) AS AVG_CPU,
	ROUND(MIN((SYS_TIME+USER_TIME) * 100 / REAL_TIME)) AS MIN_CPU,
	ROUND(MAX((SYS_TIME+USER_TIME) * 100 / REAL_TIME)) AS MAX_CPU
FROM   BENCH_RUN
WHERE  SYS_TIME IS NOT NULL AND USER_TIME IS NOT NULL
AND    REAL_TIME > 0
AND    STATUS <> 'FAILED' AND STATUS <> 'OUTL'
AND    WITH_OUTPUT = 'N'
GROUP  BY SYS, MACHINE, BENCH, FILE_ID
ORDER  BY SYS, MACHINE, BENCH, FILE_ID;


-- ============================================================================
-- Integrity Constraints / Consistency Check - Query Result must be Empty!
-- ============================================================================

CREATE VIEW INTEGRITY_ERROR AS
SELECT 'Number of lines in INPUT_FILE and edges in INPUT_GRAPH differ for: ' ||
	F.FILE_ID AS ERROR_MESSAGE
FROM  INPUT_FILE F, INPUT_GRAPH G
WHERE F.FILE_ID = G.FILE_ID
AND   F.LINES <> G.NUM_EDGEs;

-- Further checks can be added with UNION ALL

