-- ============================================================================
-- Project:	rbench - Logic Programming and Database Benchmarks
-- Filename:	db/v_estimate.sql
-- Purpose:	Benchmark DB: Views to Support Research on Runtime Estimation
-- Last Change:	11.10.2019
-- Language:	SQL (Tested with PostgreSQL)
-- Author:	Stefan Brass
-- EMail:	brass@informatik.uni-halle.de
-- WWW:		http://www.informatik.uni-halle.de/~brass/
-- Address:	Feldschloesschen 15, D-06120 Halle (Saale), GERMANY
-- Copyright:	(c) 2018-2019 by Stefan Brass
-- License:	See file "LICENSE" for copying conditions.
-- Note:	There is no warranty at all - this code may contain bugs.
-- ============================================================================


-- ############################################################################
-- Support for Reasearch on Runtime Estimation:
-- ############################################################################


-- ============================================================================
-- System Speed in Real Time:
-- ============================================================================

CREATE VIEW SPEED_REAL AS
SELECT SYS, FILE_ID, ROUND((TC_COST/REAL_T)/1000) as SPEED, TC_COST, REAL_T
FROM   TCFF_PLOT_REAL
WHERE  REAL_T > 0
ORDER  BY SYS, SPEED;

-- ============================================================================
-- System Speed in Real Time With Startup Time:
-- ============================================================================

CREATE VIEW SPEED_REAL_DIFF AS
SELECT P.SYS, P.FILE_ID,
       ROUND(((P.TC_COST-S.TC_COST)/(P.REAL_T-S.REAL_T))/1000) as SPEED,
       P.TC_COST, P.REAL_T,
       (P.TC_COST-S.TC_COST) AS DIFF_TC_COST, (P.REAL_T-S.REAL_T) AS DIFF_REAL_T
FROM   TCFF_PLOT_REAL P, TCFF_PLOT_REAL S
WHERE  P.REAL_T-S.REAL_T > 0
AND    P.SYS = S.SYS
AND    S.FILE_ID = 'm4_2ki'
ORDER  BY SYS, SPEED;

-- ============================================================================
-- System Speed in CPU Time:
-- ============================================================================

CREATE VIEW SPEED_CPU AS
SELECT SYS, FILE_ID, ROUND((TC_COST/CPU_T)/1000) as SPEED, TC_COST, CPU_T
FROM   TCFF_PLOT_CPU
WHERE  CPU_T > 0
ORDER  BY SYS, SPEED;

-- ============================================================================
-- Memory Efficiency:
-- ============================================================================

CREATE VIEW MEM_EFFICIENCY AS
SELECT SYS, FILE_ID, ROUND(MEM_MB*1024*1024/FACTS) as BYTES_PER_FACT,
       FACTS, MEM_MB
FROM   TCFF_PLOT_MEM
WHERE  FACTS > 0
ORDER  BY SYS, BYTES_PER_FACT;

-- ============================================================================
-- Compute Run Time Estimates (Real Time):
-- ============================================================================

CREATE VIEW ESTIMATE_FORMULA AS
SELECT R.SYS, R.BENCH, E.STARTUP, E.LOAD_IDX, E.RULE_APP, E.RESULT,
       E.BENCH AS FORMULA_FOR,
       F.FILE_ID,
       ROUND(CAST(
         E.STARTUP/1000 +
         (F.LINES * (LOG(F.LINES)/LOG(2)) * E.LOAD_IDX / 1000000000) +
         (C.INST * E.RULE_APP / 1000000000) +
         (C.SIZE * E.RESULT / 1000000000) AS NUMERIC),
       3) AS ESTIMATE,
       R.REAL_T
FROM   BEST_RESULT R, INPUT_FILE F, BENCH_COST C, ESTIMATE_PAR E
WHERE  R.SYS = E.SYS
AND    C.BENCH = R.BENCH
AND    C.FILE_ID = F.FILE_ID
AND    R.FILE_ID = F.FILE_ID
ORDER  BY R.SYS, E.STARTUP, E.LOAD_IDX, E.RULE_APP, R.BENCH, E.RESULT,
       ESTIMATE;

-- ============================================================================
-- Check System Speed Estimates (Real Time):
-- ============================================================================

CREATE VIEW ESTIMATE_ERR AS
SELECT E.SYS, E.STARTUP, E.LOAD_IDX, E.RULE_APP, E.RESULT, E.FORMULA_FOR,
       E.BENCH, E.FILE_ID, E.ESTIMATE, E.REAL_T,
       E.REAL_T - E.ESTIMATE AS DIFF,
       ROUND((E.REAL_T - E.ESTIMATE) * 100 / E.ESTIMATE, 3) AS DIFF_PCT
FROM   ESTIMATE_FORMULA E
ORDER  BY E.BENCH, E.SYS, E.STARTUP, E.LOAD_IDX, E.RULE_APP,
       E.RESULT, DIFF_PCT;

-- ============================================================================
-- Count the Number of Wrong Estimates (more than 20%):
-- ============================================================================

CREATE VIEW NUM_BAD_ESTIMATES AS
SELECT BENCH, SYS, STARTUP, LOAD_IDX, RULE_APP, RESULT,
	COUNT(*) AS BAD, SUM(ABS(DIFF)) AS ABS_ERR
FROM   ESTIMATE_ERR
WHERE  ABS(DIFF_PCT) > 20
GROUP  BY BENCH, SYS, STARTUP, LOAD_IDX, RULE_APP, RESULT
ORDER  BY BENCH, SYS, STARTUP, LOAD_IDX, RULE_APP, RESULT;

-- ============================================================================
-- Count the Number of Good Estimates (max. 20% error):
-- ============================================================================

CREATE VIEW NUM_GOOD_ESTIMATES AS
SELECT BENCH, SYS, STARTUP, LOAD_IDX, RULE_APP, RESULT,
	COUNT(*) AS GOOD, SUM(ABS(DIFF)) AS ABS_ERR
FROM   ESTIMATE_ERR
WHERE  ABS(DIFF_PCT) <= 20
GROUP  BY BENCH, SYS, STARTUP, LOAD_IDX, RULE_APP, RESULT
ORDER  BY BENCH, SYS, STARTUP, LOAD_IDX, RULE_APP, RESULT;

-- ============================================================================
-- Evaluate Estimates:
-- ============================================================================

CREATE VIEW EVAL_ESTIMATES AS
SELECT BENCH, SYS, STARTUP AS S, LOAD_IDX AS L, RULE_APP AS A, RESULT AS R,
	SUM(CASE WHEN ABS(DIFF_PCT) <= 20 THEN 1 ELSE 0 END) AS GOOD,
	SUM(CASE WHEN DIFF_PCT < -20 THEN 1 ELSE 0 END) AS LESS,
	SUM(CASE WHEN DIFF_PCT > 20 THEN 1 ELSE 0 END) AS MORE,
	ROUND(AVG(ABS(DIFF_PCT))) AS AVG_ERR
FROM   ESTIMATE_ERR
GROUP  BY BENCH, SYS, STARTUP, LOAD_IDX, RULE_APP, RESULT
ORDER  BY BENCH, SYS, STARTUP, LOAD_IDX, RULE_APP, RESULT;

-- ============================================================================
-- Generate C++ Objects with Graph Data and Runtime (for Optimization):
-- ============================================================================

CREATE VIEW ESTIMATE_DATA AS
SELECT R.SYS, R.BENCH, R.IMPL,
       'data_c("' || G.FILE_ID || '", ' ||
		G.NUM_NODES || ', ' ||
		G.NUM_EDGES || ', ' ||
		'INT64_C(' || C.SIZE || '), ' ||
		'INT64_C(' || C.INST || '), ' ||
		C.ITER  || ', ' ||
		ROUND(R.REAL_T*1000)|| '),' AS OBJ
FROM	INPUT_GRAPH G, INPUT_FILE F, BENCH_COST C, BEST_RESULT R
WHERE   G.FILE_ID = F.FILE_ID AND R.FILE_ID = F.FILE_ID
AND     C.FILE_ID = F.FILE_ID AND C.BENCH = 'tcff'
ORDER BY R.SYS, R.BENCH, F.SEQ_NO;

-- ============================================================================
-- Export All Data for Runtime Estimation as CSV:
-- ============================================================================

CREATE VIEW ESTIMATE_DATA_CSV AS
SELECT R.SYS, R.BENCH, R.IMPL, F.FILE_ID, F.SEQ_NO,
       '"' ||   R.SYS       || '","' ||
		R.BENCH     || '","' ||
		R.IMPL      || '","' ||
		G.FILE_ID   || '","' ||
		G.NUM_NODES || '","' ||
		G.NUM_EDGES || '","' ||
		C.SIZE      || '","' ||
		C.INST      || '","' ||
		C.ITER      || '","' ||
		ROUND(R.REAL_T*1000) || '"' AS CSV
FROM	INPUT_GRAPH G, INPUT_FILE F, BENCH_COST C, BEST_RESULT R
WHERE   R.BENCH = C.BENCH
AND     R.FILE_ID = F.FILE_ID
AND     G.FILE_ID = F.FILE_ID
AND     C.FILE_ID = F.FILE_ID
UNION ALL
SELECT '' AS SYS, '' AS BENCH, '' AS IMPL, '' AS FILE_ID, 0 AS SEQ_NO,
       '"' ||   'SYS'       || '","' ||
		'BENCH'     || '","' ||
		'IMPL'      || '","' ||
		'FILE'      || '","' ||
		'NODES'     || '","' ||
		'EDGES'     || '","' ||
		'SIZE'      || '","' ||
		'COST'      || '","' ||
		'ITER'      || '","' ||
		'TIME_MS'   || '"' AS CSV
FROM DUMMY_TAB
ORDER BY SYS, BENCH, SEQ_NO;

-- ============================================================================
-- Load time per million edges in s:
-- ============================================================================

CREATE VIEW LOAD_EFFICIENCY AS
SELECT R.SYS, R.BENCH, R.IMPL, R.LOAD_T, G.NUM_EDGES,
		ROUND(R.LOAD_T * 1000 * 1000 / G.NUM_EDGES, 3)
			AS LOAD_PER_M_EDGES
FROM    INPUT_GRAPH G, INPUT_FILE F, BEST_RESULT R
WHERE   G.FILE_ID = F.FILE_ID AND R.FILE_ID = F.FILE_ID
ORDER BY R.SYS, R.BENCH, F.SEQ_NO;

-- ============================================================================
-- Search Range for Parameter Values:
-- ============================================================================

CREATE VIEW PAR_RANGE AS
SELECT R.SYS, G.FILE_ID,
		ROUND(R.REAL_T * 1000 * 1000 / G.NUM_EDGES, 3) AS PAR_LOAD,
		ROUND(R.REAL_T * 1000 * 1000 / C.INST, 3)      AS PAR_RULE,
		ROUND(R.REAL_T * 1000 * 1000 / C.SIZE, 3)      AS PAR_RESULT,
		ROUND(R.REAL_T * 1000 * 1000 / G.NUM_NODES, 3) AS PAR_DATA,
		ROUND(R.REAL_T * 1000 * 1000 / C.ITER, 3)      AS PAR_ITER
FROM    INPUT_GRAPH G, INPUT_FILE F, BENCH_COST C, BEST_RESULT R
WHERE   G.FILE_ID = F.FILE_ID
AND     R.FILE_ID = F.FILE_ID AND R.BENCH = 'tcff'
AND     C.FILE_ID = F.FILE_ID AND C.BENCH = 'tcff'
ORDER BY R.SYS, R.BENCH, F.SEQ_NO;



