-- ============================================================================
-- Project:	rbench - Logic Programming and Database Benchmarks
-- Filename:	db/v_tcff.sql
-- Purpose:	Benchmark DB: Views for Data Analysis of TCFF Benchmark Results
-- Last Change:	04.10.2019
-- Language:	SQL (Tested with PostgreSQL)
-- Author:	Stefan Brass
-- EMail:	brass@informatik.uni-halle.de
-- WWW:		http://www.informatik.uni-halle.de/~brass/
-- Address:	Feldschloesschen 15, D-06120 Halle (Saale), GERMANY
-- Copyright:	(c) 2018-2019 by Stefan Brass
-- License:	See file "LICENSE" for copying conditions.
-- Note:	There is no warranty at all - this code may contain bugs.
-- ============================================================================


-- ############################################################################
-- Views for Plots of Performance Data vs. Cost Measures (only for TCFF):
-- ############################################################################


-- ============================================================================
-- Plot of Real Time vs. Cost for TCFF Benchmark:
-- ============================================================================

-- This looks only at the runtimes of the standard program version,
-- best implementation, on the standard machine.

CREATE VIEW TCFF_PLOT_REAL AS
SELECT	R.SYS,
	R.FILE_ID,
	F.SEQ_NO AS NO,
	C.INST AS TC_COST,
	R.REAL_T,
	'"' || R.FILE_ID || '","' || C.INST || '","' || R.REAL_T || '"'
		AS CSV,
	R.FILE_ID || '	' || C.INST || '	' || R.REAL_T
		AS TSV
FROM	RESULT R, INPUT_FILE F, INPUT_GRAPH G, BENCH_COST C, BENCH_IMPL I
WHERE   R.BENCH = 'tcff' AND R.FILE_ID = F.FILE_ID
AND     G.FILE_ID = F.FILE_ID
AND     C.BENCH = 'tcff' AND C.FILE_ID = F.FILE_ID
AND	I.BENCH = 'tcff' AND I.IMPL = R.IMPL AND I.SYS = R.SYS
AND	I.STATUS = 'B'
UNION
SELECT  DISTINCT P.SYS, '', -1, 0 AS TC_COST, 0 AS REAL_T,
	'"df","cost","tavg"' as CSV,
	'df	cost	tavg' as TSV
FROM	TEST_SYS P
WHERE	EXISTS(SELECT * FROM RESULT R
		WHERE R.BENCH = 'tcff' AND R.SYS = P.SYS)
ORDER BY SYS, NO;

-- ============================================================================
-- Plot of CPU Time vs. Cost for TCFF Benchmark:
-- ============================================================================

-- This looks only at the runtimes of the standard program version,
-- best implementation, on the standard machine.

CREATE VIEW TCFF_PLOT_CPU AS
SELECT	R.SYS,
	R.FILE_ID,
	F.SEQ_NO AS NO,
	C.INST AS TC_COST,
	R.CPU_T,
	'"' || R.FILE_ID || '","' || C.INST || '","' || R.CPU_T || '"'
		AS CSV,
	R.FILE_ID || '	' || C.INST || '	' || R.CPU_T
		AS TSV
FROM	RESULT R, INPUT_FILE F, INPUT_GRAPH G, BENCH_COST C, BENCH_IMPL I
WHERE   R.BENCH = 'tcff' AND R.FILE_ID = F.FILE_ID
AND     G.FILE_ID = F.FILE_ID
AND     C.BENCH = 'tcff' AND C.FILE_ID = F.FILE_ID
AND	I.BENCH = 'tcff' AND I.IMPL = R.IMPL AND I.SYS = R.SYS
AND	I.STATUS = 'B'
UNION
SELECT  DISTINCT P.SYS, '', -1, 0 AS TC_COST, 0 AS CPU_T,
	'"df","cost","tavg"' as CSV,
	'df	cost	tavg' as TSV
FROM	TEST_SYS P
WHERE	EXISTS(SELECT * FROM RESULT R
		WHERE R.BENCH = 'tcff' AND R.SYS = P.SYS)
ORDER BY SYS, NO;

-- ============================================================================
-- Plot of Memory vs. Number of Tuples for TCFF Benchmark:
-- ============================================================================

-- This looks only at the runtimes of the standard program version,
-- best implementation, on the standard machine.

CREATE VIEW TCFF_PLOT_MEM AS
SELECT	R.SYS,
	R.FILE_ID,
	F.SEQ_NO AS NO,
	G.NUM_EDGES + C.SIZE AS FACTS,
	R.MEM_MB,
	'"' || R.FILE_ID || '","' || (G.NUM_EDGES+C.SIZE) || '","' ||
		R.MEM_MB || '"' AS CSV,
	R.FILE_ID || '	' || (G.NUM_EDGES+C.SIZE) || '	' ||
		R.MEM_MB AS TSV
FROM	RESULT R, INPUT_FILE F, INPUT_GRAPH G, BENCH_COST C, BENCH_IMPL I
WHERE   R.BENCH = 'tcff' AND R.FILE_ID = F.FILE_ID
AND     G.FILE_ID = F.FILE_ID
AND     C.BENCH = 'tcff' AND C.FILE_ID = F.FILE_ID
AND	I.BENCH = 'tcff' AND I.IMPL = R.IMPL AND I.SYS = R.SYS
AND	I.STATUS = 'B'
UNION
SELECT  DISTINCT P.SYS, '', -1, 0 AS FACTS, 0 AS CPU_T,
	'"df","facts","mem"' as CSV,
	'df	facts	mem' as TSV
FROM	TEST_SYS P
WHERE	EXISTS(SELECT * FROM RESULT R
		WHERE R.BENCH = 'tcff' AND R.SYS = P.SYS)
ORDER BY SYS, NO;


-- ############################################################################
-- Benchmark Results for TCFF - Comparison of Different Systems:
-- ############################################################################


-- ============================================================================
-- Benchmark Results for TCFF (Comparison of Various Systems on CPU Time):
-- ============================================================================

-- This looks only at the runtimes of the standard program version,
-- best implementation, on the standard machine.

CREATE VIEW TCFF_CPU AS
SELECT	BAM.FILE_ID,
	BAM.CPU_T AS BAM_CPU,
	XSB.CPU_T AS XSB_CPU,
	(CASE WHEN BAM.CPU_T=0 THEN NULL ELSE ROUND(XSB.CPU_T/BAM.CPU_T,1) END)
		AS XSB_F,
	YAP.CPU_T AS YAP_CPU,
	(CASE WHEN BAM.CPU_T=0 THEN NULL ELSE ROUND(YAP.CPU_T/BAM.CPU_T,1) END)
		AS YAP_F,
	JENA.CPU_T AS JENA_CPU,
	(CASE WHEN BAM.CPU_T=0 THEN NULL ELSE ROUND(JENA.CPU_T/BAM.CPU_T,1) END)
		AS JENA_F,
	SQLITE.CPU_T AS SQLITE_CPU,
	(CASE WHEN BAM.CPU_T=0 THEN NULL ELSE ROUND(SQLITE.CPU_T/BAM.CPU_T,1)
		END) AS SQLITE_F
FROM	(SELECT * FROM BEST_RESULT WHERE SYS='bam' AND BENCH='tcff') BAM
	LEFT JOIN
	(SELECT * FROM BEST_RESULT WHERE SYS='xsb' AND BENCH='tcff') XSB
	ON XSB.FILE_ID = BAM.FILE_ID
	LEFT JOIN
	(SELECT * FROM BEST_RESULT WHERE SYS='yap' AND BENCH='tcff') YAP
	ON YAP.FILE_ID = BAM.FILE_ID
	LEFT JOIN
	(SELECT * FROM BEST_RESULT WHERE SYS='jena' AND BENCH='tcff') JENA
	ON JENA.FILE_ID = BAM.FILE_ID
	LEFT JOIN
	(SELECT * FROM BEST_RESULT WHERE SYS='sqlite3' AND BENCH='tcff') SQLITE
	ON SQLITE.FILE_ID = BAM.FILE_ID
	JOIN INPUT_FILE F
        ON BAM.FILE_ID = F.FILE_ID
ORDER BY F.SEQ_NO;

-- ============================================================================
-- Benchmark Results for TCFF (Comparison of Various Systems on REAL Time):
-- ============================================================================

-- This looks only at the runtimes of the standard program version,
-- best implementation, on the standard machine.
-- This version contains the runtime factors compared with BAM.

CREATE VIEW TCFF_REAL AS
SELECT	BAM.FILE_ID,
        F.SEQ_NO,
	BAM.REAL_T AS BAM_REAL,
	XSB.REAL_T AS XSB_REAL,
	(CASE WHEN BAM.REAL_T=0 THEN NULL
		ELSE ROUND(XSB.REAL_T/BAM.REAL_T,1) END)
		AS XSB_F,
	YAP.REAL_T AS YAP_REAL,
	(CASE WHEN BAM.REAL_T=0 THEN NULL
		ELSE ROUND(YAP.REAL_T/BAM.REAL_T,1) END)
		AS YAP_F,
	SQLITE.REAL_T AS SQLITE_REAL,
	(CASE WHEN BAM.REAL_T=0 THEN NULL
		ELSE ROUND(SQLITE.REAL_T/BAM.REAL_T,1) END)
		AS SQLITE_F,
	PSQL.REAL_T AS PSQL_REAL,
	(CASE WHEN BAM.REAL_T=0 THEN NULL
		ELSE ROUND(PSQL.REAL_T/BAM.REAL_T,1) END)
		AS PSQL_F,
	MARIA.REAL_T AS MARIA_REAL,
	(CASE WHEN BAM.REAL_T=0 THEN NULL
		ELSE ROUND(MARIA.REAL_T/BAM.REAL_T,1) END)
		AS MARIA_F,
	JENA.REAL_T AS JENA_REAL,
	(CASE WHEN BAM.REAL_T=0 THEN NULL
		ELSE ROUND(JENA.REAL_T/BAM.REAL_T,1) END)
		AS JENA_F,
	SOUFFLE.REAL_T AS SOUFFLE_REAL,
	(CASE WHEN BAM.REAL_T=0 THEN NULL
		ELSE ROUND(SOUFFLE.REAL_T/BAM.REAL_T,1) END)
		AS SOUFFLE_F
FROM	(SELECT * FROM BEST_RESULT WHERE SYS='bam' AND BENCH='tcff') BAM
	LEFT JOIN
	(SELECT * FROM BEST_RESULT WHERE SYS='xsb' AND BENCH='tcff') XSB
	ON XSB.FILE_ID = BAM.FILE_ID
	LEFT JOIN
	(SELECT * FROM BEST_RESULT WHERE SYS='yap' AND BENCH='tcff') YAP
	ON YAP.FILE_ID = BAM.FILE_ID
	LEFT JOIN
	(SELECT * FROM BEST_RESULT WHERE SYS='sqlite3' AND BENCH='tcff') SQLITE
	ON SQLITE.FILE_ID = BAM.FILE_ID
	LEFT JOIN
	(SELECT * FROM BEST_RESULT WHERE SYS='psql' AND BENCH='tcff') PSQL
	ON PSQL.FILE_ID = BAM.FILE_ID
	LEFT JOIN
	(SELECT * FROM BEST_RESULT WHERE SYS='mariadb' AND BENCH='tcff') MARIA
	ON MARIA.FILE_ID = BAM.FILE_ID
	LEFT JOIN
	(SELECT * FROM BEST_RESULT WHERE SYS='jena' AND BENCH='tcff') JENA
	ON JENA.FILE_ID = BAM.FILE_ID
	LEFT JOIN
	(SELECT * FROM BEST_RESULT WHERE SYS='souffle' AND BENCH='tcff')SOUFFLE
	ON SOUFFLE.FILE_ID = BAM.FILE_ID
	JOIN INPUT_FILE F
        ON BAM.FILE_ID = F.FILE_ID
ORDER BY F.SEQ_NO;

-- ============================================================================
-- Benchmark Results for TCFF (Comparison on REAL Time, Only Factors):
-- ============================================================================

-- This looks only at the runtimes of the standard program version,
-- best implementation, on the standard machine.

-- It is a projection of TCFF_REAL that shows only the factors compared to BAM.
-- This makes the output rows a little smaller.

CREATE VIEW TCFF_REAL_FACTORS AS
SELECT	FILE_ID, BAM_REAL,
	XSB_F, YAP_F, SQLITE_F, PSQL_F, MARIA_F, JENA_F, SOUFFLE_F
FROM	TCFF_REAL
ORDER BY SEQ_NO;

-- ============================================================================
-- Benchmark Results for TCFF (Comparison of Various Systems on REAL Time):
-- ============================================================================

-- This looks only at the runtimes of the standard program version,
-- best implementation, on the standard machine.

CREATE VIEW TCFF_REAL_LATEX AS
SELECT	FILE_ID,
	RPAD(REPLACE(FILE_ID, '_', '\_'), 12) || '&' ||
	TO_CHAR(BAM_REAL, '9990.999') || '&' ||
	(CASE WHEN XSB_F IS NULL THEN '      '
		ELSE TO_CHAR(XSB_F, '990.9') END) || '&' ||
	(CASE WHEN YAP_F IS NULL THEN '      '
		ELSE TO_CHAR(YAP_F, '990.9') END) || '&' ||
	(CASE WHEN SQLITE_F IS NULL THEN '      '
		ELSE TO_CHAR(SQLITE_F, '990.9') END) || '&' ||
	(CASE WHEN PSQL_F IS NULL THEN '      '
		ELSE TO_CHAR(PSQL_F, '990.9') END) || '&' ||
	(CASE WHEN MARIA_F IS NULL THEN '      '
		ELSE TO_CHAR(MARIA_F, '990.9') END) || '&' ||
	(CASE WHEN JENA_F IS NULL THEN '      '
		ELSE TO_CHAR(JENA_F, '990.9') END) || '&' ||
	(CASE WHEN SOUFFLE_F IS NULL THEN '      '
		ELSE TO_CHAR(SOUFFLE_F, '990.9') END) || '\\'
	AS LATEX
FROM	TCFF_REAL
ORDER BY SEQ_NO;

-- ============================================================================
-- Benchmark Results for TCFF (Comparison of Various Systems on REAL Time):
-- ============================================================================

-- This looks only at the runtimes of the standard program version,
-- best implementation, on the standard machine.

-- This version contains the runtime factors compared with XSB
-- and rounds the factors to two decimal places after the decimal point.
-- Otherwise it is identical to TCFF_REAL (which compares with BAM).

CREATE VIEW TCFF_REAL_XSB AS
SELECT	XSB.FILE_ID,
        F.SEQ_NO,
	XSB.REAL_T AS XSB_REAL,
	BAM.REAL_T AS BAM_REAL,
	(CASE WHEN XSB.REAL_T=0 THEN NULL
		ELSE ROUND(BAM.REAL_T/XSB.REAL_T,2) END)
		AS BAM_F,
	YAP.REAL_T AS YAP_REAL,
	(CASE WHEN XSB.REAL_T=0 THEN NULL
		ELSE ROUND(YAP.REAL_T/XSB.REAL_T,2) END)
		AS YAP_F,
	SQLITE.REAL_T AS SQLITE_REAL,
	(CASE WHEN XSB.REAL_T=0 THEN NULL
		ELSE ROUND(SQLITE.REAL_T/XSB.REAL_T,2) END)
		AS SQLITE_F,
	PSQL.REAL_T AS PSQL_REAL,
	(CASE WHEN XSB.REAL_T=0 THEN NULL
		ELSE ROUND(PSQL.REAL_T/XSB.REAL_T,2) END)
		AS PSQL_F,
	MARIA.REAL_T AS MARIA_REAL,
	(CASE WHEN XSB.REAL_T=0 THEN NULL
		ELSE ROUND(MARIA.REAL_T/XSB.REAL_T,2) END)
		AS MARIA_F,
	JENA.REAL_T AS JENA_REAL,
	(CASE WHEN XSB.REAL_T=0 THEN NULL
		ELSE ROUND(JENA.REAL_T/XSB.REAL_T,2) END)
		AS JENA_F,
	SOUFFLE.REAL_T AS SOUFFLE_REAL,
	(CASE WHEN XSB.REAL_T=0 THEN NULL
		ELSE ROUND(SOUFFLE.REAL_T/XSB.REAL_T,2) END)
		AS SOUFFLE_F
FROM	(SELECT * FROM BEST_RESULT WHERE SYS='xsb' AND BENCH='tcff') XSB
	LEFT JOIN
	(SELECT * FROM BEST_RESULT WHERE SYS='bam' AND BENCH='tcff') BAM
	ON BAM.FILE_ID = XSB.FILE_ID
	LEFT JOIN
	(SELECT * FROM BEST_RESULT WHERE SYS='yap' AND BENCH='tcff') YAP
	ON YAP.FILE_ID = XSB.FILE_ID
	LEFT JOIN
	(SELECT * FROM BEST_RESULT WHERE SYS='sqlite3' AND BENCH='tcff') SQLITE
	ON SQLITE.FILE_ID = XSB.FILE_ID
	LEFT JOIN
	(SELECT * FROM BEST_RESULT WHERE SYS='psql' AND BENCH='tcff') PSQL
	ON PSQL.FILE_ID = XSB.FILE_ID
	LEFT JOIN
	(SELECT * FROM BEST_RESULT WHERE SYS='mariadb' AND BENCH='tcff') MARIA
	ON MARIA.FILE_ID = XSB.FILE_ID
	LEFT JOIN
	(SELECT * FROM BEST_RESULT WHERE SYS='jena' AND BENCH='tcff') JENA
	ON JENA.FILE_ID = XSB.FILE_ID
	LEFT JOIN
	(SELECT * FROM BEST_RESULT WHERE SYS='souffle' AND BENCH='tcff')SOUFFLE
	ON SOUFFLE.FILE_ID = XSB.FILE_ID
	JOIN INPUT_FILE F
        ON XSB.FILE_ID = F.FILE_ID
ORDER BY F.SEQ_NO;

-- ============================================================================
-- Benchmark Results for TCFF (Comparison on REAL Time, Only Factors):
-- ============================================================================

-- This looks only at the runtimes of the standard program version,
-- best implementation, on the standard machine.

-- It is a projection of TCFF_REAL that shows only the factors
-- compared to XSB.
-- This makes the output rows a little smaller.

CREATE VIEW TCFF_REAL_XSB_FACTORS AS
SELECT	FILE_ID, XSB_REAL,
	BAM_F, YAP_F, SQLITE_F, PSQL_F, MARIA_F, JENA_F, SOUFFLE_F
FROM	TCFF_REAL_XSB
ORDER BY SEQ_NO;


-- ============================================================================
-- Benchmark Results for TCFF (Average Factors Compared to XSB):
-- ============================================================================

-- This looks only at the runtimes of the standard program version,
-- best implementation, on the standard machine.

CREATE VIEW TCFF_REAL_XSB_AVG AS
SELECT ROUND(AVG(BAM_F),2) AS BAM,
       ROUND(AVG(YAP_F),2) AS YAP,
       ROUND(AVG(SQLITE_F),2) AS SQLITE,
       ROUND(AVG(PSQL_F),2) AS PSQL,
       ROUND(AVG(MARIA_F),2) AS MARIA,
       ROUND(AVG(JENA_F),2) AS JENA,
       ROUND(AVG(SOUFFLE_F),2) AS SOUFFLE
FROM   TCFF_REAL_XSB;


-- ============================================================================
-- Benchmark Results for TCFF (Comparison of Various Systems on REAL Time):
-- ============================================================================

-- This looks only at the runtimes of the standard program version,
-- best implementation, on the standard machine.

CREATE VIEW TCFF_REAL_XSB_LATEX AS
SELECT	FILE_ID,
	RPAD(REPLACE(FILE_ID, '_', '\_'), 10) || '&' ||
	TO_CHAR(XSB_REAL, '9990.999') || '&' ||
	(CASE WHEN BAM_F IS NULL     THEN '       '
		ELSE TO_CHAR(BAM_F, '990.99') END) || '&' ||
	(CASE WHEN YAP_F IS NULL     THEN '       '
		ELSE TO_CHAR(YAP_F, '990.99') END) || '&' ||
	(CASE WHEN SQLITE_F IS NULL  THEN '       '
		ELSE TO_CHAR(SQLITE_F, '990.99') END) || '&' ||
	(CASE WHEN PSQL_F IS NULL    THEN '       '
		ELSE TO_CHAR(PSQL_F, '990.99') END) || '&' ||
	(CASE WHEN MARIA_F IS NULL   THEN '       '
		ELSE TO_CHAR(MARIA_F, '990.99') END) || '&' ||
	(CASE WHEN JENA_F IS NULL    THEN '       '
		ELSE TO_CHAR(JENA_F, '990.99') END) || '&' ||
	(CASE WHEN SOUFFLE_F IS NULL THEN '       '
		ELSE TO_CHAR(SOUFFLE_F, '990.99') END) || '\\'
	AS LATEX
FROM	TCFF_REAL_XSB
ORDER BY SEQ_NO;


-- ============================================================================
-- Benchmark Results for TCFF (Comparison of Various Systems on REAL Time):
-- ============================================================================

-- This looks only at the runtimes of the standard program version,
-- best implementation, on the standard machine.

CREATE VIEW TCFF_REAL_XSB_HTML AS
SELECT -2 AS SEQ_NO, NULL AS FILE_ID,
	'<table>' AS HTML
FROM    DUMMY_TAB
UNION ALL
SELECT -1 AS SEQ_NO, NULL AS FILE_ID,
	'<tr>' ||
	     '<th class="colGraph">'   || 'Graph'          || '</th>' || '
	' || '<th class="colXSBTime">' || 'XSB (s)'        || '</th>' || '
	' || '<th class="colBAM"    >' || 'BAM'            || '</th>' || '
	' || '<th class="colYAP"    >' || 'YAP'            || '</th>' || '
	' || '<th class="colSQLite" >' || 'SQLite'         || '</th>' || '
	' || '<th class="colPSQL"   >' || 'PostgreSQL'     || '</th>' || '
	' || '<th class="colMaria"  >' || 'MariaDB'        || '</th>' || '
	' || '<th class="colJena"   >' || 'Jena'           || '</th>' || '
	' || '<th class="colSouffle">' || 'Souffl&eacute;' || '</th>' || '
	' || '</tr>'
	AS HTML
FROM    DUMMY_TAB
UNION ALL
SELECT	SEQ_NO, FILE_ID,
	'<tr>' ||
	'<td class="colGraph">' || FILE_ID || '</td>' || '
	' || '<td class="colXSBTime">' || TO_CHAR(XSB_REAL, '9990.999') ||
		'&nbsp;s' || '</td>' || '
	' || '<td class="colBAM"    >' ||
		(CASE WHEN BAM_F IS NULL     THEN '&nbsp; '
			ELSE TO_CHAR(BAM_F,     '990.99') END) || '</td>' || '
	' || '<td class="colYAP"    >' ||
		(CASE WHEN YAP_F IS NULL     THEN '&nbsp; '
			ELSE TO_CHAR(YAP_F,     '990.99') END) || '</td>' || '
	' || '<td class="colSQLite" >' ||
		(CASE WHEN SQLITE_F IS NULL  THEN '&nbsp; '
			ELSE TO_CHAR(SQLITE_F,  '990.99') END) || '</td>' || '
	' || '<td class="colPSQL"   >' ||
		(CASE WHEN PSQL_F IS NULL    THEN '&nbsp; '
			ELSE TO_CHAR(PSQL_F,    '990.99') END) || '</td>' || '
	' || '<td class="colMaria"  >' ||
		(CASE WHEN MARIA_F IS NULL   THEN '&nbsp; '
			ELSE TO_CHAR(MARIA_F,   '990.99') END) || '</td>' || '
	' || '<td class="colJena"   >' ||
		(CASE WHEN JENA_F IS NULL    THEN '&nbsp; '
			ELSE TO_CHAR(JENA_F,    '990.99') END) || '</td>' || '
	' || '<td class="colSouffle">' ||
		(CASE WHEN SOUFFLE_F IS NULL THEN '&nbsp; '
			ELSE TO_CHAR(SOUFFLE_F, '990.99') END) || '</td>' || '
	' || '</tr>'
	AS HTML
FROM	TCFF_REAL_XSB
UNION ALL
SELECT 1000000 AS SEQ_NO, NULL AS FILE_ID,
	'</table>' AS HTML
FROM    DUMMY_TAB
ORDER BY SEQ_NO;


-- ============================================================================
-- Benchmark Results for TCFF (Comparison of Various Systems on Memory):
-- ============================================================================

-- This looks only at the runtimes of the standard program version,
-- best implementation, on the standard machine.

CREATE VIEW TCFF_MEM AS
SELECT	BAM.FILE_ID,
	BAM.MEM_MB AS BAM_MEM,
	XSB.MEM_MB AS XSB_MEM,
	(CASE WHEN BAM.MEM_MB=0 THEN NULL
		ELSE ROUND(XSB.MEM_MB/BAM.MEM_MB,1) END)
		AS XSB_F,
	YAP.MEM_MB AS YAP_MEM,
	(CASE WHEN BAM.MEM_MB=0 THEN NULL
		ELSE ROUND(YAP.MEM_MB/BAM.MEM_MB,1) END)
		AS YAP_F,
	JENA.MEM_MB AS JENA_MEM,
	(CASE WHEN BAM.MEM_MB=0 THEN NULL
		ELSE ROUND(JENA.MEM_MB/BAM.MEM_MB,1) END)
		AS JENA_F,
	SQLITE.MEM_MB AS SQLITE_MEM,
	(CASE WHEN BAM.MEM_MB=0 THEN NULL
		ELSE ROUND(SQLITE.MEM_MB/BAM.MEM_MB,1) END)
		AS SQLITE_F
FROM	(SELECT * FROM BEST_RESULT WHERE SYS='bam' AND BENCH='tcff') BAM
	LEFT JOIN
	(SELECT * FROM BEST_RESULT WHERE SYS='xsb' AND BENCH='tcff') XSB
	ON XSB.FILE_ID = BAM.FILE_ID
	LEFT JOIN
	(SELECT * FROM BEST_RESULT WHERE SYS='yap' AND BENCH='tcff') YAP
	ON YAP.FILE_ID = BAM.FILE_ID
	LEFT JOIN
	(SELECT * FROM BEST_RESULT WHERE SYS='jena' AND BENCH='tcff') JENA
	ON JENA.FILE_ID = BAM.FILE_ID
	LEFT JOIN
	(SELECT * FROM BEST_RESULT WHERE SYS='sqlite3' AND BENCH='tcff') SQLITE
	ON SQLITE.FILE_ID = BAM.FILE_ID
	JOIN INPUT_FILE F
        ON BAM.FILE_ID = F.FILE_ID
ORDER BY F.SEQ_NO;

-- ============================================================================
-- Benchmark Results for TCFF (Comparison of Various Systems on CPU Time):
-- ============================================================================

-- This looks only at the runtimes of the standard program version,
-- best implementation, on the standard machine.

CREATE VIEW TCFF_CPU_LATEX AS
SELECT	BAM.FILE_ID,
	RPAD(REPLACE(BAM.FILE_ID, '_', '\_'), 12) || '&' ||
	TO_CHAR(BAM.CPU_T, '9990.999') || '&' ||
	(CASE WHEN XSB.CPU_T IS NULL THEN '         &       ' ELSE
		(TO_CHAR(XSB.CPU_T, '9990.999') || '&' ||
		(CASE WHEN BAM.CPU_T=0 THEN '       '
			ELSE TO_CHAR(XSB.CPU_T/BAM.CPU_T, '9990.9') END))
	END) || '&' ||
	(CASE WHEN YAP.CPU_T IS NULL THEN '         &       ' ELSE
		(TO_CHAR(YAP.CPU_T, '9990.999') || '&' ||
		(CASE WHEN BAM.CPU_T=0 THEN '       '
			ELSE TO_CHAR(YAP.CPU_T/BAM.CPU_T, '9990.9') END))
	END) || '&' ||
	(CASE WHEN JENA.CPU_T IS NULL THEN '         &       ' ELSE
		(TO_CHAR(JENA.CPU_T,'9990.999') || '&' ||
		(CASE WHEN BAM.CPU_T=0 THEN '       '
			ELSE TO_CHAR(JENA.CPU_T/BAM.CPU_T, '9990.9') END))
	END) || '&' ||
	(CASE WHEN SQLITE.CPU_T IS NULL THEN '         &       ' ELSE
		(TO_CHAR(SQLITE.CPU_T,'9990.999') || '&' ||
		(CASE WHEN BAM.CPU_T=0 THEN '       '
			ELSE TO_CHAR(SQLITE.CPU_T/BAM.CPU_T, '9990.9') END))
	END) || '\\'
	AS LATEX
FROM	(SELECT * FROM BEST_RESULT WHERE SYS='bam' AND BENCH='tcff') BAM
	LEFT JOIN
	(SELECT * FROM BEST_RESULT WHERE SYS='xsb' AND BENCH='tcff') XSB
	ON XSB.FILE_ID = BAM.FILE_ID
	LEFT JOIN
	(SELECT * FROM BEST_RESULT WHERE SYS='yap' AND BENCH='tcff') YAP
	ON YAP.FILE_ID = BAM.FILE_ID
	LEFT JOIN
	(SELECT * FROM BEST_RESULT WHERE SYS='jena' AND BENCH='tcff') JENA
	ON JENA.FILE_ID = BAM.FILE_ID
	LEFT JOIN
	(SELECT * FROM BEST_RESULT WHERE SYS='sqlite3' AND BENCH='tcff') SQLITE
	ON SQLITE.FILE_ID = BAM.FILE_ID
	JOIN INPUT_FILE F
	ON F.FILE_ID = BAM.FILE_ID
ORDER BY F.SEQ_NO;

