// ============================================================================
// Project:	rbench - Logic Programming and Database Benchmarks
// Filename:	graph/graph.cpp
// Purpose:	Alternative Graph Generator
// Last Change:	04.10.2019
// Language:	C++
// EMail:	brass@informatik.uni-halle.de
// WWW:		http://www.informatik.uni-halle.de/~brass/
// Address:	Feldschloesschen 15, D-06120 Halle (Saale), GERMANY
// Copyright:	(c) 2019 by Stefan Brass
// License:	See file "LICENSE" for copying conditions.
// Note:	There is no warranty at all - this code may contain bugs.
// ============================================================================

// Please note: This is a quick&dirty solution.
// I am not particularly proud on the programming style.
//
// This must be compiled with the C++11 Standard, because it uses int64_t.
// E.g.:
//	g++ -std=c++11 -Wall -O2 -o graph graph.cpp

//-----------------------------------------------------------------------------
// Important:
//-----------------------------------------------------------------------------

// We have two programs for generating graphs!
// The other program is available at:
// https://gitlab.informatik.uni-halle.de/mwenzel/graphgen
// Most of our graphs have been generated with that program.
// I needed some new graphs quickly while the author of the other program
// was in holidays, therefore I started my own small graph generator.

//-----------------------------------------------------------------------------
// Usage:
//-----------------------------------------------------------------------------

// The first command line argument is the graph ID.
// The graph ID consists of a letter (lowercase or uppercase are equivalent)
// and one or two parameters (natural numbers).
// If there are two parameters, they are separated with "_".
// For instance, M1000_10 is a "Multipath" graph that consists of 10 paths
// of length 1000.

// The second and following command line parameters are the output files.
// Currently, the extension of the output files must be .P or .tsv.
// For .P, Prolog facts are generated,
// for .tsv, Tab-separated values.

// This program can generate the following graphs:
// -----------------------------------------------

// Bn: Binary tree of height n
//       2^n - 1 nodes, 2^n - 2 edges.
//       N = {1, ..., 2^n - 1}
//       E = {(i, 2*i)     | i = 1, ..., 2^{n-1} - 1} u
//           {(i, 2*i + 1) | i = 1, ..., 2^{n-1} - 1}

// Cn: Cycle with n nodes
//       n nodes, n edges.
//       N = {1, ..., n}
//       E = {(i, i+1) | i = 1, ..., n-1} u {(n, 1)}  

// Dn_k: Diamond (k copies)
// Dn:   Diamond (k = 1, i.e. one copy)
//       Each diamond consists of a bottom node, e.g. 1 for the first diamond,
//       from there edges to n nodes (e.g. 2, ..., n+1),
//       and from each of these nodes edges to a top node (e.g. n+2).
//       k * (n+2) nodes, k * 2*n edges
//       N = {1, ..., k * (n+2)}
//       E = { (i*(n+2) + 1, i*(n+2) + 1 + j) | i = 0,...,k; j = 1,...,n} u
//           { (i*(n+2) + j, i*(n+2) + n + 2) | i = 0,...,k; j = 1,...,n} u

// Kn: Complete graph with n nodes.
//       n nodes, n^2 edges.
//       N = {1, ..., n}
//       E = {(i, j) | i = 1, ..., n;  j = 1, ..., n }

// Mn_k: Multipath: k paths of length m.
//       n*k nodes, (n-1)*k edges.
//       V = {1, ..., n*k}
//       E = {(i, i+k) | i = 1, ..., (n-1)*k}

// Pn:   Path of length n
//       n nodes, n-1 edges.
//       V = {1, ..., n}
//       E = {(i, i+1) | i = 1, ..., n-1}

// Sn_k: Shortcut graph (cycle of length n with shortcuts)
//       It is required that n is divisible by k+1.
//       Each node has out degree k+1.
//       E.g. from node 1, there is one edge to node 2 (as in standard cycle),
//       then one edge to node 2 + (n/(k+1)),
//       then one edge to node 2 + 2 * (n/(k+1)),
//       and so on until node  2 + k * (n/(k+1)).
//       The next value in the sequence, 2 + (k+1) * (n/(k+1)) modulo n is 2.
//       For instance, if k=1, there is one shortcut edge spanning half the
//       cycle, e.g. node 1 has connections to node 2 and node 2 + n/2.
//       For n=10 and k=2, the edges are (1,2), (1,7), (2,3), (2,8), ...
//       The graph has n nodes, n * (k+1) edges.
//       N = {1, ..., n}
//       E = {(i, i+1) | i = 1, ..., n-1} u {(n, 1)} u // Standard Cycle
//           {(i, 1 + ((i + (n*j)/(k+1)) mod n) | i = 1,...,n; j=1,...,k }

// Tn:   Total order graph (maximum acyclic graph)
//       n vertices, n * (n-1) / 2 edges
//       V = {1, ..., n}
//       E = {(i, j) | i = 1, ..., n-1; j = i+1, ..., n }

// Un_k: Random Graph, Uniform Distribution of Node Degrees
//       n nodes, k edges

// Vn:   InVerted binary tree of height n (looks like V if edges point down)
//       2^n - 1 nodes, 2^n - 2 edges.
//       N = {1, ..., 2^n - 1}
//       E = {(2*i, i)     | i = 1, ..., 2^{n-1} - 1} u
//           {(2*i + 1, i) | i = 1, ..., 2^{n-1} - 1}


// Wn_k: Single level of edges, mesh, 2n nodes, degree k):
// Wn:   Abbreviation for Wn_n (n nodes connected to all n other nodes)
//       2*n nodes, n*k edges
//       V = {1, ..., 2n}
//       E = { (i, (i+j-1 mod n)+n+1) | 1 <= i <= n and 0 <= j < k }

// Xn_k: X-graph (n edges to central node, k from there)
// Xn:   Abbreviation for Xn_n
//       n+k+1 nodes, n+k edges
//       V = {1, ..., n+k+1}
//       E = {(i, n+1) | i = 1, ..., n} u {(n+1, n+1+j) | j = 1, ..., k}

// Yn_k: Y-graph (n nodes point to central node, then path of length k)
//       n+k nodes, n+k-1 edges:
//       V = {1,...,n+k}
//       E = {(i, n+1)} | i = 1,...,n} u {(n+i, n+i+1) | i = 1,...,k-1}

// The program code for graph g is in the procedure gen_graph_g.

//=============================================================================
// Buffer Size for Graph Parameter:
//=============================================================================

#define MAX_PAR_CHARS	16

//=============================================================================
// Maximum Number of Output Files:
//=============================================================================

#define MAX_OUT_FILES	4

//=============================================================================
// Name of the predicate or relation for the edges:
//=============================================================================

#define PRED_NAME	"par"

//=============================================================================
// Should Test/Debug Output be Written?
//=============================================================================

// 0 means no test output, 1 means test output:
#define TEST_OUTPUT 0

//=============================================================================
// Generate new version of S-Graph?
//=============================================================================

// 0 means old version (Mario Wenzel), 1 means new version (Stefan Brass):
#define S_GRAPH_NEW_VER 0

//=============================================================================
// Name of file with INSERT statements for benchmark cost information:
//=============================================================================

#define COST_FILE "bench_cost.sql"

//=============================================================================
// Name of table with benchmark cost information:
//=============================================================================

#define COST_TABLE "BENCH_COST"

//=============================================================================
// How many bytes does the bitstring in one element of the adjacency list have?
//=============================================================================

//#define BITSTRING_BYTES 4
#define BITSTRING_BYTES 32

//=============================================================================
// Should Assertions be tested or is speed of highest importance?
//=============================================================================

// If this is defined, asserstions are skipped (faster code):
//#define NDEBUG

//=============================================================================
// Include Files:
//=============================================================================

#include <iostream>
#include <fstream>
#include <cstdint>
#include <time.h>
#include <assert.h>

//=============================================================================
// Type for Large Integers (64 Bit):
//=============================================================================

typedef int64_t bigint_t;

//=============================================================================
// Byte Size:
//=============================================================================

#define BYTE_SIZE 8

//=============================================================================
// String Data Type:
//=============================================================================

typedef const char *str_t;

#define STR_NULL (static_cast<str_t>(0))

//-----------------------------------------------------------------------------
// str_eq: Compare two strings for equality.
//-----------------------------------------------------------------------------

bool	str_eq(str_t str1, str_t str2)
{
	// Avoid crashes:
	if(str1 == nullptr)
		return(str2 == nullptr);
	else if(str2 == nullptr)
		return(false);

	// Compare strings:
	while(*str1 && *str1==*str2) {
		str1++;
		str2++;
	}
	return(*str1==*str2);
}

//-----------------------------------------------------------------------------
// str_cpy: Secure string copy.
//-----------------------------------------------------------------------------

void	str_cpy(char* buf, int len, str_t str)
{
	int i;

	// Avoid crashes:
	if(buf == 0)
		return;
	else if(str == 0) {
		if(len >= 6) {
			*buf++ = '<';
			*buf++ = 'N';
			*buf++ = 'I';
			*buf++ = 'L';
			*buf++ = '>';
			*buf = '\0';
		}
		else if(len >= 1) {
			*buf = '\0';
		}
	}

	// Copy as much as possible:
	i = len;
	while(--i > 0 && *str)
		*buf++ = *str++;
	*buf = '\0';

	// Mark incomplete copy:
	if(*str && len >= 4) {
		buf[-3] = '.';
		buf[-2] = '.';
		buf[-1] = '.';
	}
}


//-----------------------------------------------------------------------------
// str_len: Determine string length.
//-----------------------------------------------------------------------------

int	str_len(str_t str)
{
	if(str == 0)
		return 0;

	int n = 0;
	while(*str++)
		n++;
	return n;
}

//-----------------------------------------------------------------------------
// is_digit: Classify characters as digits:
//-----------------------------------------------------------------------------

bool is_digit(char c) {
	switch(c) {
		case '0':
		case '1':
		case '2':
		case '3':
		case '4':
		case '5':
		case '6':
		case '7':
		case '8':
		case '9':
			return true;
		default:
			return false;
	}
}

//-----------------------------------------------------------------------------
// digit_val: Return value of a digit.
//-----------------------------------------------------------------------------

int digit_val(char digit) {
	switch(digit) {
		case '0':
			return 0;
		case '1':
			return 1;
		case '2':
			return 2;
		case '3':
			return 3;
		case '4':
			return 4;
		case '5':
			return 5;
		case '6':
			return 6;
		case '7':
			return 7;
		case '8':
			return 8;
		case '9':
			return 9;
		default:
			std::cerr << "digit_val called with non-digit: '" <<
					digit << "'\n";
			exit(1);
			return -1;
	}
}

//-----------------------------------------------------------------------------
// str_int: Translate digit sequence to int value:
//-----------------------------------------------------------------------------

long str_int(str_t digits) {
	int result = 0;
	for(str_t p = digits; *p; p++) {
		int prev_result = result;
		result = result * 10 + digit_val(*p);
		if(result/10 != prev_result) {
			std::cerr << "Overflow!\n";
			exit(2);
		}
	}
	return result;
}


//=============================================================================
// Class for Graph/Benchmark Size Measures:
//=============================================================================

class GSize {
	public:

	// Constructor:
	GSize()
	{
		num_nodes_  = -1;
		num_edges_  = -1;
		tc_size_    = -1;
		tc_iter_    = -1;
		tc_inst_    = -1;
		sg_size_    = -1;
		sg_iter_    = -1;
		sg_inst_    = -1;
	}

	// Accessor Functions:
	inline int      num_nodes() const { return num_nodes_; }
	inline int      num_edges() const { return num_edges_; }
	inline bigint_t tc_size()   const { return tc_size_; }
	inline int      tc_iter()   const { return tc_iter_; }
	inline bigint_t tc_inst()   const { return tc_inst_; }
	inline bigint_t sg_size()   const { return sg_size_; }
	inline int      sg_iter()   const { return sg_iter_; }
	inline bigint_t sg_inst()   const { return sg_inst_; }

	// Set functions:
	inline void set_num_nodes(int n)    { num_nodes_ = n; }
	inline void set_num_edges(int n)    { num_edges_ = n; }
	inline void set_tc_size(bigint_t n) { tc_size_   = n; }
	inline void set_tc_iter(int n)      { tc_iter_   = n; }
	inline void set_tc_inst(bigint_t n) { tc_inst_   = n; }
	inline void set_sg_size(bigint_t n) { sg_size_   = n; }
	inline void set_sg_iter(int n)      { sg_iter_   = n; }
	inline void set_sg_inst(bigint_t n) { sg_inst_   = n; }

	// Print defined values:
	void print() {
		if(num_nodes_ >= 0)
			std::cout << "\tNodes:   " << num_nodes_ << "\n";
		if(num_edges_ >= 0)
			std::cout << "\tEdges:   " << num_edges_ << "\n";
		if(tc_iter_ >= 0)
			std::cout << "\tTC Iter: " << tc_iter_ << "\n";
		if(tc_size_ >= 0)
			std::cout << "\tTC Size: " << tc_size_ << "\n";
		if(tc_inst_ >= 0)
			std::cout << "\tTC Inst: " << tc_inst_ << "\n";
		if(sg_iter_ >= 0)
			std::cout << "\tSG Iter: " << sg_iter_ << "\n";
		if(sg_size_ >= 0)
			std::cout << "\tSG Size: " << sg_size_ << "\n";
		if(sg_inst_ >= 0)
			std::cout << "\tSG Inst: " << sg_inst_ << "\n";
	}

	// Check consistency of two graph size objects:
	bool consistent(GSize *gsize) {

		// Compare number of nodes (if defined in both objects):
		if(num_nodes_ != -1 && gsize->num_nodes_ != -1) {
			if(num_nodes_ != gsize->num_nodes_)
				return false;
		}

		// Compare number of edges:
		if(num_edges_ != -1 && gsize->num_edges_ != -1) {
			if(num_edges_ != gsize->num_edges_)
				return false;
		}

		// Compare transitive closure iterations:
		if(tc_iter_ != -1 && gsize->tc_iter_ != -1) {
			if(tc_iter_ != gsize->tc_iter_)
				return false;
		}

		// Compare transitive closure size:
		if(tc_size_ != -1 && gsize->tc_size_ != -1) {
			if(tc_size_ != gsize->tc_size_)
				return false;
		}

		// Compare transitive closure rule instances:
		if(tc_inst_ != -1 && gsize->tc_inst_ != -1) {
			if(tc_inst_ != gsize->tc_inst_)
				return false;
		}

		// Compare same generation iterations:
		if(sg_iter_ != -1 && gsize->sg_iter_ != -1) {
			if(sg_iter_ != gsize->sg_iter_)
				return false;
		}

		// Compare same generation size:
		if(sg_size_ != -1 && gsize->sg_size_ != -1) {
			if(sg_size_ != gsize->sg_size_)
				return false;
		}

		// Compare same generation rule instances:
		if(sg_inst_ != -1 && gsize->sg_inst_ != -1) {
			if(sg_inst_ != gsize->sg_inst_)
				return false;
		}

		// Ok:
		return true;
	}

	// Attributes:
	private:
	int      num_nodes_;
	int      num_edges_;
	bigint_t tc_size_;
	int      tc_iter_;
	bigint_t tc_inst_;
	bigint_t sg_size_;
	int      sg_iter_;
	bigint_t sg_inst_;
};

//-----------------------------------------------------------------------------
// Pointer Type for Graph Size Data:
//-----------------------------------------------------------------------------

typedef GSize *gsize_t;

#define GSIZE_NULL (static_cast<gsize_t>(0))


//=============================================================================
// Type for Nodes in a Graph:
//=============================================================================

// In order to make the algorithms for computing problem sizes independent
// from the data structures used for representing graphs, we define a type
// for graph nodes here.
// Currently, the graph nodes are represented as positive integers starting
// from 1.
// The graph data structures use a linked list of bitmaps to represent
// sets of nodes.
// These bitmaps could also represent the number 0, which is not used.
// However, it would complicate the programs, and probably slow down them,
// to distinguish between bit positions (starting from 0) and node numbers
// (starting from 1).

typedef int node_t;

#define NODE_NULL	0

//=============================================================================
// Class for Linked List Element in Node Set (contains bit array):
//=============================================================================

class NodeSetElem {
	public:

	// Normal Constructor:
	NodeSetElem(int start_value)
	{
		// Check parameter (last bits are represented in bit array):
		assert(start_value % (BITSTRING_BYTES * 8) == 0);

		// Initialize attributes:
		next_  = (NodeSetElem *) 0;
		start_ = start_value;
		unsigned char *p = bits_;
		for(int i = BITSTRING_BYTES; i > 0; i--)
			*p++ = '\0';
	}

	// Constructor that copies a given node (except next pointer):
	NodeSetElem(NodeSetElem *elem) {
		next_  = static_cast<NodeSetElem *>(0);
		start_ = elem->start_;
		unsigned char *to   = bits_;
		unsigned char *from = elem->bits_;
		for(int i = BITSTRING_BYTES; i > 0; i--)
			*to++ = *from++;
	}

	// Get start node number (to be added to all numbers in bit array):
	inline int start() const {
		return start_;
	}

	// Get link to next list elem:
	inline NodeSetElem *next() const {
		return next_;
	}

	// Set link to next list elem:
	inline void set_next(NodeSetElem *next_elem) {
		next_ = next_elem;
	}

	// Get bit in bit array:
	inline bool bit(int index) const {

		// Check validity of parameter, part I:
		assert(index >= 0);

		// Determine position in bit array:
		int bit_no = index % 8;
		index = index / 8;

		// Check parameter of parameter, part II:
		assert(index < BITSTRING_BYTES);

		// Get bit:
		return (bits_[index] & ((0x1) << bit_no)) != 0;
	}

	// Set bit in bit array:
	inline void set_bit(int index) {

		// Check validity of parameter, part I:
		assert(index >= 0);

		// Determine position in bit array:
		int bit_no = index % 8;
		index = index / 8;

		// Check parameter of parameter, part II:
		assert(index < BITSTRING_BYTES);

		// Set bit:
		bits_[index] |= (0x1) << bit_no;
	}

	// Find next set bit in bit array with position >= index:
	inline int next_bit(int index) {

		// Check validity of parameter, part I:
		assert(index >= 0);

		// Determine position in bit array:
		int bit_no = index % 8;
		index = index / 8;

		// Check parameter, part II:
		assert(index < BITSTRING_BYTES);

		// Find next set bit (-1 if there is no further bit):
		unsigned char byte = bits_[index];
		if(byte != 0) {
			switch(bit_no) {
				case 0:
					if(byte & 0x1)
						return (index*8);
					/* FALLTHROUGH */
				case 1:
					if(byte & 0x2)
						return (index*8 + 1);
					/* FALLTHROUGH */
				case 2:
					if(byte & 0x4)
						return (index*8 + 2);
					/* FALLTHROUGH */
				case 3:
					if(byte & 0x8)
						return (index*8 + 3);
					/* FALLTHROUGH */
				case 4:
					if(byte & 0x10)
						return (index*8 + 4);
					/* FALLTHROUGH */
				case 5:
					if(byte & 0x20)
						return (index*8 + 5);
					/* FALLTHROUGH */
				case 6:
					if(byte & 0x40)
						return (index*8 + 6);
					/* FALLTHROUGH */
				case 7:
					if(byte & 0x80)
						return (index*8 + 7);
			}
		}
		while(++index < BITSTRING_BYTES) {
			byte = bits_[index];
			if(byte != 0) {
				if(byte & 0x1)
					return (index*8);
				if(byte & 0x2)
					return (index*8 + 1);
				if(byte & 0x4)
					return (index*8 + 2);
				if(byte & 0x8)
					return (index*8 + 3);
				if(byte & 0x10)
					return (index*8 + 4);
				if(byte & 0x20)
					return (index*8 + 5);
				if(byte & 0x40)
					return (index*8 + 6);
				if(byte & 0x80)
					return (index*8 + 7);
			}
		}

		// No set bit found:
		return -1;
	}

	// Copy start and bitmap from another element (sets next to null):
	inline void copy_data(NodeSetElem *other_elem) {
		next_  = static_cast<NodeSetElem *>(0);
		start_ = other_elem->start_;

		// Copy bitmap:
		unsigned char *from = other_elem->bits_;
		unsigned char *to   = bits_;
		for(int i = BITSTRING_BYTES; i-- > 0; )
			*to++ = *from++;
	}

	// Attributes:
	private:
	NodeSetElem *next_;
	int start_;
	unsigned char bits_[BITSTRING_BYTES];
};

//-----------------------------------------------------------------------------
// Pointer Type for Node Number List Element:
//-----------------------------------------------------------------------------

typedef NodeSetElem *node_set_elem_t;

#define NODE_SET_ELEM_NULL (static_cast<node_set_elem_t>(0))


//=============================================================================
// Forward declaration for NodeSetScan:
//=============================================================================

class NodeSetScan;

//=============================================================================
// Class for Node Set (represented as linked list of bit arrays):
//=============================================================================

// Note that this actually manages a set of node numbers,
// i.e. non-negative integers.

class NodeSet {

	public:

	// Constructor:
	NodeSet() {
		list_ = NODE_SET_ELEM_NULL;
	}

	// Destructor:
	~NodeSet() {
		free_list(list_);
	}

	// Insert a node number:
	void insert(node_t n) {

		// Check parameter:
		assert(n >= 1);

		// Compute position in bitmap list:
		int index = n % (BITSTRING_BYTES * 8);
		int start = n - index;

		// Handle empty list:
		if(list_ == NODE_SET_ELEM_NULL) {
			list_ = new NodeSetElem(start);
			list_->set_bit(index);
			return;
		}

		// Handle insertion into first element:
		if(list_->start() == start) {
			list_->set_bit(index);
			return;
		}

		// Handle insertion before first element:
		if(list_->start() > start) {
			node_set_elem_t elem = new NodeSetElem(start);
			elem->set_bit(index);
			elem->set_next(list_);
			list_ = elem;
		}

		// Now we can have a pointer to the previous element:
		node_set_elem_t prev = list_;
		node_set_elem_t next = prev->next();
		while(next != NODE_SET_ELEM_NULL) {
			int next_start = next->start();
			// Insert into next element:
			if(next_start == start) {
				next->set_bit(index);
				return;
			}
			// Insert before next element:
			if(next_start > start) {
				node_set_elem_t elem = new NodeSetElem(start);
				elem->set_bit(index);
				elem->set_next(next);
				prev->set_next(elem);
			}
			// Now we know that next_start < start, move on:
			prev = next;
			next = next->next();
		}

		// Insert node at the very end of the list:
		node_set_elem_t elem = new NodeSetElem(start);
		elem->set_bit(index);
		prev->set_next(elem);
	}

	// Check whether a nonnegative integer exists in the set:
	bool contains(node_t n) {

		// Check parameter:
		assert(n >= 1);

		// Compute position in bitmap list:
		int index = n % (BITSTRING_BYTES * 8);
		int start = n - index;

		// Search list element with requested start value:
		for(node_set_elem_t elem = list_; elem; elem = elem->next()) {
			int elem_start = elem->start();
			if(elem_start == start)
				return elem->bit(index);
			if(elem_start > start)
				return false;
		}
		return false;
	}

	// Check whether node set is empty:
	inline bool is_empty() {
		// List elements are only constructed when needed,
		// i.e. there are no empty bitmaps.
		return list_ == NODE_SET_ELEM_NULL;
	}

	// Copy a given node set into this node set (overwrites existing data):
	void copy(NodeSet *node_set) {

		// The current list is only used as recycle material for
		// NodeSetElem objects.
		// The objects themselves are overwritten.
		// Currently, all calls to the copy function are called
		// with a superset of nodes. This is not a requirement,
		// but this motivates that the NodeSet does not keep its
		// own free list, but directly deletes any superfluous nodes.
		// With the current usage, this will anyway not happen.

		// List of nodes to be used (recycled):
		node_set_elem_t nodes = list_;

		// List of nodes to be copied:
		node_set_elem_t from = node_set->list_;

		// Handle empty list:
		if(from == NODE_SET_ELEM_NULL) {
			list_ = NODE_SET_ELEM_NULL;
			free_list(nodes);
			return;
		}

		// First element is treated specially, because no previous one:
		node_set_elem_t first;
		if(nodes != NODE_SET_ELEM_NULL) {
			// Get node from recycle list:
			first = nodes;
			nodes = nodes->next();

			// Copy node (except next pointer):
			first->copy_data(from);
		}
		else {
			first = new NodeSetElem(from);
		}

		// This is the start of the list:
		list_ = first;
		node_set_elem_t prev = first;

		// First element was copied:
		from = from->next();

		// Copy rest of list, using existing nodes as long as possible:
		for( ; from && nodes; from = from->next()) {

			// Get node from recycle list:
			node_set_elem_t elem = nodes;
			nodes = nodes->next();

			// Copy node (except next pointer):
			elem->copy_data(from);

			// Link new element to constructed list:
			prev->set_next(elem);

			// The copied node is now the previous node:
			prev = elem;
		}

		// For the remaining elements, allocate new nodes:
		for( ; from; from = from->next()) {
			node_set_elem_t elem = new NodeSetElem(from);

			// Link new element to "to" list (via prev):
			prev->set_next(elem);

			// The previous element pointer is always one behind:
			prev = elem;
		}

		// If from list is shorter, delete remaining recycle elements:
		free_list(nodes);
	}

	// Auxiliary function to free all (remaining) list elements:
	private:
	void free_list(node_set_elem_t list) {
		node_set_elem_t next;
		for(node_set_elem_t elem = list; elem; elem = next) {
			next = elem->next();
			delete elem;
		}
	}

	// Attributes:
	private:
	node_set_elem_t list_;

	// The NodeSetScan constructor and reset must be able to access list_:
	friend NodeSetScan;
};

//-----------------------------------------------------------------------------
// Pointer Type for Node Set (really a set of non-negative integers):
//-----------------------------------------------------------------------------

typedef NodeSet *node_set_t;

#define NODE_SET_NULL (static_cast<node_set_t>(0))


//=============================================================================
// Class for Scan/Cursor over Node Set (set of non-negative integers):
//=============================================================================

class NodeSetScan {
	public:

	// Constructor:
	NodeSetScan(NodeSet *node_set)
	{
		// Remember node set for possible reset of scan:
		node_set_ = node_set;

		// Set current bitmap list elem:
		curr_elem_  = node_set_->list_; // Permitted by friend decl.
		if(curr_elem_ != NODE_SET_ELEM_NULL)
			start_ = curr_elem_->start();
		else
			start_ = 0;

		// Next bit that will be checked in current bitmap list elem:
		next_index_ = 0;
	}

	// Get next node number in set (NODE_NULL if there is no further one):
	node_t next() {
		// Check whether we are at the end of the list:
		if(curr_elem_ == NODE_SET_ELEM_NULL)
			return NODE_NULL;

		// Try to find next value in current list element:
		if(next_index_ < BITSTRING_BYTES * 8) {
			int bit = curr_elem_->next_bit(next_index_);
			if(bit >= 0) {
				next_index_ = bit + 1;
				return start_ + bit;
			}
		}

		// Move to next list element:
		curr_elem_ = curr_elem_->next();
		if(curr_elem_ == NODE_SET_ELEM_NULL)
			return NODE_NULL;

		// We know that list elements are non-empty:
		start_ = curr_elem_->start();
		int bit = curr_elem_->next_bit(0);
		next_index_ = bit + 1;
		return start_ + bit;
	}

	// Reset scan to start:
	void reset() {
		curr_elem_ = node_set_->list_;
		if(curr_elem_ != NODE_SET_ELEM_NULL)
			start_ = curr_elem_->start();
		else
			start_ = 0;
		next_index_ = 0;
	}

	// Attributes:
	private:
	node_set_t node_set_;
	node_set_elem_t curr_elem_;
	int start_;
	int next_index_;
};

//-----------------------------------------------------------------------------
// Pointer Type for Node Set Scan:
//-----------------------------------------------------------------------------

typedef NodeSetScan *node_set_scan_t;

#define NODE_SET_SCAN_NULL (static_cast<node_set_scan_t>(0))

//-----------------------------------------------------------------------------
// Macro for Loops over the Nodes in a Node Set Scan:
//-----------------------------------------------------------------------------

#define NODE_LOOP(N, S)	for(node_t N = S.next(); N != NODE_NULL; N = S.next())

//=============================================================================
// Class for Graphs:
//=============================================================================

class Graph {
	public:

	// Constructor:
	Graph(int graph_num_nodes) : out_nodes_()
	{
		num_nodes_ = graph_num_nodes;
		incoming_  = new NodeSet[num_nodes_ + 1];// Index 0 is not used
		outgoing_  = new NodeSet[num_nodes_ + 1];// Index 0 is not used
		changed_   = false;
	}

	// Destructor:
	~Graph() {
		delete[] incoming_;
		delete[] outgoing_;
	}

	// Accessor Functions:
	inline int num_nodes() const { return num_nodes_; }

	// Store edge in adjacency matrix:
	void store_edge(node_t from, node_t to) {

		// Check nodes:
		assert(from > 0);
		assert(from <= num_nodes_);
		assert(to > 0);
		assert(to <= num_nodes_);

		// Store edge:
		incoming_[to].insert(from);
		outgoing_[from].insert(to);

		// Mark change:
		changed_ = true;
	}

	// Check whether edge exists:
	bool edge_exists(node_t from, node_t to) const {

		// Check nodes:
		assert(from > 0);
		assert(from <= num_nodes_);
		assert(to > 0);
		assert(to <= num_nodes_);

		// Look up edge:
		return outgoing_[from].contains(to);
	}

	// Return set of nodes reachable from a given node:
	node_set_t outgoing(node_t from) const {

		// Check parameter node:
		assert(from > 0);
		assert(from <= num_nodes_);

		// Look up edge:
		return &(outgoing_[from]);
	}


	// Access the set of nodes from which this node is reachable:
	node_set_t incoming(node_t to) {

		// Check parameter node:
		assert(to > 0);
		assert(to <= num_nodes_);

		// Return set of nodes:
		return &(incoming_[to]);
	}

	// Access the set of nodes that have at least one outgoing edge:
	node_set_t out_nodes() {

		// Usually, this is only called after the graph was fully
		// constructed.
		// Therefore, it seems more efficient to compute the node set
		// only once instead of doing an insertion for every inserted
		// edge.

		// Do we need to recompute the set?
		if(changed_) {
			for(int i = 1; i <= num_nodes_; i++) {
				if(!outgoing_[i].is_empty())
					out_nodes_.insert(i);
			}
			changed_ = false;
		}

		// Return precomputed node set:
		return &out_nodes_;
	}

	// Copy a graph (of the same size) into this graph:
	void copy(Graph *g)
	{
		// We can copy only graphs of the same size:
		assert(g->num_nodes_ == num_nodes_);

		// Copy bit array lists:
		for(int i = 1; i <= num_nodes_; i++) {
			incoming_[i].copy(&(g->incoming_[i]));
			outgoing_[i].copy(&(g->outgoing_[i]));
		}
		changed_ = true;
	}

	// Attributes:
	private:
	int num_nodes_;
	bool changed_;
	node_set_t incoming_;
	node_set_t outgoing_;
	NodeSet out_nodes_;
};

//-----------------------------------------------------------------------------
// Pointer Type for Graph Data:
//-----------------------------------------------------------------------------

typedef Graph *graph_t;

#define GRAPH_NULL (static_cast<graph_t>(0))


//=============================================================================
// Simple Computation of Transitive Closure:
//=============================================================================

class TC {
	public:

	// Constructor:
	TC(graph_t input_graph)
	{
		input_ = input_graph;
		num_nodes_ = input_graph->num_nodes();
		tc_ = new Graph(num_nodes_);
		tc_prev_ = new Graph(num_nodes_);
		rule_inst_ = 0;
		tc_size_ = 0;
		iter_ = 0;
		num_edges_ = 0;

		// Compute transitive closure. First nonrecursive rule:
		bool changed = false;
		std::cout << "\tIteration 0.\n";
		{ // xs is used locally in this rule (avoids shadowing warning)
			// tc(X, Y) :- input(X,Y).
			NodeSetScan xs(input_->out_nodes());
			NODE_LOOP(x, xs) {
				NodeSetScan ys(input_->outgoing(x));
				NODE_LOOP(y, ys) {
					tc_->store_edge(x, y);
					rule_inst_++;
					tc_size_++;
					num_edges_++;
					changed = true;
				}
			}
		}

		// Naive iteration:
		while(changed) {
			std::cout << "\tIteration " << (iter_ + 1) << ".\n";
			changed = false;
			tc_prev_->copy(tc_);
			iter_++;
			// tc(X, Z) :- input(X,Y), tc_prev(Y, Z).
			NodeSetScan xs(input_->out_nodes());
			NODE_LOOP(x, xs) {
				NodeSetScan ys(input_->outgoing(x));
				NODE_LOOP(y, ys) {
					NodeSetScan zs(tc_prev_->outgoing(y));
					NODE_LOOP(z, zs) {
						if(tc_->edge_exists(x,z))
							continue; // duplicate
						tc_->store_edge(x,z);
						tc_size_++;
						changed = true;
					}
				}
			}
		}

		// Computation of Rule Instances:
		std::cout << "\tComputation of Rule Instances ...\n";
		// Instances of the non-recursive rule were counted above.
		NodeSetScan xs(input_->out_nodes());
		NODE_LOOP(x, xs) {
			NodeSetScan ys(input_->outgoing(x));
			NODE_LOOP(y, ys) {
				NodeSetScan zs(tc_->outgoing(y));
				NODE_LOOP(z, zs) {
					rule_inst_++;
				}
			}
		}
	}

	// Accessor Functions:
	inline int num_nodes() const { return num_nodes_; }
	inline int num_edges() const { return num_edges_; }
	inline int tc_size() const { return tc_size_; }
	inline int iter() const { return iter_; }
	inline int rule_inst() const { return rule_inst_; }

	// Pack graph size data into object:
	void export_gsize(gsize_t gsize) {
		gsize->set_num_nodes(num_nodes_);
		gsize->set_num_edges(num_edges_);
		gsize->set_tc_iter(iter_);
		gsize->set_tc_size(tc_size_);
		gsize->set_tc_inst(rule_inst_);
	}

	// Print TC:
	void print() {
		NodeSetScan xs(tc_->out_nodes());
		NODE_LOOP(x, xs) {
			NodeSetScan ys(tc_->outgoing(x));
			NODE_LOOP(y, ys) {
				std::cout << "\ttc(" << x << "," << y <<
							").\n";
			}
		}
		std::cout << "\n";
	}

	// Attributes:
	private:
	int num_nodes_;
	int num_edges_;
	graph_t input_;
	graph_t tc_;
	graph_t tc_prev_;
	bigint_t rule_inst_;
	bigint_t tc_size_;
	int iter_;
};

//-----------------------------------------------------------------------------
// Pointer Type for TC Data:
//-----------------------------------------------------------------------------

typedef TC *tc_t;

#define TC_NULL (static_cast<tc_t>(0))


//=============================================================================
// Simple Computation of Same Generation:
//=============================================================================

class SG {
	public:

	// Constructor:
	SG(graph_t input_graph)
	{
		input_ = input_graph;
		num_nodes_ = input_graph->num_nodes();
		sg_ = new Graph(num_nodes_);
		sg_prev_ = new Graph(num_nodes_);
		rule_inst_ = 0;
		sg_size_ = 0;
		iter_ = 0;
		num_edges_ = 0;

		// Compute same generation cousins:
		bool changed = false;
		std::cout << "\tIteration 0.\n";
		// sg(Y, Y) :- input(X, Y).
		{ // xs is used locally in this rule (avoids shadowing warning)
			NodeSetScan xs(input_->out_nodes());
			NODE_LOOP(x, xs) {
				NodeSetScan ys(input_->outgoing(x));
				NODE_LOOP(y, ys) {
					num_edges_++;
					rule_inst_++;
					if(sg_->edge_exists(y, y))
						continue; // duplicate
					sg_->store_edge(y, y);
					sg_size_++;
					changed = true;
				}
			}
		}

		// Instances of recursive rule:
		bigint_t rec_inst = 0;
		
		// Naive iteration:
		while(changed) {
			std::cout << "\tIteration " << (iter_ + 1) << ".\n";
			changed = false;
			rec_inst = 0;
			sg_prev_->copy(sg_);
			iter_++;
			// sg(X, Y) :- input(X, A), sg(A, B), input(Y, B).
			NodeSetScan as(sg_prev_->out_nodes());
			NODE_LOOP(a, as) {
				NodeSetScan bs(sg_prev_->outgoing(a));
				NODE_LOOP(b, bs) {
					NodeSetScan xs(input_->incoming(a));
					NODE_LOOP(x, xs) {
						NodeSetScan ys(
							input_->incoming(b));
						NODE_LOOP(y, ys) {
							rec_inst++;
							if(sg_->
							    edge_exists(x,y))
								continue;
							sg_->store_edge(x,y);
							sg_size_++;
							changed = true;
						}
					}
				}
			}
		}

		// Add instances of recursive rule to rule instances:
		rule_inst_ += rec_inst;
			// Note that this is outside the iteration.
			// I.e. it is the number of rule instances
			// considered in the last iteration.
	}

	// Accessor Functions:
	inline int num_nodes() const { return num_nodes_; }
	inline int num_edges() const { return num_edges_; }
	inline int sg_size() const { return sg_size_; }
	inline int iter() const { return iter_; }
	inline int rule_inst() const { return rule_inst_; }

	// Pack graph size data into object:
	void export_gsize(gsize_t gsize) {
		gsize->set_num_nodes(num_nodes_);
		gsize->set_num_edges(num_edges_);
		gsize->set_sg_iter(iter_);
		gsize->set_sg_size(sg_size_);
		gsize->set_sg_inst(rule_inst_);
	}

	// Print SG:
	void print() {
		NodeSetScan xs(sg_->out_nodes());
		NODE_LOOP(x, xs) {
			NodeSetScan ys(sg_->outgoing(x));
			NODE_LOOP(y, ys) {
				std::cout << "\tsg(" << x << "," << y <<
						").\n";
			}
		}
		std::cout << "\n";
	}

	// Attributes:
	private:
	int num_nodes_;
	int num_edges_;
	graph_t input_;
	graph_t sg_;
	graph_t sg_prev_;
	bigint_t rule_inst_;
	bigint_t sg_size_;
	int iter_;
};

//-----------------------------------------------------------------------------
// Pointer Type for TC Data:
//-----------------------------------------------------------------------------

typedef TC *tc_t;

#define TC_NULL (static_cast<tc_t>(0))


//=============================================================================
// Data Type for File Formats:
//=============================================================================

typedef enum { FORMAT_P, FORMAT_TSV, FORMAT_CSV, FORMAT_SQL } format_t;

str_t format_name(format_t f) {
	switch(f) {
		case FORMAT_P:
			return ".P";
		case FORMAT_TSV:
			return ".tsv";
		case FORMAT_CSV:
			return ".csv";
		case FORMAT_SQL:
			return ".sql";
		default:
			return "UNKNOWN FORMAT";
	}
}

//=============================================================================
// Class for Output of Edges (possibly later in different formats):
//=============================================================================

class Output {
	public:

	Output()
	{
		num_files_ = 0;
		num_nodes_ = 0;
		num_edges_ = 0;
		graph_     = GRAPH_NULL;
	}

	// Accessor Functions:
	inline int num_files() const { return num_files_; }
	inline int num_nodes() const { return num_nodes_; }
	inline int num_edges() const { return num_edges_; }
	inline graph_t graph() const { return graph_; }

	// Add output file:
	void add_file(str_t filename) {
		// Check that there is still space in the array:
		if(num_files_ >= MAX_OUT_FILES) {
			std::cerr << "Too many output files!\n";
			exit(3);
		}

		// Check file extension:
		int n = str_len(filename);
		if(n > 2 && filename[n-2] == '.' && filename[n-1] == 'P')
			format_[num_files_] = FORMAT_P;
		else if(n > 4 && filename[n-4] == '.' &&
			filename[n-3] == 't' && filename[n-2] == 's' &&
			filename[n-1] == 'v')
			format_[num_files_] = FORMAT_TSV;
		else if(n > 4 && filename[n-4] == '.' &&
			filename[n-3] == 'c' && filename[n-2] == 's' &&
			filename[n-1] == 'v')
			format_[num_files_] = FORMAT_CSV;
		else if(n > 4 && filename[n-4] == '.' &&
			filename[n-3] == 's' && filename[n-2] == 'q' &&
			filename[n-1] == 'l')
			format_[num_files_] = FORMAT_SQL;
		else {
			std::cerr << "Output file '" << filename <<
				"' has unknown extension.\n";
			exit(4);
		}

		// Open file:
		std::ofstream *output = files_ + num_files_;
		output->open(filename);
		if(output->fail()) {
			std::cerr << "Opening output file '" << filename <<
				"' failed.\n";
			exit(5);
		}
		num_files_++;
	}

	// Set number of nodes:
	void set_nodes(int n) {

		// The graph data structure depends on the number of nodes.
		// Therefore, the number of nodes can be set only once:
		assert(num_nodes_ == 0);

		// Store number of nodes:
		num_nodes_ = n;

		// Create graph data structure:
		graph_ = new Graph(n);
	}

	// Write an edge to output file:
	void write_edge(int from, int to) {

		// The number of nodes must be set first:
		assert(num_nodes_ > 0);

		// Check validity of parameter values:
		assert(from > 0);
		assert(from <= num_nodes_);
		assert(to > 0);
		assert(to <= num_nodes_);

		// Check that there is at least one file open:
		assert(num_files_ > 0);

		// Write edge to every file:
		for(int i = 0; i < num_files_; i++) {
			std::ofstream &file = files_[i];
			switch(format_[i]) {
				case FORMAT_P:
					file << PRED_NAME << "(" <<
						from << ", " << to <<
						")." << "\n";
					break;
				case FORMAT_TSV:
					file << from << "\t" << to << "\n";
					break;
				case FORMAT_CSV:
					file << from << "," << to << "\n";
					break;
				case FORMAT_SQL:
					file << "INSERT INTO " << PRED_NAME <<
						" VALUES (" <<
						from << "," << to << ");\n";
					break;
			}
		}

		// Store edge in graph data structure:
		graph_->store_edge(from, to);

		// Increment number of written edges:
		num_edges_++;
	}

	// Check whether edge exists:
	bool edge_exists(int from, int to) const {
		return graph_->edge_exists(from, to);
	}

	// Close output files:
	void close_files() {
		for(int i = 0; i < num_files_; i++) {
			std::ofstream &file = files_[i];
			if(file.bad()) {
				std::cerr << "Problem with output file " <<
					"(Format " <<
					format_name(format_[i]) << ").\n";
				exit(6);
			}
			file.close();
		}
	}

	// Attributes:
	private:
	int num_files_;
	int num_nodes_;
	int num_edges_;
	std::ofstream files_[MAX_OUT_FILES];
	format_t format_[MAX_OUT_FILES];
	graph_t graph_;
};

//-----------------------------------------------------------------------------
// Pointer Type for Output Class:
//-----------------------------------------------------------------------------

typedef Output *output_t;

#define OUTPUT_NULL (static_cast<output_t>(0))

//=============================================================================
// Auxiliary Functions for Graph Generators:
//=============================================================================


//-----------------------------------------------------------------------------
// Error message for graphs that have no second parameter:
//-----------------------------------------------------------------------------

void no_second_par(str_t graph_name) {
	std::cerr << "Graph '" << graph_name << "' has no second parameter.\n";
	exit(7);
}

//-----------------------------------------------------------------------------
// Error message for graphs with second parameter, which is not set:
//-----------------------------------------------------------------------------

void second_par_missing(str_t graph_name) {
	std::cerr << "Graph '" << graph_name << "' needs a second parameter.\n";
	exit(8);
}

//-----------------------------------------------------------------------------
// Error message if parameter value is too small:
//-----------------------------------------------------------------------------

void par_too_small(str_t graph_name, str_t par_name, int min) {
	std::cerr << "Value of parameter '" << par_name << "' " <<
		"for Graph '" << graph_name << "' too small: " <<
		"Minimum " << min << ".\n";
	exit(9);
}

//-----------------------------------------------------------------------------
// Compute n^2:
//-----------------------------------------------------------------------------

bigint_t square(int n) {
	bigint_t m = n;
	m *= n;
	return m;
}

//-----------------------------------------------------------------------------
// Compute Power of 2:
//-----------------------------------------------------------------------------

bigint_t pow2(int n) {
	bigint_t pow = 1;
	for(int i = 0; i < n; i++)
		pow *= 2;
	return pow;
}

//-----------------------------------------------------------------------------
// Prime Number test:
//-----------------------------------------------------------------------------

bool is_prime(int p) {
	for(int i = 2; i < p; i++) {
		if(p % i == 0)
			return false;
	}
	return true;
}

//-----------------------------------------------------------------------------
// Compute Prime Number Greater Than or Equal To a Number:
//-----------------------------------------------------------------------------

int next_prime(int p) {
	while(!is_prime(p)) {
		p++;
		if(p < 0) {
			std::cerr <<
				"Overflow in computation of prime number\n";
			exit(10);
		}
	}
	return p;
}

//-----------------------------------------------------------------------------
// Random number generator (to make random graphs deterministic and portable):
//-----------------------------------------------------------------------------

// The code is from
// https://de.wikipedia.org/wiki/KISS_(Zufallszahlengenerator)
// The same generator is the main random number generator mentioned in
// http://www0.cs.ucl.ac.uk/staff/d.jones/GoodPracticeRNG.pdf

static uint32_t x = 123456789; // seed can be arbitrary,
static uint32_t y = 362436000; // as long as y != 0 and
static uint32_t z = 521288629; // z,c are not both 0
static uint32_t c = 7654321;

uint32_t rand_KISS() {
   uint64_t t;

   // Linear congruential generator:
   x = 69069 * x + 12345;

   // Xorshift:
   y ^= y << 13;
   y ^= y >> 17;
   y ^= y << 5;

   // Multiply-with-carry:
   t = (uint64_t)698769069 * z + c;
   c = t >> 32;
   z = (uint32_t) t;

   return x + y + z;
}

//-----------------------------------------------------------------------------
// Compute Random Number from 0 to n (inclusive):
//-----------------------------------------------------------------------------

int rand_until(int n) {
	//int r = rand();
	uint32_t r = rand_KISS();

	int result = r % (n+1);
	//	This is the simple version that can be used when n is much
	//	smaller than the maximal random number that is generated.
	//	Theoretically, the numbers are not completely evenly
	//	distributed, because the last set of n+1 numbers before
	//	the maximal random number is usually not complete
	//	(unless the number of results that the random number generates
	//	is dividable by n+1).
	//	However, if n is not more than 4 million (e.g. we generate
	//	not more than 4M edges), the error from a uniform distribution
	//	is less than one promille.

	//int result = (int) (((double) r) * (n+1) / (0xFFFFFFFF + 1.0));
	//	This version avoids the slight problem, but uses double.
	//	The divisor is the maximal result of rand_KISS + 1:
	//	4294967296.0
	//	When double is converted to int, no rounding is done,
	//	but the floor function is used.
	//	Therefore, the result can never be n+1 or larger.

	if(result < 0 || result > n) {
		std::cerr << "Error in random number generator!\n";
		exit(11);
	}
#if TEST_OUTPUT
	std::cout << "rand_until(" << n << ") = " << result <<
		"\t[rand = " << r << "]\n";
#endif
	return result;
}

//-----------------------------------------------------------------------------
// Random Shuffling of Array:
//-----------------------------------------------------------------------------

void rand_shuffle(int arr[], int len) {
	for(int i = len-1; i > 0; i--) {
		int j = rand_until(i);
		int swap = arr[i];
		arr[i] = arr[j];
		arr[j] = swap;
	}

#if TEST_OUTPUT
	std::cout << "\n";
	std::cout << "Result of Shuffle (Array Length: " << len << ")\n";
	for(int i = 0; i < len; i++)
		std::cout << "\t arr[" << i << "] = " << arr[i] << "\n";
	std::cout << "\n";
#endif
}


//=============================================================================
// Generator Functions for Graphs:
//=============================================================================

//-----------------------------------------------------------------------------
// K: Complete Graph:
//-----------------------------------------------------------------------------

void gen_graph_k(int n, int k, output_t out, gsize_t gsize) {

	// Second parameter has no meaning:
	if(k != -1)
		no_second_par("K");

	// Set number of nodes:
	out->set_nodes(n);

	// Generate edges:
	for(int i = 1; i <= n; i++) {
		for(int j = 1; j <= n; j++) {
			out->write_edge(i, j);
		}
	}

	// Use Formulas for graph/benchmark size:
	gsize->set_num_nodes(n);
	gsize->set_num_edges(n * n);
	gsize->set_tc_iter(1);
	bigint_t n2 = n;
	n2 *= n; // n2 = n^2
	bigint_t n3 = n2 * n;
	gsize->set_tc_size(n2);
	gsize->set_tc_inst(n2 + n3);

	// SG Size:
	bigint_t n4 = n2 * n2;
	gsize->set_sg_size(n2);
	gsize->set_sg_inst(n2 + n4);
}

//-----------------------------------------------------------------------------
// T: Total Order Graph (Maximum Acyclic Graph):
//-----------------------------------------------------------------------------

void gen_graph_t(int n, int k, output_t out, gsize_t gsize) {

	// Second parameter has no meaning:
	if(k != -1)
		no_second_par("T");

	// Set number of nodes:
	out->set_nodes(n);

	// Generate edges:
	for(int i = 1; i <= n; i++) {
		for(int j = i+1; j <= n; j++) {
			out->write_edge(i, j);
		}
	}

	// Use Formulas for graph/benchmark size:
	gsize->set_num_nodes(n);
	gsize->set_num_edges(n * (n-1) / 2);
	gsize->set_tc_iter(1);
	bigint_t s = n;
	s *= (n-1);
	s /= 2; // s = n * (n-1) / 2
	gsize->set_tc_size(s);
	bigint_t r = n;
	r *= (n-1);
	r *= (n+1);
	r /= 6; // r = 1/6 * n * (n-1) * (n+1)
	gsize->set_tc_inst(r);

	// SG Size:
	bigint_t m = n;
	bigint_t n2 = m * m;
	bigint_t n3 = n2 * m;
	bigint_t n4 = n2 * n2;
	gsize->set_sg_size(1 + (m -1) * (m - 1));
	gsize->set_sg_inst((n4 - 6 * n3 + 19 * n2 - 22 * m)/4 + 2);
}

//-----------------------------------------------------------------------------
// P: Path:
//-----------------------------------------------------------------------------

void gen_graph_p(int n, int k, output_t out, gsize_t gsize) {

	// Second parameter has no meaning:
	if(k != -1)
		no_second_par("P");

	// Set number of nodes:
	out->set_nodes(n);

	// Generate edges:
	for(int i = 1; i < n; i++) {
			out->write_edge(i, i+1);
	}

	// Use Formulas for graph/benchmark size:
	gsize->set_num_nodes(n);
	gsize->set_num_edges(n-1);
	gsize->set_tc_iter(n-1);
	bigint_t m = n;
	m *= (n-1);
	m /= 2; // m = n * (n-1) / 2
	gsize->set_tc_size(m);
	gsize->set_tc_inst(m);

	// SG Size:
	gsize->set_sg_size(n);
	gsize->set_sg_inst(2 * (n - 1));
}

//-----------------------------------------------------------------------------
// M: Multipath:
//-----------------------------------------------------------------------------

void gen_graph_m(int n, int k, output_t out, gsize_t gsize) {

	// Check that second parameter is defined:
	if(k < 0) {
		second_par_missing("M");
	}

	// Set number of nodes:
	out->set_nodes(n*k);

	// Generate edges:
	for(int i = 1; i <= (n - 1) * k; i++) {
		out->write_edge(i, i+k);
	}

	// Use Formulas for graph/benchmark size:
	gsize->set_num_nodes(n*k);
	gsize->set_num_edges((n-1)*k);
	gsize->set_tc_iter(n-1);
	bigint_t m = n; // m is n as a bigint_t (so that 64bit * etc. is used)
	bigint_t s = k * (m * (m-1) / 2);
	gsize->set_tc_size(s);
	bigint_t r = k * ((m-1) + (m-2)*(m-1)/2);
	gsize->set_tc_inst(r);

	// SG Size:
	gsize->set_sg_size(k * n);
	gsize->set_sg_inst(2 * k * (n - 1));
}

//-----------------------------------------------------------------------------
// C: Cycle:
//-----------------------------------------------------------------------------

void gen_graph_c(int n, int k, output_t out, gsize_t gsize) {

	// Second parameter has no meaning:
	if(k != -1)
		no_second_par("C");

	// Set number of nodes:
	out->set_nodes(n);

	// Generate edges:
	for(int i = 1; i < n; i++) {
			out->write_edge(i, i+1);
	}
	out->write_edge(n, 1);

	// Use Formulas for graph/benchmark size:
	gsize->set_num_nodes(n);
	gsize->set_num_edges(n);
	gsize->set_tc_iter(n);
	bigint_t n2 = n;
	n2 *= n; // n2 = n^2
	gsize->set_tc_size(n2);
	gsize->set_tc_inst(n2 + n);

	// SG Size:
	gsize->set_sg_size(n);
	gsize->set_sg_inst(2 * n);
}

//-----------------------------------------------------------------------------
// S: Cycle with Shortcuts.
//-----------------------------------------------------------------------------

void gen_graph_s(int n, int k, output_t out, gsize_t gsize) {

	// Check that second parameter is defined:
	if(k < 0) {
		second_par_missing("S");
	}

	// Check divisibility condition:
	if(n % (k+1) != 0) {
		std::cerr << "S[n,k] requires that n is divisible by k+1.\n";
		exit(12);
	}

	// Check that k is not too large:
	if(n < (k+1)) {
		std::cerr << "S[n,k] requires that n > k.\n";
		exit(13);
	}

	// Set number of nodes:
	out->set_nodes(n);

	// Generate edges, standard cycle:
	for(int i = 1; i < n; i++) {
		out->write_edge(i, i+1);
	}
	out->write_edge(n, 1);

	// Generate edges, shortcuts:
	for(int j = 1; j <= k; j++) {
		int skip =  (n*j)/(k+1);
		for(int i = 1; i <= n; i++) {
#if S_GRAPH_NEW_VER
			out->write_edge(i, 1 + (i + skip) % n);
#else
			out->write_edge(i, 1 + (i-1 + skip) % n);
#endif
		}
	}

	// Use Formulas for graph/benchmark size:
	gsize->set_num_nodes(n);
	gsize->set_num_edges(n * (k+1));
	gsize->set_tc_iter(n / (k+1));
	bigint_t m = n; // m is n as a bigint_t (so that 64bit * etc. is used)
	bigint_t s = m * m;
	gsize->set_tc_size(s);
	bigint_t r = (k + 1) * (m + m*m);
	gsize->set_tc_inst(r);

	// SG Size:
#if S_GRAPH_NEW_VER
	gsize->set_sg_size(m * (k+1));
	//gsize->set_sg_inst(m*(k+1) + (k+1) * (m * (k+1)) * (k+1));
	//The following is the same, slightly shorter:
	gsize->set_sg_inst(m * (k+1) * (1 + (k+1) * (k+1)));
	if(k > 0 && n > 1)
		gsize->set_sg_iter(2);
	else
		gsize->set_sg_iter(1);
#else
	if(k == 1 && n % 4 == 2) {
		gsize->set_sg_size(m * m / 2);
		gsize->set_sg_inst(2 * (m * m + m));
	}
	else {
		gsize->set_sg_size(m * m);
		gsize->set_sg_inst(m * (k + 1) + m * m * (k+1) * (k+1));
	}
#endif
}

//-----------------------------------------------------------------------------
// B: Binary Tree:
//-----------------------------------------------------------------------------

void gen_graph_b(int n, int k, output_t out, gsize_t gsize) {

	// Second parameter has no meaning:
	if(k != -1)
		no_second_par("B");

	// The tree height must be minimally 2:
	if(n < 2)
		par_too_small("B", "tree height", 2);

	// Compute 2^{n-1}:
	int pow = 1;
	for(int i = 1; i < n; i++) // i.e. n-1 times
		pow *= 2;

	// Set number of nodes:
	out->set_nodes(pow * 2 - 1);

	// Generate edges:
	for(int i = 1; i <= pow - 1; i++) {
		out->write_edge(i, 2*i);
		out->write_edge(i, 2*i + 1);
	}

	// Use Formulas for graph/benchmark size:
	gsize->set_num_nodes(pow * 2 - 1);
	gsize->set_num_edges(pow * 2 - 2);
	gsize->set_tc_iter(n - 1);
	bigint_t s = pow * 2;
	s *= (n - 2); 
	s += 2; // s = (n - 2) * 2^n + 2
	gsize->set_tc_size(s);
	gsize->set_tc_inst(s);

	// SG Size:
	gsize->set_sg_size(pow2(n) - 1);
	gsize->set_sg_inst(pow2(n+1) - 4);
}

//-----------------------------------------------------------------------------
// V: Inverse Binary Tree (edges pointing to the root):
//-----------------------------------------------------------------------------

void gen_graph_v(int n, int k, output_t out, gsize_t gsize) {

	// Second parameter has no meaning:
	if(k != -1)
		no_second_par("V");

	// The tree height must be minimally 2:
	if(n < 2)
		par_too_small("V", "tree height", 2);

	// Compute 2^{n-1}:
	int pow = 1;
	for(int i = 1; i < n; i++) // i.e. n-1 times
		pow *= 2;

	// Set number of nodes:
	out->set_nodes(pow * 2 - 1);

	// Generate edges:
	for(int i = 1; i <= pow - 1; i++) {
		out->write_edge(2*i, i);
		out->write_edge(2*i + 1, i);
	}

	// Use Formulas for graph/benchmark size:
	gsize->set_num_nodes(pow * 2 - 1);
	gsize->set_num_edges(pow * 2 - 2);
	gsize->set_tc_iter(n - 1);
	bigint_t s = pow * 2;
	s *= (n - 2); 
	s += 2; // s = (n - 2) * 2^n + 2
	gsize->set_tc_size(s);
	gsize->set_tc_inst(s);

	// SG Size:
	bigint_t sum = 0;
	for(int i = 0; i <= n-1; i++)
		sum += pow2(2*i);
	gsize->set_sg_size(sum);
	sum = 0;
	for(int i = 0; i <= n-2; i++)
		sum += pow2(2*i);
	gsize->set_sg_inst(pow2(n) - 2 + 4 * sum);
}

//-----------------------------------------------------------------------------
// Y-Graph (Many nodes pointing to central node, plus path from there):
//-----------------------------------------------------------------------------

void gen_graph_y(int n, int k, output_t out, gsize_t gsize) {

	// Check that second parameter is defined:
	if(k < 0) {
		second_par_missing("Y");
	}

	// Set number of nodes:
	out->set_nodes(n + k);

	// Generate edges:
	for(int i = 1; i <= n; i++) {
		out->write_edge(i, n + 1);
	}
	for(int i = n + 2; i <= n + k; i++) {
		out->write_edge(i - 1, i);
	}

	// Use Formulas for graph/benchmark size:
	gsize->set_num_nodes(n + k);
	gsize->set_num_edges(n + (k-1));
	gsize->set_tc_iter(k);
	bigint_t m = k;
	m *= (k-1);
	m /= 2;
	bigint_t s = n;
	s *= k;
	s += m; // s = n * k + k*(k-1)/2
	gsize->set_tc_size(s);
	gsize->set_tc_inst(s);

	// SG Size:
	gsize->set_sg_size(square(n) + k);
	gsize->set_sg_inst(square(n) + n + 2 * k - 2);
}

//-----------------------------------------------------------------------------
// X-Graph (n edges to central node, k edges from there):
//-----------------------------------------------------------------------------

void gen_graph_x(int n, int k, output_t out, gsize_t gsize) {

	// Default for second parameter:
	if(k < 1)
		k = n;

	// Set number of nodes:
	out->set_nodes(n + k + 1);

	// Generate edges:
	for(int i = 1; i <= n; i++) {
		out->write_edge(i, n + 1);
	}
	for(int i = 1; i <= k; i++) {
		out->write_edge(n + 1, n + 1 + i);
	}

	// Use Formulas for graph/benchmark size:
	gsize->set_num_nodes(n + k + 1);
	gsize->set_num_edges(n + k);
	gsize->set_tc_iter(2);
	bigint_t s = n;
	s *= k;
	s += n + k; // n + k + n*k
	gsize->set_tc_size(s);
	gsize->set_tc_inst(s);

	// SG Size:
	gsize->set_sg_size(k + 1 + square(n));
	gsize->set_sg_inst(2*k + n + square(n));
}

//-----------------------------------------------------------------------------
// W-Graph (Bipartite Graph, Single level of edges, mesh):
//-----------------------------------------------------------------------------

void gen_graph_w(int n, int k, output_t out, gsize_t gsize) {

	// Default for second parameter:
	if(k < 1)
		k = n;

	// Set number of nodes:
	out->set_nodes(2 * n);

	// Generate edges:
	for(int i = 1; i <= n; i++) {
		for(int j = 0; j < k; j++) {
			out->write_edge(i, n + 1 + (i+j-1) % n);
		}
	}

	// Use Formulas for graph/benchmark size:
	gsize->set_num_nodes(2 * n);
	gsize->set_num_edges(n * k);
	gsize->set_tc_iter(1);
	gsize->set_tc_size(n * k);
	gsize->set_tc_inst(n * k);

	// SG Size:
	bigint_t m = n;
	bigint_t min = 2 * k;
	if(n + 1 < min)
		min = n + 1;
	gsize->set_sg_size(m * min);
	gsize->set_sg_inst(m * (k + k * k));
}

//-----------------------------------------------------------------------------
// D-Graph ("Diamond", one node to many, from there to one - k copies):
//-----------------------------------------------------------------------------

void gen_graph_d(int n, int k, output_t out, gsize_t gsize) {

	// Default for second parameter:
	if(k < 1)
		k = 1;

	// Set number of nodes:
	out->set_nodes((n + 2) * k);

	// Generate edges:
	for(int i = 0; i < k; i++) {
		int offset = i * (n+2);
		for(int j = 1; j <= n; j++) {
			out->write_edge(offset + 1, offset + 1 + j);
		}
		for(int j = 1; j <= n; j++) {
			out->write_edge(offset + 1 + j, offset + n + 2);
		}
	}

	// Use Formulas for graph/benchmark size:
	gsize->set_num_nodes(k * (n + 2));
	gsize->set_num_edges(k * (2 * n));
	gsize->set_tc_iter(2);
	gsize->set_tc_size(k * (2 * n + 1));
	gsize->set_tc_inst(k * (3 * n));
}

//-----------------------------------------------------------------------------
// Joint Procedure for U-Graph and A-Graph (does the real work):
//-----------------------------------------------------------------------------

// The graph generation algorithm is from OpenRuleBench with slight
// modifications.

void gen_graph_u_a(int n, int k, output_t out, gsize_t gsize, bool acyclic) {

	// Set number of nodes:
	out->set_nodes(n);

	// Compute next two prime numbers greater than n:
	int p1 = next_prime(n + 1);
	int p2 = next_prime(p1 + 1);

	// Allocate and fill lists of nodes:
	int *a1 = new int[p1];
	int *a2 = new int[p2];
	for(int i = 0; i < p1; i++) {
		a1[i] = (i < n)? (i+1) : 0;
	}
	for(int i = 0; i < p2; i++) {
		a2[i] = (i < n)? (i+1) : 0;
	}

	// Do the random shuffling:
	rand_shuffle(a1, p1);
	rand_shuffle(a2, p2);

	// Generate edges:
	int generated_edges = 0;
	for(int i = 0; generated_edges < k; i++) {

		// Ensure termination:
		if(i >= p1 * p2) {
			std::cerr <<
				"Number of edges too large in random graph.\n";
			exit(14);
		}

		// Draw the two nodes of a candidate edge:
		int node1 = a1[i % p1];
		int node2 = a2[i % p2];

		// Skip edges with dummy nodes (added to make lists coprime):
		if(node1 == 0)
			continue;
		if(node2 == 0)
			continue;

		// For acyclic graphs, orient edges towards the greater node:
		if(acyclic) {
			// Skip loops:
			if(node1 == node2)
				continue;

			// Make edge pointing towards the greater node:
			if(node1 > node2) {
				int swap = node1;
				node1 = node2;
				node2 = swap;
			}

			// Because we changed edges, duplicates are possible:
			if(out->edge_exists(node1, node2))
				continue;
		}

		// Ok, edge has passed all tests:
		out->write_edge(node1, node2);
		generated_edges++;
	}

	// Free allocated memory:
	delete[] a1;
	delete[] a2;

	// There are no formulas for the benchmark size, only edges and nodes:
	gsize->set_num_nodes(n);
	gsize->set_num_edges(k);
}

//-----------------------------------------------------------------------------
// U-Graph (Random Graph with Uniform Distribution of Node Degrees):
//-----------------------------------------------------------------------------

// This Graph Generation Algorithm is more or less copied from OpenRuleBench.
// The first parameter n is the number of nodes.
// The second parameter k the number of edges.
// The two next prime numbers p1 and p2 greater than n are computed
// and two arrays a1 and a2 of size p1 and p2 are generated.
// These arrays are filled with the numbers 1..n and p1/p2 - n entries
// with the value 0.
// The two arrays are randomly shuffled and then a1[i mod p1]
// is connected with a2[i mod p2] and all connections with 0 are ignored.

void gen_graph_u(int n, int k, output_t out, gsize_t gsize) {

	// Check that second parameter is defined:
	if(k < 0) {
		second_par_missing("U");
	}

	// Check relation between n and k:
	if(k > n * n) {
		std::cerr <<
			"Impossible number of edges (too large) in U-Graph";
		exit(15);
	}

	// Call joint procedure for graphs U and A:
	gen_graph_u_a(n, k, out, gsize, false);
}

//-----------------------------------------------------------------------------
// A-Graph (Acyclic Random Graph):
//-----------------------------------------------------------------------------

// This is similar to the OpenRuleBench algorithm, but improved.
// OpenRuleBench made the Graph acyclic by ordering generated edges
// (as in the U-Graph, see above) towards the greater node.
// However, this generates some duplicate edges.
// Furthermore, the OpenRuleBench scripts allowed loops (edges from a node
// to itself). Therefore, the graphs were not acyclic, although claimed
// to be so.
//
// We improved this as follows:
// Loops are immediately skipped and a new edges is generated.
// We do orient the edges towards the greater node as in OpenRuleBench,
// but we use a duplicate check:
// Duplicate edges are skipped.
//
// Note that here the node degrees are not uniform:
// Because we orient all edges towards the greater node,
// node 1 has an in-degree of 0 and the last node (node n) an out-degree of 0.
// However, the total degree (sum of in and out degree) is quite uniformly
// distributed.

void gen_graph_a(int n, int k, output_t out, gsize_t gsize) {

	// Check that second parameter is defined:
	if(k < 0) {
		second_par_missing("A");
	}

	// Check relation between n and k:
	if(k > n * (n-1) / 2) {
		std::cerr <<
			"Impossible number of edges (too large) in A-Graph";
		exit(16);
	}

	// Call joint procedure for graphs U and A:
	gen_graph_u_a(n, k, out, gsize, true);
}


//=============================================================================
// The main Function:
//=============================================================================


int main(int argc, str_t argv[])
{
	// Print welcome:
	// std::cout << "Welcome to the Graph Data Generator.\n";
	// std::cout << "\n";

	// Compute elapsed time in seconds:
	time_t start_time;
	time(&start_time);

	// The program should be called with the graph and the output files:
	if(argc < 3) {
		std::cerr <<
			"Usage: ./graph [-atsv] GraphID OutputFile1 ...\n";
		exit(17);
	}

	// Initialize options:
	bool opt_tc = false;
	bool opt_sg = false;
	bool opt_verbose = false;

	// Parse Options:
	int argno = 1;
	while(argno < argc && argv[argno][0] == '-') {
		str_t opts = argv[argno++];
		opts++; // Skip '-'
		while(*opts) {
			switch(*opts++) {
			case 'a':
				// Compute all benchmarks:
				opt_tc = true;
				opt_sg = true;
				break;
			case 's':
				// Compute same generation:
				opt_sg = true;
				break;
			case 't':
				// Compute transitive closure:
				opt_tc = true;
				break;
			case 'v':
				// Verbose output (e.g. show benchmark result):
				opt_verbose = true;
				break;
			default:
				// Unknown option:
				std::cerr << "Unknown option: " <<
						"'" << *opts << "'\n";
				exit(18);
			}
		}
	}

	// Get graph parameters, first code:
	if(argno >= argc) {
		std::cerr << "Command line argument for Graph ID missing.\n";
		exit(19);
	}
	str_t graph_id = argv[argno++];
	str_t p = graph_id;
	char graph_code = *p++;
	if(graph_code == '\0') {
		std::cerr << "Impossible empty Graph ID.\n";
		exit(20);
	}

	// First parameter:
	char par_chars[MAX_PAR_CHARS+1];
	int i;
	for(i = 0; *p && is_digit(*p) && i < MAX_PAR_CHARS; i++)
		par_chars[i] = *p++;
	par_chars[i] = 0;
	if(i == 0) {
		std::cerr << "First parameter in graph ID missing.\n";
		exit(21);
	}
	int par1 = str_int(par_chars);
	if(*p == 'k') {
		p++;
		if(*p == 'i') {
			p++;
			par1 *= 1024;
		}
		else {
			par1 *= 1000;
			if(*p == '5') {
				p++;
				par1 += 500;
			}
		}
	}
	else if(*p == 'm') {
		p++;
		par1 *= 1000000;
		if(*p == '5') {
			p++;
			par1 += 500000;
		}
	}

	// Second parameter:
	int par2 = -1;
	if(*p == '_') {
		p++;
		for(i = 0; *p && is_digit(*p) && i < MAX_PAR_CHARS; i++)
			par_chars[i] = *p++;
		par_chars[i] = 0;
		if(i == 0) {
			std::cerr << "Second parameter in graph ID missing.\n";
			exit(22);
		}
		par2 = str_int(par_chars);
		if(*p == 'k') {
			p++;
			if(*p == 'i') {
				p++;
				par2 *= 1024;
			}
			else {
				par2 *= 1000;
				if(*p == '5') {
					p++;
					par2 += 500;
				}
			}
		}
		else if(*p == 'm') {
			p++;
			par2 *= 1000000;
			if(*p == '5') {
				p++;
				par2 += 500000;
			}
		}
	}

	// Check that we have successfully parsed the entire graph ID:
	if(*p != '\0') {
		std::cerr << "Unexpected characters at the end of graph ID.\n";
		exit(23);
	}

	// Some feedback about graph to be generated:
	std::cout << "Generating graph " << graph_id << " ...\n";

	// Open output file(s):
	if(argno == argc) {
		std::cerr << "At least one output file must be specified.\n";
		exit(24);
	}
	Output output;
	while(argno < argc)
		output.add_file(argv[argno++]);

	// Create object for graph/benchmark size (computed with formula):
	GSize formula;

	// Select generator:
	switch(graph_code) {
		case 'b':
		case 'B':
			// Binary tree
			gen_graph_b(par1, par2, &output, &formula);
			break;
		case 'c':
		case 'C':
			// Cycle
			gen_graph_c(par1, par2, &output, &formula);
			break;
		case 'd':
		case 'D':
			// Diamond
			gen_graph_d(par1, par2, &output, &formula);
			break;
		case 'k':
		case 'K':
			// Complete graph
			gen_graph_k(par1, par2, &output, &formula);
			break;
		case 'm':
		case 'M':
			// Multipath
			gen_graph_m(par1, par2, &output, &formula);
			break;
		case 'p':
		case 'P':
			// Path
			gen_graph_p(par1, par2, &output, &formula);
			break;
		case 's':
		case 'S':
			// Shortcut graph (cycle with shortcuts)
			gen_graph_s(par1, par2, &output, &formula);
			break;
		case 't':
		case 'T':
			// Total order graph (maximum acyclic graph)
			gen_graph_t(par1, par2, &output, &formula);
			break;
		case 'u':
		case 'U':
			// Random Graph, Uniform Distribution of Node Degrees
			gen_graph_u(par1, par2, &output, &formula);
			break;
		case 'v':
		case 'V':
			// InVerted bin. tree (looks like V if edges point down)
			gen_graph_v(par1, par2, &output, &formula);
			break;
		case 'w':
		case 'W':
			// Single level of edges, mesh, 2n nodes, degree k):
			gen_graph_w(par1, par2, &output, &formula);
			break;
		case 'x':
		case 'X':
			// X-graph (n edges to central node, k from there)
			gen_graph_x(par1, par2, &output, &formula);
			break;
		case 'y':
		case 'Y':
			// Y-graph (many nodes point to central node, then path)
			gen_graph_y(par1, par2, &output, &formula);
			break;
		case 'a':
		case 'A':
			// Acyclic random graph
			gen_graph_a(par1, par2, &output, &formula);
			break;
		default:
			std::cerr << "Unknown graph type '" << graph_code <<
				"'.\n";
			exit(25);
	}

	// Output number of edges written:
	std::cout << "    Number of edges written: " <<
			output.num_edges() << ".\n";
	std::cout << "\n";

	// Close output file (in case we interrupt computation):
	output.close_files();

	// Create object for graph/benchmark size computed from graph:
	GSize gsize;

	// Compute transitive closure data:
	if(opt_tc) {
		std::cout << "    Compute transitive closure data ...\n";
		TC tc(output.graph());
		std::cout << "\tdone\n";
		std::cout << "\n";

		// Show computation result:
		if(opt_verbose)
			tc.print();

		// Store computed size data in graph size object:
		tc.export_gsize(&gsize);
	}

	// Compute same generation data:
	if(opt_sg) {
		std::cout << "    Compute same generation data ...\n";
		SG sg(output.graph());
		std::cout << "\tdone\n";
		std::cout << "\n";

		// Show computation result:
		if(opt_verbose)
			sg.print();

		// Store computed size data in graph size object:
		sg.export_gsize(&gsize);
	}

	// Print size data from formula:
	std::cout << "    Size Data Computed With Formulas:\n";
	//std::cout << "    ================================\n";
	formula.print();
	std::cout << "\n";

	// If size data was computed from graph, print that and check result:
	if(opt_tc || opt_sg) {
		std::cout << "    Size Data Computed From Graph:\n";
		//std::cout << "    =============================\n";
		gsize.print();
		std::cout << "\n";

		// Check consistency:
		if(gsize.consistent(&formula))
			std::cout << "    Values are consistent.\n";
		else {
			std::cout << "    *** VALUES DIFFER! ***\n";
			std::cerr << "Inconsistent size values detected!\n";
			exit(26);
		}
		std::cout << "\n";
	}

	// Write cost data. First open output file in append mode:
	std::ofstream cost_file;
	cost_file.open(COST_FILE, std::ios::out|std::ios::app);
	if(cost_file.fail() || !cost_file.is_open()) {
		std::cerr << "Opening '" << COST_FILE << "' failed.\n";
		exit(27);
	}

	// Get tc_size where defined (from formula or computed from graph):
	bigint_t tc_size = formula.tc_size();
	if(tc_size < 0)
		tc_size = gsize.tc_size();

	// Get tc_iter where defined:
	int tc_iter = formula.tc_iter();
	if(tc_iter < 0)
		tc_iter = gsize.tc_iter();

	// Get tc_inst where defined:
	bigint_t tc_inst = formula.tc_inst();
	if(tc_inst < 0)
		tc_inst = gsize.tc_inst();

	// If all three are defined, write cost information:
	if(tc_size >= 0 && tc_iter >= 0 && tc_inst >= 0) {
		cost_file << "INSERT INTO " << COST_TABLE << " VALUES(" <<
			"'tcff'" << "," << "'" << graph_id << "'" << "," <<
			tc_size << "," << tc_iter << "," << tc_inst << ");\n";
	}

	// Get sg_size where defined (from formula or computed from graph):
	bigint_t sg_size = formula.sg_size();
	if(sg_size < 0)
		sg_size = gsize.sg_size();

	// Get sg_iter where defined:
	int sg_iter = formula.sg_iter();
	if(sg_iter < 0)
		sg_iter = gsize.sg_iter();

	// Get sg_inst where defined:
	bigint_t sg_inst = formula.sg_inst();
	if(sg_inst < 0)
		sg_inst = gsize.sg_inst();

	// If all three are defined, write cost information:
	if(sg_size >= 0 && sg_iter >= 0 && sg_inst >= 0) {
		cost_file << "INSERT INTO " << COST_TABLE << " VALUES(" <<
			"'sgff'" << "," << "'" << graph_id << "'" << "," <<
			sg_size << "," << sg_iter << "," << sg_inst << ");\n";
	}

	// Close file:
	cost_file.close();
	if(cost_file.fail()) {
		std::cerr << "Closing '" << COST_FILE << "' failed.\n";
		exit(28);
	}

	// Print elapsed time in seconds:
	time_t end_time;
	time(&end_time);
	int seconds = (int) difftime(end_time, start_time);
	std::cout << "    Elapsed time: " << seconds << "s\n";
	std::cout << "\n";
}

