import java.io.File;
//import java.io.FileReader;
import java.io.FileInputStream;
import java.io.InputStreamReader;
import java.io.BufferedReader;
import java.io.IOException;
import java.io.FileNotFoundException;
import java.io.UnsupportedEncodingException;

import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.ResultSet;
//import java.sql.ResultSetMetaData;
import java.sql.SQLException;
import java.sql.Statement;
import java.sql.PreparedStatement;

// ============================================================================
// Start of Class for DBLP Benchmark:
// ============================================================================

public class DBLP {


	// --------------------------------------------------------------------
	// Constants (Database Connection):
	// --------------------------------------------------------------------

	// Class with JDBC Driver:
	private static final String JDBC_DRIVER = "org.hsqldb.jdbcDriver";

	// URI prefix for JDBC connection:
	private static final String JDBC_URI_PREFIX = "jdbc:hsqldb:file:";

	// Database file name without extension (database name):
	private static final String DB_FILE_NAME = "db";

	// Database Account (system administrator):
	private static final String DB_ACCOUNT = "SA";

	// Database Password (empty, database is only accessible on this pc):
	private static final String DB_PASSWORD = "";

	// File extension of lock file of HSQLDB (including "."):
	// private static final String DB_LOCK_EXT = ".lck";

	// File extension of script file of HSQLDB (including "."):
	// private static final String DB_SCRIPT_EXT = ".script";

	// DBLP data file (in csv-format):
	private static final String DBLP_DATA = "../data_csv/dblp.csv";


	// --------------------------------------------------------------------
	// Static Attributes:
	// --------------------------------------------------------------------

	// Database connection (session):
	private static Connection dbConnection = null;

	// Current prepared statement:
	private static PreparedStatement currStmt = null;

	// SQL text of current prepared statement (for debugging):
	//private static String currSQL = null;

	// Current result set (after query was executed):
	//private static ResultSet currResult = null;

	// Number of rows processed:
	private static int totalRows = 0;

	// --------------------------------------------------------------------
	// connect - connect to database
	// --------------------------------------------------------------------

	/**
	 * Connect to database (HSQLDB).
	 * @param dir Absolute path to directory where database is stored
	 *            (as Java File object).
	 * @throws Exception if connection failed
	 */

	public static void connect(File dir) throws Exception {
		// The following command loads and initializes the JDBC driver,
		// which then registers itself with the driver manager.
		// Since Java 6 and current JDBC drivers, this is no longer
		// necessary, but it does not hurt:
		try {
			Class.forName(JDBC_DRIVER);
		} catch(ClassNotFoundException e) {
			System.err.println("Driver not found.");
			throw e;
		} catch(Exception e) {
			System.err.println("error loading JDBC driver.");
			throw e;
		}
		
		// Construct connection URI:
		File dbFilePath = new File(dir, DB_FILE_NAME);
		String uri = JDBC_URI_PREFIX + dbFilePath.getAbsolutePath();

		// Connect:
		try {
			dbConnection =
				DriverManager.getConnection(uri, DB_ACCOUNT,
						DB_PASSWORD);
			System.out.println("AutoCommit Mode: " +
					dbConnection.getAutoCommit());
			// Autocommit mode is probably default anyway:
			dbConnection.setAutoCommit(true);
		}
		catch(SQLException e) {
			System.err.println("Error during connect operation.");
			throw  e;
		}
	}

	// --------------------------------------------------------------------
	// shutdown - shutdown database (must be done at the end!)
	// --------------------------------------------------------------------

	/**
	 * Shutdown the database.
	 * It is important to do this at the end.
	 * Otherwise the lock file remains and changed data might not be
	 * written to the database file.
	 * @throws SQLException if an error occurred during shutdown.
	 */

	public static void shutdown() throws SQLException {
		// Do nothing if there is no open connection:
		if(dbConnection == null)
			return;
	
		// Shutdown the database:
		Statement s = null;
		try {
			dbConnection.commit();
			s = dbConnection.createStatement();
			s.executeUpdate("SHUTDOWN");
			dbConnection.close();
		}
		catch(SQLException e) {
			System.err.println("Error in shutdown operation");
			throw e;
		}
		finally {
			if(s != null) {
				try {
					s.close();
				}
				catch(SQLException e) {
					System.err.println(
					"Error while closing statement");
					throw e;
				}
			}
		}
	}


	// --------------------------------------------------------------------
	// createTable - create table (when connecting for the first time).
	// --------------------------------------------------------------------

	/**
	 * Create table when connecting to a fresh database for the first
	 * time.
	 * Note that {@link #connect(File) connect(dir)} must be called
	 * before this method is called, i.e. the connection to the database
	 * must already be open.
	 * HSQLDB automatically creates a database if it does not exist.
	 * Therefore, the purpose of this method is only to create the table
	 * in a fresh database.
	 *
	 * @throws SQLException if some error occurs
	 */

	public static void createTable() throws SQLException {
		// The database connection must be open:
		//assert dbConnection != null :
		//	"DB must be open before tables can be created";

		// Create table statements:
		String sql =
			"CREATE CACHED TABLE ATT(" +
			"    DOC_ID    VARCHAR(70) NOT NULL, " +
			"    ATTRIBUTE VARCHAR(20) NOT NULL, " +
			"    VALUE     VARCHAR(2000) NOT NULL)";
		// NOTE: VARACHR(60) for ID is not enough
		// See data file line 76150:
	// 'journals/fgcs/LeighRJJJSSSAWVLSPGKGLVDBCPDWLTRPCDWSKKHKCZBG06'
		// VARCHAR(1200) for VALUE is not enough
		// See line 3644132 in data file: 1700 characters.


		// Create table:
		Statement s = null;
		try {
			s = dbConnection.createStatement();
			s.executeUpdate(sql);
		}
		catch(SQLException e) {
			System.err.println("Error while creating table.");
			throw e;
		}
		finally {
			if(s != null) {
				try {
					s.close();
				}
				catch(SQLException e) {
					System.err.println("Error while " +
						"closing statement");
					throw e;
				}
			}
		}
	}


	// --------------------------------------------------------------------
	// createIndex - create index (when connecting for the first time).
	// --------------------------------------------------------------------

	/**
	 * Create index when connecting to a fresh database for the first
	 * time.
	 * Note that {@link #connect(File) connect(dir)} must be called
	 * before this method is called, i.e. the connection to the database
	 * must already be open.
	 * HSQLDB automatically creates a database if it does not exist.
	 * Therefore, the purpose of this method is only to create the table
	 * in a fresh database.
	 *
	 * @throws SQLException if some error occurs
	 */

	public static void createIndex() throws SQLException {
		// The database connection must be open:
		//assert dbConnection != null :
		//	"DB must be open before tables can be created";

		// Create table statements:
		String sql =
			//"CREATE INDEX ATT_INDEX ON ATT(ATTRIBUTE,DOC_ID)";
			"CREATE INDEX ATT_INDEX ON ATT(ATTRIBUTE,DOC_ID,VALUE)";

		// Create index:
		Statement s = null;
		try {
			s = dbConnection.createStatement();
			s.executeUpdate(sql);
		}
		catch(SQLException e) {
			System.err.println("Error while creating index.");
			throw e;
		}
		finally {
			if(s != null) {
				try {
					s.close();
				}
				catch(SQLException e) {
					System.err.println("Error while " +
						"closing statement");
					throw e;
				}
			}
		}
	}

	// --------------------------------------------------------------------
	// prepareStmt - Parepare an SQL Statement for Excecution.
	// --------------------------------------------------------------------

	/**
	 * Prepare an SQL statement for execution.
	 * This simple database interface can have only one statement
	 * open at the time. Thus the previous statement must be closed
	 * before prepareStmt can be called.
	 * The advantage is that other classes do not need to work with
	 * JDBC objects directly - everything is kept here in static variables.
	 * Of course, a database connection must be open before prepareStmt
	 * can be called.
	 *
	 * @param sql The SQL statement to be prepared.
	 * @throws SQLException if an error occurred during the prepare
	 * 		operation, e.g. the SQL statement contained a
	 * 		syntax error.
	 */

	public static void prepareStmt(String sql) throws SQLException {

		// Check parameter and state:
		//assert sql != null : "SQL Statement cannot be null";
		//assert dbConnection != null : "DB conection must be open";
		//assert currStmt == null :
		//		"Previous statement must be finished";
		//assert currResult == null : "Previous result not closed";

		// Prepare the statement:
		try {
			//currSQL = sql;
			currStmt = dbConnection.prepareStatement(sql);
		}
		catch(SQLException e) {
			System.err.println("Error preparing statement.");
			throw e;
		}
	}


	// --------------------------------------------------------------------
	// execQuerySingleInt - Execute Prepared SQL Statement as Query.
	// --------------------------------------------------------------------

	/**
	 * Executes the current prepared statement as a query.
	 * In this special case, the query must return exactly one row
	 * which has a single column of type int.
	 * A typical application is a count() aggregation query.
	 * Of course, a statement must have been prepared before,
	 * see {@link #prepareStmt(String) prepareStmt(sql)}.
	 * This statement must be a query.
	 *
	 * @return the value of the first column of the result.
	 * @throws SQLException if an error occurred during the execution
	 *		of the prepared statement.
	 */

	public static int execQuerySingleInt() throws SQLException {

		// Check state:
		//assert dbConnection != null : "DB conection must be open";
		//assert currStmt != null : "No Statement open";
		//assert currResult == null : "Previous result not closed";

		// Execute the statement:
		try {
			ResultSet rs = currStmt.executeQuery();
			if(!rs.next()) {
				System.err.println("Query result is empty");
				return -1;
			}
			int value = rs.getInt(1);
			if(rs.next()) {
				System.err.println("There is more than one " +
							"Result!");
			}
			rs.close();
			return value;
		}
		catch(SQLException e) {
			System.err.println("Error while executing SQL " +
						"statement");
			throw e;
		}
	}


	// --------------------------------------------------------------------
	// execUpdate - Execute Prepared SQL Statement as Update.
	// --------------------------------------------------------------------

	/**
	 * Executes the current prepared statement as an update.
	 * The number of modified/inserted rows is returned.
	 * Of course, a statement must have been prepared before,
	 * see {@link #prepareStmt(String) prepareStmt(sql)}.
	 * This statement must be an INSERT/UPDATE/DELETE or a DDL statement
	 * (i.e. something which does not return a query result).
	 *
	 * @return the number of rows affected by the update.
	 *		In rare cases, -1 might be returned, if the
	 *		interface does not know how many rowsv are updated,
	 *		e.g. if the statement is a call of a stored procedure.
	 *		A negative result does not necessarily mean that
	 *		an error occurred.
	 * @throws Exception if an error occurred during the execution
	 *		of the prepared statement.
	 */

	public static int execUpdate() throws SQLException {

		// Check state:
		//assert dbConnection != null : "DB conection must be open";
		//assert currStmt != null : "No Statement open";
		//assert currResult == null : "Previous result not closed";

		// Remember that an update was executed:
		//DB.wasUpdated = true;

		// Execute the statement, return number of affected rows:
		try {
			return currStmt.executeUpdate();
		}
		catch(SQLException e) {
			System.err.println("Error occurred during update");
			throw e;
		}
	}


	// --------------------------------------------------------------------
	// loadData - Load DBLP data from csv-file.
	// --------------------------------------------------------------------

	public static void loadData() throws Exception {

		// Create file object, check existence:
		File dblp_data = new File(DBLP_DATA);
		if(!dblp_data.exists()) {
			System.err.println("Data file '" + DBLP_DATA +
						"' does not exist.");
			return;
		}

		// Open file:
		FileInputStream inputStream;
		try {
			inputStream = new FileInputStream(dblp_data);
		}
		catch(FileNotFoundException e) {
			System.err.println("Error while opening '" +
					DBLP_DATA + "'");
			throw e;
		}

		// Create InputStreamReader for ISO Latin 1 Encoding:
		InputStreamReader reader;
		try {
			reader = new InputStreamReader(inputStream,
							"ISO-8859-1");
		}
		catch(UnsupportedEncodingException e) {
			System.err.println("ISO Latin 1 Encoding " +
						"is not supported!");
			throw e;
		}

		// Create buffered reader:
		BufferedReader bufReader = new BufferedReader(reader);

		// Read lines:
		totalRows = 0;
		try {
			String line = bufReader.readLine();
			while(line != null) {
				String insert = "INSERT INTO ATT VALUES(" +
						line + ")";
				prepareStmt(insert);
				int rows = execUpdate();
				if(rows != 1) {
					System.err.println("Insertion  of row "
						+ totalRows + " failed");
					return;
				}
				totalRows++;
				if(totalRows % 10000 == 0) {
					System.out.println(totalRows +
							" rows inserted.");
					System.out.flush();
				}
				line = bufReader.readLine();
			}
		}
		catch(IOException e) {
			System.err.println("Error while reading from '" +
					DBLP_DATA + "'.");
			throw e;
		}
		catch(SQLException e) {
			System.err.println("Error while inserting data: " +
				" row " + (totalRows+1));
			throw e;
		}

		// File load successfully finished:
		System.out.println("File load done: " + totalRows + " rows.");
	}

	// --------------------------------------------------------------------
	// runQuery - Run the benchmark query.
	// --------------------------------------------------------------------

	public static int runQuery() throws Exception {
		String query =
			"SELECT COUNT(*) " +
			"FROM   ATT ATT_TITLE, " +
			"       ATT ATT_YEAR, " +
			"       ATT ATT_AUTHOR, " +
			"       ATT ATT_MONTH " +
			"WHERE  ATT_TITLE.ATTRIBUTE = 'title' " +
			"  AND  ATT_YEAR.ATTRIBUTE  = 'year' " +
			"  AND  ATT_AUTHOR.ATTRIBUTE  = 'author' " +
			"  AND  ATT_MONTH.ATTRIBUTE  = 'month' " +
			"  AND  ATT_TITLE.DOC_ID = ATT_YEAR.DOC_ID " +
			"  AND  ATT_TITLE.DOC_ID = ATT_AUTHOR.DOC_ID " +
			"  AND  ATT_TITLE.DOC_ID = ATT_MONTH.DOC_ID";
		prepareStmt(query);
		int result = execQuerySingleInt();
		return result;
	}


	// --------------------------------------------------------------------
	// Main program:
	// --------------------------------------------------------------------

	static public void main(String[] args) {
		long time_start = System.currentTimeMillis();
		System.out.println("Welcome to DBLP Benchmark for HSQLDB");

		// Print memory:
		Runtime runtime = Runtime.getRuntime();
		long mem = runtime.totalMemory();
		System.out.println("Total memory: " + (mem/1024/1024) + " MB");
		mem = runtime.maxMemory();
		System.out.println("Max. memory:  " + (mem/1024/1024) + " MB");

		try {
			// Directory for database:
			File dir = new File("db");

			// Connect to Database:
			long time_before = System.currentTimeMillis();
			connect(dir);
			long time_after = System.currentTimeMillis();
			System.out.println("Time for connect to database: " +
				(time_after - time_before) + " ms");

			// Create table and fill database:
			if(args.length > 0 && args[0].equals("init")) {
				// Create table:
				time_before = System.currentTimeMillis();
				createTable();
				time_after = System.currentTimeMillis();
				System.out.println("Time for creating table: "+
					(time_after - time_before) + " ms");

				// Create index:
				time_before = System.currentTimeMillis();
				createIndex();
				time_after = System.currentTimeMillis();
				System.out.println("Time for creating index: "+
					(time_after - time_before) + " ms");

				// Load data:
				System.out.println("Loading data ...");
				time_before = System.currentTimeMillis();
				loadData();
				time_after = System.currentTimeMillis();
				System.out.println("Time for loading data: "+
					(time_after - time_before) + " ms");
				
			}

			// Run benchmark query:
			System.out.println("Benchmark query runs ...");
			time_before = System.currentTimeMillis();
			int result = runQuery();
			time_after = System.currentTimeMillis();
			System.out.println("Result of benchmark query: " +
						result);
			System.out.println("Time for benchmark query:  " +
				(time_after - time_before) + " ms");

			// Shutdown database:
			System.out.println("Shutting down database ...");
			time_before = System.currentTimeMillis();
			shutdown();
			time_after = System.currentTimeMillis();
			System.out.println("Time for shutdown database: " +
				(time_after - time_before) + " ms");
		} catch(Throwable e) {
			System.err.println("Exception occurred:");
			System.err.println(e);
			e.printStackTrace();
			System.err.println("Number of rows processed so far: "
						+ totalRows);
		}
		long time_stop = System.currentTimeMillis();
		System.out.println("Total time:   " +
			(time_stop - time_start) + " ms");

		// Print memory:
		mem = runtime.totalMemory();
		System.out.println("Total memory: " + (mem/1024/1024) + " MB");
	}


	// --------------------------------------------------------------------
	// End of Class:
	// --------------------------------------------------------------------
}
