-- ============================================================================
-- Project:	Deductive Database
-- Filename:	bench_db.sql
-- Purpose:	Database for Benchmark Runs
-- Last Change:	01.10.2018
-- Language:	SQL (Tested with PostgreSQL)
-- Author:	Stefan Brass
-- EMail:	brass@informatik.uni-halle.de
-- WWW:		http://www.informatik.uni-halle.de/~brass/
-- Address:	Feldschloesschen 15, D-06120 Halle (Saale), GERMANY
-- Copyright:	(c) 2016-2018 by Stefan Brass
-- License:	See file "LICENSE" for copying conditions.
-- Note:	There is no warranty at all - this code may contain bugs.
-- ============================================================================


-- ============================================================================
-- Delete Old Versions of the Tables:
-- ============================================================================

-- When this script is executed the first time,
-- these commands generate errors (because the tables/views do not exist).

DROP VIEW TCFF_FILE_IDS;
DROP VIEW BENCH_FILE_LIST;

DROP VIEW GRAPH_DATA_MD;
DROP VIEW DATA_FILE_MD;
DROP VIEW BENCH_IMPL_MD;

DROP VIEW  PAR_RANGE;
DROP VIEW  LOAD_EFFICIENCY;
DROP VIEW  ESTIMATE_DATA;

DROP VIEW  NUM_BAD_ESTIMATES;
DROP VIEW  ESTIMATE_ERR;
DROP VIEW  ESTIMATE_FORMULA;
DROP VIEW  SPEED_REAL;
DROP VIEW  SPEED_REAL_DIFF;
DROP VIEW  SPEED_CPU;
DROP VIEW  MEM_EFFICIENCY;

DROP VIEW  PCT_CPU;
DROP VIEW  PCT_CPU_INPUT;
DROP VIEW  REAL_TOO_LARGE;
DROP VIEW  OUTLIER_MEM;
DROP VIEW  OUTLIER_CPU;
DROP VIEW  OUTLIER_REAL;

DROP VIEW  GRAPH_DATA_HTML;
DROP VIEW  GRAPH_DATA_LATEX;

DROP VIEW  TCFF_CPU;

DROP VIEW  TCFF_REAL_LATEX;
DROP VIEW  TCFF_REAL_FACTORS;
DROP VIEW  TCFF_REAL;

DROP VIEW  TCFF_REAL_XSB_HTML;
DROP VIEW  TCFF_REAL_XSB_LATEX;
DROP VIEW  TCFF_REAL_XSB_FACTORS;
DROP VIEW  TCFF_REAL_XSB;

DROP VIEW  TCFF_MEM;
DROP VIEW  TCFF_CPU_LATEX;
DROP VIEW  PLOT_MEM;
DROP VIEW  PLOT_CPU;
DROP VIEW  PLOT_REAL;

DROP VIEW  BEST_IMPL_CPU;
DROP VIEW  BEST_IMPL_REAL;

DROP VIEW  XSB;
DROP VIEW  BAM;
DROP VIEW  YAP;
DROP VIEW  SQLITE;
DROP VIEW  PSQL;
DROP VIEW  MARIA_COMPARE;
DROP VIEW  MARIA;
DROP VIEW  SOUFFLE;
DROP VIEW  JENA;
DROP VIEW  DATASCRIPT;

DROP VIEW  BEST_RESULT;
DROP VIEW  RESULT;
DROP VIEW  RUN_STATISTICS;
DROP VIEW  RUN_STAT;

DROP TABLE ESTIMATE_PAR;

DROP TABLE DUMMY_TAB;

DROP TABLE BENCH_RUN;
DROP TABLE BENCH_IMPL;
DROP TABLE GRAPH_DATA;
DROP TABLE GRAPH_TYPE;
DROP TABLE DATA_FILE;
DROP TABLE PROG_INSTALL;
DROP TABLE PROG_VER;
DROP TABLE PROGRAM;
DROP TABLE PROG_TYPE;
DROP TABLE BENCHMARK;
DROP TABLE INPUT_TYPE;
DROP TABLE MACHINE;

-- ============================================================================
-- List of Machines on which Benchmarks are Executed:
-- ============================================================================

CREATE TABLE MACHINE(
	MACHINE     VARCHAR(20)  NOT NULL,
	SEQ_NO      NUMERIC(2)   NOT NULL,
	STATUS      CHAR(1)      NOT NULL,
        MODEL       VARCHAR(60)  NULL,
	YEAR        NUMERIC(4)   NULL,
        MONTH       NUMERIC(2)   NULL,
        PRICE       NUMERIC(7)   NULL,
	NUM_CPUS    NUMERIC(4)   NOT NULL,
	CPU         VARCHAR(60)  NOT NULL,
	CPU_CORES   NUMERIC(4)   NULL,
        CPU_THREADS NUMERIC(4)   NULL,
	GHZ         NUMERIC(5,2) NULL,
	ARCH        VARCHAR(20)  NULL,
	RAM         NUMERIC(8,3) NOT NULL,
	NUM_DISKS   NUMERIC(4)   NULL,
        DISK        VARCHAR(40)  NULL,
	SSD         CHAR(1)      NULL,
	OS          VARCHAR(40)  NULL,
	OS_CORE     VARCHAR(40)  NULL,
	CONSTRAINT MACHINE_PK
		PRIMARY KEY(MACHINE),
	CONSTRAINT MACHINE_KEY_SEQ
		UNIQUE (SEQ_NO),
	CONSTRAINT MACHINE_SEQ_NO_POSITIVE
		CHECK(SEQ_NO > 0),
        CONSTRAINT MACHINE_YEAR_VALID
		CHECK(YEAR >= 2000),
        CONSTRAINT MACHINE_MONTH_VALID
                CHECK(MONTH >= 1 AND MONTH <= 12),
        CONSTRAINT MACHINE_PRICE_NON_NEGATIVE
                CHECK(PRICE >= 0),
	CONSTRAINT MACHINE_STATUS_S_OR_SPACE
		CHECK(STATUS IN ('S', ' ')),
		-- The machine with STATUS='S' is the standard machine.
		-- Only one row of the table should have STATUS='S'.
		-- Some views select only benchmark results on the standard
		-- machine.
		-- In future, there many be more STATUS values, e.g. for
		-- comparing the standard machine to an alternative machine.
	CONSTRAINT MACHINE_NUM_CPUS_POSITIVE
		CHECK(NUM_CPUS > 0),
	CONSTRAINT MACHINE_CPU_CORES_POSITIVE
		CHECK(CPU_CORES > 0),
	CONSTRAINT MACHINE_CPU_THREADS_POSITIVE
		CHECK(CPU_THREADS > 0),
	CONSTRAINT MACHINE_RAM_POSITIVE
		CHECK(RAM > 0),
	CONSTRAINT MACHINE_NUM_DISKS_POSITIVE
		CHECK(NUM_DISKS > 0),
	CONSTRAINT MACHINE_SSD_YN
		CHECK(SSD IN ('Y','N')));

INSERT INTO MACHINE VALUES(
	'dbs1', 1, 'S',
	'HP Blade Server, HPE BL460c Gen9',
        2016, 10, 3319,
	2, 'Intel(R) Xeon(R) CPU E5-2630 v4 @ 2.20GHz', 10, 20, 2.2, 'x86_64',
	64,
	NULL, NULL, NULL,
	'Debian GNU/Linux 8.10 (jessie)',
	'Linux 3.16.0-5-amd64 x86_64');

INSERT INTO MACHINE VALUES(
	'bach', 2, ' ',
	'silentmaxx Workstation KENKO Fanless Xeon E5',
	2017, 12, 1933,
	1, 'Xeon Intel 2011 E5-1620v4 4/8x 3,5GHz', 4, 8, 3.5, 'x86_64',
	16,
	1, 'Samsung SSD 850 EVO 500GB', 'Y',
	'CentOS Linux release 7.4.1708 (Core)',
	'Linux 3.10.0-693.21.1.el7.x86_64 x86_64');

-- Note that the CORE might not be current. It changes with every update.
-- It was current when we started the benchmarks in May 2018.
-- It gives some impression of the machine and its operating system.

-- ============================================================================
-- List of Input Data (Problem Instance) Types:
-- ============================================================================


CREATE TABLE INPUT_TYPE(
	ITYPE       VARCHAR(8)  NOT NULL,
	SEQ_NO      NUMERIC(2)  NOT NULL,
	HEADLINE    VARCHAR(80) NOT NULL,
	DESCRIPTION VARCHAR(80) NOT NULL,
	CONSTRAINT INPUT_TYPE_SEQ_NO_NONNEGATIVE
		CHECK(SEQ_NO >= 0),
	CONSTRAINT INPUT_TYPE_PK
		PRIMARY KEY(ITYPE),
	CONSTRAINT INPUT_TYPE_SEQ_NO_UNIQUE
		UNIQUE(SEQ_NO));

INSERT INTO INPUT_TYPE VALUES('tc', 1,
	'Input Files for the TC Benchmarks (Directed Graphs)',
	'Directed graph with edges stored in predicate par');

INSERT INTO INPUT_TYPE VALUES('j1', 2,
	'Input Files for Join 1 Benchmark',
	'Directed graph with edges in five relations: c2, c3, c4, d1, d2');

INSERT INTO INPUT_TYPE VALUES('dblp', 3,
	'Input Files for DBLP Benchmark',
	'DBLP data in an EAV table att');

INSERT INTO INPUT_TYPE VALUES('wine', 4,
	'Input File for Wine Ontology Benchmark',
	'Wine ontology facts');


-- ============================================================================
-- List of Benchmarks:
-- ============================================================================


CREATE TABLE BENCHMARK(
	BENCH      VARCHAR(20)  NOT NULL,
	SEQ_NO     NUMERIC(2)   NOT NULL,
	ITYPE      VARCHAR(8)   NULL,
	HEADLINE   VARCHAR(80)  NOT NULL,
	CONSTRAINT BENCHMARK_SEQ_NONNEGATIVE
		CHECK(SEQ_NO >= 0),
	CONSTRAINT BENCHMARK_PK
		PRIMARY KEY (BENCH),
	CONSTRAINT BENCHMARK_KEY_SEQ
		UNIQUE (SEQ_NO),
	CONSTRAINT BENCHMARK_REF_INPUT_TYPE
		FOREIGN KEY (ITYPE) REFERENCES INPUT_TYPE);

INSERT INTO BENCHMARK VALUES (
	'dblp', 1, 'dblp',
	'DBLP Benchmark');

INSERT INTO BENCHMARK VALUES (
	'j1axy', 2, 'j1',
	'The Join1 Benchmark, Query a(X,Y)');

INSERT INTO BENCHMARK VALUES (
	'j1axyd', 3, 'j1',
	'The Join1 Benchmark (without duplicate elimination)');

INSERT INTO BENCHMARK VALUES (
	'tcff', 4, 'tc',
	'Transitive Closure Benchmark without Query Bindings');

INSERT INTO BENCHMARK VALUES (
	'tcbf', 5, 'tc',
	'Transitive Closure Benchmark with the First Argument Bound');

INSERT INTO BENCHMARK VALUES (
	'tcbfs', 6, 'tc',
	'Transitive Closure Benchmark, First Argument Bound, with SLDMagic');

INSERT INTO BENCHMARK VALUES (
	'tcfb', 7, 'tc',
	'Transitive Closure Benchmark with the Second Argument Bound');

INSERT INTO BENCHMARK VALUES (
	'wine', 8, 'wine',
	'Wine Ontology Benchmark');

INSERT INTO BENCHMARK VALUES (
	'loadd', 9, 'dblp',
	'Test of Loader for DBLP Data');

INSERT INTO BENCHMARK VALUES (
	'loadt', 10, 'tc',
	'Test of Loader for Transitive Closure');

INSERT INTO BENCHMARK VALUES (
	'set1', 11, NULL,
	'Test of Set Insertion and Lookup (One Column)');

INSERT INTO BENCHMARK VALUES (
	'set2', 12, NULL,
	'Test of Set Insertion and Lookup (Two Columns)');

-- ============================================================================
-- Program Types (for Subheadlines in the list of programs to be tested):
-- ============================================================================

CREATE TABLE PROG_TYPE(
	PTYPE      VARCHAR(10)  NOT NULL,
	SEQ_NO     NUMERIC(2)   NOT NULL,
	HEADLINE   VARCHAR(80)  NOT NULL,
	CONSTRAINT PROG_TYPE_SEQ_NONNEGATIVE
		CHECK(SEQ_NO >= 0),
	CONSTRAINT PROG_TYPE_PK
		PRIMARY KEY (PTYPE),
	CONSTRAINT PROG_TYPE_KEY_SEQ
		UNIQUE (SEQ_NO));

INSERT INTO PROG_TYPE VALUES ('Prolog',  1, 'Prolog Systems');
INSERT INTO PROG_TYPE VALUES ('Datalog', 2, 'Datalog Systems');
INSERT INTO PROG_TYPE VALUES ('RDB',     3, 'Relational Database Systems');
INSERT INTO PROG_TYPE VALUES ('Graph',   4, 'Graph Databases and RDF Stores');


-- ============================================================================
-- List of Programs to be Tested/Compared:
-- ============================================================================

CREATE TABLE PROGRAM(
	PROG        VARCHAR(20)  NOT NULL,
	SEQ_NO      NUMERIC(2)   NOT NULL,
	FULL_NAME   VARCHAR(40)  NOT NULL,
	PTYPE       VARCHAR(10)  NOT NULL,
	URL         VARCHAR(80)  NULL,
	REMARK      VARCHAR(800) NULL,
	CONSTRAINT PROGRAM_SEQ_NONNEGATIVE
		CHECK(SEQ_NO >= 0),
	CONSTRAINT PROGRAM_PK
		PRIMARY KEY (PROG),
	CONSTRAINT PROGRAM_KEY_SEQ
		UNIQUE (SEQ_NO),
	CONSTRAINT PROGRAM_REF_PROG_TYPE
		FOREIGN KEY(PTYPE) REFERENCES PROG_TYPE);

-- Prolog Systems (with Extensions):

INSERT INTO PROGRAM VALUES (
	'xsb', 10, 'XSB', 'Prolog',
	'http://xsb.sourceforge.net/',
	NULL);

INSERT INTO PROGRAM VALUES (
	'yap', 11, 'YAProlog', 'Prolog',
	'https://www.dcc.fc.up.pt/~vsc/Yap/',
	NULL);

INSERT INTO PROGRAM VALUES (
	'swipl', 12, 'SWI-Prolog', 'Prolog',
	'http://www.swi-prolog.org/',
	NULL);

INSERT INTO PROGRAM VALUES (
	'mercury', 13, 'Melbourne Mercury', 'Prolog',
	'http://www.mercurylang.org/',
	NULL);

-- Datalog Systems:

INSERT INTO PROGRAM VALUES (
	'bam', 21, 'Push (bam)', 'Datalog',
	'http://users.informatik.uni-halle.de/~brass/push/',
	NULL);

INSERT INTO PROGRAM VALUES (
	'ydb', 22, 'Push (ydb)', 'Datalog',
	'http://users.informatik.uni-halle.de/~brass/push/',
	NULL);

INSERT INTO PROGRAM VALUES (
	'dlv', 23, 'DLV', 'Datalog',
	'http://www.dlvsystem.com/',
	NULL);

INSERT INTO PROGRAM VALUES (
	'souffle', 24, 'Souffle', 'Datalog',
	'http://souffle-lang.org/',
	NULL);

INSERT INTO PROGRAM VALUES (
	'datascript', 25, 'Datascript', 'Datalog',
	'https://github.com/tonsky/datascript',
	NULL);

-- Relational Databases:

INSERT INTO PROGRAM VALUES (
	'hsql', 31, 'HSQLDB', 'RDB',
	'http://hsqldb.org/',
	NULL);

INSERT INTO PROGRAM VALUES (
	'sqlite3', 32, 'SQLite 3', 'RDB',
	'https://www.sqlite.org/index.html',
	NULL);

INSERT INTO PROGRAM VALUES (
	'mariadb', 33, 'MariaDB', 'RDB',
	'https://www.mariadb.com',
	NULL);

INSERT INTO PROGRAM VALUES (
	'psql', 34, 'PostgreSQL', 'RDB',
	'https://www.postgresql.org',
	NULL);

-- Graph Databases and RDF Stores:

INSERT INTO PROGRAM VALUES (
	'jena', 41, 'Apache Jena', 'Graph',
	'https://jena.apache.org/',
	NULL);


-- ============================================================================
-- Table for Program Versions:
-- ============================================================================

-- This table contains information about program versions.
--
-- VER_ADDTEXT should be concatenated with VER to get the full name
-- of the version.
--
-- This table also permits to store additional information about a version
-- in the column REMARK (e.g., the major changes).
--
-- Note that this table describes a version of the source code.
-- PROG_INSTALL (below) describes a version of the program binary,
-- which also depends on the machine, the compiler, compiler settings,
-- and configuration settings.

CREATE TABLE PROG_VER (
	PROG        VARCHAR(20)  NOT NULL,
	VER         VARCHAR(20)  NOT NULL,
	VER_ADDTEXT VARCHAR(80)  NOT NULL,
	VER_DATE    DATE         NULL,
	REMARK      VARCHAR(800) NULL,
	CONSTRAINT PROG_VER_PK
		PRIMARY KEY (PROG, VER),
	CONSTRAINT PROG_VER_REF_PROGRAM
		FOREIGN KEY (PROG) REFERENCES PROGRAM);

-- Prolog Systems (with Extensions):

INSERT INTO PROG_VER VALUES (
	'xsb', '3.7.', ' (Clan MacGregor)', '2016-07-06',
	NULL);

INSERT INTO PROG_VER VALUES (
	'xsb', '3.8.0', ' (Three-Buck Chuck)', '2017-10-28',
	NULL);

INSERT INTO PROG_VER VALUES (
	'yap', '6.2.2', ' (x86_64-linux)', '2011-10-21',
	NULL);

INSERT INTO PROG_VER VALUES (
	'yap', '6.3.3', ' (x86_64-linux)', '2013-01-20',
	NULL);

INSERT INTO PROG_VER VALUES (
	'swipl', '7.7.15', ' (x86_64-linux)', '2018-06-01',
	NULL);

INSERT INTO PROG_VER VALUES (
	'swipl', '7.5.15', ' (x86_64-linux)', '2017-09-12',
	NULL);

INSERT INTO PROG_VER VALUES (
	'swipl', '7.6.4', ' (x86_64-linux)', '2018-01-12',
	NULL);

INSERT INTO PROG_VER VALUES (
	'swipl', '7.7.7', ' (x86_64-linux)', '2017-12-21',
	NULL);

-- Datalog Systems:

INSERT INTO PROG_VER VALUES (
	'ydb', '0.1', '', '2016-11-21',
	NULL);

INSERT INTO PROG_VER VALUES (
	'bam', '0.1a', '', '2018-04-24',
	NULL);

INSERT INTO PROG_VER VALUES (
	'souffle', '1.0', '', NULL,
	NULL);

INSERT INTO PROG_VER VALUES (
	'datascript', '0.16.6', '', '2018-06-06',
	NULL);

-- Relational Databases:

INSERT INTO PROG_VER VALUES (
	'sqlite3', '3.8.7.1', '', '2014-10-29',
	NULL);

INSERT INTO PROG_VER VALUES (
	'sqlite3', '3.22.0', '', '2018-01-22',
	NULL);

INSERT INTO PROG_VER VALUES (
	'sqlite3', '3.24.0', '', '2018-06-04',
	NULL);

INSERT INTO PROG_VER VALUES (
	'sqlite3', '3.25.1', '', '2018-09-18',
	NULL);

INSERT INTO PROG_VER VALUES (
	'sqlite3', '3.25.2', '', '2018-09-25',
	NULL);

INSERT INTO PROG_VER VALUES (
	'psql', '9.4.15', '', '2017-11-09',
	NULL);

INSERT INTO PROG_VER VALUES (
	'mariadb', '10.0.35', '', '2018-05-03',
	NULL);

INSERT INTO PROG_VER VALUES (
	'mariadb', '10.3.9', '', '2018-08-15',
	NULL);

-- Graph Databases and RDF Stores:

INSERT INTO PROG_VER VALUES (
	'jena', '3.7.0', '', '2018-04-05',
	NULL);

INSERT INTO PROG_VER VALUES (
	'jena', '3.8.0', '', '2018-06-25',
	NULL);


-- ============================================================================
-- Table for Installed Program Versions on Machines:
-- ============================================================================

-- This table lists program versions that are or were installed
-- on a test machine.
-- They are identified here by a number INSTALL that is unique together with
-- the program name and the machine.
-- These three elements (program, installation ID, and machine)
-- are also encoded in the file names with the test results.
--
-- Whereas PROG_VER describes a version of the source code,
-- PROG_INSTALL describes a binary program.
-- For instance, if the same source code is compiled with g++ and with clang,
-- these are two different entries in PROG_INSTALL, both referring to the
-- same entry in PROG_VER.
-- Also different configuration settings may give different binaries,
-- and therefore different entries in PROG_INSTALL.
--
-- I try to make sure that the same version of a program
-- gets the same INSTALL number even on different machines,
-- but the database schema does not require this
-- (and it is impossible when there are really different installations
--  of the same program version).
--
-- Only program installations that are listed here can appear in the test runs.
--
-- This table also define a current version (more precisely, installation)
-- of a program on a machine (by STATUS='C').
-- Some views show only results for the current version.
-- It is also possible to define a previous version (STATUS='P') as reference,
-- if the result of a program update should be evaluated.
-- Finally, one can mark an alternative version (STATUS='A'),
-- e.g. compiled with a different compiler or compiler options,
-- which can also be used for comparison.

CREATE TABLE PROG_INSTALL (
	PROG        VARCHAR(20)  NOT NULL,
	VER         VARCHAR(20)  NOT NULL,
	MACHINE     VARCHAR(20)  NOT NULL,
	INSTALL     NUMERIC(4)   NOT NULL,
	COMPILER    VARCHAR(80)  NOT NULL,
	STATUS      CHAR(1)      NOT NULL,
	REMARK      VARCHAR(800) NULL,
	CONSTRAINT PROG_INSTALL_PK
		PRIMARY KEY (PROG, INSTALL, MACHINE),
	CONSTRAINT PROG_INSTALL_REF_PROG_VER
		FOREIGN KEY (PROG, VER) REFERENCES PROG_VER,
	CONSTRAINT PROG_INSTALL_REF_MACHINE
		FOREIGN KEY (MACHINE) REFERENCES MACHINE,
	CONSTRAINT PROG_INSTALL_STATUS_VALID
		CHECK(STATUS IN ('C','P','A',' ')));
	-- 'C': Current version
	-- 'P': Previous version
	-- 'A': Alternate version
	-- ' ': Any other version
	-- Views may select only benchmark results for the current version,
	-- or help to compare the current with the previous version.

-- Prolog Systems:

INSERT INTO PROG_INSTALL VALUES (
	'xsb', '3.7.', 'dbs1', 1,
	'',
	'C',
	'x86_64-unknown-linux-gnu 64 bits; ' ||
		'mode: optimal; engine: slg-wam; scheduling: local');

INSERT INTO PROG_INSTALL VALUES (
	'xsb', '3.8.0', 'dbs1', 2,
	'',
	'C',
	'x86_64-unknown-linux-gnu 64 bits; ' ||
		'mode: optimal; engine: slg-wam; scheduling: local');

INSERT INTO PROG_INSTALL VALUES (
	'xsb', '3.8.0', 'bach', 2,
	'',
	'C',
	'x86_64-unknown-linux-gnu 64 bits; ' ||
		'mode: optimal; engine: slg-wam; scheduling: local');

INSERT INTO PROG_INSTALL VALUES (
	'yap', '6.2.2', 'dbs1', 1,
	'',
	'C',
	NULL);

INSERT INTO PROG_INSTALL VALUES (
	'yap', '6.3.3', 'bach', 2,
	'',
	'C',
	NULL);

INSERT INTO PROG_INSTALL VALUES (
	'swipl', '7.7.15', 'dbs1', 1,
	'',
	'C',
	NULL);

INSERT INTO PROG_INSTALL VALUES (
	'swipl', '7.5.15', 'dbs1', 2,
	'',
	'P',
	NULL);

INSERT INTO PROG_INSTALL VALUES (
	'swipl', '7.6.4', 'dbs1', 3,
	'',
	'C',
	NULL);

INSERT INTO PROG_INSTALL VALUES (
	'swipl', '7.7.7', 'dbs1', 4,
	'',
	'C',
	NULL);

-- Datalog Systems:

INSERT INTO PROG_INSTALL VALUES (
	'ydb', '0.1', 'bach', 1,
	'gcc version 4.8.5 (-O2)',
	'C',
	NULL);

INSERT INTO PROG_INSTALL VALUES (
	'ydb', '0.1', 'dbs1', 1,
	'gcc version 4.9.2 (-O2)',
	'C',
	NULL);

INSERT INTO PROG_INSTALL VALUES (
	'bam', '0.1a', 'bach', 1,
	'gcc version 4.8.5 (-O2)',
	'C',
	NULL);

INSERT INTO PROG_INSTALL VALUES (
	'bam', '0.1a', 'dbs1', 1,
	'gcc version 4.9.2 (-O2)',
	'C',
	NULL);

INSERT INTO PROG_INSTALL VALUES (
	'souffle', '1.0', 'dbs1', 1,
	'',
	'C',
	NULL);

INSERT INTO PROG_INSTALL VALUES (
	'datascript', '0.16.6', 'dbs1', 1,
	'node 10.5.0',
	'C',
	NULL);

-- Relational Databases:

INSERT INTO PROG_INSTALL VALUES (
	'sqlite3', '3.8.7.1', 'dbs1', 1,
	'',
	'P',
	NULL);

INSERT INTO PROG_INSTALL VALUES (
	'sqlite3', '3.22.0', 'dbs1', 2,
	'',
	'P',
	NULL);

INSERT INTO PROG_INSTALL VALUES (
	'sqlite3', '3.24.0', 'dbs1', 3,
	'',
	'P',
	NULL);

INSERT INTO PROG_INSTALL VALUES (
	'sqlite3', '3.25.1', 'dbs1', 4,
	'',
	'C',
	NULL);

INSERT INTO PROG_INSTALL VALUES (
	'sqlite3', '3.25.2', 'dbs1', 5,
	'',
	'A',
	NULL);

INSERT INTO PROG_INSTALL VALUES (
	'psql', '9.4.15', 'dbs1', 1,
	'',
	'C',
	NULL);

INSERT INTO PROG_INSTALL VALUES (
	'mariadb', '10.0.35', 'dbs1', 1,
	'',
	'P',
	NULL);

INSERT INTO PROG_INSTALL VALUES (
	'mariadb', '10.3.9', 'dbs1', 2,
	'',
	'C',
	NULL);

-- Graph Databases and RDF Stores:

INSERT INTO PROG_INSTALL VALUES (
	'jena', '3.7.0', 'dbs1', 1,
	'openJDK 8',
	'P',
	NULL);

INSERT INTO PROG_INSTALL VALUES (
	'jena', '3.8.0', 'dbs1', 2,
	'openJDK 8',
	'C',
	NULL);


-- ============================================================================
-- Table for Implementation Variants of Benchmarks:
-- ============================================================================

CREATE TABLE BENCH_IMPL(
	PROG        VARCHAR(20)  NOT NULL,
	BENCH       VARCHAR(20)  NOT NULL,
	IMPL        VARCHAR(20)  NOT NULL,
	STATUS      CHAR(1)      NOT NULL,
	DESCRIPTION VARCHAR(80)  NULL,
	CONSTRAINT BENCH_IMPL_PK
		PRIMARY KEY (PROG, BENCH, IMPL),
	CONSTRAINT BENCH_IMPL_REF_BENCHMARK
		FOREIGN KEY (BENCH) REFERENCES BENCHMARK,
	CONSTRAINT BENCH_IMPL_REF_PROGRAM
		FOREIGN KEY (PROG) REFERENCES PROGRAM,
	CONSTRAINT BENCH_IMPL_STATUS_VALID
		CHECK(STATUS IN ('B','A',' ')));

-- STATUS='B' marks the implementation variant (option settings)
-- that is considered best and is used in comparisons between systems.
-- One can use the views BEST_IMPL_REAL and BEST_IMPL_CPU to check
-- whether these implementation variants are really best.
-- Sometimes the best implementation depends on the specific data file,
-- but it would somehow be unfair to adapt the options to the data file.

-- STATUS='A' marks an interesting alternative variants
-- that appear in some result tables.
-- There should be only one "best" version,
-- but there can be several alterative versions.

-- STATUS=' ' means that the variant was tested
-- in order to prove that the best version is really good,
-- but it will not be listed in summary tables.

-- ----------------------------------------------------------------------------
-- Benchmark Implementations for bam:
-- ----------------------------------------------------------------------------

-- Join1 Implementations for bam:

INSERT INTO BENCH_IMPL VALUES (
	'bam', 'j1axy', '1', ' ',
	'Join1, a(X,Y) [V1: RelAttr, -goto, fullDupCh]');

INSERT INTO BENCH_IMPL VALUES (
	'bam', 'j1axy', '2', ' ',
	'Join1, a(X,Y) [V2: no goto, bitmap-dupcheck]');

INSERT INTO BENCH_IMPL VALUES (
	'bam', 'j1axy', '3', 'B',
	'Join1, a(X,Y) [V3: goto, bitmap-dupC, cursor]');

INSERT INTO BENCH_IMPL VALUES (
	'bam', 'j1axy', '4', 'A',
	'Join1, a(X,Y) [V4: functions, bitmap-dupC]');

INSERT INTO BENCH_IMPL VALUES (
	'bam', 'j1axy', '5', ' ',
	'Join1, a(X,Y) [V5: fun without args, bitmap]');

INSERT INTO BENCH_IMPL VALUES (
	'bam', 'j1axy', '6', 'A',
	'Join1, a(X,Y) [V6: Abstract Machine, bitmap]');

-- Transitive Closure Implementations for bam (TCFF - Both Arguments Free):

INSERT INTO BENCH_IMPL VALUES (
	'bam', 'tcff', '1', ' ',
	'tc(X,Y) [V1: RelLocalVar]');

INSERT INTO BENCH_IMPL VALUES (
	'bam', 'tcff', '2', 'A',
	'tc(X,Y) [V2: Cursor Interface, switch]');

INSERT INTO BENCH_IMPL VALUES (
	'bam', 'tcff', '3', 'A',
	'tc(X,Y) [V3: Seminaive Evaluation]');

INSERT INTO BENCH_IMPL VALUES (
	'bam', 'tcff', '4', 'A',
	'tc(X,Y) [V4: Functions]');

	-- Not really best, but this is the BAM:
INSERT INTO BENCH_IMPL VALUES (
	'bam', 'tcff', '5', 'B',
	'tc(X,Y) [V5: Abstract Machine]');

-- Data Loader Tests for bam:

INSERT INTO BENCH_IMPL VALUES (
	'bam', 'loadt', '1', 'A',
	'transitive closure data file (with handcrafted loader)');

INSERT INTO BENCH_IMPL VALUES (
	'bam', 'loadt', '2', 'B',
	'transitive closure data file (with general loader)');

-- Set Implementation Tests for bam (One Column):

INSERT INTO BENCH_IMPL VALUES (
	'bam', 'set1', '1', 'B',
	'Set with 1 column, extendible Hash table');

-- Set Implementation Tests for bam (Two Columns):

INSERT INTO BENCH_IMPL VALUES (
	'bam', 'set2', '1', 'B',
	'Set with 2 columns, extendible Hash table');

-- ----------------------------------------------------------------------------
-- Benchmark Implementations for XSB:
-- ----------------------------------------------------------------------------

-- DBLP Benchmark for XSB:

INSERT INTO BENCH_IMPL VALUES (
	'xsb', 'dblp', 'dyn', 'A',
	'load_dyn, no index');

INSERT INTO BENCH_IMPL VALUES (
	'xsb', 'dblp', 'dync', 'A',
	'load_dync, no index');

INSERT INTO BENCH_IMPL VALUES (
	'xsb', 'dblp', 'dynin', 'A',
	'load_dyn, index on col 1+2');

INSERT INTO BENCH_IMPL VALUES (
	'xsb', 'dblp', 'dyncin', 'B',
	'load_dync, index on col 1+2');

INSERT INTO BENCH_IMPL VALUES (
	'xsb', 'dblp', 'dyntr', 'A',
	'load_dyn, trie index');

INSERT INTO BENCH_IMPL VALUES (
	'xsb', 'dblp', 'dynctr', 'A',
	'load_dync, trie index');

-- Join1 Benchmark for XSB:

INSERT INTO BENCH_IMPL VALUES (
	'xsb', 'j1axyd', 'none', 'B',
	'Join1 with duplicates, a(X,Y) [V0: no tabling]');

INSERT INTO BENCH_IMPL VALUES (
	'xsb', 'j1axy', 'a', ' ',
	'Join1, a(X,Y) [V1: only a tabled]');

INSERT INTO BENCH_IMPL VALUES (
	'xsb', 'j1axy', 'all', ' ',
	'Join1, a(X,Y) [V2: all derived preds tabled (subsumptive)]');

INSERT INTO BENCH_IMPL VALUES (
	'xsb', 'j1axy', 'allv', 'B',
	'Join1, a(X,Y) [V4: all derived preds tabled, variant tabled]');

INSERT INTO BENCH_IMPL VALUES (
	'xsb', 'j1axy', 'alli', ' ',
	'Join1, a(X,Y) [V4: all derived preds tabled, with indexes]');

INSERT INTO BENCH_IMPL VALUES (
	'xsb', 'j1axy', 'allt', 'A',
	'Join1, a(X,Y) [V5: all derived preds tabled, with trie index]');

-- TCFF Benchmark for XSB:

INSERT INTO BENCH_IMPL VALUES (
	'xsb', 'tcff', 'dyn', ' ',
	'tc(X,Y) [V1: load_dyn, trie-index for par]');

-- INSERT INTO BENCH_IMPL VALUES (
--	'xsb', 'tcff', 'dync', 'B',
--	'tc(X,Y) [V2: load_dync, trie-index for par]');

INSERT INTO BENCH_IMPL VALUES (
	'xsb', 'tcff', '', 'B',
	'tc(X,Y) [V2: load_dync, trie-index for par]');

-- TCBF Benchmark for XSB:

INSERT INTO BENCH_IMPL VALUES (
	'xsb', 'tcbf', 'dyn', ' ',
	'tc(1,X) [V1: load_dyn, trie-index for par]');

-- INSERT INTO BENCH_IMPL VALUES (
--	'xsb', 'tcbf',  'dync', 'B',
--	'tc(1,X) [V2: load_dync, trie-index for par]');

INSERT INTO BENCH_IMPL VALUES (
	'xsb', 'tcbf',  '', 'B',
	'tc(1,X) [V2: load_dync, trie-index for par]');

INSERT INTO BENCH_IMPL VALUES (
	'xsb', 'tcbfs', 'slddc', 'B',
	'tc(1,X) [V3: SLDMagic, load_dync, trie-index]');

-- TCFB Benchmark for XSB:

INSERT INTO BENCH_IMPL VALUES (
	'xsb', 'tcfb',  '', 'B',
	'tc(X,1) [unknown implementation]');

-- Wine Ontology Benchmark for XSB:

INSERT INTO BENCH_IMPL VALUES (
	'xsb', 'wine', 'is', 'B',
	'wine ontology [V1: load_dync, subsumptive tabling, std. indexes]');


-- ----------------------------------------------------------------------------
-- Benchmark Implemenations for YAP:
-- ----------------------------------------------------------------------------

-- DBLP Benchmark for YAP:

INSERT INTO BENCH_IMPL VALUES (
	'yap', 'dblp', '1', 'B',
	'DBLP Benchmark');

-- Join1 Benchmark for YAP:

INSERT INTO BENCH_IMPL VALUES (
	'yap', 'j1axyd', 'none', 'B',
	'Join1 with duplicates, a(X,Y) [V0: no tabling]');

INSERT INTO BENCH_IMPL VALUES (
	'yap', 'j1axy', 'a', ' ',
	'Join1, a(X,Y) [V1: only a tabled]');

INSERT INTO BENCH_IMPL VALUES (
	'yap', 'j1axy', 'all', 'B',
	'Join1, a(X,Y) [V2: all derived preds tabled]');

-- TCFF Benchmark for YAP:

-- INSERT INTO BENCH_IMPL VALUES (
--	'yap', 'tcff', 'tab', 'B',
--	'tc(X,Y) [V1: tc tabled]');

INSERT INTO BENCH_IMPL VALUES (
	'yap', 'tcff', '', 'B',
	'tc(X,Y) [V1: tc tabled]');

-- TCBF Benchmark for YAP:

INSERT INTO BENCH_IMPL VALUES (
	'yap', 'tcbf', '', 'B',
	'tc(1,X) [V1: tc tabled]');

INSERT INTO BENCH_IMPL VALUES (
	'yap', 'tcbfs', 'sldm', 'B',
	'tc(1,X) [V2: with SLDMagic, all preds tabled]');

-- TCFB Benchmark for YAP:

INSERT INTO BENCH_IMPL VALUES (
	'yap', 'tcfb', '', 'B',
	'tc(X,1) [V1: tc tabled]');

-- Wine Ontology Benchmark for YAP:

INSERT INTO BENCH_IMPL VALUES (
	'yap', 'wine', 'tab', 'B',
	'wine ontology [V1: table 225 derived preds, reorder body lits]');


-- ----------------------------------------------------------------------------
-- Benchmark Implemenations for SWI Prolog:
-- ----------------------------------------------------------------------------

-- DBLP Benchmark for swipl:

INSERT INTO BENCH_IMPL VALUES (
	'swipl', 'dblp', '', 'B',
	'DBLP Benchmark');

-- Join1 Benchmark for swipl:

INSERT INTO BENCH_IMPL VALUES (
	'swipl', 'j1axyd', '', 'B',
	'Join1 with duplicates, a(X,Y)');

INSERT INTO BENCH_IMPL VALUES (
	'swipl', 'j1axy', '', 'B',
	'Join1, a(X,Y)');

-- TCFF Benchmark for swipl:

INSERT INTO BENCH_IMPL VALUES (
	'swipl', 'tcff', '', 'B',
	'tc(X,Y) [tc tabled]');

-- TCBF Benchmark for swipl:

INSERT INTO BENCH_IMPL VALUES (
	'swipl', 'tcbf', '', 'B',
	'tc(1,X) [tc tabled]');

INSERT INTO BENCH_IMPL VALUES (
	'swipl', 'tcbfs', 'sldm', 'B',
	'tc(1,X) [with SLDMagic, all preds tabled]');

-- TCFB Benchmark for swipl:

INSERT INTO BENCH_IMPL VALUES (
	'swipl', 'tcfb', '', 'B',
	'tc(X,1) [tc tabled]');

-- Wine Ontology Benchmark for swipl:

INSERT INTO BENCH_IMPL VALUES (
	'swipl', 'wine', '', 'B',
	'wine ontology');


-- ----------------------------------------------------------------------------
-- Benchmark implementations for SQLite3:
-- ----------------------------------------------------------------------------

INSERT INTO BENCH_IMPL VALUES (
	'sqlite3', 'tcff', '', 'B',
	'transitive closure [ff] for SQLite3');

INSERT INTO BENCH_IMPL VALUES (
	'sqlite3', 'tcbf', '', 'B',
	'transitive closure [bf] for SQLite3');

INSERT INTO BENCH_IMPL VALUES (
	'sqlite3', 'tcfb', '', 'B',
	'transitive closure [fb] for SQLite3');


-- ----------------------------------------------------------------------------
-- Benchmark implementations for PostgreSQL:
-- ----------------------------------------------------------------------------

INSERT INTO BENCH_IMPL VALUES (
	'psql', 'tcff', '', 'B',
	'transitive closure [ff] for PostgreSQL');

INSERT INTO BENCH_IMPL VALUES (
	'psql', 'tcbf', '', 'B',
	'transitive closure [bf] for PostgreSQL');

INSERT INTO BENCH_IMPL VALUES (
	'psql', 'tcfb', '', 'B',
	'transitive closure [fb] for PostgreSQL');


-- ----------------------------------------------------------------------------
-- Benchmark implementations for MariaDB:
-- ----------------------------------------------------------------------------

INSERT INTO BENCH_IMPL VALUES (
	'mariadb', 'tcff', 'm', 'A',
	'transitive closure for MariaDB, storage manager MEMORY');

INSERT INTO BENCH_IMPL VALUES (
	'mariadb', 'tcff', 'i', 'B',
	'transitive closure for MariaDB, storage manager InnoDB');

INSERT INTO BENCH_IMPL VALUES (
	'mariadb', 'tcbf', '', 'B',
	'transitive closure for MariaDB, unknown storage manager');

INSERT INTO BENCH_IMPL VALUES (
	'mariadb', 'tcfb', '', 'B',
	'transitive closure for MariaDB, unknown storage manager');


-- ----------------------------------------------------------------------------
-- Benchmark implementations for Apache Jena
-- ----------------------------------------------------------------------------

INSERT INTO BENCH_IMPL VALUES (
	'jena', 'tcff', '', 'B',
	'transitive closure [ff] with Property Paths for Apache Jena');

INSERT INTO BENCH_IMPL VALUES (
	'jena', 'tcbf', '', 'B',
	'transitive closure [bf] with Property Paths for Apache Jena');

INSERT INTO BENCH_IMPL VALUES (
	'jena', 'tcfb', '', 'B',
	'transitive closure [fb] with Property Paths for Apache Jena');


-- ----------------------------------------------------------------------------
-- Benchmark implementations for DataScript:
-- ----------------------------------------------------------------------------

INSERT INTO BENCH_IMPL VALUES (
	'datascript', 'tcff', '', 'B',
	'transitive closure for DataScript');


-- ----------------------------------------------------------------------------
-- Benchmark Implemenations for Souffle:
-- ----------------------------------------------------------------------------

-- TCFF Benchmark for Souffle:

INSERT INTO BENCH_IMPL VALUES (
	'souffle', 'tcff', '', 'B',
	'tc(X,Y) [V1: compiled, -j1]');


-- ============================================================================
-- Table for Data Files (Input Files to Benchmarks):
-- ============================================================================

CREATE TABLE DATA_FILE(
	FILE_ID     VARCHAR(10)  NOT NULL,
	SEQ_NO      NUMERIC(3)   NOT NULL,
	ITYPE       VARCHAR(8)   NOT NULL,
	LINES       NUMERIC(12)  NOT NULL, -- should be the number of facts
	BYTES       NUMERIC(12)  NOT NULL, -- refers to Datalog facts version
	FILENAME    VARCHAR(40)  NOT NULL, -- without extension
--	DESCRIPTION VARCHAR(80)  NULL,
	CONSTRAINT DATA_FILE_PK
		PRIMARY KEY (FILE_ID),
	CONSTRAINT DATA_FILE_KEY2
		UNIQUE (SEQ_NO),
	CONSTRAINT DATA_FILE_SEQ_NO_NOT_NEGATIVE
		CHECK(SEQ_NO >= 0),
	CONSTRAINT DATA_FILE_REF_INPUT_TYPE
		FOREIGN KEY (ITYPE) REFERENCES INPUT_TYPE,
	CONSTRAINT DATA_FILE_LINES_NOT_NEGATIVE
		CHECK(LINES >= 0),
	CONSTRAINT DATA_FILE_BYTES_NOT_NEGATIVE
		CHECK(BYTES >= 0));

-- wc was used to compute lines and bytes

INSERT INTO DATA_FILE VALUES
  ('k465',     101, 'tc',   216225,    3141075, 'tc_complete_465');
INSERT INTO DATA_FILE VALUES
  ('k500',     102, 'tc',   250000,    3640000, 'tc_complete_500');
INSERT INTO DATA_FILE VALUES
  ('k585',     103, 'tc',   342225,    5004675, 'tc_complete_585');
INSERT INTO DATA_FILE VALUES
  ('k670',     104, 'tc',   448900,    6586100, 'tc_complete_670');
INSERT INTO DATA_FILE VALUES
  ('k735',     105, 'tc',   540225,    7941675, 'tc_complete_735');
INSERT INTO DATA_FILE VALUES
  ('k795',     106, 'tc',   632025,    9305475, 'tc_complete_795');
INSERT INTO DATA_FILE VALUES
  ('k845',     107, 'tc',   714025,   10524475, 'tc_complete_845');
INSERT INTO DATA_FILE VALUES
  ('k890',     108, 'tc',   792100,   11685700, 'tc_complete_890');
INSERT INTO DATA_FILE VALUES
  ('k930',     109, 'tc',   864900,   12768900, 'tc_complete_930');
INSERT INTO DATA_FILE VALUES
  ('k965',     110, 'tc',   931225,   13756075, 'tc_complete_965');
INSERT INTO DATA_FILE VALUES
  ('k1k',      111, 'tc',  1000000,   14780000, 'tc_complete_1000');
INSERT INTO DATA_FILE VALUES
  ('k2k',      112, 'tc',  4000000,   63560000, 'tc_complete_2000');
INSERT INTO DATA_FILE VALUES
  ('t500',     121, 'tc',   124750,    1816360, 'tc_total_500');
INSERT INTO DATA_FILE VALUES
  ('t585',     122, 'tc',   170820,    2498060, 'tc_total_585');
INSERT INTO DATA_FILE VALUES
  ('t735',     123, 'tc',   269745,    3965435, 'tc_total_735');
INSERT INTO DATA_FILE VALUES
  ('t845',     124, 'tc',   356590,    5256010, 'tc_total_845');
INSERT INTO DATA_FILE VALUES
  ('t930',     125, 'tc',   431985,    6377585, 'tc_total_930');
INSERT INTO DATA_FILE VALUES
  ('t1k',      126, 'tc',   499500,    7382610, 'tc_total_1000');
INSERT INTO DATA_FILE VALUES
  ('t1065',    127, 'tc',   566580,    8450820, 'tc_total_1065');
INSERT INTO DATA_FILE VALUES
  ('t1120',    128, 'tc',   626640,    9410790, 'tc_total_1120');
INSERT INTO DATA_FILE VALUES
  ('t1170',    129, 'tc',   683865,   10328115, 'tc_total_1170');
INSERT INTO DATA_FILE VALUES
  ('t1215',    130, 'tc',   737505,   11190045, 'tc_total_1215');
INSERT INTO DATA_FILE VALUES
  ('t1260',    131, 'tc',   793170,   12086400, 'tc_total_1260');
INSERT INTO DATA_FILE VALUES
  ('t2k',      132, 'tc',  1999000,   31764110, 'tc_total_2000');
INSERT INTO DATA_FILE VALUES
  ('c1k',      141, 'tc',     1000,      14780, 'tc_cycle_1000');
INSERT INTO DATA_FILE VALUES
  ('c2k',      142, 'tc',     2000,      31780, 'tc_cycle_2000');
INSERT INTO DATA_FILE VALUES
  ('c3k',      143, 'tc',     3000,      48780, 'tc_cycle_3000');
INSERT INTO DATA_FILE VALUES
  ('c4k',      144, 'tc',     4000,      65780, 'tc_cycle_4000');
INSERT INTO DATA_FILE VALUES
  ('c10k',     145, 'tc',    10000,     167780, 'tc_cycle_10000');
INSERT INTO DATA_FILE VALUES
  ('c14k1',    146, 'tc',    14100,     245680, 'tc_cycle_14100');
INSERT INTO DATA_FILE VALUES
  ('c17k3',    147, 'tc',    17300,     306480, 'tc_cycle_17300');
INSERT INTO DATA_FILE VALUES
  ('c20k',     148, 'tc',    20000,     357780, 'tc_cycle_20000');
INSERT INTO DATA_FILE VALUES
  ('c22k4',    149, 'tc',    22400,     403380, 'tc_cycle_22400');
INSERT INTO DATA_FILE VALUES
  ('c24k5',    150, 'tc',    24500,     443280, 'tc_cycle_24500');
INSERT INTO DATA_FILE VALUES
  ('c26k5',    151, 'tc',    26500,     481280, 'tc_cycle_26500');
INSERT INTO DATA_FILE VALUES
  ('c28k3',    152, 'tc',    28300,     515480, 'tc_cycle_28300');
INSERT INTO DATA_FILE VALUES
  ('c30k',     153, 'tc',    30000,     547780, 'tc_cycle_30000');
INSERT INTO DATA_FILE VALUES
  ('c31k6',    154, 'tc',    31600,     578180, 'tc_cycle_31600');
INSERT INTO DATA_FILE VALUES
  ('s1k_1',    161, 'tc',     2000,      29560, 'tc_shorted_cycle_1000_1');
INSERT INTO DATA_FILE VALUES
  ('s1k_2',    162, 'tc',     3000,      44340, 'tc_shorted_cycle_1000_2');
INSERT INTO DATA_FILE VALUES
  ('s1k_3',    163, 'tc',     4000,      59120, 'tc_shorted_cycle_1000_3');
INSERT INTO DATA_FILE VALUES
  ('s1k_4',    164, 'tc',     5000,      73900, 'tc_shorted_cycle_1000_4');
INSERT INTO DATA_FILE VALUES
  ('s1k_5',    165, 'tc',     6000,      88680, 'tc_shorted_cycle_1000_5');
INSERT INTO DATA_FILE VALUES
  ('s1k2_1',   166, 'tc',     2400,      36360, 'tc_shorted_cycle_1200_1');
INSERT INTO DATA_FILE VALUES
  ('s1k2_2',   167, 'tc',     3600,      54540, 'tc_shorted_cycle_1200_2');
INSERT INTO DATA_FILE VALUES
  ('s1k2_4',   168, 'tc',     6000,      90900, 'tc_shorted_cycle_1200_4');
INSERT INTO DATA_FILE VALUES
  ('s1k2_5',   169, 'tc',     3357,      50265, 'tc_shorted_cycle_1200_5');
INSERT INTO DATA_FILE VALUES
  ('s2k_1',    170, 'tc',     4000,      63560, 'tc_shorted_cycle_2000_1');
INSERT INTO DATA_FILE VALUES
  ('s2k_2',    171, 'tc',     6000,      95340, 'tc_shorted_cycle_2000_2');
INSERT INTO DATA_FILE VALUES
  ('s2k_3',    172, 'tc',     8000,     127120, 'tc_shorted_cycle_2000_3');
INSERT INTO DATA_FILE VALUES
  ('s2k_4',    173, 'tc',    10000,     158900, 'tc_shorted_cycle_2000_4');
INSERT INTO DATA_FILE VALUES
  ('s2k_5',    174, 'tc',    12000,     190680, 'tc_shorted_cycle_2000_5');
INSERT INTO DATA_FILE VALUES
  ('s2k4_1',   175, 'tc',     4800,      77160, 'tc_shorted_cycle_2400_1');
INSERT INTO DATA_FILE VALUES
  ('s2k4_2',   176, 'tc',     7200,     115740, 'tc_shorted_cycle_2400_2');
INSERT INTO DATA_FILE VALUES
  ('s2k4_4',   177, 'tc',    12000,     192900, 'tc_shorted_cycle_2400_4');
INSERT INTO DATA_FILE VALUES
  ('s2k4_5',   178, 'tc',    14400,     231480, 'tc_shorted_cycle_2400_5');
INSERT INTO DATA_FILE VALUES
  ('s4k8_1',   179, 'tc',     9600,     158760, 'tc_shorted_cycle_4800_1');
INSERT INTO DATA_FILE VALUES
  ('s4k8_2',   180, 'tc',    14400,     238140, 'tc_shorted_cycle_4800_2');
INSERT INTO DATA_FILE VALUES
  ('s4k8_4',   181, 'tc',    24000,     396900, 'tc_shorted_cycle_4800_4');
INSERT INTO DATA_FILE VALUES
  ('s4k8_5',   182, 'tc',    28800,     476280, 'tc_shorted_cycle_4800_5');
INSERT INTO DATA_FILE VALUES
  ('s6k3_1',   183, 'tc',    12600,     209760, 'tc_shorted_cycle_6300_1');
INSERT INTO DATA_FILE VALUES
  ('s6k3_2',   184, 'tc',    18900,     314640, 'tc_shorted_cycle_6300_2');
INSERT INTO DATA_FILE VALUES
  ('s6k3_4',   185, 'tc',    31500,     524400, 'tc_shorted_cycle_6300_4');
INSERT INTO DATA_FILE VALUES
  ('s6k3_5',   186, 'tc',    37800,     629280, 'tc_shorted_cycle_6300_5');
INSERT INTO DATA_FILE VALUES
  ('s7k2_1',   187, 'tc',    14400,     240360, 'tc_shorted_cycle_7200_1');
INSERT INTO DATA_FILE VALUES
  ('s7k2_2',   188, 'tc',    21600,     360540, 'tc_shorted_cycle_7200_2');
INSERT INTO DATA_FILE VALUES
  ('s7k2_4',   189, 'tc',    36000,     600900, 'tc_shorted_cycle_7200_4');
INSERT INTO DATA_FILE VALUES
  ('s7k2_5',   190, 'tc',    43200,     721080, 'tc_shorted_cycle_7200_5');
INSERT INTO DATA_FILE VALUES
  ('s9k_1',    191, 'tc',    18000,     301560, 'tc_shorted_cycle_9000_1');
INSERT INTO DATA_FILE VALUES
  ('s9k_2',    192, 'tc',    27000,     452340, 'tc_shorted_cycle_9000_2');
INSERT INTO DATA_FILE VALUES
  ('s9k_4',    193, 'tc',    45000,     753900, 'tc_shorted_cycle_9000_4');
INSERT INTO DATA_FILE VALUES
  ('s9k_5',    194, 'tc',    54000,     904680, 'tc_shorted_cycle_9000_5');
INSERT INTO DATA_FILE VALUES
  ('s12k_1',   195, 'tc',    24000,     411560, 'tc_shorted_cycle_12000_1');
INSERT INTO DATA_FILE VALUES
  ('s12k_2',   196, 'tc',    36000,     617340, 'tc_shorted_cycle_12000_2');
INSERT INTO DATA_FILE VALUES
  ('s12k_4',   197, 'tc',    60000,    1028900, 'tc_shorted_cycle_12000_4');
INSERT INTO DATA_FILE VALUES
  ('s12k_5',   198, 'tc',    72000,    1234680, 'tc_shorted_cycle_12000_5');
INSERT INTO DATA_FILE VALUES
  ('s13k5_1',  199, 'tc',    27000,     468560, 'tc_shorted_cycle_13500_1');
INSERT INTO DATA_FILE VALUES
  ('s13k5_2',  200, 'tc',    40500,     702840, 'tc_shorted_cycle_13500_2');
INSERT INTO DATA_FILE VALUES
  ('s13k5_4',  201, 'tc',    67500,    1171400, 'tc_shorted_cycle_13500_4');
INSERT INTO DATA_FILE VALUES
  ('s13k5_5',  202, 'tc',    81000,    1405680, 'tc_shorted_cycle_13500_5');
INSERT INTO DATA_FILE VALUES
  ('s15k9_1',  203, 'tc',    31800,     559760, 'tc_shorted_cycle_15900_1');
INSERT INTO DATA_FILE VALUES
  ('s15k9_2',  204, 'tc',    47700,     839640, 'tc_shorted_cycle_15900_2');
INSERT INTO DATA_FILE VALUES
  ('s15k9_4',  205, 'tc',    79500,    1399400, 'tc_shorted_cycle_15900_4');
INSERT INTO DATA_FILE VALUES
  ('s15k9_5',  206, 'tc',    95400,    1679280, 'tc_shorted_cycle_15900_5');
INSERT INTO DATA_FILE VALUES
  ('s18k6_1',  207, 'tc',    37200,     662360, 'tc_shorted_cycle_18600_1');
INSERT INTO DATA_FILE VALUES
  ('s18k6_2',  208, 'tc',    55800,     993540, 'tc_shorted_cycle_18600_2');
INSERT INTO DATA_FILE VALUES
  ('s18k6_4',  209, 'tc',    93000,    1655900, 'tc_shorted_cycle_18600_4');
INSERT INTO DATA_FILE VALUES
  ('s18k6_5',  210, 'tc',   111600,    1987080, 'tc_shorted_cycle_18600_5');
INSERT INTO DATA_FILE VALUES
  ('p1k',      221, 'tc',      999,      14767, 'tc_path_1000');
INSERT INTO DATA_FILE VALUES
  ('p2k',      222, 'tc',     1999,      31766, 'tc_path_2000');
INSERT INTO DATA_FILE VALUES
  ('p3k',      223, 'tc',     2999,      48766, 'tc_path_3000');
INSERT INTO DATA_FILE VALUES
  ('p4k',      224, 'tc',     3999,      65766, 'tc_path_4000');
INSERT INTO DATA_FILE VALUES
  ('p14k1',    225, 'tc',    14099,     245665, 'tc_path_14100');
INSERT INTO DATA_FILE VALUES
  ('p20k',     226, 'tc',    19999,     357765, 'tc_path_20000');
INSERT INTO DATA_FILE VALUES
  ('p24k5',    227, 'tc',    24499,     443265, 'tc_path_24500');
INSERT INTO DATA_FILE VALUES
  ('p28k3',    228, 'tc',    28299,     515465, 'tc_path_28300');
INSERT INTO DATA_FILE VALUES
  ('p31k6',    229, 'tc',    31599,     578165, 'tc_path_31600');
INSERT INTO DATA_FILE VALUES
  ('p34k6',    230, 'tc',    34599,     635165, 'tc_path_34600');
INSERT INTO DATA_FILE VALUES
  ('p37k4',    231, 'tc',    37399,     688365, 'tc_path_37400');
INSERT INTO DATA_FILE VALUES
  ('p40k',     232, 'tc',    39999,     737765, 'tc_path_40000');
INSERT INTO DATA_FILE VALUES
  ('p42k4',    233, 'tc',    42399,     783365, 'tc_path_42400');
INSERT INTO DATA_FILE VALUES
  ('p44k7',    234, 'tc',    44699,     827065, 'tc_path_44700');
INSERT INTO DATA_FILE VALUES
  ('m4_2ki',   241, 'tc',     6144,     103341, 'tc_multipath_4_2048');
INSERT INTO DATA_FILE VALUES
  ('m16_512',  242, 'tc',     7680,     128966, 'tc_multipath_16_512');
INSERT INTO DATA_FILE VALUES
  ('m64_128',  243, 'tc',     8064,     135110, 'tc_multipath_64_128');
INSERT INTO DATA_FILE VALUES
  ('m256_32',  244, 'tc',     8160,     136579, 'tc_multipath_256_32');
INSERT INTO DATA_FILE VALUES
  ('m1ki_8',   245, 'tc',     8184,     136938, 'tc_multipath_1024_8');
INSERT INTO DATA_FILE VALUES
  ('m4ki_2',   246, 'tc',     8190,     137022, 'tc_multipath_4096_2');
INSERT INTO DATA_FILE VALUES
  ('b17',      251, 'tc',   131070,    2488088, 'tc_bintree_17');
INSERT INTO DATA_FILE VALUES
  ('b18',      252, 'tc',   262142,    5171672, 'tc_bintree_18');
INSERT INTO DATA_FILE VALUES
  ('b19',      253, 'tc',   524286,   10676696, 'tc_bintree_19');
INSERT INTO DATA_FILE VALUES
  ('b20',      254, 'tc',  1048574,   21735320, 'tc_bintree_20');
INSERT INTO DATA_FILE VALUES
  ('b21',      255, 'tc',  2097150,   44901144, 'tc_bintree_21');
INSERT INTO DATA_FILE VALUES
  ('b22',      256, 'tc',  4194302,   93135640, 'tc_bintree_22');
INSERT INTO DATA_FILE VALUES
  ('b23',      257, 'tc',  8388606,  189604632, 'tc_bintree_23');
INSERT INTO DATA_FILE VALUES
  ('b24',      258, 'tc', 16777214,  389319832, 'tc_bintree_24');
INSERT INTO DATA_FILE VALUES
  ('b25',      259, 'tc', 33554430,  805527448, 'tc_bintree_25');
INSERT INTO DATA_FILE VALUES
  ('b26',      260, 'tc', 67108862, 1644388248, 'tc_bintree_26');
INSERT INTO DATA_FILE VALUES
  ('y500_4k',  271, 'tc',     4499,      74378, 'tc_y_500_4000');
INSERT INTO DATA_FILE VALUES
  ('y500_8k',  272, 'tc',     8499,     142378, 'tc_y_500_8000');
INSERT INTO DATA_FILE VALUES
  ('y1k_4k',   273, 'tc',     4999,      83876, 'tc_y_1000_4000');
INSERT INTO DATA_FILE VALUES
  ('y1k_8k',   274, 'tc',     8999,     151876, 'tc_y_1000_8000');
INSERT INTO DATA_FILE VALUES
  ('y1k_15k',  275, 'tc',    15999,     282877, 'tc_y_1000_15000');
INSERT INTO DATA_FILE VALUES
  ('y1k_25k',  276, 'tc',    25999,     472877, 'tc_y_1000_25000');
INSERT INTO DATA_FILE VALUES
  ('y1k_35k',  277, 'tc',    35999,     662877, 'tc_y_1000_35000');
INSERT INTO DATA_FILE VALUES
  ('y5k_15k',  278, 'tc',    19999,     358877, 'tc_y_5000_15000');
INSERT INTO DATA_FILE VALUES
  ('y5k_25k',  279, 'tc',    29999,     548877, 'tc_y_5000_25000');
INSERT INTO DATA_FILE VALUES
  ('y5k_35k',  280, 'tc',    39999,     738877, 'tc_y_5000_35000');
INSERT INTO DATA_FILE VALUES
  ('y10k_15k', 281, 'tc',    24999,     463875, 'tc_y_10000_15000');
INSERT INTO DATA_FILE VALUES
  ('y10k_25k', 282, 'tc',    34999,     653875, 'tc_y_10000_25000');
INSERT INTO DATA_FILE VALUES
  ('y10k_35k', 283, 'tc',    44999,     843875, 'tc_y_10000_35000');
INSERT INTO DATA_FILE VALUES
  ('u1k_50k',  301, 'tc',    50000,     739011, 'tc_uniform_1000_50000');
INSERT INTO DATA_FILE VALUES
  ('u1k_125k', 302, 'tc',   125000,    1847511, 'tc_uniform_1000_125000');
INSERT INTO DATA_FILE VALUES
  ('u1k_250k', 303, 'tc',   250000,    3695016, 'tc_uniform_1000_250000');
INSERT INTO DATA_FILE VALUES
  ('u2k_200k', 304, 'tc',   200000,    3178014, 'tc_uniform_2000_200000');
INSERT INTO DATA_FILE VALUES
  ('u2k_250k', 305, 'tc',   250000,    3972534, 'tc_uniform_2000_250000');
INSERT INTO DATA_FILE VALUES
  ('u2k_500k', 306, 'tc',   500000,    7944982, 'tc_uniform_2000_500000');
INSERT INTO DATA_FILE VALUES
  ('u2k_1m',   307, 'tc',  1000000,   15890006, 'tc_uniform_2000_1000000');
INSERT INTO DATA_FILE VALUES
  ('a1k_50k',  311, 'tc',    50000,     738966, 'tc_acyc_1000_50000');
INSERT INTO DATA_FILE VALUES
  ('a1k_125k', 312, 'tc',   125000,    1847531, 'tc_acyc_1000_125000');
INSERT INTO DATA_FILE VALUES
  ('a1k_250k', 313, 'tc',   250000,    3695001, 'tc_acyc_1000_250000');
INSERT INTO DATA_FILE VALUES
  ('a2k_200k', 314, 'tc',   200000,    3177896, 'tc_acyc_2000_200000');
INSERT INTO DATA_FILE VALUES
  ('a2k_500k', 315, 'tc',   500000,    7944738, 'tc_acyc_2000_500000');
INSERT INTO DATA_FILE VALUES
  ('a2k_1m',   316, 'tc',  1000000,   15890152, 'tc_acyc_2000_1000000');

INSERT INTO DATA_FILE VALUES
  ('f1',  401, 'tc',    50000,   689318,'tc_d1000_parsize50000_xsb_nocyc');
INSERT INTO DATA_FILE VALUES
  ('f2',  402, 'tc',    50000,   689318,'tc_d1000_parsize50000_xsb_cyc');
INSERT INTO DATA_FILE VALUES
  ('f3',  403,  'tc',  250000,  3446506,'tc_d1000_parsize250000_xsb_nocyc');
INSERT INTO DATA_FILE VALUES
  ('f4',  404,  'tc',  250000,  3446506,'tc_d1000_parsize250000_xsb_cyc');
INSERT INTO DATA_FILE VALUES
  ('f5',  405,  'tc',  500000,  6893031,'tc_d1000_parsize500000_xsb_nocyc');
INSERT INTO DATA_FILE VALUES
  ('f6',  406,  'tc',  500000,  6893031,'tc_d1000_parsize500000_xsb_cyc');
INSERT INTO DATA_FILE VALUES
  ('f7',  407,  'tc',  500000,  7446477,'tc_d2000_parsize500000_xsb_nocyc');
INSERT INTO DATA_FILE VALUES
  ('f8',  408,  'tc',  500000,  7446477,'tc_d2000_parsize500000_xsb_cyc');
INSERT INTO DATA_FILE VALUES
  ('f9',  409,  'tc', 1000000, 14892982,'tc_d2000_parsize1000000_xsb_nocyc');
INSERT INTO DATA_FILE VALUES
  ('f10', 410,  'tc', 1000000, 14892982,'tc_d2000_parsize1000000_xsb_cyc');

INSERT INTO DATA_FILE VALUES
  ('10k',  501, 'j1',   50000,   639300,'d1000_relsize10000_xsb_cyc');
INSERT INTO DATA_FILE VALUES
  ('50k',  502, 'j1',  250000,  3196504,'d1000_relsize50000_xsb_cyc');
INSERT INTO DATA_FILE VALUES
  ('250k', 503, 'j1', 1250000, 15982480,'d1000_relsize250000_xsb_cyc');

INSERT INTO DATA_FILE VALUES
  ('std',  601,  'dblp', 2437867, 128444409,'dblp');
INSERT INTO DATA_FILE VALUES
  ('big',  602,  'dblp',50221773,3082908204,'dblp_big');

INSERT INTO DATA_FILE VALUES
  ('wdat', 701,  'wine',     654,     26386, 'wine_data');


-- ============================================================================
-- Graph Types (A Subclass of Data Files are Directed Graphs):
-- ============================================================================


CREATE TABLE GRAPH_TYPE(
	GTYPE      CHAR(1)      NOT NULL,
	SEQ_NO     NUMERIC(2)   NOT NULL,
	NAME       VARCHAR(80)  NOT NULL,
	CONSTRAINT GRAPH_TYPE_SEQ_NONNEGATIVE
		CHECK(SEQ_NO >= 0),
	CONSTRAINT GRAPH_TYPE_PK
		PRIMARY KEY (GTYPE),
	CONSTRAINT GRAPH_TYPE_KEY_SEQ
		UNIQUE (SEQ_NO));

INSERT INTO GRAPH_TYPE VALUES (
	'K',  1, 'Complete Graph (with loops)');
INSERT INTO GRAPH_TYPE VALUES (
	'T',  2, 'Total Order Graph (i.e. maximal acyclic graph)');
INSERT INTO GRAPH_TYPE VALUES (
	'C',  3, 'Cycle Graph');
INSERT INTO GRAPH_TYPE VALUES (
	'S',  4, 'Cycle Graph with Shortcuts');
INSERT INTO GRAPH_TYPE VALUES (
	'P',  5, 'Path');
INSERT INTO GRAPH_TYPE VALUES (
	'M',  6, 'Multi-Path');
INSERT INTO GRAPH_TYPE VALUES (
	'B',  7, 'Binary Tree');
INSERT INTO GRAPH_TYPE VALUES (
	'Y',  8, 'Y-Graph (many nodes pointing to first node of path)');
INSERT INTO GRAPH_TYPE VALUES (
	'U',  9, 'Random Graph with Uniform Node Degrees');
INSERT INTO GRAPH_TYPE VALUES (
	'A', 10, 'Random Acyclic Graph with Uniform Node Degrees');


-- ============================================================================
-- More Data on Data Files That are Directed Graphs:
-- ============================================================================


CREATE TABLE GRAPH_DATA (
	FILE_ID        VARCHAR(10) NOT NULL,
	GTYPE          CHAR(1)     NOT NULL,
	PARAM_1        NUMERIC(12) NULL,
	PARAM_2        NUMERIC(12) NULL,
	NUM_NODES      NUMERIC(12) NOT NULL,
	NUM_EDGES      NUMERIC(12) NOT NULL,
	DUP_EDGES      NUMERIC(12) NOT NULL,
	LOOPS          CHAR(1)     NOT NULL,
	MIN_IN_DEGREE  NUMERIC(12) NOT NULL,
	MAX_IN_DEGREE  NUMERIC(12) NOT NULL,
	MIN_OUT_DEGREE NUMERIC(12) NOT NULL,
	MAX_OUT_DEGREE NUMERIC(12) NOT NULL,
	CYCLES         CHAR(1)     NOT NULL,
	TC_SIZE        NUMERIC(12) NOT NULL,
	NUM_ITER       NUMERIC(12) NOT NULL,
	COST           NUMERIC(12) NOT NULL,
	CONSTRAINT GRAPH_DATA_PK
		PRIMARY KEY(FILE_ID),
	CONSTRAINT GRAPH_DATA_REF_DATA_FILE
		FOREIGN KEY(FILE_ID) REFERENCES DATA_FILE,
	CONSTRAINT GRAPH_DATA_REF_GRAPH_TYPE
		FOREIGN KEY (GTYPE) REFERENCES GRAPH_TYPE,
	CONSTRAINT GRAPH_DATA_PARAM_1_NONNEGATIVE
		CHECK(PARAM_1 >= 0),
	CONSTRAINT GRAPH_DATA_PARAM_2_NONNEGATIVE
		CHECK(PARAM_2 >= 0),
	CONSTRAINT GRAPH_DATA_NUM_NODES_NONNEGATIVE
		CHECK(NUM_NODES >= 0),
	CONSTRAINT GRAPH_DATA_NUM_EDGES_NONNEGATIVE
		CHECK(NUM_EDGES >= 0),
	CONSTRAINT GRAPH_DATA_DUP_EDGES_NONNEGATIVE
		CHECK(DUP_EDGES >= 0),
	CONSTRAINT GRAPH_DATA_LOOPS_YES_NO
		CHECK(LOOPS  in ('Y','N')),
	CONSTRAINT GRAPH_DATA_MIN_IN_DEGREE_NONNEGATIVE
		CHECK(MIN_IN_DEGREE >= 0),
	CONSTRAINT GRAPH_DATA_MAX_IN_DEGREE_NONNEGATIVE
		CHECK(MAX_IN_DEGREE >= 0),
	CONSTRAINT GRAPH_DATA_MIN_OUT_DEGREE_NONNEGATIVE
		CHECK(MIN_OUT_DEGREE >= 0),
	CONSTRAINT GRAPH_DATA_MAX_OUT_DEGREE_NONNEGATIVE
		CHECK(MAX_OUT_DEGREE >= 0),
	CONSTRAINT GRAPH_DATA_CYCLES_YES_NO
		CHECK(CYCLES IN ('Y','N')),
	CONSTRAINT GRAPH_DATA_TC_SIZE_NONNEGATIVE
		CHECK(TC_SIZE >= 0),
	CONSTRAINT GRAPH_DATA_NUM_ITER_NONNEGATIVE
		CHECK(NUM_ITER >= 0),
	CONSTRAINT GRAPH_DATA_COST_NONNEGATIVE
		CHECK(COST >= 0)
);


-- PARAM_1 and PARAM_2 are the parameters for the Data Generation Program

-- DUP_EDGES is the number of duplicate edges,
-- i.e. the number of node pairs (x,y) such that there are at least two edges
-- between these nodes.
-- Normally, it should be 0.
-- However, the OpenRuleBench graph generation program generates such
-- duplicate edges for acyclic graphs.
-- Therefore, our graph analysis checks the input files for this case.

-- LOOPS='Y' means that there are edges of the form (x,x).

-- COST is the number of applicable rule instances in the standard
-- tail-recursive Prolog program for computing the transitive closure:
--	tc(X, Y) :- par(X, Y).
--	tc(X, Z) :- par(X, Y), tc(Y, Z).
-- COST is the number of rule instances that semi-naive evaluation would
-- consider.
-- It can be computed as the sum of the number of edges
-- (i.e. the size of the par relation) and
-- the size of the join of par and tc.

INSERT INTO GRAPH_DATA VALUES('b17','B',17,NULL,
131071,131070,0,'N',0,1,0,2,'N',1966082,16,1966082);

INSERT INTO GRAPH_DATA VALUES('b18','B',18,NULL,
262143,262142,0,'N',0,1,0,2,'N',4194306,17,4194306);

INSERT INTO GRAPH_DATA VALUES('b19','B',19,NULL,
524287,524286,0,'N',0,1,0,2,'N',8912898,18,8912898);

INSERT INTO GRAPH_DATA VALUES('c1k','C',1000,NULL,
1000,1000,0,'N',1,1,1,1,'Y',1000000,1000,1001000);

INSERT INTO GRAPH_DATA VALUES('c2k','C',2000,NULL,
2000,2000,0,'N',1,1,1,1,'Y',4000000,2000,4002000);

INSERT INTO GRAPH_DATA VALUES('c3k','C',3000,NULL,
3000,3000,0,'N',1,1,1,1,'Y',9000000,3000,9003000);

INSERT INTO GRAPH_DATA VALUES('c4k','C',4000,NULL,
4000,4000,0,'N',1,1,1,1,'Y',16000000,4000,16004000);

INSERT INTO GRAPH_DATA VALUES('k500','K',500,NULL,
500,250000,0,'Y',500,500,500,500,'Y',250000,1,125250000);

INSERT INTO GRAPH_DATA VALUES('k1k','K',1000,NULL,
1000,1000000,0,'Y',1000,1000,1000,1000,'Y',1000000,1,1001000000);

INSERT INTO GRAPH_DATA VALUES('k2k','K',2000,NULL,
2000,4000000,0,'Y',2000,2000,2000,2000,'Y',4000000,1,8004000000);

INSERT INTO GRAPH_DATA VALUES('t500','T',500,NULL,
500,124750,0,'N',0,499,0,499,'N',124750,1,62126000);

INSERT INTO GRAPH_DATA VALUES('t1k','T',1000,NULL,
1000,499500,0,'N',0,999,0,999,'N',499500,1,166666500);

INSERT INTO GRAPH_DATA VALUES('t2k','T',2000,NULL,
2000,1999000,0,'N',0,1999,0,1999,'N',1999000,1,1333333000);

INSERT INTO GRAPH_DATA VALUES('m4ki_2','M', 4096, 2,
8192,8190,0,'N',0,1,0,1,'N',16773120,4095,16773120);

INSERT INTO GRAPH_DATA VALUES('m1ki_8','M', 1024, 8,
8192,8184,0,'N',0,1,0,1,'N',4190208,1023,4190208);

INSERT INTO GRAPH_DATA VALUES('m256_32','M', 256, 32,
8192,8160,0,'N',0,1,0,1,'N',1044480,255,1044480);

INSERT INTO GRAPH_DATA VALUES('m64_128','M',64,128,
8192,8064,0,'N',0,1,0,1,'N',258048,63,258048);

INSERT INTO GRAPH_DATA VALUES('m16_512','M', 16, 512,
8192,7680,0,'N',0,1,0,1,'N',61440,15,61440);

INSERT INTO GRAPH_DATA VALUES('m4_2ki','M', 4, 2048,
8192,6144,0,'N',0,1,0,1,'N',12288,3,12288);

INSERT INTO GRAPH_DATA VALUES('p1k','P',1000,NULL,
1000,999,0,'N',0,1,0,1,'N',499500,999,499500);

INSERT INTO GRAPH_DATA VALUES('p2k','P',2000,NULL,
2000,1999,0,'N',0,1,0,1,'N',1999000,1999,1999000);

INSERT INTO GRAPH_DATA VALUES('p3k','P',3000,NULL,
3000,2999,0,'N',0,1,0,1,'N',4498500,2999,4498500);

INSERT INTO GRAPH_DATA VALUES('p4k','P',4000,NULL,
4000,3999,0,'N',0,1,0,1,'N',7998000,3999,7998000);

INSERT INTO GRAPH_DATA VALUES('s2k_1','S', 2000, 1,
2000,4000,0,'N',2,2,2,2,'Y',4000000,1000,8004000);

INSERT INTO GRAPH_DATA VALUES('s2k_2','S', 2000, 2,
2000,6000,0,'N',3,3,3,3,'Y',4000000,288,12006000);

INSERT INTO GRAPH_DATA VALUES('s2k_3','S', 2000, 3,
2000,8000,0,'N',4,4,4,4,'Y',4000000,500,16008000);

INSERT INTO GRAPH_DATA VALUES('s2k_4','S', 2000, 4,
2000,10000,0,'N',5,5,5,5,'Y',4000000,400,20010000);

INSERT INTO GRAPH_DATA VALUES('s2k_5','S', 2000, 5,
2000,12000,0,'N',6,6,6,6,'Y',4000000,127,24012000);

INSERT INTO GRAPH_DATA VALUES('s1k_1','S', 1000, 1,
1000,2000,0,'N',2,2,2,2,'Y',1000000,500,2002000);

INSERT INTO GRAPH_DATA VALUES('s1k_2','S', 1000, 2,
1000,3000,0,'N',3,3,3,3,'Y',1000000,201,3003000);

INSERT INTO GRAPH_DATA VALUES('s1k_3','S', 1000, 3,
1000,4000,0,'N',4,4,4,4,'Y',1000000,250,4004000);

INSERT INTO GRAPH_DATA VALUES('s1k_4','S', 1000, 4,
1000,5000,0,'N',5,5,5,5,'Y',1000000,200,5005000);

INSERT INTO GRAPH_DATA VALUES('s1k_5','S', 1000, 5,
1000,6000,0,'N',6,6,6,6,'Y',1000000,42,6006000);

INSERT INTO GRAPH_DATA VALUES('u1k_50k','U', 1000, 50000,
1000,50000,0,'N',46,51,46,51,'Y',1000000,3,50050000);

INSERT INTO GRAPH_DATA VALUES('u1k_125k','U', 1000, 125000,
1000,125000,0,'N',122,127,122,127,'Y',1000000,2,125125000);

INSERT INTO GRAPH_DATA VALUES('u1k_250k','U', 1000, 250000,
1000,250000,0,'N',248,251,248,251,'Y',1000000,2,250250000);

INSERT INTO GRAPH_DATA VALUES('u2k_200k','U', 2000, 200000,
2000,200000,0,'N',98,101,98,101,'Y',4000000,3,400200000);

INSERT INTO GRAPH_DATA VALUES('u2k_500k','U', 2000, 500000,
2000,500000,0,'N',248,251,248,251,'Y',4000000,2,1000500000);

INSERT INTO GRAPH_DATA VALUES('u2k_1m','U', 2000, 1000000,
2000,1000000,0,'N',499,502,499,502,'Y',4000000,2,2001000000);

INSERT INTO GRAPH_DATA VALUES('a1k_50k','A', 1000, 50000,
1000,50000,0,'N',0,102,0,102,'N',471621,8,15269270);

INSERT INTO GRAPH_DATA VALUES('a1k_125k','A', 1000, 125000,
1000,125000,0,'N',0,250,0,250,'N',492680,5,40853746);

INSERT INTO GRAPH_DATA VALUES('a1k_250k','A', 1000, 250000,
1000,250000,0,'N',0,500,0,501,'N',497763,4,82941061);

INSERT INTO GRAPH_DATA VALUES('a2k_200k','A', 2000, 200000,
2000,200000,0,'N',0,201,0,201,'N',1944729,8,128420988);

INSERT INTO GRAPH_DATA VALUES('a2k_500k','A', 2000, 500000,
2000,500000,0,'N',0,500,0,500,'N',1985321,6,330666855);

INSERT INTO GRAPH_DATA VALUES('a2k_1m','A', 2000, 1000000,
2000,1000000,0,'N',0,1000,0,1000,'N',1995698,4,666033613);

INSERT INTO GRAPH_DATA VALUES('y1k_4k','Y', 1000, 4000,
5000,4999,0,'N',0,1000,0,1,'N',11998000,4000,11998000);

INSERT INTO GRAPH_DATA VALUES('y1k_8k','Y', 1000, 8000,
9000,8999,0,'N',0,1000,0,1,'N',39996000,8000,39996000);

INSERT INTO GRAPH_DATA VALUES('y500_4k','Y', 500, 4000,
4500,4499,0,'N',0,500,0,1,'N',9998000,4000,9998000);

INSERT INTO GRAPH_DATA VALUES('y500_8k','Y', 500, 8000,
8500,8499,0,'N',0,500,0,1,'N',35996000,8000,35996000);


-- ============================================================================
-- Table for Results of Benchmark Runs (Test Runs):
-- ============================================================================


CREATE TABLE BENCH_RUN(
	RUN_DATE    DATE         NOT NULL,
	RUN_NO      NUMERIC(4)   NOT NULL,
	PROG        VARCHAR(20)  NOT NULL,
	INSTALL  NUMERIC(4)   NOT NULL,
	MACHINE     VARCHAR(20)  NOT NULL,
	BENCH       VARCHAR(20)  NOT NULL,
	IMPL        VARCHAR(20)  NOT NULL,
	FILE_ID     VARCHAR(10)  NOT NULL,
	WITH_OUTPUT CHAR(1)      NOT NULL,
	LOAD_TIME   NUMERIC(10,3) NULL,
	EXEC_TIME   NUMERIC(10,3) NULL,
	TOTAL_TIME  NUMERIC(10,3) NULL,
	USER_TIME   NUMERIC(10,3) NULL,
	SYS_TIME    NUMERIC(10,3) NULL,
	REAL_TIME   NUMERIC(10,3) NOT NULL,
	MEM         NUMERIC(10,3) NULL,
	STATUS      VARCHAR(80)  NULL,
	REMARK      VARCHAR(800) NULL,
        CONSTRAINT  BENCH_RUN_RUN_NO_POSITIVE
		CHECK(RUN_NO > 0),
        CONSTRAINT  BENCH_RUN_WITH_OUTPUT_BOOL
		CHECK(WITH_OUTPUT IN ('Y','N')),
        CONSTRAINT  BENCH_RUN_LOAD_TIME_NONNEGATIVE
		CHECK(LOAD_TIME >= 0),
        CONSTRAINT  BENCH_RUN_EXEC_TIME_NONNEGATIVE
		CHECK(EXEC_TIME >= 0),
        CONSTRAINT  BENCH_RUN_TOTAL_TIME_NONNEGATIVE
		CHECK(TOTAL_TIME >= 0),
        CONSTRAINT  BENCH_RUN_USER_TIME_NONNEGATIVE
		CHECK(USER_TIME >= 0),
        CONSTRAINT  BENCH_RUN_SYS_TIME_NONNEGATIVE
		CHECK(SYS_TIME >= 0),
        CONSTRAINT  BENCH_RUN_REAL_TIME_NONNEGATIVE
		CHECK(REAL_TIME >= 0),
        CONSTRAINT  BENCH_RUN_MEM_NONNEGATIVE
		CHECK(MEM >= 0),
        CONSTRAINT  BENCH_RUN_STATUS_VALID
		CHECK(STATUS IN ('OK', 'FAILED', 'OUTL',
				'Warn', 'Warn1', 'Warn2', 'Warn3', 'Warn4')),
        CONSTRAINT  BENCH_RUN_PK
		PRIMARY KEY(RUN_DATE,RUN_NO,MACHINE,PROG,INSTALL),
        CONSTRAINT  BENCH_RUN_REF_BENCH_IMPL
		FOREIGN KEY(PROG, BENCH, IMPL) REFERENCES BENCH_IMPL,
	CONSTRAINT  BENCH_RUN_REF_PROG_INSTALL
		FOREIGN KEY(PROG, INSTALL, MACHINE) REFERENCES PROG_INSTALL,
	CONSTRAINT  BENCH_RUN_REF_DATA_FILE
		FOREIGN KEY(FILE_ID) REFERENCES DATA_FILE);

-- All times are stored in seconds.
-- MEM is the maximum resident set size.


-- ============================================================================
-- Experimental Table for Runtime Estimation Parameters:
-- ============================================================================


CREATE TABLE ESTIMATE_PAR(
	PROG        VARCHAR(20)  NOT NULL,
	BENCH       VARCHAR(20)  NOT NULL,
	STARTUP     NUMERIC(10)  NOT NULL,
	LOAD_IDX    NUMERIC(10)  NOT NULL,
	RULE_APP    NUMERIC(10)  NOT NULL,
	RESULT      NUMERIC(10)  NOT NULL,
	HITS        NUMERIC(10)  NOT NULL,
        CONSTRAINT  ESTIMATE_PAR_STARTUP_NONNEGATIVE
		CHECK(STARTUP >= 0),
        CONSTRAINT  ESTIMATE_PAR_LOAD_IDX_NONNEGATIVE
		CHECK(LOAD_IDX >= 0),
        CONSTRAINT  ESTIMATE_PAR_RULE_APP_NONNEGATIVE
		CHECK(RULE_APP >= 0),
        CONSTRAINT  ESTIMATE_PAR_RESULT_NONNEGATIVE
		CHECK(RESULT >= 0),
        CONSTRAINT  ESTIMATE_PAR_HITS_NONNEGATIVE
		CHECK(HITS >= 0),
        CONSTRAINT  ESTIMATE_PAR_PK
		PRIMARY KEY(PROG,BENCH,STARTUP,LOAD_IDX,RULE_APP,RESULT),
        CONSTRAINT  ESTIMATE_PAR_REF_PROG
		FOREIGN KEY(PROG) REFERENCES PROGRAM,
        CONSTRAINT  ESTIMATE_PAR_REF_BENCH
		FOREIGN KEY(BENCH) REFERENCES BENCHMARK);

--                                               START  LOAD R_APP RESULT HITS
INSERT INTO ESTIMATE_PAR VALUES( 'xsb',   'tcff',  125,  100,  107,   390, 41);

INSERT INTO ESTIMATE_PAR VALUES( 'yap',   'tcff',   30,   72,  263,   350, 32);

INSERT INTO ESTIMATE_PAR VALUES( 'bam',   'tcff',    0,  244,   35,    51, 29);

INSERT INTO ESTIMATE_PAR VALUES('jena',   'tcff', 1510, 2030,  164,  1590, 42);
INSERT INTO ESTIMATE_PAR VALUES('jena',   'tcff', 1605, 1140,  174,  1578, 42);
INSERT INTO ESTIMATE_PAR VALUES('jena',   'tcff', 1705, 1130,  172,  1578, 42);
INSERT INTO ESTIMATE_PAR VALUES('jena',   'tcff', 1805, 1130,  172,  1566, 42);
INSERT INTO ESTIMATE_PAR VALUES('jena',   'tcff', 1905, 1120,  172,  1566, 42);
INSERT INTO ESTIMATE_PAR VALUES('jena',   'tcff', 2005, 1120,  172,  1554, 42);
INSERT INTO ESTIMATE_PAR VALUES('jena',   'tcff', 2105, 1110,  172,  1554, 42);
INSERT INTO ESTIMATE_PAR VALUES('jena',   'tcff', 2205, 1110,  170,  1548, 42);
INSERT INTO ESTIMATE_PAR VALUES('jena',   'tcff', 2305, 1100,  172,  1542, 42);

INSERT INTO ESTIMATE_PAR VALUES('psql',   'tcff',   75,  570,  440,  1020, 38);
INSERT INTO ESTIMATE_PAR VALUES('psql',   'tcff',  105,  180,  446,  1116, 38);

INSERT INTO ESTIMATE_PAR VALUES('sqlite3','tcff',   50,  380,  643,  1524, 42);
INSERT INTO ESTIMATE_PAR VALUES('sqlite3','tcff',  105,    0,  628,  1818, 42);

INSERT INTO ESTIMATE_PAR VALUES('mariadb','tcff',    0, 1210,  422,  1362, 19);
-- InnoDB:                                           0  1210   422   1362  19
-- Memory:                                           5,  410,  507,   822, 18

INSERT INTO ESTIMATE_PAR VALUES('souffle','tcff',   96,   68,    6,   909, 24);
INSERT INTO ESTIMATE_PAR VALUES('souffle','tcff',  100,   40,    7,   906, 24);
INSERT INTO ESTIMATE_PAR VALUES('souffle','tcff',  100,   26,    7,   909, 24);
INSERT INTO ESTIMATE_PAR VALUES('souffle','tcff',  105,   30,    7,   894, 24);
INSERT INTO ESTIMATE_PAR VALUES('souffle','tcff',  120,   26,    7,   897, 24);

-- ============================================================================
-- Dummy Table with Exactly One Row Used in Some Views:
-- ============================================================================


CREATE TABLE DUMMY_TAB(
	ATTR NUMERIC(1)
);

INSERT INTO DUMMY_TAB VALUES(0);


-- ############################################################################
-- Views for Statistical Evaluation of Run Data:
-- ############################################################################


-- ============================================================================
-- Full Statistical Evaluation of Run Data:
-- ============================================================================

CREATE VIEW RUN_STATISTICS AS
SELECT BENCH, IMPL, FILE_ID, MACHINE, PROG, INSTALL, WITH_OUTPUT,
	-- Load Time:
	ROUND(AVG(LOAD_TIME),3) AS AVG_LOAD_TIME,
	MIN(LOAD_TIME) AS MIN_LOAD_TIME,
	MAX(LOAD_TIME) AS MAX_LOAD_TIME,
	MAX(LOAD_TIME)-MIN(LOAD_TIME) AS DIFF_LOAD_TIME,
	STDDEV(LOAD_TIME) AS SDEV_LOAD_TIME,
	-- Exec Time:
	ROUND(AVG(EXEC_TIME),3) AS AVG_EXEC_TIME,
	MIN(EXEC_TIME) AS MIN_EXEC_TIME,
	MAX(EXEC_TIME) AS MAX_EXEC_TIME,
	MAX(EXEC_TIME)-MIN(EXEC_TIME) AS DIFF_EXEC_TIME,
	STDDEV(EXEC_TIME) AS SDEV_EXEC_TIME,
	-- Total Time:
	ROUND(AVG(TOTAL_TIME),3) AS AVG_TOTAL_TIME,
	MIN(TOTAL_TIME) AS MIN_TOTAL_TIME,
	MAX(TOTAL_TIME) AS MAX_TOTAL_TIME,
	MAX(TOTAL_TIME)-MIN(TOTAL_TIME) AS DIFF_TOTAL_TIME,
	STDDEV(TOTAL_TIME) AS SDEV_TOTAL_TIME,
	-- User Mode Time:
	ROUND(AVG(USER_TIME),3) AS AVG_USER_TIME,
	MIN(USER_TIME) AS MIN_USER_TIME,
	MAX(USER_TIME) AS MAX_USER_TIME,
	MAX(USER_TIME)-MIN(USER_TIME) AS DIFF_USER_TIME,
	STDDEV(USER_TIME) AS SDEV_USER_TIME,
	-- System Mode Time:
	ROUND(AVG(SYS_TIME),3) AS AVG_SYS_TIME,
	MIN(SYS_TIME) AS MIN_SYS_TIME,
	MAX(SYS_TIME) AS MAX_SYS_TIME,
	MAX(SYS_TIME)-MIN(SYS_TIME) AS DIFF_SYS_TIME,
	STDDEV(SYS_TIME) AS SDEV_SYS_TIME,
	-- CPU Time:
	ROUND(AVG(SYS_TIME+USER_TIME),3) AS AVG_CPU_TIME,
	MIN(SYS_TIME+USER_TIME) AS MIN_CPU_TIME,
	MAX(SYS_TIME+USER_TIME) AS MAX_CPU_TIME,
	MAX(SYS_TIME+USER_TIME)-MIN(SYS_TIME+USER_TIME) AS DIFF_CPU_TIME,
	STDDEV(SYS_TIME+USER_TIME) AS SDEV_CPU_TIME,
	-- Real Time:
	ROUND(AVG(REAL_TIME),3) AS AVG_REAL_TIME,
	MIN(REAL_TIME) AS MIN_REAL_TIME,
	MAX(REAL_TIME) AS MAX_REAL_TIME,
	MAX(REAL_TIME)-MIN(REAL_TIME) AS DIFF_REAL_TIME,
	STDDEV(REAL_TIME) AS SDEV_REAL_TIME,
	-- Memory:
	ROUND(AVG(MEM),3) AS AVG_MEM,
	MIN(MEM) AS MIN_MEM,
	MAX(MEM) AS MAX_MEM,
	MAX(MEM)-MIN(MEM) AS DIFF_MEM,
	STDDEV(MEM) AS SDEV_MEM,
	-- Number of Runs:
	COUNT(*) AS NUM_BENCH_RUN
FROM BENCH_RUN
WHERE STATUS <> 'FAILED' AND STATUS <> 'OUTL'
GROUP BY MACHINE, PROG, INSTALL, BENCH, IMPL, FILE_ID, WITH_OUTPUT
ORDER BY MACHINE, PROG, INSTALL, BENCH, IMPL, FILE_ID, WITH_OUTPUT;

-- ============================================================================
-- Statistical Evaluation of Run Data, without STDDEV - This works for SQLite:
-- ============================================================================

CREATE VIEW RUN_STAT AS
SELECT BENCH, IMPL, FILE_ID, MACHINE, PROG, INSTALL, WITH_OUTPUT,
	-- Load Time:
	ROUND(AVG(LOAD_TIME),3) AS AVG_LOAD_TIME,
	MIN(LOAD_TIME) AS MIN_LOAD_TIME,
	MAX(LOAD_TIME) AS MAX_LOAD_TIME,
	MAX(LOAD_TIME)-MIN(LOAD_TIME) AS DIFF_LOAD_TIME,
	-- Exec Time:
	ROUND(AVG(EXEC_TIME),3) AS AVG_EXEC_TIME,
	MIN(EXEC_TIME) AS MIN_EXEC_TIME,
	MAX(EXEC_TIME) AS MAX_EXEC_TIME,
	MAX(EXEC_TIME)-MIN(EXEC_TIME) AS DIFF_EXEC_TIME,
	-- Total Time:
	ROUND(AVG(TOTAL_TIME),3) AS AVG_TOTAL_TIME,
	MIN(TOTAL_TIME) AS MIN_TOTAL_TIME,
	MAX(TOTAL_TIME) AS MAX_TOTAL_TIME,
	MAX(TOTAL_TIME)-MIN(TOTAL_TIME) AS DIFF_TOTAL_TIME,
	-- User Mode Time:
	ROUND(AVG(USER_TIME),3) AS AVG_USER_TIME,
	MIN(USER_TIME) AS MIN_USER_TIME,
	MAX(USER_TIME) AS MAX_USER_TIME,
	MAX(USER_TIME)-MIN(USER_TIME) AS DIFF_USER_TIME,
	-- System Mode Time:
	ROUND(AVG(SYS_TIME),3) AS AVG_SYS_TIME,
	MIN(SYS_TIME) AS MIN_SYS_TIME,
	MAX(SYS_TIME) AS MAX_SYS_TIME,
	MAX(SYS_TIME)-MIN(SYS_TIME) AS DIFF_SYS_TIME,
	-- CPU Time:
	ROUND(AVG(SYS_TIME+USER_TIME),3) AS AVG_CPU_TIME,
	MIN(SYS_TIME+USER_TIME) AS MIN_CPU_TIME,
	MAX(SYS_TIME+USER_TIME) AS MAX_CPU_TIME,
	MAX(SYS_TIME+USER_TIME)-MIN(SYS_TIME+USER_TIME) AS DIFF_CPU_TIME,
	-- Real Time:
	ROUND(AVG(REAL_TIME),3) AS AVG_REAL_TIME,
	MIN(REAL_TIME) AS MIN_REAL_TIME,
	MAX(REAL_TIME) AS MAX_REAL_TIME,
	MAX(REAL_TIME)-MIN(REAL_TIME) AS DIFF_REAL_TIME,
	-- Memory:
	ROUND(AVG(MEM),3) AS AVG_MEM,
	MIN(MEM) AS MIN_MEM,
	MAX(MEM) AS MAX_MEM,
	MAX(MEM)-MIN(MEM) AS DIFF_MEM,
	-- Number of Runs:
	COUNT(*) AS NUM_BENCH_RUN
FROM BENCH_RUN
WHERE STATUS <> 'FAILED' AND STATUS <> 'OUTL'
GROUP BY MACHINE, PROG, INSTALL, BENCH, IMPL, FILE_ID, WITH_OUTPUT
ORDER BY MACHINE, PROG, INSTALL, BENCH, IMPL, FILE_ID, WITH_OUTPUT;

-- ============================================================================
-- Benchmark Results for Current Version on the Standard Machine without Output:
-- ============================================================================

CREATE VIEW RESULT AS
SELECT R.BENCH AS BENCH,
	R.IMPL AS IMPL,
	R.FILE_ID AS FILE_ID,
	R.PROG AS PROG,
	R.AVG_LOAD_TIME AS LOAD_T,
	R.AVG_EXEC_TIME AS EXEC_T,
	R.AVG_TOTAL_TIME AS TOTAL_T,
	R.AVG_CPU_TIME AS CPU_T,
	R.AVG_REAL_TIME AS REAL_T,
	R.AVG_MEM AS MEM_MB
FROM   RUN_STAT R, PROG_INSTALL V, MACHINE M, DATA_FILE F
WHERE  R.MACHINE = M.MACHINE AND V.MACHINE = R.MACHINE AND M.STATUS = 'S'
AND    R.PROG = V.PROG AND R.INSTALL = V.INSTALL AND V.STATUS = 'C'
AND    R.WITH_OUTPUT = 'N'
AND    R.FILE_ID = F.FILE_ID
ORDER BY R.BENCH, R.IMPL, F.SEQ_NO, R.PROG;

-- ============================================================================
-- Benchmark Results (Current Ver., Std. Machine, No Output, Best Settings):
-- ============================================================================

CREATE VIEW BEST_RESULT AS
SELECT R.BENCH AS BENCH,
	R.IMPL AS IMPL,
	R.FILE_ID AS FILE_ID,
	R.PROG AS PROG,
	R.AVG_LOAD_TIME AS LOAD_T,
	R.AVG_EXEC_TIME AS EXEC_T,
	R.AVG_TOTAL_TIME AS TOTAL_T,
	R.AVG_CPU_TIME AS CPU_T,
	R.AVG_REAL_TIME AS REAL_T,
	R.AVG_MEM AS MEM_MB
FROM   RUN_STAT R, PROG_INSTALL V, MACHINE M, BENCH_IMPL I, DATA_FILE F
WHERE  R.MACHINE = M.MACHINE AND V.MACHINE = R.MACHINE AND M.STATUS = 'S'
AND    R.PROG = V.PROG AND R.INSTALL = V.INSTALL AND V.STATUS = 'C'
AND    R.WITH_OUTPUT = 'N'
AND    I.PROG = R.PROG AND I.BENCH = R.BENCH AND I.IMPL = R.IMPL
AND    I.STATUS = 'B'
AND    R.FILE_ID = F.FILE_ID
ORDER BY R.BENCH, R.IMPL, F.SEQ_NO, R.PROG;


-- ############################################################################
-- Outlier Detection, Checking of Results for Plausibility:
-- ############################################################################


-- ============================================================================
-- Detect Outliers with regard to Real Time:
-- ============================================================================

CREATE VIEW OUTLIER_REAL AS
SELECT R.REAL_TIME, S.AVG_REAL_TIME, R.PROG, R.INSTALL, R.STATUS,
       R.BENCH, R.IMPL, R.FILE_ID, R.MACHINE, R.WITH_OUTPUT,
       R.RUN_DATE, R.RUN_NO
FROM   BENCH_RUN R, RUN_STAT S
WHERE  R.STATUS <> 'FAILED'
AND    R.BENCH       = S.BENCH
AND    R.IMPL        = S.IMPL
AND    R.FILE_ID     = S.FILE_ID
AND    R.MACHINE     = S.MACHINE
AND    R.PROG        = S.PROG
AND    R.INSTALL     = S.INSTALL
AND    R.WITH_OUTPUT = S.WITH_OUTPUT
AND    R.REAL_TIME   > 2.0 * S.AVG_REAL_TIME
AND    R.REAL_TIME   > S.AVG_REAL_TIME + 1
ORDER BY MACHINE, PROG, INSTALL, BENCH, IMPL, FILE_ID, WITH_OUTPUT;

-- ============================================================================
-- Detect Outliers with regard to CPU Time:
-- ============================================================================

CREATE VIEW OUTLIER_CPU AS
SELECT R.SYS_TIME+R.USER_TIME AS CPU_TIME, S.AVG_CPU_TIME,
       R.PROG, R.INSTALL, R.STATUS,
       R.BENCH, R.IMPL, R.FILE_ID, R.MACHINE, R.WITH_OUTPUT,
       R.RUN_DATE, R.RUN_NO
FROM   BENCH_RUN R, RUN_STAT S
WHERE  R.STATUS <> 'FAILED'
AND    R.BENCH       = S.BENCH
AND    R.IMPL        = S.IMPL
AND    R.FILE_ID     = S.FILE_ID
AND    R.MACHINE     = S.MACHINE
AND    R.PROG        = S.PROG
AND    R.INSTALL     = S.INSTALL
AND    R.WITH_OUTPUT = S.WITH_OUTPUT
AND    R.SYS_TIME+R.USER_TIME > 2.0 * S.AVG_CPU_TIME
AND    R.SYS_TIME+R.USER_TIME > S.AVG_CPU_TIME + 1
ORDER BY MACHINE, PROG, INSTALL, BENCH, IMPL, FILE_ID, WITH_OUTPUT;

-- ============================================================================
-- Detect Outliers with regard to Memory:
-- ============================================================================

CREATE VIEW OUTLIER_MEM AS
SELECT R.MEM, S.AVG_MEM, R.PROG, R.INSTALL, R.STATUS,
       R.BENCH, R.IMPL, R.FILE_ID, R.MACHINE, R.WITH_OUTPUT,
       R.RUN_DATE, R.RUN_NO
FROM   BENCH_RUN R, RUN_STAT S
WHERE  R.STATUS <> 'FAILED'
AND    R.BENCH       = S.BENCH
AND    R.IMPL        = S.IMPL
AND    R.FILE_ID     = S.FILE_ID
AND    R.MACHINE     = S.MACHINE
AND    R.PROG        = S.PROG
AND    R.INSTALL     = S.INSTALL
AND    R.WITH_OUTPUT = S.WITH_OUTPUT
AND    R.MEM         > 2.0 * S.AVG_MEM
AND    R.MEM         > S.AVG_MEM + 5
ORDER BY MACHINE, PROG, INSTALL, BENCH, IMPL, FILE_ID, WITH_OUTPUT;

-- ============================================================================
-- Check where Realtime is much more than CPU time (at least double):
-- ============================================================================

CREATE VIEW REAL_TOO_LARGE AS
SELECT R.REAL_TIME, S.AVG_REAL_TIME,
       R.SYS_TIME+R.USER_TIME AS CPU_TIME, S.AVG_CPU_TIME,
       R.PROG, R.INSTALL, R.STATUS,
       R.BENCH, R.IMPL, R.FILE_ID, R.MACHINE, R.WITH_OUTPUT,
       R.RUN_DATE, R.RUN_NO
FROM   BENCH_RUN R, RUN_STAT S
WHERE  R.STATUS <> 'FAILED'
AND    R.BENCH       = S.BENCH
AND    R.IMPL        = S.IMPL
AND    R.FILE_ID     = S.FILE_ID
AND    R.MACHINE     = S.MACHINE
AND    R.PROG        = S.PROG
AND    R.INSTALL     = S.INSTALL
AND    R.WITH_OUTPUT = S.WITH_OUTPUT
AND    R.REAL_TIME   > 2.0 * (R.SYS_TIME+R.USER_TIME)
AND    R.REAL_TIME   > (R.SYS_TIME+R.USER_TIME) + 1
ORDER BY MACHINE, PROG, INSTALL, BENCH, IMPL, FILE_ID, WITH_OUTPUT;

-- ============================================================================
-- Compare Real Time with CPU Time:
-- ============================================================================

CREATE VIEW PCT_CPU AS
SELECT PROG, RUN_DATE, MACHINE, BENCH,
	ROUND(AVG((SYS_TIME+USER_TIME) * 100 / REAL_TIME)) AS AVG_CPU,
	ROUND(MIN((SYS_TIME+USER_TIME) * 100 / REAL_TIME)) AS MIN_CPU,
	ROUND(MAX((SYS_TIME+USER_TIME) * 100 / REAL_TIME)) AS MAX_CPU
FROM   BENCH_RUN
WHERE  SYS_TIME IS NOT NULL AND USER_TIME IS NOT NULL
AND    REAL_TIME > 0
AND    STATUS <> 'FAILED' AND STATUS <> 'OUTL'
AND    WITH_OUTPUT = 'N'
GROUP  BY PROG, RUN_DATE, MACHINE, BENCH
ORDER  BY PROG, RUN_DATE, MACHINE, BENCH;


-- ============================================================================
-- Compare Real Time with CPU Time:
-- ============================================================================

CREATE VIEW PCT_CPU_INPUT AS
SELECT PROG, MACHINE, BENCH, FILE_ID,
	ROUND(AVG((SYS_TIME+USER_TIME) * 100 / REAL_TIME)) AS AVG_CPU,
	ROUND(MIN((SYS_TIME+USER_TIME) * 100 / REAL_TIME)) AS MIN_CPU,
	ROUND(MAX((SYS_TIME+USER_TIME) * 100 / REAL_TIME)) AS MAX_CPU
FROM   BENCH_RUN
WHERE  SYS_TIME IS NOT NULL AND USER_TIME IS NOT NULL
AND    REAL_TIME > 0
AND    STATUS <> 'FAILED' AND STATUS <> 'OUTL'
AND    WITH_OUTPUT = 'N'
GROUP  BY PROG, MACHINE, BENCH, FILE_ID
ORDER  BY PROG, MACHINE, BENCH, FILE_ID;


-- ############################################################################
-- Views with Benchmark Results for a Specific System:
-- ############################################################################


-- ============================================================================
-- Benchmark Results for BAM (Current Version, Standard Machine, No Output):
-- ============================================================================

CREATE VIEW BAM AS
SELECT BENCH,
	IMPL,
	FILE_ID AS FILE,
	LOAD_T,
	EXEC_T,
	-- TOTAL_T,
	CPU_T,
	REAL_T,
	MEM_MB
FROM RESULT
WHERE PROG = 'bam';

-- ============================================================================
-- Benchmark Results for XSB (Current Version, Standard Machine, No Output):
-- ============================================================================

CREATE VIEW XSB AS
SELECT BENCH,
	IMPL,
	FILE_ID AS FILE,
	LOAD_T,
	EXEC_T,
	-- TOTAL_T,
	CPU_T,
	REAL_T,
	MEM_MB
FROM RESULT
WHERE PROG = 'xsb';

-- ============================================================================
-- Benchmark Results for YAP (Current Version, Standard Machine, No Output):
-- ============================================================================

CREATE VIEW YAP AS
SELECT BENCH,
	IMPL,
	FILE_ID AS FILE,
	LOAD_T,
	EXEC_T,
	-- TOTAL_T,
	CPU_T,
	REAL_T,
	MEM_MB
FROM RESULT
WHERE PROG = 'yap';

-- ============================================================================
-- Benchmark Results for SQLite (Current Ver., Standard Machine, No Output):
-- ============================================================================

CREATE VIEW SQLITE AS
SELECT BENCH,
	IMPL,
	FILE_ID AS FILE,
	-- LOAD_T,
	-- EXEC_T,
	-- TOTAL_T,
	CPU_T,
	REAL_T,
	MEM_MB
FROM RESULT
WHERE PROG = 'sqlite3';

-- ============================================================================
-- Benchmark Results for PostgreSQL (Current Ver., Standard Machine, No Output):
-- ============================================================================

CREATE VIEW PSQL AS
SELECT BENCH,
	IMPL,
	FILE_ID,
	-- LOAD_T,
	-- EXEC_T,
	-- TOTAL_T,
	CPU_T,
	REAL_T,
	MEM_MB
FROM RESULT
WHERE PROG = 'psql';

-- ============================================================================
-- Benchmark Results for MariaDB (Current Ver., Standard Machine, No Output):
-- ============================================================================

CREATE VIEW MARIA AS
SELECT BENCH,
	IMPL,
	FILE_ID,
	-- LOAD_T,
	-- EXEC_T,
	-- TOTAL_T,
	CPU_T,
	REAL_T,
	MEM_MB
FROM RESULT
WHERE PROG = 'mariadb';

-- ============================================================================
-- Benchmark Results for Souffle (Current Ver., Standard Machine, No Output):
-- ============================================================================

CREATE VIEW SOUFFLE AS
SELECT BENCH,
	IMPL,
	FILE_ID,
	-- LOAD_T,
	-- EXEC_T,
	-- TOTAL_T,
	CPU_T,
	REAL_T,
	MEM_MB
FROM RESULT
WHERE PROG = 'souffle';

-- ============================================================================
-- Benchmark Results for Jena (Current Version, Standard Machine, No Output):
-- ============================================================================

CREATE VIEW JENA AS
SELECT BENCH,
	IMPL,
	FILE_ID,
	-- LOAD_T,
	-- EXEC_T,
	-- TOTAL_T,
	CPU_T,
	REAL_T,
	MEM_MB
FROM RESULT
WHERE PROG = 'jena';

-- ============================================================================
-- Benchmark Results for DataScript (Current Ver., Standard Machine, No Output):
-- ============================================================================

CREATE VIEW DATASCRIPT AS
SELECT BENCH,
	IMPL,
	FILE_ID AS FILE,
	-- LOAD_T,
	-- EXEC_T,
	-- TOTAL_T,
	CPU_T,
	REAL_T,
	MEM_MB
FROM RESULT
WHERE PROG = 'datascript';


-- ############################################################################
-- Views to Find/Check Best Option Settings for a System and a Benchmark:
-- ############################################################################


-- ============================================================================
-- Best Option Settings (in Real Time) for a Program for a Benchmark:
-- ============================================================================

-- This looks only at the runtimes of the standard program version
-- on the standard machine.

CREATE VIEW BEST_IMPL_REAL AS
SELECT	R.BENCH,
	R.FILE_ID,
	R.PROG,
	R.IMPL,
	R.REAL_T,
	I.STATUS
FROM	RESULT R, BENCHMARK B, DATA_FILE F, BENCH_IMPL I
WHERE   R.BENCH = B.BENCH AND R.FILE_ID = F.FILE_ID
AND	I.BENCH = R.BENCH AND I.IMPL = R.IMPL AND I.PROG = R.PROG
AND NOT EXISTS(SELECT * FROM RESULT R2
		WHERE R2.BENCH = R.BENCH AND R2.PROG = R.PROG
		AND   R2.FILE_ID = R.FILE_ID
		AND   R2.REAL_T < R.REAL_T)
-- Eliminate trivial tests with run time 0: All options are best.
AND EXISTS(SELECT * FROM RESULT R3
		WHERE R3.BENCH = R.BENCH AND R3.PROG = R.PROG
		AND   R3.FILE_ID = R.FILE_ID
		AND   R3.REAL_T > 0)
ORDER BY B.SEQ_NO, F.SEQ_NO;


-- ============================================================================
-- Best Option Settings (in CPU Time) for a Program for a Benchmark:
-- ============================================================================

-- This looks only at the runtimes of the standard program version
-- on the standard machine.

CREATE VIEW BEST_IMPL_CPU AS
SELECT	R.BENCH,
	R.FILE_ID,
	R.PROG,
	R.IMPL,
	R.CPU_T,
	I.STATUS
FROM	RESULT R, BENCHMARK B, DATA_FILE F, BENCH_IMPL I
WHERE   R.BENCH = B.BENCH AND R.FILE_ID = F.FILE_ID
AND	I.BENCH = R.BENCH AND I.IMPL = R.IMPL AND I.PROG = R.PROG
AND NOT EXISTS(SELECT * FROM RESULT R2
		WHERE R2.BENCH = R.BENCH AND R2.PROG = R.PROG
		AND   R2.FILE_ID = R.FILE_ID
		AND   R2.CPU_T < R.CPU_T)
-- Eliminate trivial tests with run time 0: All options are best.
AND EXISTS(SELECT * FROM RESULT R3
		WHERE R3.BENCH = R.BENCH AND R3.PROG = R.PROG
		AND   R3.FILE_ID = R.FILE_ID
		AND   R3.CPU_T > 0)
ORDER BY B.SEQ_NO, F.SEQ_NO;

-- ============================================================================
-- Compare Storage Managers for MariaDB:
-- ============================================================================

CREATE VIEW MARIA_COMPARE AS
SELECT INNO.BENCH,
	INNO.FILE_ID,
        F.LINES,
	INNO.REAL_T AS INNO_REAL_T,
	MEM.REAL_T AS MEM_REAL_T,
	CASE WHEN INNO.REAL_T > 0 THEN
                ROUND((INNO.REAL_T-MEM.REAL_T)*100/INNO.REAL_T)
                ELSE NULL END AS MEM_BETTER_PCT,
	CASE WHEN INNO.REAL_T < MEM.REAL_T THEN 'Y' ELSE ' ' END AS INNO_BETTER
FROM MARIA INNO, MARIA MEM, DATA_FILE F
WHERE INNO.BENCH = MEM.BENCH AND INNO.FILE_ID = MEM.FILE_ID AND
        INNO.IMPL = 'i' AND MEM.IMPL = 'm' AND
	F.FILE_ID = INNO.FILE_ID
ORDER BY INNO.BENCH, F.SEQ_NO;


-- ############################################################################
-- Views for Plots of Performance Data vs. Cost Measures (only for TCFF):
-- ############################################################################


-- ============================================================================
-- Plot of Real Time vs. Cost for TCFF Benchmark:
-- ============================================================================

-- This looks only at the runtimes of the standard program version,
-- best implementation, on the standard machine.

CREATE VIEW PLOT_REAL AS
SELECT	R.PROG,
	R.FILE_ID,
	F.SEQ_NO AS NO,
	G.COST,
	R.REAL_T,
	'"' || R.FILE_ID || '","' || G.COST || '","' || R.REAL_T || '"' AS CSV,
	R.FILE_ID || '	' || G.COST || '	' || R.REAL_T AS TSV
FROM	RESULT R, DATA_FILE F, GRAPH_DATA G, BENCH_IMPL I
WHERE   R.BENCH = 'tcff' AND R.FILE_ID = F.FILE_ID AND R.FILE_ID = G.FILE_ID
AND	I.BENCH = R.BENCH AND I.IMPL = R.IMPL AND I.PROG = R.PROG
AND	I.STATUS = 'B'
UNION
SELECT  DISTINCT P.PROG, '', -1, 0 AS COST, 0 AS REAL_T,
	'"df","cost","tavg"' as CSV,
	'df	cost	tavg' as TSV
FROM	PROGRAM P
WHERE	EXISTS(SELECT * FROM RESULT R
		WHERE R.BENCH = 'tcff' AND R.PROG = P.PROG)
ORDER BY PROG, NO;

-- ============================================================================
-- Plot of CPU Time vs. Cost for TCFF Benchmark:
-- ============================================================================

-- This looks only at the runtimes of the standard program version,
-- best implementation, on the standard machine.

CREATE VIEW PLOT_CPU AS
SELECT	R.PROG,
	R.FILE_ID,
	F.SEQ_NO AS NO,
	G.COST,
	R.CPU_T,
	'"' || R.FILE_ID || '","' || G.COST || '","' || R.CPU_T || '"' AS CSV,
	R.FILE_ID || '	' || G.COST || '	' || R.CPU_T AS TSV
FROM	RESULT R, DATA_FILE F, GRAPH_DATA G, BENCH_IMPL I
WHERE   R.BENCH = 'tcff' AND R.FILE_ID = F.FILE_ID AND R.FILE_ID = G.FILE_ID
AND	I.BENCH = R.BENCH AND I.IMPL = R.IMPL AND I.PROG = R.PROG
AND	I.STATUS = 'B'
UNION
SELECT  DISTINCT P.PROG, '', -1, 0 AS COST, 0 AS CPU_T,
	'"df","cost","tavg"' as CSV,
	'df	cost	tavg' as TSV
FROM	PROGRAM P
WHERE	EXISTS(SELECT * FROM RESULT R
		WHERE R.BENCH = 'tcff' AND R.PROG = P.PROG)
ORDER BY PROG, NO;

-- ============================================================================
-- Plot of Memory vs. Number of Tuples for TCFF Benchmark:
-- ============================================================================

-- This looks only at the runtimes of the standard program version,
-- best implementation, on the standard machine.

CREATE VIEW PLOT_MEM AS
SELECT	R.PROG,
	R.FILE_ID,
	F.SEQ_NO AS NO,
	G.NUM_EDGES + G.TC_SIZE AS FACTS,
	R.MEM_MB,
	'"' || R.FILE_ID || '","' || (G.NUM_EDGES+G.TC_SIZE) || '","' ||
		R.MEM_MB || '"' AS CSV,
	R.FILE_ID || '	' || (G.NUM_EDGES+G.TC_SIZE) || '	' ||
		R.MEM_MB AS TSV
FROM	RESULT R, DATA_FILE F, GRAPH_DATA G, BENCH_IMPL I
WHERE   R.BENCH = 'tcff' AND R.FILE_ID = F.FILE_ID AND R.FILE_ID = G.FILE_ID
AND	I.BENCH = R.BENCH AND I.IMPL = R.IMPL AND I.PROG = R.PROG
AND	I.STATUS = 'B'
UNION
SELECT  DISTINCT P.PROG, '', -1, 0 AS FACTS, 0 AS CPU_T,
	'"df","facts","mem"' as CSV,
	'df	facts	mem' as TSV
FROM	PROGRAM P
WHERE	EXISTS(SELECT * FROM RESULT R
		WHERE R.BENCH = 'tcff' AND R.PROG = P.PROG)
ORDER BY PROG, NO;


-- ############################################################################
-- Benchmark Results for TCFF - Comparison of Different Systems:
-- ############################################################################


-- ============================================================================
-- Benchmark Results for TCFF (Comparison of Various Systems on CPU Time):
-- ============================================================================

-- This looks only at the runtimes of the standard program version,
-- best implementation, on the standard machine.

CREATE VIEW TCFF_CPU AS
SELECT	BAM.FILE_ID,
	BAM.CPU_T AS BAM_CPU,
	XSB.CPU_T AS XSB_CPU,
	(CASE WHEN BAM.CPU_T=0 THEN NULL ELSE ROUND(XSB.CPU_T/BAM.CPU_T,1) END)
		AS XSB_F,
	YAP.CPU_T AS YAP_CPU,
	(CASE WHEN BAM.CPU_T=0 THEN NULL ELSE ROUND(YAP.CPU_T/BAM.CPU_T,1) END)
		AS YAP_F,
	JENA.CPU_T AS JENA_CPU,
	(CASE WHEN BAM.CPU_T=0 THEN NULL ELSE ROUND(JENA.CPU_T/BAM.CPU_T,1) END)
		AS JENA_F,
	SQLITE.CPU_T AS SQLITE_CPU,
	(CASE WHEN BAM.CPU_T=0 THEN NULL ELSE ROUND(SQLITE.CPU_T/BAM.CPU_T,1)
		END) AS SQLITE_F
FROM	(SELECT * FROM BEST_RESULT WHERE PROG='bam'AND BENCH='tcff') BAM
	LEFT JOIN
	(SELECT * FROM BEST_RESULT WHERE PROG='xsb' AND BENCH='tcff') XSB
	ON XSB.FILE_ID = BAM.FILE_ID
	LEFT JOIN
	(SELECT * FROM BEST_RESULT WHERE PROG='yap' AND BENCH='tcff') YAP
	ON YAP.FILE_ID = BAM.FILE_ID
	LEFT JOIN
	(SELECT * FROM BEST_RESULT WHERE PROG='jena' AND BENCH='tcff') JENA
	ON JENA.FILE_ID = BAM.FILE_ID
	LEFT JOIN
	(SELECT * FROM BEST_RESULT WHERE PROG='sqlite3' AND BENCH='tcff') SQLITE
	ON SQLITE.FILE_ID = BAM.FILE_ID
	JOIN DATA_FILE F
        ON BAM.FILE_ID = F.FILE_ID
ORDER BY F.SEQ_NO;

-- ============================================================================
-- Benchmark Results for TCFF (Comparison of Various Systems on REAL Time):
-- ============================================================================

-- This looks only at the runtimes of the standard program version,
-- best implementation, on the standard machine.
-- This version contains the runtime factors compared with BAM.

CREATE VIEW TCFF_REAL AS
SELECT	BAM.FILE_ID,
        F.SEQ_NO,
	BAM.REAL_T AS BAM_REAL,
	XSB.REAL_T AS XSB_REAL,
	(CASE WHEN BAM.REAL_T=0 THEN NULL
		ELSE ROUND(XSB.REAL_T/BAM.REAL_T,1) END)
		AS XSB_F,
	YAP.REAL_T AS YAP_REAL,
	(CASE WHEN BAM.REAL_T=0 THEN NULL
		ELSE ROUND(YAP.REAL_T/BAM.REAL_T,1) END)
		AS YAP_F,
	SQLITE.REAL_T AS SQLITE_REAL,
	(CASE WHEN BAM.REAL_T=0 THEN NULL
		ELSE ROUND(SQLITE.REAL_T/BAM.REAL_T,1) END)
		AS SQLITE_F,
	PSQL.REAL_T AS PSQL_REAL,
	(CASE WHEN BAM.REAL_T=0 THEN NULL
		ELSE ROUND(PSQL.REAL_T/BAM.REAL_T,1) END)
		AS PSQL_F,
	MARIA.REAL_T AS MARIA_REAL,
	(CASE WHEN BAM.REAL_T=0 THEN NULL
		ELSE ROUND(MARIA.REAL_T/BAM.REAL_T,1) END)
		AS MARIA_F,
	JENA.REAL_T AS JENA_REAL,
	(CASE WHEN BAM.REAL_T=0 THEN NULL
		ELSE ROUND(JENA.REAL_T/BAM.REAL_T,1) END)
		AS JENA_F,
	SOUFFLE.REAL_T AS SOUFFLE_REAL,
	(CASE WHEN BAM.REAL_T=0 THEN NULL
		ELSE ROUND(SOUFFLE.REAL_T/BAM.REAL_T,1) END)
		AS SOUFFLE_F
FROM	(SELECT * FROM BEST_RESULT WHERE PROG='bam'AND BENCH='tcff') BAM
	LEFT JOIN
	(SELECT * FROM BEST_RESULT WHERE PROG='xsb' AND BENCH='tcff') XSB
	ON XSB.FILE_ID = BAM.FILE_ID
	LEFT JOIN
	(SELECT * FROM BEST_RESULT WHERE PROG='yap' AND BENCH='tcff') YAP
	ON YAP.FILE_ID = BAM.FILE_ID
	LEFT JOIN
	(SELECT * FROM BEST_RESULT WHERE PROG='sqlite3' AND BENCH='tcff') SQLITE
	ON SQLITE.FILE_ID = BAM.FILE_ID
	LEFT JOIN
	(SELECT * FROM BEST_RESULT WHERE PROG='psql' AND BENCH='tcff') PSQL
	ON PSQL.FILE_ID = BAM.FILE_ID
	LEFT JOIN
	(SELECT * FROM BEST_RESULT WHERE PROG='mariadb' AND BENCH='tcff') MARIA
	ON MARIA.FILE_ID = BAM.FILE_ID
	LEFT JOIN
	(SELECT * FROM BEST_RESULT WHERE PROG='jena' AND BENCH='tcff') JENA
	ON JENA.FILE_ID = BAM.FILE_ID
	LEFT JOIN
	(SELECT * FROM BEST_RESULT WHERE PROG='souffle' AND BENCH='tcff')SOUFFLE
	ON SOUFFLE.FILE_ID = BAM.FILE_ID
	JOIN DATA_FILE F
        ON BAM.FILE_ID = F.FILE_ID
ORDER BY F.SEQ_NO;

-- ============================================================================
-- Benchmark Results for TCFF (Comparison on REAL Time, Only Factors):
-- ============================================================================

-- This looks only at the runtimes of the standard program version,
-- best implementation, on the standard machine.

-- It is a projection of TCFF_REAL that shows only the factors compared to BAM.
-- This makes the output rows a little smaller.

CREATE VIEW TCFF_REAL_FACTORS AS
SELECT	FILE_ID, BAM_REAL,
	XSB_F, YAP_F, SQLITE_F, PSQL_F, MARIA_F, JENA_F, SOUFFLE_F
FROM	TCFF_REAL
ORDER BY SEQ_NO;

-- ============================================================================
-- Benchmark Results for TCFF (Comparison of Various Systems on REAL Time):
-- ============================================================================

-- This looks only at the runtimes of the standard program version,
-- best implementation, on the standard machine.

CREATE VIEW TCFF_REAL_LATEX AS
SELECT	FILE_ID,
	RPAD(REPLACE(FILE_ID, '_', '\_'), 12) || '&' ||
	TO_CHAR(BAM_REAL, '9990.999') || '&' ||
	(CASE WHEN XSB_F IS NULL THEN '      '
		ELSE TO_CHAR(XSB_F, '990.9') END) || '&' ||
	(CASE WHEN YAP_F IS NULL THEN '      '
		ELSE TO_CHAR(YAP_F, '990.9') END) || '&' ||
	(CASE WHEN SQLITE_F IS NULL THEN '      '
		ELSE TO_CHAR(SQLITE_F, '990.9') END) || '&' ||
	(CASE WHEN PSQL_F IS NULL THEN '      '
		ELSE TO_CHAR(PSQL_F, '990.9') END) || '&' ||
	(CASE WHEN MARIA_F IS NULL THEN '      '
		ELSE TO_CHAR(MARIA_F, '990.9') END) || '&' ||
	(CASE WHEN JENA_F IS NULL THEN '      '
		ELSE TO_CHAR(JENA_F, '990.9') END) || '&' ||
	(CASE WHEN SOUFFLE_F IS NULL THEN '      '
		ELSE TO_CHAR(SOUFFLE_F, '990.9') END) || '\\'
	AS LATEX
FROM	TCFF_REAL
ORDER BY SEQ_NO;

-- ============================================================================
-- Benchmark Results for TCFF (Comparison of Various Systems on REAL Time):
-- ============================================================================

-- This looks only at the runtimes of the standard program version,
-- best implementation, on the standard machine.

-- This version contains the runtime factors compared with XSB
-- and rounds the factors to two decimal places after the decimal point.
-- Otherwise it is identical to TCFF_REAL (which compares with BAM).

CREATE VIEW TCFF_REAL_XSB AS
SELECT	XSB.FILE_ID,
        F.SEQ_NO,
	XSB.REAL_T AS XSB_REAL,
	BAM.REAL_T AS BAM_REAL,
	(CASE WHEN XSB.REAL_T=0 THEN NULL
		ELSE ROUND(BAM.REAL_T/XSB.REAL_T,2) END)
		AS BAM_F,
	YAP.REAL_T AS YAP_REAL,
	(CASE WHEN XSB.REAL_T=0 THEN NULL
		ELSE ROUND(YAP.REAL_T/XSB.REAL_T,2) END)
		AS YAP_F,
	SQLITE.REAL_T AS SQLITE_REAL,
	(CASE WHEN XSB.REAL_T=0 THEN NULL
		ELSE ROUND(SQLITE.REAL_T/XSB.REAL_T,2) END)
		AS SQLITE_F,
	PSQL.REAL_T AS PSQL_REAL,
	(CASE WHEN XSB.REAL_T=0 THEN NULL
		ELSE ROUND(PSQL.REAL_T/XSB.REAL_T,2) END)
		AS PSQL_F,
	MARIA.REAL_T AS MARIA_REAL,
	(CASE WHEN XSB.REAL_T=0 THEN NULL
		ELSE ROUND(MARIA.REAL_T/XSB.REAL_T,2) END)
		AS MARIA_F,
	JENA.REAL_T AS JENA_REAL,
	(CASE WHEN XSB.REAL_T=0 THEN NULL
		ELSE ROUND(JENA.REAL_T/XSB.REAL_T,2) END)
		AS JENA_F,
	SOUFFLE.REAL_T AS SOUFFLE_REAL,
	(CASE WHEN XSB.REAL_T=0 THEN NULL
		ELSE ROUND(SOUFFLE.REAL_T/XSB.REAL_T,2) END)
		AS SOUFFLE_F
FROM	(SELECT * FROM BEST_RESULT WHERE PROG='xsb'AND BENCH='tcff') XSB
	LEFT JOIN
	(SELECT * FROM BEST_RESULT WHERE PROG='bam' AND BENCH='tcff') BAM
	ON BAM.FILE_ID = XSB.FILE_ID
	LEFT JOIN
	(SELECT * FROM BEST_RESULT WHERE PROG='yap' AND BENCH='tcff') YAP
	ON YAP.FILE_ID = XSB.FILE_ID
	LEFT JOIN
	(SELECT * FROM BEST_RESULT WHERE PROG='sqlite3' AND BENCH='tcff') SQLITE
	ON SQLITE.FILE_ID = XSB.FILE_ID
	LEFT JOIN
	(SELECT * FROM BEST_RESULT WHERE PROG='psql' AND BENCH='tcff') PSQL
	ON PSQL.FILE_ID = XSB.FILE_ID
	LEFT JOIN
	(SELECT * FROM BEST_RESULT WHERE PROG='mariadb' AND BENCH='tcff') MARIA
	ON MARIA.FILE_ID = XSB.FILE_ID
	LEFT JOIN
	(SELECT * FROM BEST_RESULT WHERE PROG='jena' AND BENCH='tcff') JENA
	ON JENA.FILE_ID = XSB.FILE_ID
	LEFT JOIN
	(SELECT * FROM BEST_RESULT WHERE PROG='souffle' AND BENCH='tcff')SOUFFLE
	ON SOUFFLE.FILE_ID = XSB.FILE_ID
	JOIN DATA_FILE F
        ON XSB.FILE_ID = F.FILE_ID
ORDER BY F.SEQ_NO;

-- ============================================================================
-- Benchmark Results for TCFF (Comparison on REAL Time, Only Factors):
-- ============================================================================

-- This looks only at the runtimes of the standard program version,
-- best implementation, on the standard machine.

-- It is a projection of TCFF_REAL that shows only the factors compared to BAM.
-- This makes the output rows a little smaller.

CREATE VIEW TCFF_REAL_XSB_FACTORS AS
SELECT	FILE_ID, XSB_REAL,
	BAM_F, YAP_F, SQLITE_F, PSQL_F, MARIA_F, JENA_F, SOUFFLE_F
FROM	TCFF_REAL_XSB
ORDER BY SEQ_NO;


-- ============================================================================
-- Benchmark Results for TCFF (Comparison of Various Systems on REAL Time):
-- ============================================================================

-- This looks only at the runtimes of the standard program version,
-- best implementation, on the standard machine.

CREATE VIEW TCFF_REAL_XSB_LATEX AS
SELECT	FILE_ID,
	RPAD(REPLACE(FILE_ID, '_', '\_'), 10) || '&' ||
	TO_CHAR(XSB_REAL, '9990.999') || '&' ||
	(CASE WHEN BAM_F IS NULL     THEN '       '
		ELSE TO_CHAR(BAM_F, '990.99') END) || '&' ||
	(CASE WHEN YAP_F IS NULL     THEN '       '
		ELSE TO_CHAR(YAP_F, '990.99') END) || '&' ||
	(CASE WHEN SQLITE_F IS NULL  THEN '       '
		ELSE TO_CHAR(SQLITE_F, '990.99') END) || '&' ||
	(CASE WHEN PSQL_F IS NULL    THEN '       '
		ELSE TO_CHAR(PSQL_F, '990.99') END) || '&' ||
	(CASE WHEN MARIA_F IS NULL   THEN '       '
		ELSE TO_CHAR(MARIA_F, '990.99') END) || '&' ||
	(CASE WHEN JENA_F IS NULL    THEN '       '
		ELSE TO_CHAR(JENA_F, '990.99') END) || '&' ||
	(CASE WHEN SOUFFLE_F IS NULL THEN '       '
		ELSE TO_CHAR(SOUFFLE_F, '990.99') END) || '\\'
	AS LATEX
FROM	TCFF_REAL_XSB
ORDER BY SEQ_NO;


-- ============================================================================
-- Benchmark Results for TCFF (Comparison of Various Systems on REAL Time):
-- ============================================================================

-- This looks only at the runtimes of the standard program version,
-- best implementation, on the standard machine.

CREATE VIEW TCFF_REAL_XSB_HTML AS
SELECT -2 AS SEQ_NO, NULL AS FILE_ID,
	'<table>' AS HTML
FROM    DUMMY_TAB
UNION ALL
SELECT -1 AS SEQ_NO, NULL AS FILE_ID,
	'<tr>' ||
	     '<th class="colGraph">'   || 'Graph'          || '</th>' || '
	' || '<th class="colXSBTime">' || 'XSB (s)'        || '</th>' || '
	' || '<th class="colBAM"    >' || 'BAM'            || '</th>' || '
	' || '<th class="colYAP"    >' || 'YAP'            || '</th>' || '
	' || '<th class="colSQLite" >' || 'SQLite'         || '</th>' || '
	' || '<th class="colPSQL"   >' || 'PostgreSQL'     || '</th>' || '
	' || '<th class="colMaria"  >' || 'MariaDB'        || '</th>' || '
	' || '<th class="colJena"   >' || 'Jena'           || '</th>' || '
	' || '<th class="colSouffle">' || 'Souffl&eacute;' || '</th>' || '
	' || '</tr>'
	AS HTML
FROM    DUMMY_TAB
UNION ALL
SELECT	SEQ_NO, FILE_ID,
	'<tr>' ||
	'<td class="colGraph">' || FILE_ID || '</td>' || '
	' || '<td class="colXSBTime">' || TO_CHAR(XSB_REAL, '9990.999') ||
		'&nbsp;s' || '</td>' || '
	' || '<td class="colBAM"    >' ||
		(CASE WHEN BAM_F IS NULL     THEN '&nbsp; '
			ELSE TO_CHAR(BAM_F,     '990.99') END) || '</td>' || '
	' || '<td class="colYAP"    >' ||
		(CASE WHEN YAP_F IS NULL     THEN '&nbsp; '
			ELSE TO_CHAR(YAP_F,     '990.99') END) || '</td>' || '
	' || '<td class="colSQLite" >' ||
		(CASE WHEN SQLITE_F IS NULL  THEN '&nbsp; '
			ELSE TO_CHAR(SQLITE_F,  '990.99') END) || '</td>' || '
	' || '<td class="colPSQL"   >' ||
		(CASE WHEN PSQL_F IS NULL    THEN '&nbsp; '
			ELSE TO_CHAR(PSQL_F,    '990.99') END) || '</td>' || '
	' || '<td class="colMaria"  >' ||
		(CASE WHEN MARIA_F IS NULL   THEN '&nbsp; '
			ELSE TO_CHAR(MARIA_F,   '990.99') END) || '</td>' || '
	' || '<td class="colJena"   >' ||
		(CASE WHEN JENA_F IS NULL    THEN '&nbsp; '
			ELSE TO_CHAR(JENA_F,    '990.99') END) || '</td>' || '
	' || '<td class="colSouffle">' ||
		(CASE WHEN SOUFFLE_F IS NULL THEN '&nbsp; '
			ELSE TO_CHAR(SOUFFLE_F, '990.99') END) || '</td>' || '
	' || '</tr>'
	AS HTML
FROM	TCFF_REAL_XSB
UNION ALL
SELECT 1000000 AS SEQ_NO, NULL AS FILE_ID,
	'</table>' AS HTML
FROM    DUMMY_TAB
ORDER BY SEQ_NO;


-- ============================================================================
-- Benchmark Results for TCFF (Comparison of Various Systems on Memory):
-- ============================================================================

-- This looks only at the runtimes of the standard program version,
-- best implementation, on the standard machine.

CREATE VIEW TCFF_MEM AS
SELECT	BAM.FILE_ID,
	BAM.MEM_MB AS BAM_MEM,
	XSB.MEM_MB AS XSB_MEM,
	(CASE WHEN BAM.MEM_MB=0 THEN NULL
		ELSE ROUND(XSB.MEM_MB/BAM.MEM_MB,1) END)
		AS XSB_F,
	YAP.MEM_MB AS YAP_MEM,
	(CASE WHEN BAM.MEM_MB=0 THEN NULL
		ELSE ROUND(YAP.MEM_MB/BAM.MEM_MB,1) END)
		AS YAP_F,
	JENA.MEM_MB AS JENA_MEM,
	(CASE WHEN BAM.MEM_MB=0 THEN NULL
		ELSE ROUND(JENA.MEM_MB/BAM.MEM_MB,1) END)
		AS JENA_F,
	SQLITE.MEM_MB AS SQLITE_MEM,
	(CASE WHEN BAM.MEM_MB=0 THEN NULL
		ELSE ROUND(SQLITE.MEM_MB/BAM.MEM_MB,1) END)
		AS SQLITE_F
FROM	(SELECT * FROM BEST_RESULT WHERE PROG='bam'AND BENCH='tcff') BAM
	LEFT JOIN
	(SELECT * FROM BEST_RESULT WHERE PROG='xsb' AND BENCH='tcff') XSB
	ON XSB.FILE_ID = BAM.FILE_ID
	LEFT JOIN
	(SELECT * FROM BEST_RESULT WHERE PROG='yap' AND BENCH='tcff') YAP
	ON YAP.FILE_ID = BAM.FILE_ID
	LEFT JOIN
	(SELECT * FROM BEST_RESULT WHERE PROG='jena' AND BENCH='tcff') JENA
	ON JENA.FILE_ID = BAM.FILE_ID
	LEFT JOIN
	(SELECT * FROM BEST_RESULT WHERE PROG='sqlite3' AND BENCH='tcff') SQLITE
	ON SQLITE.FILE_ID = BAM.FILE_ID
	JOIN DATA_FILE F
        ON BAM.FILE_ID = F.FILE_ID
ORDER BY F.SEQ_NO;

-- ============================================================================
-- Benchmark Results for TCFF (Comparison of Various Systems on CPU Time):
-- ============================================================================

-- This looks only at the runtimes of the standard program version,
-- best implementation, on the standard machine.

CREATE VIEW TCFF_CPU_LATEX AS
SELECT	BAM.FILE_ID,
	RPAD(REPLACE(BAM.FILE_ID, '_', '\_'), 12) || '&' ||
	TO_CHAR(BAM.CPU_T, '9990.999') || '&' ||
	(CASE WHEN XSB.CPU_T IS NULL THEN '         &       ' ELSE
		(TO_CHAR(XSB.CPU_T, '9990.999') || '&' ||
		(CASE WHEN BAM.CPU_T=0 THEN '       '
			ELSE TO_CHAR(XSB.CPU_T/BAM.CPU_T, '9990.9') END))
	END) || '&' ||
	(CASE WHEN YAP.CPU_T IS NULL THEN '         &       ' ELSE
		(TO_CHAR(YAP.CPU_T, '9990.999') || '&' ||
		(CASE WHEN BAM.CPU_T=0 THEN '       '
			ELSE TO_CHAR(YAP.CPU_T/BAM.CPU_T, '9990.9') END))
	END) || '&' ||
	(CASE WHEN JENA.CPU_T IS NULL THEN '         &       ' ELSE
		(TO_CHAR(JENA.CPU_T,'9990.999') || '&' ||
		(CASE WHEN BAM.CPU_T=0 THEN '       '
			ELSE TO_CHAR(JENA.CPU_T/BAM.CPU_T, '9990.9') END))
	END) || '&' ||
	(CASE WHEN SQLITE.CPU_T IS NULL THEN '         &       ' ELSE
		(TO_CHAR(SQLITE.CPU_T,'9990.999') || '&' ||
		(CASE WHEN BAM.CPU_T=0 THEN '       '
			ELSE TO_CHAR(SQLITE.CPU_T/BAM.CPU_T, '9990.9') END))
	END) || '\\'
	AS LATEX
FROM	(SELECT * FROM BEST_RESULT WHERE PROG='bam'AND BENCH='tcff') BAM
	LEFT JOIN
	(SELECT * FROM BEST_RESULT WHERE PROG='xsb' AND BENCH='tcff') XSB
	ON XSB.FILE_ID = BAM.FILE_ID
	LEFT JOIN
	(SELECT * FROM BEST_RESULT WHERE PROG='yap' AND BENCH='tcff') YAP
	ON YAP.FILE_ID = BAM.FILE_ID
	LEFT JOIN
	(SELECT * FROM BEST_RESULT WHERE PROG='jena' AND BENCH='tcff') JENA
	ON JENA.FILE_ID = BAM.FILE_ID
	LEFT JOIN
	(SELECT * FROM BEST_RESULT WHERE PROG='sqlite3' AND BENCH='tcff') SQLITE
	ON SQLITE.FILE_ID = BAM.FILE_ID
	JOIN DATA_FILE F
	ON F.FILE_ID = BAM.FILE_ID
ORDER BY F.SEQ_NO;


-- ############################################################################
-- Graph Data in Various Formats:
-- ############################################################################


-- ============================================================================
-- Graph Data as LaTeX:
-- ============================================================================

CREATE VIEW GRAPH_DATA_LATEX AS
SELECT	G.FILE_ID,
	REPLACE(G.FILE_ID, '_', '\_') || '&' ||
	G.NUM_NODES || '&' || G.NUM_EDGES || '&' ||
--	G.MIN_IN_DEGREE || '&' || G.MAX_IN_DEGREE || '&' ||
--	G.MIN_OUT_DEGREE || '&' || G.MAX_OUT_DEGREE || '&' ||
        (CASE WHEN G.MIN_IN_DEGREE = G.MAX_IN_DEGREE
		THEN G.MIN_IN_DEGREE || '&'
		ELSE G.MIN_IN_DEGREE || '--' || G.MAX_IN_DEGREE || '&' END) ||
        (CASE WHEN G.MIN_OUT_DEGREE = G.MAX_OUT_DEGREE
		THEN G.MIN_OUT_DEGREE || '&'
		ELSE G.MIN_OUT_DEGREE || '--' || G.MAX_OUT_DEGREE || '&' END)||
	(CASE WHEN CYCLES='Y' THEN 'yes'
		WHEN CYCLES='N' THEN 'no'
		ELSE '?' END) || '&' ||
	G.TC_SIZE || '&' ||
		G.NUM_ITER || '&' ||
		G.COST || '\\'
		AS LATEX
FROM	GRAPH_DATA G, DATA_FILE F
WHERE   G.FILE_ID = F.FILE_ID
ORDER BY F.SEQ_NO;

-- ============================================================================
-- Graph Data as HTML:
-- ============================================================================

CREATE VIEW GRAPH_DATA_HTML AS
SELECT -2 AS SEQ_NO, NULL AS FILE_ID,
	'<table>' AS HTML
FROM    DUMMY_TAB
UNION ALL
SELECT -1 AS SEQ_NO, NULL AS FILE_ID,
	'<tr>' ||
	'<th class="colGraph">' || 'Graph'          || '</th>' || '
	' || '<th class="colNodes"   >' || 'Nodes'           || '</th>' || '
	' || '<th class="colEdges"   >' || 'Edges'           || '</th>' || '
	' || '<th class="colInDeg"   >' || 'In-Deg.'         || '</th>' || '
	' || '<th class="colOutDeg"  >' || 'Out-Deg.'        || '</th>' || '
	' || '<th class="colCyc"     >' || 'Cyc.'            || '</th>' || '
	' || '<th class="colTCSize"  >' || 'TC&nbsp;Size.'   || '</th>' || '
	' || '<th class="colIter"    >' || 'Iter.'           || '</th>' || '
	' || '<th class="colRuleInst">' || 'Rule&nbsp;Inst.' || '</th>' || '
	' || '</tr>'
	AS HTML
FROM    DUMMY_TAB
UNION ALL
SELECT	F.SEQ_NO, F.FILE_ID,
	'<tr>' ||
	'<td class="colGraph">' ||
		'<a href="data_tsv/' || F.FILENAME || '.tsv">' ||
		F.FILE_ID || '</a>' || '</td>' || '
	' || '<td class="colNodes"   >' || NUM_NODES || '</td>' || '
	' || '<td class="colEdges"   >' || NUM_EDGES || '</td>' || '
	' || '<td class="colInDeg"   >' ||
		(CASE WHEN G.MIN_IN_DEGREE = G.MAX_IN_DEGREE
			THEN CAST(G.MIN_IN_DEGREE AS VARCHAR)
			ELSE G.MIN_IN_DEGREE || '-' || G.MAX_IN_DEGREE END)
	                                              || '</td>' || '
	' || '<td class="colOutDeg"  >' ||
		(CASE WHEN G.MIN_OUT_DEGREE = G.MAX_OUT_DEGREE
			THEN CAST(G.MIN_OUT_DEGREE AS VARCHAR)
			ELSE G.MIN_OUT_DEGREE || '-' || G.MAX_OUT_DEGREE END)
	                                              || '</td>' || '
	' || '<td class="colCyc"     >' ||
		(CASE WHEN CYCLES='Y' THEN 'yes'
			WHEN CYCLES='N' THEN 'no'
			ELSE '?' END)                 || '</td>' || '
	' || '<td class="colTCSize"  >' || TC_SIZE   || '</td>' || '
	' || '<td class="colIter"    >' || NUM_ITER  || '</td>' || '
	' || '<td class="colRuleInst">' || COST      || '</td>' || '
	' || '</tr>'
	AS HTML
FROM	GRAPH_DATA G, DATA_FILE F
WHERE   G.FILE_ID = F.FILE_ID
UNION ALL
SELECT 1000000 AS SEQ_NO, NULL AS FILE_ID,
	'</table>' AS HTML
FROM    DUMMY_TAB
ORDER BY SEQ_NO;


-- ############################################################################
-- Support for Reasearch on Runtime Estimation:
-- ############################################################################


-- ============================================================================
-- System Speed in Real Time:
-- ============================================================================

CREATE VIEW SPEED_REAL AS
SELECT PROG, FILE_ID, ROUND((COST/REAL_T)/1000) as SPEED, COST, REAL_T
FROM   PLOT_REAL
WHERE  REAL_T > 0
ORDER  BY PROG, SPEED;

-- ============================================================================
-- System Speed in Real Time With Startup Time:
-- ============================================================================

CREATE VIEW SPEED_REAL_DIFF AS
SELECT P.PROG, P.FILE_ID,
       ROUND(((P.COST-S.COST)/(P.REAL_T-S.REAL_T))/1000) as SPEED,
       P.COST, P.REAL_T,
       (P.COST-S.COST) AS DIFF_COST, (P.REAL_T-S.REAL_T) AS DIFF_REAL_T
FROM   PLOT_REAL P, PLOT_REAL S
WHERE  P.REAL_T-S.REAL_T > 0
AND    P.PROG = S.PROG
AND    S.FILE_ID = 'm4_2ki'
ORDER  BY PROG, SPEED;

-- ============================================================================
-- System Speed in CPU Time:
-- ============================================================================

CREATE VIEW SPEED_CPU AS
SELECT PROG, FILE_ID, ROUND((COST/CPU_T)/1000) as SPEED, COST, CPU_T
FROM   PLOT_CPU
WHERE  CPU_T > 0
ORDER  BY PROG, SPEED;

-- ============================================================================
-- Memory Efficiency:
-- ============================================================================

CREATE VIEW MEM_EFFICIENCY AS
SELECT PROG, FILE_ID, ROUND(MEM_MB*1024*1024/FACTS) as BYTES_PER_FACT,
       FACTS, MEM_MB
FROM   PLOT_MEM
WHERE  FACTS > 0
ORDER  BY PROG, BYTES_PER_FACT;

-- ============================================================================
-- Compute Run Time Estimates (Real Time):
-- ============================================================================

CREATE VIEW ESTIMATE_FORMULA AS
SELECT R.PROG, E.STARTUP, E.LOAD_IDX, E.RULE_APP, E.RESULT,
       G.FILE_ID,
       ROUND(CAST(
         E.STARTUP +
         (G.NUM_EDGES * (LOG(G.NUM_EDGES)/LOG(2)) * E.LOAD_IDX / 1000000000) +
         (G.COST * E.RULE_APP / 1000000000) +
         (G.TC_SIZE * E.RESULT / 1000000000) AS NUMERIC),
       3) AS ESTIMATE,
       R.REAL_T
FROM   RESULT R, GRAPH_DATA G, ESTIMATE_PAR E
WHERE  R.PROG = E.PROG AND E.BENCH = 'tcff' AND R.BENCH = 'tcff'
AND    R.FILE_ID = G.FILE_ID
ORDER  BY R.PROG, E.STARTUP, E.LOAD_IDX, E.RULE_APP, E.RESULT, ESTIMATE;

-- ============================================================================
-- Check System Speed Estimates (Real Time):
-- ============================================================================

CREATE VIEW ESTIMATE_ERR AS
SELECT E.PROG, E.STARTUP, E.LOAD_IDX, E.RULE_APP, E.RESULT,
       E.FILE_ID, E.ESTIMATE, E.REAL_T,
       E.REAL_T - E.ESTIMATE AS DIFF,
       ROUND((E.REAL_T - E.ESTIMATE) * 100 / E.ESTIMATE) AS DIFF_PCT
FROM   ESTIMATE_FORMULA E
ORDER  BY E.PROG, E.STARTUP, E.LOAD_IDX, E.RULE_APP, E.RESULT, DIFF_PCT;

-- ============================================================================
-- Count the Number of Wrong Estimates (more than 2s wrong and more than 20%):
-- ============================================================================

CREATE VIEW NUM_BAD_ESTIMATES AS
SELECT PROG, STARTUP, LOAD_IDX, RULE_APP, RESULT,
	COUNT(*) AS BAD_ESTIMATES, SUM(ABS(DIFF)) AS ABS_ERR
FROM   ESTIMATE_ERR
WHERE  ABS(DIFF_PCT) > 20
GROUP  BY PROG, STARTUP, LOAD_IDX, RULE_APP, RESULT
ORDER  BY PROG, STARTUP, LOAD_IDX, RULE_APP, RESULT;

-- ============================================================================
-- Generate C++ Objects with Graph Data and Runtime (for Optimization):
-- ============================================================================

CREATE VIEW ESTIMATE_DATA AS
SELECT R.PROG, R.BENCH, R.IMPL,
       'data_c("' || G.FILE_ID || '", ' ||
		G.NUM_NODES || ', ' ||
		G.NUM_EDGES || ', ' ||
		'INT64_C(' || G.TC_SIZE || '), ' ||
		'INT64_C(' || G.COST    || '), ' ||
		G.NUM_ITER  || ', ' ||
		ROUND(R.REAL_T*1000)|| '),' AS OBJ
FROM	GRAPH_DATA G, DATA_FILE F, BEST_RESULT R
WHERE   G.FILE_ID = F.FILE_ID AND R.FILE_ID = F.FILE_ID
ORDER BY R.PROG, R.BENCH, F.SEQ_NO;

-- ============================================================================
-- Load time per million edges in s:
-- ============================================================================

CREATE VIEW LOAD_EFFICIENCY AS
SELECT R.PROG, R.BENCH, R.IMPL, R.LOAD_T, G.NUM_EDGES,
		ROUND(R.LOAD_T * 1000 * 1000 / G.NUM_EDGES, 3)
			AS LOAD_PER_M_EDGES
FROM    GRAPH_DATA G, DATA_FILE F, BEST_RESULT R
WHERE   G.FILE_ID = F.FILE_ID AND R.FILE_ID = F.FILE_ID
ORDER BY R.PROG, R.BENCH, F.SEQ_NO;

-- ============================================================================
-- Search Range for Parameter Values:
-- ============================================================================

CREATE VIEW PAR_RANGE AS
SELECT R.PROG, G.FILE_ID,
		ROUND(R.REAL_T * 1000 * 1000 / G.NUM_EDGES, 3) AS PAR_LOAD,
		ROUND(R.REAL_T * 1000 * 1000 / G.COST, 3)      AS PAR_RULE,
		ROUND(R.REAL_T * 1000 * 1000 / G.TC_SIZE, 3)   AS PAR_RESULT,
		ROUND(R.REAL_T * 1000 * 1000 / G.NUM_NODES, 3) AS PAR_DATA,
		ROUND(R.REAL_T * 1000 * 1000 / G.NUM_ITER, 3)  AS PAR_ITER
FROM    GRAPH_DATA G, DATA_FILE F, BEST_RESULT R
WHERE   G.FILE_ID = F.FILE_ID AND R.FILE_ID = F.FILE_ID
AND     R.BENCH = 'tcff'
ORDER BY R.PROG, R.BENCH, F.SEQ_NO;



-- ############################################################################
-- Output Table Data in Markdown Format (for Documentation)
-- ############################################################################


-- ============================================================================
-- BENCH_IMPL_MD:
-- ============================================================================

CREATE VIEW BENCH_IMPL_MD AS
SELECT 0 AS SECTION, ' ' AS PROG, ' ' AS BENCH, ' ' AS IMPL,
	'|PROG|BENCH|IMPL|STATUS|DESCRIPTION|' AS MD
UNION ALL
SELECT 1 AS SECTION, ' ' AS PROG, ' ' AS BENCH, ' ' AS IMPL,
	'|----|-----|----|------|-----------|' AS MD
UNION ALL
SELECT 2 AS LINE, PROG, BENCH, IMPL,
       '|' || PROG || '|' ||  BENCH || '|' || IMPL || '|' || STATUS ||
	'|' || DESCRIPTION || '|' AS MD
FROM   BENCH_IMPL
ORDER BY SECTION, PROG, BENCH, IMPL;

-- ============================================================================
-- DATA_FILE_MD:
-- ============================================================================

CREATE VIEW DATA_FILE_MD AS
SELECT 0 AS SECTION, 0 AS SEQ_NO,
	'|FILE_ID|SEQ_NO|ITYPE|LINES|BYTES|FILENAME|' AS MD
UNION ALL
SELECT 1 AS SECTION, 0 AS SEQ_NO,
	'|-------|-----:|-----|----:|----:|--------|' AS MD
UNION ALL
SELECT 2 AS SECTION, SEQ_NO,
       '|' || FILE_ID || '|' ||  SEQ_NO || '|' || ITYPE || '|' || LINES ||
	'|' || BYTES || '|' || FILENAME || '|' AS MD
FROM   DATA_FILE
ORDER BY SECTION, SEQ_NO;


-- ============================================================================
-- GRAPH_DATA_MD:
-- ============================================================================

CREATE VIEW GRAPH_DATA_MD AS
SELECT 0 AS SECTION, 0 AS SEQ_NO,
	'|FILE_ID|' ||
	'GTYPE|PARAM_1|PARAM_2|' ||
	'NUM_NODES|NUM_EDGES|' ||
	'DUP_EDGES|LOOPS|' ||
	'MIN_IN_DEGREE|MAX_IN_DEGREE|' ||
	'MIN_OUT_DEGREE|MAX_OUT_DEGREE|' ||
	'CYCLES|TC_SIZE|NUM_ITER|COST|' AS MD
UNION ALL
SELECT 1 AS SECTION, 0 AS SEQ_NO,
	'|-------|' ||
	'-----|------:|------:|' ||
	'--------:|--------:|' ||
	'--------:|:---:|' ||
	'------------:|------------:|' ||
	'-------------:|-------------:|' ||
	':----:|------:|-------:|---:|' AS MD
UNION ALL
SELECT 2 AS SECTION, F.SEQ_NO,
       '|' || G.FILE_ID || '|' ||
	GTYPE || '|' ||
	(CASE WHEN PARAM_1 IS NULL THEN ' ' ELSE CAST(PARAM_1 AS VARCHAR) END)
        || '|' ||
	(CASE WHEN PARAM_2 IS NULL THEN ' ' ELSE CAST(PARAM_2 AS VARCHAR) END)
	|| '|' ||
	NUM_NODES || '|' ||  NUM_EDGES || '|' ||
	DUP_EDGES || '|' || LOOPS || '|' ||
	MIN_IN_DEGREE || '|' || MAX_IN_DEGREE || '|' ||
	MIN_OUT_DEGREE || '|' || MAX_OUT_DEGREE || '|' ||
	CYCLES || '|' || TC_SIZE || '|' || NUM_ITER || '|' || COST || '|' AS MD
FROM   GRAPH_DATA G, DATA_FILE F
WHERE  G.FILE_ID = F.FILE_ID
ORDER BY SECTION, SEQ_NO;


-- ############################################################################
-- Support for Benchmark Execution Scripts:
-- ############################################################################

-- ============================================================================
-- BENCH_FILE_LIST: For RUN_BENCH Scripts
-- ============================================================================

-- This produces the benchmark file list for use in the RUN_BENCH scripts.
-- The result is to be used in a "case" switch in the bash script.

CREATE VIEW BENCH_FILE_LIST AS
SELECT '' AS FILE_ID, '' AS ITYPE, T.SEQ_NO AS TNO, -2 AS FNO,
	'' AS DATA_P,
	'' AS DATA_TSV
FROM INPUT_TYPE T
UNION ALL
SELECT '' AS FILE_ID, '' AS ITYPE, T.SEQ_NO AS TNO, -1 AS FNO,
	'    ' || '# ' || T.HEADLINE || ':' AS DATA_P,
	'    ' || '# ' || T.HEADLINE || ':' AS DATA_TSV
FROM INPUT_TYPE T
UNION ALL
SELECT F.FILE_ID, F.ITYPE, T.SEQ_NO AS TNO, F.SEQ_NO AS FNO,
	'    ' || RPAD((F.FILE_ID || ')'), 11) ||
	' FTYPE=' || RPAD((F.ITYPE || ';'), 11) ||
	'FILE=../data_p/' || F.FILENAME || '.P' || ' ;;' AS DATA_P,
	'    ' || RPAD((F.FILE_ID || ')'), 9) ||
	' FTYPE=' || RPAD((F.ITYPE || ';'), 9) ||
	'FILE=../data_tsv/' || F.FILENAME || '.tsv' || ' ;;' AS DATA_TSV
FROM DATA_FILE F, INPUT_TYPE T
WHERE F.ITYPE = T.ITYPE
ORDER BY TNO, FNO;

-- Note that RPAD cuts off strings that are too long (in PostgreSQL).
-- This is only safe because FILE_ID is declared as VARCHAR(10)
-- and ITYPE as VARCHAR(8).

-- ============================================================================
-- TC_FILE_IDs: For RUN_BENCH_LIST Scripts
-- ============================================================================

CREATE VIEW TC_FILE_IDS AS
SELECT -1 AS SEQ_NO, ' ' AS GTYPE,
	'TC_FILE_IDS="\' AS FILE_IDS
UNION ALL
SELECT T.SEQ_NO, T.GTYPE,
	CHR(9) || STRING_AGG(G.FILE_ID, ' ') || ' ' || '\' AS FILE_IDS
FROM GRAPH_DATA G, GRAPH_TYPE T
WHERE G.GTYPE = T.GTYPE
GROUP BY T.SEQ_NO, T.GTYPE
UNION ALL
SELECT 1000000 AS SEQ_NO, ' ' AS GTYPE,
	CHR(9) || '"' AS FILE_IDS
ORDER BY SEQ_NO;

-- CHR(9) is a TAB character.
-- PostgreSQL ignores literal TAB characters on input.

-- For MySQL, replace
--	STRING_AGG(G.FILE_ID, ' ')
-- by
--	GROUP_CONCAT((G.FILE_ID SEPARATOR ' '))

