-- ============================================================================
-- Project:	Deductive Database
-- Filename:	bench_db.sql
-- Purpose:	Database for Benchmark Runs
-- Last Change:	20.06.2018
-- Language:	SQL (Tested with PostgreSQL)
-- Author:	Stefan Brass
-- EMail:	brass@informatik.uni-halle.de
-- WWW:		http://www.informatik.uni-halle.de/~brass/
-- Address:	Feldschloesschen 15, D-06120 Halle (Saale), GERMANY
-- Copyright:	(c) 2016-2018 by Stefan Brass
-- License:	See file "LICENSE" for copying conditions.
-- Note:	There is no warranty at all - this code may contain bugs.
-- ============================================================================


-- ============================================================================
-- Delete Old Versions of the Tables:
-- ============================================================================

-- When this script is executed the first time,
-- these commands generate errors (because the tables/views do not exist).

DROP VIEW  PAR_RANGE;
DROP VIEW  LOAD_EFFICIENCY;
DROP VIEW  ESTIMATE_DATA;

DROP VIEW  NUM_BAD_ESTIMATES;
DROP VIEW  ESTIMATE_ERR;
DROP VIEW  SPEED_REAL;
DROP VIEW  SPEED_REAL_DIFF;
DROP VIEW  SPEED_CPU;
DROP VIEW  MEM_EFFICIENCY;

DROP VIEW  PCT_CPU;
DROP VIEW  PCT_CPU_INPUT;
DROP VIEW  REAL_TOO_LARGE;
DROP VIEW  OUTLIER_MEM;
DROP VIEW  OUTLIER_CPU;
DROP VIEW  OUTLIER_REAL;

DROP VIEW  GRAPH_DATA_LATEX;

DROP VIEW  TCFF_CPU;
DROP VIEW  TCFF_REAL;
DROP VIEW  TCFF_MEM;
DROP VIEW  TCFF_REAL_LATEX;
DROP VIEW  TCFF_CPU_LATEX;
DROP VIEW  PLOT_MEM;
DROP VIEW  PLOT_CPU;
DROP VIEW  PLOT_REAL;

DROP VIEW  BEST_IMPL_CPU;
DROP VIEW  BEST_IMPL_REAL;

DROP VIEW  XSB;
DROP VIEW  BAM;
DROP VIEW  YAP;
DROP VIEW  JENA;
DROP VIEW  DATASCRIPT;
DROP VIEW  SQLITE;

DROP VIEW  BEST_RESULT;
DROP VIEW  RESULT;
DROP VIEW  RUN_STATISTICS;
DROP VIEW  RUN_STAT;

DROP TABLE SPEED_ESTIMATE;

DROP TABLE BENCH_RUN;
DROP TABLE BENCH_IMPL;
DROP TABLE BENCH_INPUT;
DROP TABLE GRAPH_DATA;
DROP TABLE DATA_FILE;
DROP TABLE PROG_VER;
DROP TABLE PROGRAM;
DROP TABLE BENCHMARK;
DROP TABLE MACHINE;

-- ============================================================================
-- List of Machines on which Benchmarks are Executed:
-- ============================================================================

CREATE TABLE MACHINE(
	MACHINE     VARCHAR(20)  NOT NULL,
	SEQ_NO      NUMERIC(2)   NOT NULL,
	STATUS      CHAR(1)      NOT NULL,
        MODEL       VARCHAR(60)  NULL,
	YEAR        NUMERIC(4)   NULL,
        MONTH       NUMERIC(2)   NULL,
        PRICE       NUMERIC(7)   NULL,
	NUM_CPUS    NUMERIC(4)   NOT NULL,
	CPU         VARCHAR(60)  NOT NULL,
	CPU_CORES   NUMERIC(4)   NULL,
        CPU_THREADS NUMERIC(4)   NULL,
	GHZ         NUMERIC(5,2) NULL,
	ARCH        VARCHAR(20)  NULL,
	RAM         NUMERIC(8,3) NOT NULL,
	NUM_DISKS   NUMERIC(4)   NULL,
        DISK        VARCHAR(40)  NULL,
	SSD         CHAR(1)      NULL,
	OS          VARCHAR(40)  NULL,
	OS_CORE     VARCHAR(40)  NULL,
	CONSTRAINT MACHINE_PK
		PRIMARY KEY(MACHINE),
	CONSTRAINT MACHINE_KEY_SEQ
		UNIQUE (SEQ_NO),
	CONSTRAINT MACHINE_SEQ_NO_POSITIVE
		CHECK(SEQ_NO > 0),
        CONSTRAINT MACHINE_YEAR_VALID
		CHECK(YEAR >= 2000),
        CONSTRAINT MACHINE_MONTH_VALID
                CHECK(MONTH >= 1 AND MONTH <= 12),
        CONSTRAINT MACHINE_PRICE_NON_NEGATIVE
                CHECK(PRICE >= 0),
	CONSTRAINT MACHINE_STATUS_S_OR_SPACE
		CHECK(STATUS IN ('S', ' ')),
		-- The machine with STATUS='S' is the standard machine.
		-- Only one row of the table should have STATUS='S'.
		-- Some views select only benchmark results on the standard
		-- machine.
		-- In future, there many be more STATUS values, e.g. for
		-- comparing the standard machine to an alternative machine.
	CONSTRAINT MACHINE_NUM_CPUS_POSITIVE
		CHECK(NUM_CPUS > 0),
	CONSTRAINT MACHINE_CPU_CORES_POSITIVE
		CHECK(CPU_CORES > 0),
	CONSTRAINT MACHINE_CPU_THREADS_POSITIVE
		CHECK(CPU_THREADS > 0),
	CONSTRAINT MACHINE_RAM_POSITIVE
		CHECK(RAM > 0),
	CONSTRAINT MACHINE_NUM_DISKS_POSITIVE
		CHECK(NUM_DISKS > 0),
	CONSTRAINT MACHINE_SSD_YN
		CHECK(SSD IN ('Y','N')));

INSERT INTO MACHINE VALUES(
	'bach', 1, ' ',
	'silentmaxx Workstation KENKO Fanless Xeon E5',
	2017, 12, 1933,
	1, 'Xeon Intel 2011 E5-1620v4 4/8x 3,5GHz', 4, 8, 3.5, 'x86_64',
	16,
	1, 'Samsung SSD 850 EVO 500GB', 'Y',
	'CentOS Linux release 7.4.1708 (Core)',
	'Linux 3.10.0-693.21.1.el7.x86_64 x86_64');

INSERT INTO MACHINE VALUES(
	'dbs1', 2, 'S',
	'HP Blade Server, HPE BL460c Gen9',
        2016, 10, 3319,
	2, 'Intel(R) Xeon(R) CPU E5-2630 v4 @ 2.20GHz', 10, 20, 2.2, 'x86_64',
	64,
	NULL, NULL, NULL,
	'Debian GNU/Linux 8.10 (jessie)',
	'Linux 3.16.0-5-amd64 x86_64');

-- Note that the CORE might not be current. It changes with every update.
-- It was current when we started the benchmarks in May 2018.
-- It gives some impression of the machine and its operating system.

-- ============================================================================
-- List of Benchmarks:
-- ============================================================================


CREATE TABLE BENCHMARK(
	BENCH      VARCHAR(20)  NOT NULL,
	SEQ_NO     NUMERIC(2)   NOT NULL,
	HEADLINE   VARCHAR(80)  NOT NULL,
	CONSTRAINT BENCHMARK_SEQ_NONNEGATIVE
		CHECK(SEQ_NO >= 0),
	CONSTRAINT BENCHMARK_PK
		PRIMARY KEY (BENCH),
	CONSTRAINT BENCHMARK_KEY_SEQ
		UNIQUE (SEQ_NO));

INSERT INTO BENCHMARK VALUES (
	'dblp', 1,
	'DBLP Benchmark');

INSERT INTO BENCHMARK VALUES (
	'j1axy', 2,
	'The Join1 Benchmark, Query a(X,Y)');

INSERT INTO BENCHMARK VALUES (
	'j1axyd', 3,
	'The Join1 Benchmark (without duplicate elimination)');

INSERT INTO BENCHMARK VALUES (
	'tcff', 4,
	'Transitive Closure Benchmark without Query Bindings');

INSERT INTO BENCHMARK VALUES (
	'tcbf', 5,
	'Transitive Closure Benchmark with the First Argument Bound');

INSERT INTO BENCHMARK VALUES (
	'tcbfs', 6,
	'Transitive Closure Benchmark, first arg bound, with SLDMagic');

INSERT INTO BENCHMARK VALUES (
	'wine', 7,
	'Wine Ontology Benchmark');

INSERT INTO BENCHMARK VALUES (
	'loadd', 8,
	'Test of Loader for DBLP Data');

INSERT INTO BENCHMARK VALUES (
	'loadt', 9,
	'Test of Loader for Transitive Closure');

INSERT INTO BENCHMARK VALUES (
	'set1', 10,
	'Test of Set Insertion and Lookup (One Column)');

INSERT INTO BENCHMARK VALUES (
	'set2', 11,
	'Test of Set Insertion and Lookup (Two Columns)');

-- ============================================================================
-- List of Programs to be Tested/Compared:
-- ============================================================================

CREATE TABLE PROGRAM(
	PROG        VARCHAR(20)  NOT NULL,
	SEQ_NO      NUMERIC(2)   NOT NULL,
	FULL_NAME   VARCHAR(40)  NOT NULL,
	URL         VARCHAR(80)  NULL,
	REMARK      VARCHAR(800) NULL,
	CONSTRAINT PROGRAM_SEQ_NONNEGATIVE
		CHECK(SEQ_NO >= 0),
	CONSTRAINT PROGRAM_PK
		PRIMARY KEY (PROG),
	CONSTRAINT PROGRAM_KEY_SEQ
		UNIQUE (SEQ_NO));

INSERT INTO PROGRAM VALUES (
	'ydb', 1, 'Push (ydb)',
	'http://users.informatik.uni-halle.de/~brass/push/',
	NULL);

INSERT INTO PROGRAM VALUES (
	'bam', 2, 'Push (bam)',
	'http://users.informatik.uni-halle.de/~brass/push/',
	NULL);

INSERT INTO PROGRAM VALUES (
	'xsb', 3, 'XSB',
	'http://xsb.sourceforge.net/',
	NULL);

INSERT INTO PROGRAM VALUES (
	'yap', 4, 'YAProlog',
	'https://www.dcc.fc.up.pt/~vsc/Yap/',
	NULL);

INSERT INTO PROGRAM VALUES (
	'souffle', 5, 'Souffle',
	'http://souffle-lang.org/',
	NULL);

INSERT INTO PROGRAM VALUES (
	'dlv', 6, 'DLV',
	'http://www.dlvsystem.com/',
	NULL);

INSERT INTO PROGRAM VALUES (
	'mercury', 7, 'Melbourne Mercury',
	'http://www.mercurylang.org/',
	NULL);

INSERT INTO PROGRAM VALUES (
	'hsql', 8, 'HSQLDB',
	'http://hsqldb.org/',
	NULL);

INSERT INTO PROGRAM VALUES (
	'swipl', 9, 'SWI-Prolog',
	'http://www.swi-prolog.org/',
	NULL);

INSERT INTO PROGRAM VALUES (
	'jena', 10, 'Apache Jena',
	'https://jena.apache.org/',
	NULL);

INSERT INTO PROGRAM VALUES (
	'datascript', 11, 'Datascript',
	'https://github.com/tonsky/datascript',
	NULL);

INSERT INTO PROGRAM VALUES (
	'sqlite3', 12, 'SQLite 3',
	'https://www.sqlite.org/index.html',
	NULL);

INSERT INTO PROGRAM VALUES (
	'mariadb', 13, 'MariaDB',
	'https://www.mariadb.com',
	NULL);

INSERT INTO PROGRAM VALUES (
	'psql', 14, 'PostgreSQL',
	'https://www.postgresql.org',
	NULL);

-- ============================================================================
-- Table for Installed Program Versions on Machines:
-- ============================================================================

-- This table lists program versions
-- that are or were installed on a test machine.
-- They are identified here by a number that is unique together with
-- the program name and the machine.
-- These three elements (Program, Version number, and machine)
-- are also encoded in the file names with the test results.
--
-- Note that we use a simple sequential number VER for the version in the key,
-- not the real version number (which can be an arbitrary string).
-- The real version number is available in column VER_NAME
-- and can be used for output.
--
-- I try to make sure that the same VER_NAME of a program gets the same VER
-- number even on different machines, but the database schema does not
-- require this.
-- It is possible that a VER_NAME corresponds to different VER numbers,
-- for instance, if a program is compiled with different compilers or
-- compiler settings. This would require a new VER number.
--
-- Only program versions that are listed here can appear in the test runs.
--
-- This table also define a current version of a program on a machine
-- (by STATUS='C').  Some views show only results for the current version.
-- It is also possible to define a previous version (STATUS='P') as reference,
-- if the result of a program update should be evaluated.
-- Finally, one can mark an alternative version (STATUS='A'),
-- e.g. compiled with a different compiler or compiler options,
-- which can also be used for comparison.
--
-- VER_ADDTEXT should be concatenated with VER_NAME
-- to get the full name of the version.
-- This table also permits to store additional information about a version.
-- For instance, XSB outputs configuration information when started.

CREATE TABLE PROG_VER (
	PROG        VARCHAR(20)  NOT NULL,
	VER         NUMERIC(4)   NOT NULL,
	MACHINE     VARCHAR(20)  NOT NULL,
	VER_NAME    VARCHAR(20)  NOT NULL,
	VER_ADDTEXT VARCHAR(80)  NOT NULL,
	VER_DATE    DATE         NULL,
	COMPILER    VARCHAR(80)  NOT NULL,
	STATUS      CHAR(1)      NOT NULL,
	REMARK      VARCHAR(800) NULL,
	CONSTRAINT PROG_VER_PK
		PRIMARY KEY (PROG, VER, MACHINE),
	CONSTRAINT PROG_VER_REF_PROGRAM
		FOREIGN KEY (PROG) REFERENCES PROGRAM,
	CONSTRAINT PROG_VER_REF_MACHINE
		FOREIGN KEY (MACHINE) REFERENCES MACHINE,
	CONSTRAINT PROG_VER_STATUS_VALID
		CHECK(STATUS IN ('C','P','A',' ')));
	-- 'C': Current version
	-- 'P': Previous version
	-- 'A': Alternate version
	-- ' ': Any other version
	-- Views may select only benchmark results for the current version,
	-- or help to compare the current with the previous version.

INSERT INTO PROG_VER VALUES (
	'ydb', 1, 'bach',
	'0.1', '', '2016-11-21',
	'gcc version 4.8.5 (-O2)',
	'C',
	NULL);

INSERT INTO PROG_VER VALUES (
	'ydb', 1, 'dbs1',
	'0.1', '', '2016-11-21',
	'gcc version 4.9.2 (-O2)',
	'C',
	NULL);

INSERT INTO PROG_VER VALUES (
	'bam', 1, 'bach',
	'0.1a', '', '2018-04-24',
	'gcc version 4.8.5 (-O2)',
	'C',
	NULL);

INSERT INTO PROG_VER VALUES (
	'bam', 1, 'dbs1',
	'0.1a', '', '2018-04-24',
	'gcc version 4.9.2 (-O2)',
	'C',
	NULL);

INSERT INTO PROG_VER VALUES (
	'xsb', 1, 'dbs1',
	'3.7.', ' (Clan MacGregor)', '2016-07-06',
	'',
	'C',
	'x86_64-unknown-linux-gnu 64 bits; ' ||
		'mode: optimal; engine: slg-wam; scheduling: local');

INSERT INTO PROG_VER VALUES (
	'xsb', 2, 'dbs1',
	'3.8.0', ' (Three-Buck Chuck)', '2017-10-28',
	'',
	'C',
	'x86_64-unknown-linux-gnu 64 bits; ' ||
		'mode: optimal; engine: slg-wam; scheduling: local');

INSERT INTO PROG_VER VALUES (
	'xsb', 2, 'bach',
	'3.8.0', ' (Three-Buck Chuck)', '2017-10-28',
	'',
	'C',
	'x86_64-unknown-linux-gnu 64 bits; ' ||
		'mode: optimal; engine: slg-wam; scheduling: local');

INSERT INTO PROG_VER VALUES (
	'yap', 1, 'dbs1',
	'6.2.2', ' (x86_64-linux)', '2011-10-21',
	'',
	'C',
	NULL);

INSERT INTO PROG_VER VALUES (
	'yap', 2, 'bach',
	'6.3.3', ' (x86_64-linux)', '2013-01-20',
	'',
	'C',
	NULL);

INSERT INTO PROG_VER VALUES (
	'swipl', 1, 'dbs1',
	'7.7.15', ' (x86_64-linux)', '2018-06-01',
	'',
	'C',
	NULL);

INSERT INTO PROG_VER VALUES (
	'swipl', 2, 'dbs1',
	'7.5.15', ' (x86_64-linux)', '2017-09-12',
	'',
	'P',
	NULL);

INSERT INTO PROG_VER VALUES (
	'swipl', 3, 'dbs1',
	'7.6.4', ' (x86_64-linux)', '2018-01-12',
	'',
	'C',
	NULL);

INSERT INTO PROG_VER VALUES (
	'swipl', 4, 'dbs1',
	'7.7.7', ' (x86_64-linux)', '2017-12-21',
	'',
	'C',
	NULL);

INSERT INTO PROG_VER VALUES (
	'jena', 1, 'dbs1',
	'3.7.0', '', '2018-04-05',
	'openJDK 8',
	'P',
	NULL);

INSERT INTO PROG_VER VALUES (
	'jena', 2, 'dbs1',
	'3.8.0', '', '2018-06-25',
	'openJDK 8',
	'C',
	NULL);

INSERT INTO PROG_VER VALUES (
	'datascript', 1, 'dbs1',
	'0.16.6', '', '2018-06-06',
	'node 10.5.0',
	'C',
	NULL);

INSERT INTO PROG_VER VALUES (
	'sqlite3', 1, 'dbs1',
	'3.8.7.1', '', '2014-10-29',
	'',
	'P',
	NULL);

INSERT INTO PROG_VER VALUES (
	'sqlite3', 2, 'dbs1',
	'3.22.0', '', '2018-01-22',
	'',
	'P',
	NULL);
INSERT INTO PROG_VER VALUES (
	'sqlite3', 3, 'dbs1',
	'3.24.0', '', '2018-06-04',
	'',
	'P',
	NULL);
INSERT INTO PROG_VER VALUES (
	'sqlite3', 4, 'dbs1',
	'3.25.1', '', '2018-09-18',
	'',
	'P',
	NULL);

INSERT INTO PROG_VER VALUES (
	'sqlite3', 5, 'dbs1',
	'3.25.2', '', '2018-09-25',
	'',
	'C',
	NULL);

INSERT INTO PROG_VER VALUES (
	'psql', 1, 'dbs1',
	'9.4.15', '', '2017-11-09',
	'',
	'C',
	NULL);

INSERT INTO PROG_VER VALUES (
	'mariadb', 1, 'dbs1',
	'10.0.35', '', '2018-05-03',
	'',
	'P',
	NULL);

INSERT INTO PROG_VER VALUES (
	'mariadb', 1, 'dbs1',
	'10.3.9', '', '2018-08-15',
	'',
	'P',
	NULL);

-- ============================================================================
-- Table for Implementation Variants of Benchmarks:
-- ============================================================================

CREATE TABLE BENCH_IMPL(
	PROG        VARCHAR(20)  NOT NULL,
	BENCH       VARCHAR(20)  NOT NULL,
	IMPL        VARCHAR(20)  NOT NULL,
	STATUS      CHAR(1)      NOT NULL,
	DESCRIPTION VARCHAR(80)  NULL,
	CONSTRAINT BENCH_IMPL_PK
		PRIMARY KEY (PROG, BENCH, IMPL),
	CONSTRAINT BENCH_IMPL_REF_BENCHMARK
		FOREIGN KEY (BENCH) REFERENCES BENCHMARK,
	CONSTRAINT BENCH_IMPL_REF_PROGRAM
		FOREIGN KEY (PROG) REFERENCES PROGRAM,
	CONSTRAINT BENCH_IMPL_STATUS_VALID
		CHECK(STATUS IN ('B','A',' ')));

-- STATUS='B' marks the implementation variant (option settings)
-- that is considered best and is used in comparisons between systems.
-- One can use the views BEST_IMPL_REAL and BEST_IMPL_CPU to check
-- whether these implementation variants are really best.
-- Sometimes the best implementation depends on the specific data file,
-- but it would somehow be unfair to adapt the options to the data file.

-- STATUS='A' marks an interesting alternative variants
-- that appear in some result tables.
-- There should be only one "best" version,
-- but there can be several alterative versions.

-- STATUS=' ' means that the variant was tested
-- in order to prove that the best version is really good,
-- but it will not be listed in summary tables.

-- ----------------------------------------------------------------------------
-- Benchmark Implementations for bam:
-- ----------------------------------------------------------------------------

-- Join1 Implementations for bam:

INSERT INTO BENCH_IMPL VALUES (
	'bam', 'j1axy', '1', ' ',
	'Join1, a(X,Y) [V1: RelAttr, -goto, fullDupCh]');

INSERT INTO BENCH_IMPL VALUES (
	'bam', 'j1axy', '2', ' ',
	'Join1, a(X,Y) [V2: no goto, bitmap-dupcheck]');

INSERT INTO BENCH_IMPL VALUES (
	'bam', 'j1axy', '3', 'B',
	'Join1, a(X,Y) [V3: goto, bitmap-dupC, cursor]');

INSERT INTO BENCH_IMPL VALUES (
	'bam', 'j1axy', '4', 'A',
	'Join1, a(X,Y) [V4: functions, bitmap-dupC]');

INSERT INTO BENCH_IMPL VALUES (
	'bam', 'j1axy', '5', ' ',
	'Join1, a(X,Y) [V5: fun without args, bitmap]');

INSERT INTO BENCH_IMPL VALUES (
	'bam', 'j1axy', '6', 'A',
	'Join1, a(X,Y) [V6: Abstract Machine, bitmap]');

-- Transitive Closure Implementations for bam (TCFF - Both Arguments Free):

INSERT INTO BENCH_IMPL VALUES (
	'bam', 'tcff', '1', ' ',
	'tc(X,Y) [V1: RelLocalVar]');

INSERT INTO BENCH_IMPL VALUES (
	'bam', 'tcff', '2', 'A',
	'tc(X,Y) [V2: Cursor Interface, switch]');

INSERT INTO BENCH_IMPL VALUES (
	'bam', 'tcff', '3', 'A',
	'tc(X,Y) [V3: Seminaive Evaluation]');

INSERT INTO BENCH_IMPL VALUES (
	'bam', 'tcff', '4', 'A',
	'tc(X,Y) [V4: Functions]');

	-- Not really best, but this is the BAM:
INSERT INTO BENCH_IMPL VALUES (
	'bam', 'tcff', '5', 'B',
	'tc(X,Y) [V5: Abstract Machine]');

-- Data Loader Tests for bam:

INSERT INTO BENCH_IMPL VALUES (
	'bam', 'loadt', '1', 'A',
	'transitive closure data file (with handcrafted loader)');

INSERT INTO BENCH_IMPL VALUES (
	'bam', 'loadt', '2', 'B',
	'transitive closure data file (with general loader)');

-- Set Implementation Tests for bam (One Column):

INSERT INTO BENCH_IMPL VALUES (
	'bam', 'set1', '1', 'B',
	'Set with 1 column, extendible Hash table');

-- Set Implementation Tests for bam (Two Columns):

INSERT INTO BENCH_IMPL VALUES (
	'bam', 'set2', '1', 'B',
	'Set with 2 columns, extendible Hash table');

-- ----------------------------------------------------------------------------
-- Benchmark Implementations for XSB:
-- ----------------------------------------------------------------------------

-- DBLP Benchmark for XSB:

INSERT INTO BENCH_IMPL VALUES (
	'xsb', 'dblp', 'dyn', 'A',
	'load_dyn, no index');

INSERT INTO BENCH_IMPL VALUES (
	'xsb', 'dblp', 'dync', 'A',
	'load_dync, no index');

INSERT INTO BENCH_IMPL VALUES (
	'xsb', 'dblp', 'dynin', 'A',
	'load_dyn, index on col 1+2');

INSERT INTO BENCH_IMPL VALUES (
	'xsb', 'dblp', 'dyncin', 'B',
	'load_dync, index on col 1+2');

INSERT INTO BENCH_IMPL VALUES (
	'xsb', 'dblp', 'dyntr', 'A',
	'load_dyn, trie index');

INSERT INTO BENCH_IMPL VALUES (
	'xsb', 'dblp', 'dynctr', 'A',
	'load_dync, trie index');

-- Join1 Benchmark for XSB:

INSERT INTO BENCH_IMPL VALUES (
	'xsb', 'j1axyd', 'none', 'B',
	'Join1 with duplicates, a(X,Y) [V0: no tabling]');

INSERT INTO BENCH_IMPL VALUES (
	'xsb', 'j1axy', 'a', ' ',
	'Join1, a(X,Y) [V1: only a tabled]');

INSERT INTO BENCH_IMPL VALUES (
	'xsb', 'j1axy', 'all', ' ',
	'Join1, a(X,Y) [V2: all derived preds tabled (subsumptive)]');

INSERT INTO BENCH_IMPL VALUES (
	'xsb', 'j1axy', 'allv', 'B',
	'Join1, a(X,Y) [V4: all derived preds tabled, variant tabled]');

INSERT INTO BENCH_IMPL VALUES (
	'xsb', 'j1axy', 'alli', ' ',
	'Join1, a(X,Y) [V4: all derived preds tabled, with indexes]');

INSERT INTO BENCH_IMPL VALUES (
	'xsb', 'j1axy', 'allt', 'A',
	'Join1, a(X,Y) [V5: all derived preds tabled, with trie index]');

-- TCFF Benchmark for XSB:

INSERT INTO BENCH_IMPL VALUES (
	'xsb', 'tcff', 'dyn', ' ',
	'tc(X,Y) [V1: load_dyn, trie-index for par]');

INSERT INTO BENCH_IMPL VALUES (
	'xsb', 'tcff', 'dync', 'B',
	'tc(X,Y) [V2: load_dync, trie-index for par]');

-- TCBF Benchmark for XSB:

INSERT INTO BENCH_IMPL VALUES (
	'xsb', 'tcbf', 'dyn', ' ',
	'tc(1,X) [V1: load_dyn, trie-index for par]');

INSERT INTO BENCH_IMPL VALUES (
	'xsb', 'tcbf',  'dync', 'B',
	'tc(1,X) [V2: load_dync, trie-index for par]');

INSERT INTO BENCH_IMPL VALUES (
	'xsb', 'tcbfs', 'slddc', 'B',
	'tc(1,X) [V3: SLDMagic, load_dync, trie-index]');

-- Wine Ontology Benchmark for XSB:

INSERT INTO BENCH_IMPL VALUES (
	'xsb', 'wine', 'is', 'B',
	'wine ontology [V1: load_dync, subsumptive tabling, std. indexes]');


-- ----------------------------------------------------------------------------
-- Benchmark Implemenations for YAP:
-- ----------------------------------------------------------------------------

-- DBLP Benchmark for YAP:

INSERT INTO BENCH_IMPL VALUES (
	'yap', 'dblp', '1', 'B',
	'DBLP Benchmark');

-- Join1 Benchmark for YAP:

INSERT INTO BENCH_IMPL VALUES (
	'yap', 'j1axyd', 'none', 'B',
	'Join1 with duplicates, a(X,Y) [V0: no tabling]');

INSERT INTO BENCH_IMPL VALUES (
	'yap', 'j1axy', 'a', ' ',
	'Join1, a(X,Y) [V1: only a tabled]');

INSERT INTO BENCH_IMPL VALUES (
	'yap', 'j1axy', 'all', 'B',
	'Join1, a(X,Y) [V2: all derived preds tabled]');

-- TCFF Benchmark for YAP:

INSERT INTO BENCH_IMPL VALUES (
	'yap', 'tcff', 'tab', 'B',
	'tc(X,Y) [V1: tc tabled]');

-- TBFF Benchmark for YAP:

INSERT INTO BENCH_IMPL VALUES (
	'yap', 'tcbf', 'tab', 'B',
	'tc(1,X) [V1: tc tabled]');

INSERT INTO BENCH_IMPL VALUES (
	'yap', 'tcbfs', 'sldm', 'B',
	'tc(1,X) [V2: with SLDMagic, all preds tabled]');

-- Wine Ontology Benchmark for YAP:

INSERT INTO BENCH_IMPL VALUES (
	'yap', 'wine', 'tab', 'B',
	'wine ontology [V1: table 225 derived preds, reorder body lits var->]');


-- ----------------------------------------------------------------------------
-- Benchmark implementations for Apache Jena
-- ----------------------------------------------------------------------------

INSERT INTO BENCH_IMPL VALUES (
	'jena', 'tcff', '', 'B',
	'transitive closure with Property Paths for Apache Jena');

-- ----------------------------------------------------------------------------
-- Benchmark implementations for DataScript:
-- ----------------------------------------------------------------------------

INSERT INTO BENCH_IMPL VALUES (
	'datascript', 'tcff', '', 'B',
	'transitive closure for DataScript');

-- ----------------------------------------------------------------------------
-- Benchmark implementations for SQLite3:
-- ----------------------------------------------------------------------------

INSERT INTO BENCH_IMPL VALUES (
	'sqlite3', 'tcff', '', 'B',
	'transitive closure for SQLite3');

-- ============================================================================
-- Table for Data Files (Input Files to Benchmarks):
-- ============================================================================

CREATE TABLE DATA_FILE(
	FILE_ID     VARCHAR(20)  NOT NULL,
	SEQ_NO      NUMERIC(3)   NOT NULL,
	LINES       NUMERIC(12)  NOT NULL, -- should be the number of facts
	BYTES       NUMERIC(12)  NOT NULL, -- refers to Datalog facts version
	FILENAME    VARCHAR(40)  NOT NULL, -- without extension
--	DESCRIPTION VARCHAR(80)  NULL,
	CONSTRAINT DATA_FILE_PK
		PRIMARY KEY (FILE_ID),
	CONSTRAINT DATA_FILE_KEY2
		UNIQUE (SEQ_NO),
	CONSTRAINT DATA_FILE_SEQ_NO_NOT_NEGATIVE
		CHECK(SEQ_NO >= 0),
	CONSTRAINT DATA_FILE_LINES_NOT_NEGATIVE
		CHECK(LINES >= 0),
	CONSTRAINT DATA_FILE_BYTES_NOT_NEGATIVE
		CHECK(BYTES >= 0));

-- wc was used to compute lines and bytes

INSERT INTO DATA_FILE VALUES
    ('k_1k',      1, 1000000,  14780000,'tc_kc1_n1k');
INSERT INTO DATA_FILE VALUES
    ('k_2k',      2, 4000000,  63560000,'tc_kc2_n2k');
INSERT INTO DATA_FILE VALUES
    ('a_1k',      3,  499500,   7382610,'tc_kn1_n1k');
INSERT INTO DATA_FILE VALUES
    ('a_2k',      4, 1999000,  31764110,'tc_kn2_n2k');
INSERT INTO DATA_FILE VALUES
    ('c_1k',      5,    1000,     14780,'tc_c1_n1k_');
INSERT INTO DATA_FILE VALUES
    ('c_2k',      6,    2000,     31780,'tc_c2_n2k_');
INSERT INTO DATA_FILE VALUES
    ('c_4k',      7,    4000,     65780,'tc_c3_n4k_');
INSERT INTO DATA_FILE VALUES
    ('s_2k_1',    8,    4000,     63560,'tc_s1_n2k_1');
INSERT INTO DATA_FILE VALUES
    ('s_2k_2',    9,    6000,     95340,'tc_s2_n2k_2');
INSERT INTO DATA_FILE VALUES
    ('s_2k_3',   10,    8000,    127120,'tc_s3_n2k_3');
INSERT INTO DATA_FILE VALUES
    ('s_2k_4',   11,   10000,    158900,'tc_s4_n2k_4');
INSERT INTO DATA_FILE VALUES
    ('p_1k',     12,     999,     14767,'tc_p1_n1k_');
INSERT INTO DATA_FILE VALUES
    ('p_2k',     13,    1999,     31766,'tc_p2_n2k_');
INSERT INTO DATA_FILE VALUES
    ('p_4k',     14,    3999,     65766,'tc_p3_n4k_');
INSERT INTO DATA_FILE VALUES
    ('m_4ki_2',  15,    8190,    137016,'tc_m1_n8k_2');
INSERT INTO DATA_FILE VALUES
    ('m_1ki_8',  16,    8184,    136932,'tc_m2_n8k_8');
INSERT INTO DATA_FILE VALUES
    ('m_256_32', 17,    8160,    136574,'tc_m3_n8k_32');
INSERT INTO DATA_FILE VALUES
    ('m_64_128', 18,    8064,    135106,'tc_m4_n8k_128');
INSERT INTO DATA_FILE VALUES
    ('m_16_512', 19,    7680,    128962,'tc_m5_n8k_512');
INSERT INTO DATA_FILE VALUES
    ('m_4_2ki',  20,    6144,    103338,'tc_m6_n8k_2048');
INSERT INTO DATA_FILE VALUES
    ('b_17',     21,  131070,   2488088,'tc_b1_n131k_');
INSERT INTO DATA_FILE VALUES
    ('b_18',     22,  262142,   5171672,'tc_b2_n262k_');
INSERT INTO DATA_FILE VALUES
    ('b_19',     23,  524286,  10676696,'tc_b3_n524k_');
INSERT INTO DATA_FILE VALUES
    ('u_1k_50k', 24,   50000,    739017,'tc_uc1_n1k_e50k');
INSERT INTO DATA_FILE VALUES
    ('u_1k_125k',25,  125000,   1847521,'tc_uc2_n1k_e125k');
INSERT INTO DATA_FILE VALUES
    ('u_1k_250k',26,  250000,   3694992,'tc_uc3_n1k_e250k');
INSERT INTO DATA_FILE VALUES
    ('u_2k_250k',27,  200000,   3178006,'tc_uc4_n2k_e200k');
INSERT INTO DATA_FILE VALUES
    ('u_2k_500k',28,  500000,   7945050,'tc_uc5_n2k_e500k');
INSERT INTO DATA_FILE VALUES
    ('u_2k_1m',  29, 1000000,  15890038,'tc_uc6_n2k_e1000k');
INSERT INTO DATA_FILE VALUES
    ('f_1k_50k', 30,   50000,    738969,'tc_un1_n1k_e50k');
INSERT INTO DATA_FILE VALUES
    ('f_1k_125k',31,  125000,   1847363,'tc_un2_n1k_e125k');
INSERT INTO DATA_FILE VALUES
    ('f_1k_250k',32,  250000,   3694940,'tc_un3_n1k_e250k');
INSERT INTO DATA_FILE VALUES
    ('f_2k_250k',33,  200000,   3177933,'tc_un4_n2k_e200k');
INSERT INTO DATA_FILE VALUES
    ('f_2k_500k',34,  500000,   7944766,'tc_un5_n2k_e500k');
INSERT INTO DATA_FILE VALUES
    ('f_2k_1m',  35, 1000000,  15890191,'tc_un6_n2k_e1000k');
INSERT INTO DATA_FILE VALUES
    ('f1',       40,   50000,    689318,'tc_d1000_parsize50000_xsb_nocyc');
INSERT INTO DATA_FILE VALUES
    ('f2',       41,   50000,    689318,'tc_d1000_parsize50000_xsb_cyc');
INSERT INTO DATA_FILE VALUES
    ('f3',       42,  250000,   3446506,'tc_d1000_parsize250000_xsb_nocyc');
INSERT INTO DATA_FILE VALUES
    ('f4',       43,  250000,   3446506,'tc_d1000_parsize250000_xsb_cyc');
INSERT INTO DATA_FILE VALUES
    ('f5',       44,  500000,   6893031,'tc_d1000_parsize500000_xsb_nocyc');
INSERT INTO DATA_FILE VALUES
    ('f6',       45,  500000,   6893031,'tc_d1000_parsize500000_xsb_cyc');
INSERT INTO DATA_FILE VALUES
    ('f7',       46,  500000,   7446477,'tc_d2000_parsize500000_xsb_nocyc');
INSERT INTO DATA_FILE VALUES
    ('f8',       47,  500000,   7446477,'tc_d2000_parsize500000_xsb_cyc');
INSERT INTO DATA_FILE VALUES
    ('f9',       48, 1000000,  14892982,'tc_d2000_parsize1000000_xsb_nocyc');
INSERT INTO DATA_FILE VALUES
    ('f10',      49, 1000000,  14892982,'tc_d2000_parsize1000000_xsb_cyc');

INSERT INTO DATA_FILE VALUES
    ('10k',      50,   50000,    639300,'d1000_relsize10000_xsb_cyc');
INSERT INTO DATA_FILE VALUES
    ('50k',      51,  250000,   3196504,'d1000_relsize50000_xsb_cyc');
INSERT INTO DATA_FILE VALUES
    ('250k',     52, 1250000,  15982480,'d1000_relsize250000_xsb_cyc');

INSERT INTO DATA_FILE VALUES
    ('std',      60, 2437867, 128444409,'dblp');
INSERT INTO DATA_FILE VALUES
    ('big',      61,50221773,3082908204,'dblp_big');

INSERT INTO DATA_FILE VALUES
    ('wdat',70,     654,     26386, 'wine_data');


-- ============================================================================
-- More Data on Data Files That are Directed Graphs:
-- ============================================================================

CREATE TABLE GRAPH_DATA (
	FILE_ID        VARCHAR(20) NOT NULL,
	FILE_NAME      VARCHAR(80) NOT NULL,
	NUM_NODES      NUMERIC(12) NOT NULL,
	NUM_EDGES      NUMERIC(12) NOT NULL,
	DUP_EDGES      NUMERIC(12) NOT NULL,
	LOOPS          CHAR(1)     NOT NULL,
	MIN_IN_DEGREE  NUMERIC(12) NOT NULL,
	MAX_IN_DEGREE  NUMERIC(12) NOT NULL,
	MIN_OUT_DEGREE NUMERIC(12) NOT NULL,
	MAX_OUT_DEGREE NUMERIC(12) NOT NULL,
	CYCLES         char(1)     NOT NULL,
	TC_SIZE        NUMERIC(12) NOT NULL,
	NUM_ITER       NUMERIC(12) NOT NULL,
	MAX_ITER       NUMERIC(12) NOT NULL,
	LIMIT_REACHED  char(1)     NOT NULL,
	MISSING_ROWS   NUMERIC(12) NOT NULL,
	COST           NUMERIC(12) NOT NULL,
	CONSTRAINT GRAPH_DATA_PK
		PRIMARY KEY(FILE_ID),
	CONSTRAINT GRAPH_DATA_REF_DATA_FILE
		FOREIGN KEY(FILE_ID) REFERENCES DATA_FILE,
	CONSTRAINT GRAPH_DATA_FILE_NAME_UNIQUE
		UNIQUE(FILE_NAME),
	CONSTRAINT GRAPH_DATA_NUM_NODES_NONNEGATIVE
		CHECK(NUM_NODES >= 0),
	CONSTRAINT GRAPH_DATA_NUM_EDGES_NONNEGATIVE
		CHECK(NUM_EDGES >= 0),
	CONSTRAINT GRAPH_DATA_DUP_EDGES_NONNEGATIVE
		CHECK(DUP_EDGES >= 0),
	CONSTRAINT GRAPH_DATA_LOOPS_YES_NO
		CHECK(LOOPS  in ('Y','N')),
	CONSTRAINT GRAPH_DATA_MIN_IN_DEGREE_NONNEGATIVE
		CHECK(MIN_IN_DEGREE >= 0),
	CONSTRAINT GRAPH_DATA_MAX_IN_DEGREE_NONNEGATIVE
		CHECK(MAX_IN_DEGREE >= 0),
	CONSTRAINT GRAPH_DATA_MIN_OUT_DEGREE_NONNEGATIVE
		CHECK(MIN_OUT_DEGREE >= 0),
	CONSTRAINT GRAPH_DATA_MAX_OUT_DEGREE_NONNEGATIVE
		CHECK(MAX_OUT_DEGREE >= 0),
	CONSTRAINT GRAPH_DATA_CYCLES_YES_NO
		CHECK(CYCLES IN ('Y','N')),
	CONSTRAINT GRAPH_DATA_TC_SIZE_NONNEGATIVE
		CHECK(TC_SIZE >= 0),
	CONSTRAINT GRAPH_DATA_NUM_ITER_NONNEGATIVE
		CHECK(NUM_ITER >= 0),
	CONSTRAINT GRAPH_DATA_MAX_ITER_NONNEGATIVE
		CHECK(MAX_ITER >= 0),
	CONSTRAINT GRAPH_DATA_NUM_ITER_LE_MAX_ITER
		CHECK(NUM_ITER <= MAX_ITER),
	CONSTRAINT GRAPH_DATA_LIMIT_REACHED_YES_NO
		CHECK(LIMIT_REACHED IN ('Y','N')),
	CONSTRAINT GRAPH_DATA_LIMIT_REACHED_CONSISTENT
		CHECK((LIMIT_REACHED = 'N' AND MISSING_ROWS = 0) OR
			(LIMIT_REACHED = 'Y' AND NUM_ITER = MAX_ITER)),
	CONSTRAINT GRAPH_DATA_MISSING_ROWS_NONNEGATIVE
		CHECK(MISSING_ROWS >= 0),
	CONSTRAINT GRAPH_DATA_COST_NONNEGATIVE
		CHECK(COST >= 0)
);

-- The computation of the transitive closure with iterations is done
-- only to the maximal level MAX_ITER.
-- LIMIT_REACHED indicates whether the limit was reached,
-- i.e. whether the computations of levels/iterations was incomplete.
-- In this case the number of missing rows, i.e. tc rows that need
-- more iterations than MAX_ITER, are listed in MISSING_ROWS.
-- Note that the normal computation of the transitive closure (without levels)
-- is not affected.


INSERT INTO GRAPH_DATA VALUES('b_17','tc_b1_n131k_',
131071,131070,0,'N',0,1,0,2,'N',1966082,16,100,'N',0,1966082);

INSERT INTO GRAPH_DATA VALUES('b_18','tc_b2_n262k_',
262143,262142,0,'N',0,1,0,2,'N',4194306,17,100,'N',0,4194306);

INSERT INTO GRAPH_DATA VALUES('b_19','tc_b3_n524k_',
524287,524286,0,'N',0,1,0,2,'N',8912898,18,100,'N',0,8912898);

INSERT INTO GRAPH_DATA VALUES('c_1k','tc_c1_n1k_',
1000,1000,0,'N',1,1,1,1,'Y',1000000,1000,1002,'N',0,1001000);

INSERT INTO GRAPH_DATA VALUES('c_2k','tc_c2_n2k_',
2000,2000,0,'N',1,1,1,1,'Y',4000000,2000,2002,'N',0,4002000);

INSERT INTO GRAPH_DATA VALUES('c_4k','tc_c3_n4k_',
4000,4000,0,'N',1,1,1,1,'Y',16000000,4000,4002,'N',0,16004000);

INSERT INTO GRAPH_DATA VALUES('k_1k','tc_kc1_n1k',
1000,1000000,0,'Y',1000,1000,1000,1000,'Y',1000000,1,10,'N',0,1001000000);

INSERT INTO GRAPH_DATA VALUES('k_2k','tc_kc2_n2k',
2000,4000000,0,'Y',2000,2000,2000,2000,'Y',4000000,1,1,'N',0,8004000000);

INSERT INTO GRAPH_DATA VALUES('a_1k','tc_kn1_n1k',
1000,499500,0,'N',0,999,0,999,'N',499500,1,10,'N',0,166666500);

INSERT INTO GRAPH_DATA VALUES('a_2k','tc_kn2_n2k',
2000,1999000,0,'N',0,1999,0,1999,'N',1999000,1,10,'N',0,1333333000);

INSERT INTO GRAPH_DATA VALUES('m_4ki_2','tc_m1_n8k_2',
8192,8190,0,'N',0,1,0,1,'N',16773120,4095,4100,'N',0,16773120);

INSERT INTO GRAPH_DATA VALUES('m_1ki_8','tc_m2_n8k_8',
8192,8184,0,'N',0,1,0,1,'N',4190208,1023,1100,'N',0,4190208);

INSERT INTO GRAPH_DATA VALUES('m_256_32','tc_m3_n8k_32',
8192,8160,0,'N',0,1,0,1,'N',1044480,255,400,'N',0,1044480);

INSERT INTO GRAPH_DATA VALUES('m_64_128','tc_m4_n8k_128',
8192,8064,0,'N',0,1,0,1,'N',258048,63,100,'N',0,258048);

INSERT INTO GRAPH_DATA VALUES('m_16_512','tc_m5_n8k_512',
8192,7680,0,'N',0,1,0,1,'N',61440,15,20,'N',0,61440);

INSERT INTO GRAPH_DATA VALUES('m_4_2ki','tc_m6_n8k_2048',
8192,6144,0,'N',0,1,0,1,'N',12288,3,10,'N',0,12288);

INSERT INTO GRAPH_DATA VALUES('p_1k','tc_p1_n1k_',
1000,999,0,'N',0,1,0,1,'N',499500,999,1002,'N',0,499500);

INSERT INTO GRAPH_DATA VALUES('p_2k','tc_p2_n2k_',
2000,1999,0,'N',0,1,0,1,'N',1999000,1999,2002,'N',0,1999000);

INSERT INTO GRAPH_DATA VALUES('p_4k','tc_p3_n4k_',
4000,3999,0,'N',0,1,0,1,'N',7998000,3999,4002,'N',0,7998000);

INSERT INTO GRAPH_DATA VALUES('s_2k_1','tc_s1_n2k_1',
2000,4000,0,'N',2,2,2,2,'Y',4000000,10,10,'Y',3958000,8004000);

INSERT INTO GRAPH_DATA VALUES('s_2k_2','tc_s2_n2k_2',
2000,6000,0,'N',3,3,3,3,'Y',4000000,10,10,'Y',3870000,12006000);

INSERT INTO GRAPH_DATA VALUES('s_2k_3','tc_s3_n2k_3',
2000,8000,0,'N',4,4,4,4,'Y',4000000,10,10,'Y',3918000,16008000);

INSERT INTO GRAPH_DATA VALUES('s_2k_4','tc_s4_n2k_4',
2000,10000,0,'N',5,5,5,5,'Y',4000000,10,10,'Y',3898000,20010000);

INSERT INTO GRAPH_DATA VALUES('u_1k_50k','tc_uc1_n1k_e50k',
1000,50000,0,'N',47,51,46,51,'Y',1000000,3,10,'N',0,50050000);

INSERT INTO GRAPH_DATA VALUES('u_1k_125k','tc_uc2_n1k_e125k',
1000,125000,0,'N',122,127,122,127,'Y',1000000,2,10,'N',0,125125000);

INSERT INTO GRAPH_DATA VALUES('u_1k_250k','tc_uc3_n1k_e250k',
1000,250000,0,'N',248,251,248,251,'Y',1000000,2,10,'N',0,250250000);

INSERT INTO GRAPH_DATA VALUES('u_2k_250k','tc_uc4_n2k_e200k',
2000,200000,0,'N',98,101,98,101,'Y',4000000,3,10,'N',0,400200000);

INSERT INTO GRAPH_DATA VALUES('u_2k_500k','tc_uc5_n2k_e500k',
2000,500000,0,'N',248,251,248,251,'Y',4000000,2,10,'N',0,1000500000);

INSERT INTO GRAPH_DATA VALUES('u_2k_1m','tc_uc6_n2k_e1000k',
2000,1000000,0,'N',499,502,499,502,'Y',4000000,2,10,'N',0,2001000000);

INSERT INTO GRAPH_DATA VALUES('f_1k_50k','tc_un1_n1k_e50k',
1000,50000,0,'N',0,101,0,101,'N',472863,8,10,'N',0,15421338);

INSERT INTO GRAPH_DATA VALUES('f_1k_125k','tc_un2_n1k_e125k',
1000,125000,0,'N',0,251,0,251,'N',492170,5,10,'N',0,40705777);

INSERT INTO GRAPH_DATA VALUES('f_1k_250k','tc_un3_n1k_e250k',
1000,250000,0,'N',0,500,0,501,'N',497810,5,10,'N',0,83073458);

INSERT INTO GRAPH_DATA VALUES('f_2k_250k','tc_un4_n2k_e200k',
2000,200000,0,'N',0,201,0,201,'N',1946015,8,10,'N',0,128081155);

INSERT INTO GRAPH_DATA VALUES('f_2k_500k','tc_un5_n2k_e500k',
2000,500000,0,'N',0,500,0,500,'N',1985377,6,10,'N',0,330379789);

INSERT INTO GRAPH_DATA VALUES('f_2k_1m','tc_un6_n2k_e1000k',
2000,1000000,0,'N',0,1000,0,1001,'N',1995412,4,10,'N',0,665127921);

-- ============================================================================
-- Table for Input Files to Benchmarks:
-- ============================================================================

CREATE TABLE BENCH_INPUT(
	BENCH       VARCHAR(20)  NOT NULL,
	FILE_ID     VARCHAR(20)  NOT NULL,
	CONSTRAINT BENCH_INPUT_PK
		PRIMARY KEY (BENCH, FILE_ID),
	CONSTRAINT BENCH_INPUT_REF_BENCHMARK
		FOREIGN KEY (BENCH) REFERENCES BENCHMARK,
	CONSTRAINT BENCH_INPUT_REF_DATA_FILE
		FOREIGN KEY (FILE_ID) REFERENCES DATA_FILE);

INSERT INTO BENCH_INPUT VALUES ('dblp',   'std');
INSERT INTO BENCH_INPUT VALUES ('dblp',   'big');

INSERT INTO BENCH_INPUT VALUES ('j1axy',  '10k');
INSERT INTO BENCH_INPUT VALUES ('j1axy',  '50k');
INSERT INTO BENCH_INPUT VALUES ('j1axy',  '250k');

INSERT INTO BENCH_INPUT VALUES ('j1axyd', '10k');
INSERT INTO BENCH_INPUT VALUES ('j1axyd', '50k');
INSERT INTO BENCH_INPUT VALUES ('j1axyd', '250k');

INSERT INTO BENCH_INPUT VALUES ('tcff',   'b_17');
INSERT INTO BENCH_INPUT VALUES ('tcff',   'b_18');
INSERT INTO BENCH_INPUT VALUES ('tcff',   'b_19');
INSERT INTO BENCH_INPUT VALUES ('tcff',   'c_1k');
INSERT INTO BENCH_INPUT VALUES ('tcff',   'c_2k');
INSERT INTO BENCH_INPUT VALUES ('tcff',   'c_4k');
INSERT INTO BENCH_INPUT VALUES ('tcff',   'k_1k');
INSERT INTO BENCH_INPUT VALUES ('tcff',   'k_2k');
INSERT INTO BENCH_INPUT VALUES ('tcff',   'a_1k');
INSERT INTO BENCH_INPUT VALUES ('tcff',   'a_2k');
INSERT INTO BENCH_INPUT VALUES ('tcff',   'm_4ki_2');
INSERT INTO BENCH_INPUT VALUES ('tcff',   'm_1ki_8');
INSERT INTO BENCH_INPUT VALUES ('tcff',   'm_256_32');
INSERT INTO BENCH_INPUT VALUES ('tcff',   'm_64_128');
INSERT INTO BENCH_INPUT VALUES ('tcff',   'm_16_512');
INSERT INTO BENCH_INPUT VALUES ('tcff',   'm_4_2ki');
INSERT INTO BENCH_INPUT VALUES ('tcff',   'p_1k');
INSERT INTO BENCH_INPUT VALUES ('tcff',   'p_2k');
INSERT INTO BENCH_INPUT VALUES ('tcff',   'p_4k');
INSERT INTO BENCH_INPUT VALUES ('tcff',   's_2k_1');
INSERT INTO BENCH_INPUT VALUES ('tcff',   's_2k_2');
INSERT INTO BENCH_INPUT VALUES ('tcff',   's_2k_3');
INSERT INTO BENCH_INPUT VALUES ('tcff',   's_2k_4');
INSERT INTO BENCH_INPUT VALUES ('tcff',   'u_1k_50k');
INSERT INTO BENCH_INPUT VALUES ('tcff',   'u_1k_125k');
INSERT INTO BENCH_INPUT VALUES ('tcff',   'u_1k_250k');
INSERT INTO BENCH_INPUT VALUES ('tcff',   'u_2k_250k');
INSERT INTO BENCH_INPUT VALUES ('tcff',   'u_2k_500k');
INSERT INTO BENCH_INPUT VALUES ('tcff',   'u_2k_1m');
INSERT INTO BENCH_INPUT VALUES ('tcff',   'f_1k_50k');
INSERT INTO BENCH_INPUT VALUES ('tcff',   'f_1k_125k');
INSERT INTO BENCH_INPUT VALUES ('tcff',   'f_1k_250k');
INSERT INTO BENCH_INPUT VALUES ('tcff',   'f_2k_250k');
INSERT INTO BENCH_INPUT VALUES ('tcff',   'f_2k_500k');
INSERT INTO BENCH_INPUT VALUES ('tcff',   'f_2k_1m');
INSERT INTO BENCH_INPUT VALUES ('tcff',   'f1');
INSERT INTO BENCH_INPUT VALUES ('tcff',   'f2');
INSERT INTO BENCH_INPUT VALUES ('tcff',   'f3');
INSERT INTO BENCH_INPUT VALUES ('tcff',   'f4');
INSERT INTO BENCH_INPUT VALUES ('tcff',   'f5');
INSERT INTO BENCH_INPUT VALUES ('tcff',   'f6');
INSERT INTO BENCH_INPUT VALUES ('tcff',   'f7');
INSERT INTO BENCH_INPUT VALUES ('tcff',   'f8');
INSERT INTO BENCH_INPUT VALUES ('tcff',   'f9');
INSERT INTO BENCH_INPUT VALUES ('tcff',   'f10');

INSERT INTO BENCH_INPUT VALUES ('tcbf',   'b_17');
INSERT INTO BENCH_INPUT VALUES ('tcbf',   'b_18');
INSERT INTO BENCH_INPUT VALUES ('tcbf',   'b_19');
INSERT INTO BENCH_INPUT VALUES ('tcbf',   'c_1k');
INSERT INTO BENCH_INPUT VALUES ('tcbf',   'c_2k');
INSERT INTO BENCH_INPUT VALUES ('tcbf',   'c_4k');
INSERT INTO BENCH_INPUT VALUES ('tcbf',   'k_1k');
INSERT INTO BENCH_INPUT VALUES ('tcbf',   'k_2k');
INSERT INTO BENCH_INPUT VALUES ('tcbf',   'a_1k');
INSERT INTO BENCH_INPUT VALUES ('tcbf',   'a_2k');
INSERT INTO BENCH_INPUT VALUES ('tcbf',   'm_4ki_2');
INSERT INTO BENCH_INPUT VALUES ('tcbf',   'm_1ki_8');
INSERT INTO BENCH_INPUT VALUES ('tcbf',   'm_256_32');
INSERT INTO BENCH_INPUT VALUES ('tcbf',   'm_64_128');
INSERT INTO BENCH_INPUT VALUES ('tcbf',   'm_16_512');
INSERT INTO BENCH_INPUT VALUES ('tcbf',   'm_4_2ki');
INSERT INTO BENCH_INPUT VALUES ('tcbf',   'p_1k');
INSERT INTO BENCH_INPUT VALUES ('tcbf',   'p_2k');
INSERT INTO BENCH_INPUT VALUES ('tcbf',   'p_4k');
INSERT INTO BENCH_INPUT VALUES ('tcbf',   's_2k_1');
INSERT INTO BENCH_INPUT VALUES ('tcbf',   's_2k_2');
INSERT INTO BENCH_INPUT VALUES ('tcbf',   's_2k_3');
INSERT INTO BENCH_INPUT VALUES ('tcbf',   's_2k_4');
INSERT INTO BENCH_INPUT VALUES ('tcbf',   'u_1k_50k');
INSERT INTO BENCH_INPUT VALUES ('tcbf',   'u_1k_125k');
INSERT INTO BENCH_INPUT VALUES ('tcbf',   'u_1k_250k');
INSERT INTO BENCH_INPUT VALUES ('tcbf',   'u_2k_250k');
INSERT INTO BENCH_INPUT VALUES ('tcbf',   'u_2k_500k');
INSERT INTO BENCH_INPUT VALUES ('tcbf',   'u_2k_1m');
INSERT INTO BENCH_INPUT VALUES ('tcbf',   'f_1k_50k');
INSERT INTO BENCH_INPUT VALUES ('tcbf',   'f_1k_125k');
INSERT INTO BENCH_INPUT VALUES ('tcbf',   'f_1k_250k');
INSERT INTO BENCH_INPUT VALUES ('tcbf',   'f_2k_250k');
INSERT INTO BENCH_INPUT VALUES ('tcbf',   'f_2k_500k');
INSERT INTO BENCH_INPUT VALUES ('tcbf',   'f_2k_1m');
INSERT INTO BENCH_INPUT VALUES ('tcbf',   'f1');
INSERT INTO BENCH_INPUT VALUES ('tcbf',   'f2');
INSERT INTO BENCH_INPUT VALUES ('tcbf',   'f3');
INSERT INTO BENCH_INPUT VALUES ('tcbf',   'f4');
INSERT INTO BENCH_INPUT VALUES ('tcbf',   'f5');
INSERT INTO BENCH_INPUT VALUES ('tcbf',   'f6');
INSERT INTO BENCH_INPUT VALUES ('tcbf',   'f7');
INSERT INTO BENCH_INPUT VALUES ('tcbf',   'f8');
INSERT INTO BENCH_INPUT VALUES ('tcbf',   'f9');
INSERT INTO BENCH_INPUT VALUES ('tcbf',   'f10');

INSERT INTO BENCH_INPUT VALUES ('tcbfs',  'b_17');
INSERT INTO BENCH_INPUT VALUES ('tcbfs',  'b_18');
INSERT INTO BENCH_INPUT VALUES ('tcbfs',  'b_19');
INSERT INTO BENCH_INPUT VALUES ('tcbfs',  'c_1k');
INSERT INTO BENCH_INPUT VALUES ('tcbfs',  'c_2k');
INSERT INTO BENCH_INPUT VALUES ('tcbfs',  'c_4k');
INSERT INTO BENCH_INPUT VALUES ('tcbfs',  'k_1k');
INSERT INTO BENCH_INPUT VALUES ('tcbfs',  'k_2k');
INSERT INTO BENCH_INPUT VALUES ('tcbfs',  'a_1k');
INSERT INTO BENCH_INPUT VALUES ('tcbfs',  'a_2k');
INSERT INTO BENCH_INPUT VALUES ('tcbfs',  'm_4ki_2');
INSERT INTO BENCH_INPUT VALUES ('tcbfs',  'm_1ki_8');
INSERT INTO BENCH_INPUT VALUES ('tcbfs',  'm_256_32');
INSERT INTO BENCH_INPUT VALUES ('tcbfs',  'm_64_128');
INSERT INTO BENCH_INPUT VALUES ('tcbfs',  'm_16_512');
INSERT INTO BENCH_INPUT VALUES ('tcbfs',  'm_4_2ki');
INSERT INTO BENCH_INPUT VALUES ('tcbfs',  'p_1k');
INSERT INTO BENCH_INPUT VALUES ('tcbfs',  'p_2k');
INSERT INTO BENCH_INPUT VALUES ('tcbfs',  'p_4k');
INSERT INTO BENCH_INPUT VALUES ('tcbfs',  's_2k_1');
INSERT INTO BENCH_INPUT VALUES ('tcbfs',  's_2k_2');
INSERT INTO BENCH_INPUT VALUES ('tcbfs',  's_2k_3');
INSERT INTO BENCH_INPUT VALUES ('tcbfs',  's_2k_4');
INSERT INTO BENCH_INPUT VALUES ('tcbfs',  'u_1k_50k');
INSERT INTO BENCH_INPUT VALUES ('tcbfs',  'u_1k_125k');
INSERT INTO BENCH_INPUT VALUES ('tcbfs',  'u_1k_250k');
INSERT INTO BENCH_INPUT VALUES ('tcbfs',  'u_2k_250k');
INSERT INTO BENCH_INPUT VALUES ('tcbfs',  'u_2k_500k');
INSERT INTO BENCH_INPUT VALUES ('tcbfs',  'u_2k_1m');
INSERT INTO BENCH_INPUT VALUES ('tcbfs',  'f_1k_50k');
INSERT INTO BENCH_INPUT VALUES ('tcbfs',  'f_1k_125k');
INSERT INTO BENCH_INPUT VALUES ('tcbfs',  'f_1k_250k');
INSERT INTO BENCH_INPUT VALUES ('tcbfs',  'f_2k_250k');
INSERT INTO BENCH_INPUT VALUES ('tcbfs',  'f_2k_500k');
INSERT INTO BENCH_INPUT VALUES ('tcbfs',  'f_2k_1m');
INSERT INTO BENCH_INPUT VALUES ('tcbfs',  'f1');
INSERT INTO BENCH_INPUT VALUES ('tcbfs',  'f2');
INSERT INTO BENCH_INPUT VALUES ('tcbfs',  'f3');
INSERT INTO BENCH_INPUT VALUES ('tcbfs',  'f4');
INSERT INTO BENCH_INPUT VALUES ('tcbfs',  'f5');
INSERT INTO BENCH_INPUT VALUES ('tcbfs',  'f6');
INSERT INTO BENCH_INPUT VALUES ('tcbfs',  'f7');
INSERT INTO BENCH_INPUT VALUES ('tcbfs',  'f8');
INSERT INTO BENCH_INPUT VALUES ('tcbfs',  'f9');
INSERT INTO BENCH_INPUT VALUES ('tcbfs',  'f10');
INSERT INTO BENCH_INPUT VALUES ('wine',   'wdat');

INSERT INTO BENCH_INPUT VALUES ('loadd',  'std');
INSERT INTO BENCH_INPUT VALUES ('loadd',  'big');

--INSERT INTO BENCH_INPUT VALUES ( 'loadt', ...);
--INSERT INTO BENCH_INPUT VALUES (  'set1', ...);
--INSERT INTO BENCH_INPUT VALUES (  'set2', ...);

-- ============================================================================
-- Table for Results of Benchmark Runs (Test Runs):
-- ============================================================================


CREATE TABLE BENCH_RUN(
	RUN_DATE    DATE         NOT NULL,
	RUN_NO      NUMERIC(4)   NOT NULL,
	PROG        VARCHAR(20)  NOT NULL,
	VER         NUMERIC(4)   NOT NULL,
	MACHINE     VARCHAR(20)  NOT NULL,
	BENCH       VARCHAR(20)  NOT NULL,
	IMPL        VARCHAR(20)  NOT NULL,
	FILE_ID     VARCHAR(20)  NOT NULL,
	WITH_OUTPUT CHAR(1)      NOT NULL,
	LOAD_TIME   NUMERIC(8,3) NULL,
	EXEC_TIME   NUMERIC(8,3) NULL,
	TOTAL_TIME  NUMERIC(8,3) NULL,
	USER_TIME   NUMERIC(8,3) NULL,
	SYS_TIME    NUMERIC(8,3) NULL,
	REAL_TIME   NUMERIC(8,3) NOT NULL,
	MEM         NUMERIC(8,3) NULL,
	STATUS      VARCHAR(80)  NULL,
	REMARK      VARCHAR(800) NULL,
        CONSTRAINT  RUN_NO_POSITIVE
		CHECK(RUN_NO > 0),
        CONSTRAINT  WITH_OUTPUT_BOOL
		CHECK(WITH_OUTPUT IN ('Y','N')),
        CONSTRAINT  LOAD_TIME_NONNEGATIVE
		CHECK(LOAD_TIME >= 0),
        CONSTRAINT  EXEC_TIME_NONNEGATIVE
		CHECK(EXEC_TIME >= 0),
        CONSTRAINT  TOTAL_TIME_NONNEGATIVE
		CHECK(TOTAL_TIME >= 0),
        CONSTRAINT  USER_TIME_NONNEGATIVE
		CHECK(USER_TIME >= 0),
        CONSTRAINT  SYS_TIME_NONNEGATIVE
		CHECK(SYS_TIME >= 0),
        CONSTRAINT  REAL_TIME_NONNEGATIVE
		CHECK(REAL_TIME >= 0),
        CONSTRAINT  MEM_NONNEGATIVE
		CHECK(MEM >= 0),
        CONSTRAINT  BENCH_RUN_PK
		PRIMARY KEY(RUN_DATE,RUN_NO,MACHINE,PROG,VER),
        CONSTRAINT  BENCH_RUN_REF_BENCH_IMPL
		FOREIGN KEY(PROG, BENCH, IMPL) REFERENCES BENCH_IMPL,
        CONSTRAINT  BENCH_RUN_REF_BENCH_INPUT
		FOREIGN KEY(BENCH, FILE_ID) REFERENCES BENCH_INPUT,
	CONSTRAINT  BENCH_RUN_REF_PROG_VER
		FOREIGN KEY(PROG, VER, MACHINE) REFERENCES PROG_VER);

-- ============================================================================
-- Experimental Table for Speed Values to be Tested with Data:
-- ============================================================================


CREATE TABLE SPEED_ESTIMATE(
	PROG        VARCHAR(20)  NOT NULL,
	BENCH       VARCHAR(20)  NOT NULL,
	SPEED       NUMERIC(10)  NOT NULL,
        CONSTRAINT  SPEED_POSITIVE
		CHECK(SPEED > 0),
        CONSTRAINT  SPEED_ESTIMATE_PK
		PRIMARY KEY(PROG,BENCH,SPEED),
        CONSTRAINT  SPEED_REF_PROG
		FOREIGN KEY(PROG) REFERENCES PROGRAM,
        CONSTRAINT  SPEED_REF_BENCH
		FOREIGN KEY(BENCH) REFERENCES BENCHMARK);

INSERT INTO SPEED_ESTIMATE VALUES('bam',     'tcff', 13000000);
INSERT INTO SPEED_ESTIMATE VALUES('bam',     'tcff', 14000000);
INSERT INTO SPEED_ESTIMATE VALUES('bam',     'tcff', 15000000);
INSERT INTO SPEED_ESTIMATE VALUES('bam',     'tcff', 16000000);
INSERT INTO SPEED_ESTIMATE VALUES('bam',     'tcff', 17000000);
INSERT INTO SPEED_ESTIMATE VALUES('bam',     'tcff', 18000000);
INSERT INTO SPEED_ESTIMATE VALUES('bam',     'tcff', 19000000);
INSERT INTO SPEED_ESTIMATE VALUES('bam',     'tcff', 19100000);
INSERT INTO SPEED_ESTIMATE VALUES('bam',     'tcff', 19200000);
INSERT INTO SPEED_ESTIMATE VALUES('bam',     'tcff', 19300000);
INSERT INTO SPEED_ESTIMATE VALUES('bam',     'tcff', 19400000);
INSERT INTO SPEED_ESTIMATE VALUES('bam',     'tcff', 19500000);
INSERT INTO SPEED_ESTIMATE VALUES('bam',     'tcff', 19600000);
INSERT INTO SPEED_ESTIMATE VALUES('bam',     'tcff', 19700000);
INSERT INTO SPEED_ESTIMATE VALUES('bam',     'tcff', 19800000);
INSERT INTO SPEED_ESTIMATE VALUES('bam',     'tcff', 19810000);
INSERT INTO SPEED_ESTIMATE VALUES('bam',     'tcff', 19820000);
INSERT INTO SPEED_ESTIMATE VALUES('bam',     'tcff', 19830000);
INSERT INTO SPEED_ESTIMATE VALUES('bam',     'tcff', 19840000);
INSERT INTO SPEED_ESTIMATE VALUES('bam',     'tcff', 19850000);
INSERT INTO SPEED_ESTIMATE VALUES('bam',     'tcff', 19860000);
INSERT INTO SPEED_ESTIMATE VALUES('bam',     'tcff', 19870000);
INSERT INTO SPEED_ESTIMATE VALUES('bam',     'tcff', 19880000);
INSERT INTO SPEED_ESTIMATE VALUES('bam',     'tcff', 19890000);
INSERT INTO SPEED_ESTIMATE VALUES('bam',     'tcff', 19900000);
INSERT INTO SPEED_ESTIMATE VALUES('bam',     'tcff', 19910000);
INSERT INTO SPEED_ESTIMATE VALUES('bam',     'tcff', 19920000);
INSERT INTO SPEED_ESTIMATE VALUES('bam',     'tcff', 19930000);
INSERT INTO SPEED_ESTIMATE VALUES('bam',     'tcff', 19940000);
INSERT INTO SPEED_ESTIMATE VALUES('bam',     'tcff', 19950000);
INSERT INTO SPEED_ESTIMATE VALUES('bam',     'tcff', 19960000);
INSERT INTO SPEED_ESTIMATE VALUES('bam',     'tcff', 19970000);
INSERT INTO SPEED_ESTIMATE VALUES('bam',     'tcff', 19980000);
INSERT INTO SPEED_ESTIMATE VALUES('bam',     'tcff', 19990000);
INSERT INTO SPEED_ESTIMATE VALUES('bam',     'tcff', 20000000);
INSERT INTO SPEED_ESTIMATE VALUES('bam',     'tcff', 20100000);
INSERT INTO SPEED_ESTIMATE VALUES('bam',     'tcff', 20200000);
INSERT INTO SPEED_ESTIMATE VALUES('bam',     'tcff', 20300000);
INSERT INTO SPEED_ESTIMATE VALUES('bam',     'tcff', 20400000);
INSERT INTO SPEED_ESTIMATE VALUES('bam',     'tcff', 20500000);
INSERT INTO SPEED_ESTIMATE VALUES('bam',     'tcff', 20600000);
INSERT INTO SPEED_ESTIMATE VALUES('bam',     'tcff', 20700000);
INSERT INTO SPEED_ESTIMATE VALUES('bam',     'tcff', 20710000);
INSERT INTO SPEED_ESTIMATE VALUES('bam',     'tcff', 20720000);
INSERT INTO SPEED_ESTIMATE VALUES('bam',     'tcff', 20730000);
INSERT INTO SPEED_ESTIMATE VALUES('bam',     'tcff', 20740000);
INSERT INTO SPEED_ESTIMATE VALUES('bam',     'tcff', 20750000);
INSERT INTO SPEED_ESTIMATE VALUES('bam',     'tcff', 20760000);
INSERT INTO SPEED_ESTIMATE VALUES('bam',     'tcff', 20770000);
INSERT INTO SPEED_ESTIMATE VALUES('bam',     'tcff', 20780000);
INSERT INTO SPEED_ESTIMATE VALUES('bam',     'tcff', 20790000);
INSERT INTO SPEED_ESTIMATE VALUES('bam',     'tcff', 20800000);
INSERT INTO SPEED_ESTIMATE VALUES('bam',     'tcff', 20810000);
INSERT INTO SPEED_ESTIMATE VALUES('bam',     'tcff', 20820000);
INSERT INTO SPEED_ESTIMATE VALUES('bam',     'tcff', 20830000);
INSERT INTO SPEED_ESTIMATE VALUES('bam',     'tcff', 20840000);
INSERT INTO SPEED_ESTIMATE VALUES('bam',     'tcff', 20850000);
INSERT INTO SPEED_ESTIMATE VALUES('bam',     'tcff', 20860000);
INSERT INTO SPEED_ESTIMATE VALUES('bam',     'tcff', 20870000);
INSERT INTO SPEED_ESTIMATE VALUES('bam',     'tcff', 20880000);
INSERT INTO SPEED_ESTIMATE VALUES('bam',     'tcff', 20890000);
INSERT INTO SPEED_ESTIMATE VALUES('bam',     'tcff', 20900000);
INSERT INTO SPEED_ESTIMATE VALUES('bam',     'tcff', 21000000);
INSERT INTO SPEED_ESTIMATE VALUES('bam',     'tcff', 21100000);
INSERT INTO SPEED_ESTIMATE VALUES('bam',     'tcff', 21200000);
INSERT INTO SPEED_ESTIMATE VALUES('bam',     'tcff', 21300000);
INSERT INTO SPEED_ESTIMATE VALUES('bam',     'tcff', 21400000);
INSERT INTO SPEED_ESTIMATE VALUES('bam',     'tcff', 21500000);
INSERT INTO SPEED_ESTIMATE VALUES('bam',     'tcff', 21600000);
INSERT INTO SPEED_ESTIMATE VALUES('bam',     'tcff', 21700000);
INSERT INTO SPEED_ESTIMATE VALUES('bam',     'tcff', 21800000);
INSERT INTO SPEED_ESTIMATE VALUES('bam',     'tcff', 21900000);
INSERT INTO SPEED_ESTIMATE VALUES('bam',     'tcff', 22000000);
INSERT INTO SPEED_ESTIMATE VALUES('bam',     'tcff', 23000000);
INSERT INTO SPEED_ESTIMATE VALUES('bam',     'tcff', 24000000);
INSERT INTO SPEED_ESTIMATE VALUES('bam',     'tcff', 25000000);
INSERT INTO SPEED_ESTIMATE VALUES('bam',     'tcff', 26000000);
INSERT INTO SPEED_ESTIMATE VALUES('bam',     'tcff', 27000000);
INSERT INTO SPEED_ESTIMATE VALUES('bam',     'tcff', 28000000);
INSERT INTO SPEED_ESTIMATE VALUES('bam',     'tcff', 29000000);
INSERT INTO SPEED_ESTIMATE VALUES('bam',     'tcff', 30000000);
INSERT INTO SPEED_ESTIMATE VALUES('bam',     'tcff', 31000000);
INSERT INTO SPEED_ESTIMATE VALUES('bam',     'tcff', 32000000);
INSERT INTO SPEED_ESTIMATE VALUES('bam',     'tcff', 33000000);
INSERT INTO SPEED_ESTIMATE VALUES('bam',     'tcff', 34000000);
INSERT INTO SPEED_ESTIMATE VALUES('bam',     'tcff', 35000000);

INSERT INTO SPEED_ESTIMATE VALUES('xsb',     'tcff',  5000000);
INSERT INTO SPEED_ESTIMATE VALUES('xsb',     'tcff',  6000000);
INSERT INTO SPEED_ESTIMATE VALUES('xsb',     'tcff',  7000000);
INSERT INTO SPEED_ESTIMATE VALUES('xsb',     'tcff',  8000000);
INSERT INTO SPEED_ESTIMATE VALUES('xsb',     'tcff',  8100000);
INSERT INTO SPEED_ESTIMATE VALUES('xsb',     'tcff',  8200000);
INSERT INTO SPEED_ESTIMATE VALUES('xsb',     'tcff',  8300000);
INSERT INTO SPEED_ESTIMATE VALUES('xsb',     'tcff',  8310000);
INSERT INTO SPEED_ESTIMATE VALUES('xsb',     'tcff',  8320000);
INSERT INTO SPEED_ESTIMATE VALUES('xsb',     'tcff',  8330000);
INSERT INTO SPEED_ESTIMATE VALUES('xsb',     'tcff',  8340000);
INSERT INTO SPEED_ESTIMATE VALUES('xsb',     'tcff',  8350000);
INSERT INTO SPEED_ESTIMATE VALUES('xsb',     'tcff',  8360000);
INSERT INTO SPEED_ESTIMATE VALUES('xsb',     'tcff',  8370000);
INSERT INTO SPEED_ESTIMATE VALUES('xsb',     'tcff',  8380000);
INSERT INTO SPEED_ESTIMATE VALUES('xsb',     'tcff',  8390000);
INSERT INTO SPEED_ESTIMATE VALUES('xsb',     'tcff',  8400000);
INSERT INTO SPEED_ESTIMATE VALUES('xsb',     'tcff',  8410000);
INSERT INTO SPEED_ESTIMATE VALUES('xsb',     'tcff',  8420000);
INSERT INTO SPEED_ESTIMATE VALUES('xsb',     'tcff',  8430000);
INSERT INTO SPEED_ESTIMATE VALUES('xsb',     'tcff',  8440000);
INSERT INTO SPEED_ESTIMATE VALUES('xsb',     'tcff',  8450000);
INSERT INTO SPEED_ESTIMATE VALUES('xsb',     'tcff',  8460000);
INSERT INTO SPEED_ESTIMATE VALUES('xsb',     'tcff',  8470000);
INSERT INTO SPEED_ESTIMATE VALUES('xsb',     'tcff',  8480000);
INSERT INTO SPEED_ESTIMATE VALUES('xsb',     'tcff',  8490000);
INSERT INTO SPEED_ESTIMATE VALUES('xsb',     'tcff',  8500000);
INSERT INTO SPEED_ESTIMATE VALUES('xsb',     'tcff',  8600000);
INSERT INTO SPEED_ESTIMATE VALUES('xsb',     'tcff',  8610000);
INSERT INTO SPEED_ESTIMATE VALUES('xsb',     'tcff',  8620000);
INSERT INTO SPEED_ESTIMATE VALUES('xsb',     'tcff',  8630000);
INSERT INTO SPEED_ESTIMATE VALUES('xsb',     'tcff',  8640000);
INSERT INTO SPEED_ESTIMATE VALUES('xsb',     'tcff',  8650000);
INSERT INTO SPEED_ESTIMATE VALUES('xsb',     'tcff',  8660000);
INSERT INTO SPEED_ESTIMATE VALUES('xsb',     'tcff',  8670000);
INSERT INTO SPEED_ESTIMATE VALUES('xsb',     'tcff',  8680000);
INSERT INTO SPEED_ESTIMATE VALUES('xsb',     'tcff',  8690000);
INSERT INTO SPEED_ESTIMATE VALUES('xsb',     'tcff',  8700000);
INSERT INTO SPEED_ESTIMATE VALUES('xsb',     'tcff',  8800000);
INSERT INTO SPEED_ESTIMATE VALUES('xsb',     'tcff',  8900000);
INSERT INTO SPEED_ESTIMATE VALUES('xsb',     'tcff',  9000000);
INSERT INTO SPEED_ESTIMATE VALUES('xsb',     'tcff',  9100000);
INSERT INTO SPEED_ESTIMATE VALUES('xsb',     'tcff',  9200000);
INSERT INTO SPEED_ESTIMATE VALUES('xsb',     'tcff',  9300000);
INSERT INTO SPEED_ESTIMATE VALUES('xsb',     'tcff',  9400000);
INSERT INTO SPEED_ESTIMATE VALUES('xsb',     'tcff',  9500000);
INSERT INTO SPEED_ESTIMATE VALUES('xsb',     'tcff',  9600000);
INSERT INTO SPEED_ESTIMATE VALUES('xsb',     'tcff',  9700000);
INSERT INTO SPEED_ESTIMATE VALUES('xsb',     'tcff',  9800000);
INSERT INTO SPEED_ESTIMATE VALUES('xsb',     'tcff',  9900000);
INSERT INTO SPEED_ESTIMATE VALUES('xsb',     'tcff', 10000000);
INSERT INTO SPEED_ESTIMATE VALUES('xsb',     'tcff', 10100000);
INSERT INTO SPEED_ESTIMATE VALUES('xsb',     'tcff', 10200000);
INSERT INTO SPEED_ESTIMATE VALUES('xsb',     'tcff', 10300000);
INSERT INTO SPEED_ESTIMATE VALUES('xsb',     'tcff', 10400000);
INSERT INTO SPEED_ESTIMATE VALUES('xsb',     'tcff', 10500000);
INSERT INTO SPEED_ESTIMATE VALUES('xsb',     'tcff', 10600000);
INSERT INTO SPEED_ESTIMATE VALUES('xsb',     'tcff', 10700000);
INSERT INTO SPEED_ESTIMATE VALUES('xsb',     'tcff', 10800000);
INSERT INTO SPEED_ESTIMATE VALUES('xsb',     'tcff', 10900000);
INSERT INTO SPEED_ESTIMATE VALUES('xsb',     'tcff', 11000000);
INSERT INTO SPEED_ESTIMATE VALUES('xsb',     'tcff', 11100000);
INSERT INTO SPEED_ESTIMATE VALUES('xsb',     'tcff', 11200000);
INSERT INTO SPEED_ESTIMATE VALUES('xsb',     'tcff', 11300000);
INSERT INTO SPEED_ESTIMATE VALUES('xsb',     'tcff', 11400000);
INSERT INTO SPEED_ESTIMATE VALUES('xsb',     'tcff', 11500000);
INSERT INTO SPEED_ESTIMATE VALUES('xsb',     'tcff', 11600000);
INSERT INTO SPEED_ESTIMATE VALUES('xsb',     'tcff', 11700000);
INSERT INTO SPEED_ESTIMATE VALUES('xsb',     'tcff', 11800000);
INSERT INTO SPEED_ESTIMATE VALUES('xsb',     'tcff', 11900000);
INSERT INTO SPEED_ESTIMATE VALUES('xsb',     'tcff', 12000000);
INSERT INTO SPEED_ESTIMATE VALUES('xsb',     'tcff', 12100000);
INSERT INTO SPEED_ESTIMATE VALUES('xsb',     'tcff', 12200000);
INSERT INTO SPEED_ESTIMATE VALUES('xsb',     'tcff', 12300000);
INSERT INTO SPEED_ESTIMATE VALUES('xsb',     'tcff', 12400000);
INSERT INTO SPEED_ESTIMATE VALUES('xsb',     'tcff', 12500000);
INSERT INTO SPEED_ESTIMATE VALUES('xsb',     'tcff', 12600000);
INSERT INTO SPEED_ESTIMATE VALUES('xsb',     'tcff', 12700000);
INSERT INTO SPEED_ESTIMATE VALUES('xsb',     'tcff', 12800000);
INSERT INTO SPEED_ESTIMATE VALUES('xsb',     'tcff', 12900000);
INSERT INTO SPEED_ESTIMATE VALUES('xsb',     'tcff', 13000000);

INSERT INTO SPEED_ESTIMATE VALUES('yap',     'tcff',  3000000);
INSERT INTO SPEED_ESTIMATE VALUES('yap',     'tcff',  3700000);
INSERT INTO SPEED_ESTIMATE VALUES('yap',     'tcff',  3800000);
INSERT INTO SPEED_ESTIMATE VALUES('yap',     'tcff',  3900000);
INSERT INTO SPEED_ESTIMATE VALUES('yap',     'tcff',  4000000);
INSERT INTO SPEED_ESTIMATE VALUES('yap',     'tcff',  4010000);
INSERT INTO SPEED_ESTIMATE VALUES('yap',     'tcff',  4020000);
INSERT INTO SPEED_ESTIMATE VALUES('yap',     'tcff',  4030000);
INSERT INTO SPEED_ESTIMATE VALUES('yap',     'tcff',  4040000);
INSERT INTO SPEED_ESTIMATE VALUES('yap',     'tcff',  4050000);
INSERT INTO SPEED_ESTIMATE VALUES('yap',     'tcff',  4060000);
INSERT INTO SPEED_ESTIMATE VALUES('yap',     'tcff',  4070000);
INSERT INTO SPEED_ESTIMATE VALUES('yap',     'tcff',  4080000);
INSERT INTO SPEED_ESTIMATE VALUES('yap',     'tcff',  4090000);
INSERT INTO SPEED_ESTIMATE VALUES('yap',     'tcff',  4100000);
INSERT INTO SPEED_ESTIMATE VALUES('yap',     'tcff',  4110000);
INSERT INTO SPEED_ESTIMATE VALUES('yap',     'tcff',  4120000);
INSERT INTO SPEED_ESTIMATE VALUES('yap',     'tcff',  4130000);
INSERT INTO SPEED_ESTIMATE VALUES('yap',     'tcff',  4140000);
INSERT INTO SPEED_ESTIMATE VALUES('yap',     'tcff',  4150000);
INSERT INTO SPEED_ESTIMATE VALUES('yap',     'tcff',  4160000);
INSERT INTO SPEED_ESTIMATE VALUES('yap',     'tcff',  4170000);
INSERT INTO SPEED_ESTIMATE VALUES('yap',     'tcff',  4180000);
INSERT INTO SPEED_ESTIMATE VALUES('yap',     'tcff',  4190000);
INSERT INTO SPEED_ESTIMATE VALUES('yap',     'tcff',  4200000);
INSERT INTO SPEED_ESTIMATE VALUES('yap',     'tcff',  4210000);
INSERT INTO SPEED_ESTIMATE VALUES('yap',     'tcff',  4220000);
INSERT INTO SPEED_ESTIMATE VALUES('yap',     'tcff',  4230000);
INSERT INTO SPEED_ESTIMATE VALUES('yap',     'tcff',  4240000);
INSERT INTO SPEED_ESTIMATE VALUES('yap',     'tcff',  4250000);
INSERT INTO SPEED_ESTIMATE VALUES('yap',     'tcff',  4260000);
INSERT INTO SPEED_ESTIMATE VALUES('yap',     'tcff',  4270000);
INSERT INTO SPEED_ESTIMATE VALUES('yap',     'tcff',  4280000);
INSERT INTO SPEED_ESTIMATE VALUES('yap',     'tcff',  4290000);
INSERT INTO SPEED_ESTIMATE VALUES('yap',     'tcff',  4300000);
INSERT INTO SPEED_ESTIMATE VALUES('yap',     'tcff',  4310000);
INSERT INTO SPEED_ESTIMATE VALUES('yap',     'tcff',  4320000);
INSERT INTO SPEED_ESTIMATE VALUES('yap',     'tcff',  4330000);
INSERT INTO SPEED_ESTIMATE VALUES('yap',     'tcff',  4340000);
INSERT INTO SPEED_ESTIMATE VALUES('yap',     'tcff',  4350000);
INSERT INTO SPEED_ESTIMATE VALUES('yap',     'tcff',  4360000);
INSERT INTO SPEED_ESTIMATE VALUES('yap',     'tcff',  4370000);
INSERT INTO SPEED_ESTIMATE VALUES('yap',     'tcff',  4380000);
INSERT INTO SPEED_ESTIMATE VALUES('yap',     'tcff',  4390000);
INSERT INTO SPEED_ESTIMATE VALUES('yap',     'tcff',  4400000);
INSERT INTO SPEED_ESTIMATE VALUES('yap',     'tcff',  4500000);
INSERT INTO SPEED_ESTIMATE VALUES('yap',     'tcff',  5000000);
INSERT INTO SPEED_ESTIMATE VALUES('yap',     'tcff',  6000000);
INSERT INTO SPEED_ESTIMATE VALUES('yap',     'tcff',  7000000);
INSERT INTO SPEED_ESTIMATE VALUES('yap',     'tcff',  8000000);
INSERT INTO SPEED_ESTIMATE VALUES('yap',     'tcff',  9000000);
INSERT INTO SPEED_ESTIMATE VALUES('yap',     'tcff', 10000000);
INSERT INTO SPEED_ESTIMATE VALUES('yap',     'tcff', 11000000);
INSERT INTO SPEED_ESTIMATE VALUES('yap',     'tcff', 12000000);
INSERT INTO SPEED_ESTIMATE VALUES('yap',     'tcff', 13000000);
INSERT INTO SPEED_ESTIMATE VALUES('yap',     'tcff', 14000000);
INSERT INTO SPEED_ESTIMATE VALUES('yap',     'tcff', 15000000);
INSERT INTO SPEED_ESTIMATE VALUES('yap',     'tcff', 16000000);
INSERT INTO SPEED_ESTIMATE VALUES('yap',     'tcff', 17000000);
INSERT INTO SPEED_ESTIMATE VALUES('yap',     'tcff', 18000000);
INSERT INTO SPEED_ESTIMATE VALUES('yap',     'tcff', 19000000);
INSERT INTO SPEED_ESTIMATE VALUES('yap',     'tcff', 20000000);

INSERT INTO SPEED_ESTIMATE VALUES('jena',    'tcff',  1500000);
INSERT INTO SPEED_ESTIMATE VALUES('jena',    'tcff',  1600000);
INSERT INTO SPEED_ESTIMATE VALUES('jena',    'tcff',  1700000);
INSERT INTO SPEED_ESTIMATE VALUES('jena',    'tcff',  1800000);
INSERT INTO SPEED_ESTIMATE VALUES('jena',    'tcff',  1900000);
INSERT INTO SPEED_ESTIMATE VALUES('jena',    'tcff',  2000000);
INSERT INTO SPEED_ESTIMATE VALUES('jena',    'tcff',  2100000);
INSERT INTO SPEED_ESTIMATE VALUES('jena',    'tcff',  2200000);
INSERT INTO SPEED_ESTIMATE VALUES('jena',    'tcff',  2300000);
INSERT INTO SPEED_ESTIMATE VALUES('jena',    'tcff',  2400000);
INSERT INTO SPEED_ESTIMATE VALUES('jena',    'tcff',  2500000);
INSERT INTO SPEED_ESTIMATE VALUES('jena',    'tcff',  2600000);
INSERT INTO SPEED_ESTIMATE VALUES('jena',    'tcff',  2700000);
INSERT INTO SPEED_ESTIMATE VALUES('jena',    'tcff',  2800000);
INSERT INTO SPEED_ESTIMATE VALUES('jena',    'tcff',  2900000);
INSERT INTO SPEED_ESTIMATE VALUES('jena',    'tcff',  3000000);
INSERT INTO SPEED_ESTIMATE VALUES('jena',    'tcff',  3100000);
INSERT INTO SPEED_ESTIMATE VALUES('jena',    'tcff',  3200000);
INSERT INTO SPEED_ESTIMATE VALUES('jena',    'tcff',  3300000);
INSERT INTO SPEED_ESTIMATE VALUES('jena',    'tcff',  3400000);
INSERT INTO SPEED_ESTIMATE VALUES('jena',    'tcff',  3500000);
INSERT INTO SPEED_ESTIMATE VALUES('jena',    'tcff',  4000000);
INSERT INTO SPEED_ESTIMATE VALUES('jena',    'tcff',  4400000);
INSERT INTO SPEED_ESTIMATE VALUES('jena',    'tcff',  4410000);
INSERT INTO SPEED_ESTIMATE VALUES('jena',    'tcff',  4420000);
INSERT INTO SPEED_ESTIMATE VALUES('jena',    'tcff',  4430000);
INSERT INTO SPEED_ESTIMATE VALUES('jena',    'tcff',  4440000);
INSERT INTO SPEED_ESTIMATE VALUES('jena',    'tcff',  4450000);
INSERT INTO SPEED_ESTIMATE VALUES('jena',    'tcff',  4460000);
INSERT INTO SPEED_ESTIMATE VALUES('jena',    'tcff',  4470000);
INSERT INTO SPEED_ESTIMATE VALUES('jena',    'tcff',  4480000);
INSERT INTO SPEED_ESTIMATE VALUES('jena',    'tcff',  4490000);
INSERT INTO SPEED_ESTIMATE VALUES('jena',    'tcff',  4500000);
INSERT INTO SPEED_ESTIMATE VALUES('jena',    'tcff',  4510000);
INSERT INTO SPEED_ESTIMATE VALUES('jena',    'tcff',  4520000);
INSERT INTO SPEED_ESTIMATE VALUES('jena',    'tcff',  4530000);
INSERT INTO SPEED_ESTIMATE VALUES('jena',    'tcff',  4540000);
INSERT INTO SPEED_ESTIMATE VALUES('jena',    'tcff',  4550000);
INSERT INTO SPEED_ESTIMATE VALUES('jena',    'tcff',  4560000);
INSERT INTO SPEED_ESTIMATE VALUES('jena',    'tcff',  4570000);
INSERT INTO SPEED_ESTIMATE VALUES('jena',    'tcff',  4580000);
INSERT INTO SPEED_ESTIMATE VALUES('jena',    'tcff',  4590000);
INSERT INTO SPEED_ESTIMATE VALUES('jena',    'tcff',  4600000);
INSERT INTO SPEED_ESTIMATE VALUES('jena',    'tcff',  4700000);
INSERT INTO SPEED_ESTIMATE VALUES('jena',    'tcff',  4800000);
INSERT INTO SPEED_ESTIMATE VALUES('jena',    'tcff',  4900000);
INSERT INTO SPEED_ESTIMATE VALUES('jena',    'tcff',  5000000);
INSERT INTO SPEED_ESTIMATE VALUES('jena',    'tcff',  5100000);
INSERT INTO SPEED_ESTIMATE VALUES('jena',    'tcff',  5200000);
INSERT INTO SPEED_ESTIMATE VALUES('jena',    'tcff',  5300000);
INSERT INTO SPEED_ESTIMATE VALUES('jena',    'tcff',  6000000);

INSERT INTO SPEED_ESTIMATE VALUES('sqlite3', 'tcff',   700000);
INSERT INTO SPEED_ESTIMATE VALUES('sqlite3', 'tcff',  1000000);
INSERT INTO SPEED_ESTIMATE VALUES('sqlite3', 'tcff',  1300000);
INSERT INTO SPEED_ESTIMATE VALUES('sqlite3', 'tcff',  1310000);
INSERT INTO SPEED_ESTIMATE VALUES('sqlite3', 'tcff',  1320000);
INSERT INTO SPEED_ESTIMATE VALUES('sqlite3', 'tcff',  1330000);
INSERT INTO SPEED_ESTIMATE VALUES('sqlite3', 'tcff',  1340000);
INSERT INTO SPEED_ESTIMATE VALUES('sqlite3', 'tcff',  1350000);
INSERT INTO SPEED_ESTIMATE VALUES('sqlite3', 'tcff',  1360000);
INSERT INTO SPEED_ESTIMATE VALUES('sqlite3', 'tcff',  1370000);
INSERT INTO SPEED_ESTIMATE VALUES('sqlite3', 'tcff',  1380000);
INSERT INTO SPEED_ESTIMATE VALUES('sqlite3', 'tcff',  1390000);
INSERT INTO SPEED_ESTIMATE VALUES('sqlite3', 'tcff',  1400000);
INSERT INTO SPEED_ESTIMATE VALUES('sqlite3', 'tcff',  1410000);
INSERT INTO SPEED_ESTIMATE VALUES('sqlite3', 'tcff',  1420000);
INSERT INTO SPEED_ESTIMATE VALUES('sqlite3', 'tcff',  1430000);
INSERT INTO SPEED_ESTIMATE VALUES('sqlite3', 'tcff',  1440000);
INSERT INTO SPEED_ESTIMATE VALUES('sqlite3', 'tcff',  1450000);
INSERT INTO SPEED_ESTIMATE VALUES('sqlite3', 'tcff',  1460000);
INSERT INTO SPEED_ESTIMATE VALUES('sqlite3', 'tcff',  1470000);
INSERT INTO SPEED_ESTIMATE VALUES('sqlite3', 'tcff',  1480000);
INSERT INTO SPEED_ESTIMATE VALUES('sqlite3', 'tcff',  1490000);
INSERT INTO SPEED_ESTIMATE VALUES('sqlite3', 'tcff',  1500000);
INSERT INTO SPEED_ESTIMATE VALUES('sqlite3', 'tcff',  1550000);
INSERT INTO SPEED_ESTIMATE VALUES('sqlite3', 'tcff',  1600000);
INSERT INTO SPEED_ESTIMATE VALUES('sqlite3', 'tcff',  2000000);
INSERT INTO SPEED_ESTIMATE VALUES('sqlite3', 'tcff',  2500000);
INSERT INTO SPEED_ESTIMATE VALUES('sqlite3', 'tcff',  3000000);

-- ============================================================================
-- Full Statistical Evaluation of Run Data:
-- ============================================================================

CREATE VIEW RUN_STATISTICS AS
SELECT BENCH, IMPL, FILE_ID, MACHINE, PROG, VER, WITH_OUTPUT,
	-- Load Time:
	ROUND(AVG(LOAD_TIME),3) AS AVG_LOAD_TIME,
	MIN(LOAD_TIME) AS MIN_LOAD_TIME,
	MAX(LOAD_TIME) AS MAX_LOAD_TIME,
	MAX(LOAD_TIME)-MIN(LOAD_TIME) AS DIFF_LOAD_TIME,
	STDDEV(LOAD_TIME) AS SDEV_LOAD_TIME,
	-- Exec Time:
	ROUND(AVG(EXEC_TIME),3) AS AVG_EXEC_TIME,
	MIN(EXEC_TIME) AS MIN_EXEC_TIME,
	MAX(EXEC_TIME) AS MAX_EXEC_TIME,
	MAX(EXEC_TIME)-MIN(EXEC_TIME) AS DIFF_EXEC_TIME,
	STDDEV(EXEC_TIME) AS SDEV_EXEC_TIME,
	-- Total Time:
	ROUND(AVG(TOTAL_TIME),3) AS AVG_TOTAL_TIME,
	MIN(TOTAL_TIME) AS MIN_TOTAL_TIME,
	MAX(TOTAL_TIME) AS MAX_TOTAL_TIME,
	MAX(TOTAL_TIME)-MIN(TOTAL_TIME) AS DIFF_TOTAL_TIME,
	STDDEV(TOTAL_TIME) AS SDEV_TOTAL_TIME,
	-- User Mode Time:
	ROUND(AVG(USER_TIME),3) AS AVG_USER_TIME,
	MIN(USER_TIME) AS MIN_USER_TIME,
	MAX(USER_TIME) AS MAX_USER_TIME,
	MAX(USER_TIME)-MIN(USER_TIME) AS DIFF_USER_TIME,
	STDDEV(USER_TIME) AS SDEV_USER_TIME,
	-- System Mode Time:
	ROUND(AVG(SYS_TIME),3) AS AVG_SYS_TIME,
	MIN(SYS_TIME) AS MIN_SYS_TIME,
	MAX(SYS_TIME) AS MAX_SYS_TIME,
	MAX(SYS_TIME)-MIN(SYS_TIME) AS DIFF_SYS_TIME,
	STDDEV(SYS_TIME) AS SDEV_SYS_TIME,
	-- CPU Time:
	ROUND(AVG(SYS_TIME+USER_TIME),3) AS AVG_CPU_TIME,
	MIN(SYS_TIME+USER_TIME) AS MIN_CPU_TIME,
	MAX(SYS_TIME+USER_TIME) AS MAX_CPU_TIME,
	MAX(SYS_TIME+USER_TIME)-MIN(SYS_TIME+USER_TIME) AS DIFF_CPU_TIME,
	STDDEV(SYS_TIME+USER_TIME) AS SDEV_CPU_TIME,
	-- Real Time:
	ROUND(AVG(REAL_TIME),3) AS AVG_REAL_TIME,
	MIN(REAL_TIME) AS MIN_REAL_TIME,
	MAX(REAL_TIME) AS MAX_REAL_TIME,
	MAX(REAL_TIME)-MIN(REAL_TIME) AS DIFF_REAL_TIME,
	STDDEV(REAL_TIME) AS SDEV_REAL_TIME,
	-- Memory:
	ROUND(AVG(MEM),3) AS AVG_MEM,
	MIN(MEM) AS MIN_MEM,
	MAX(MEM) AS MAX_MEM,
	MAX(MEM)-MIN(MEM) AS DIFF_MEM,
	STDDEV(MEM) AS SDEV_MEM,
	-- Number of Runs:
	COUNT(*) AS NUM_BENCH_RUN
FROM BENCH_RUN
WHERE STATUS <> 'FAILED' AND STATUS <> 'OUTL'
GROUP BY MACHINE, PROG, VER, BENCH, IMPL, FILE_ID, WITH_OUTPUT
ORDER BY MACHINE, PROG, VER, BENCH, IMPL, FILE_ID, WITH_OUTPUT;

-- ============================================================================
-- Statistical Evaluation of Run Data, without STDDEV - This works for SQLite:
-- ============================================================================

CREATE VIEW RUN_STAT AS
SELECT BENCH, IMPL, FILE_ID, MACHINE, PROG, VER, WITH_OUTPUT,
	-- Load Time:
	ROUND(AVG(LOAD_TIME),3) AS AVG_LOAD_TIME,
	MIN(LOAD_TIME) AS MIN_LOAD_TIME,
	MAX(LOAD_TIME) AS MAX_LOAD_TIME,
	MAX(LOAD_TIME)-MIN(LOAD_TIME) AS DIFF_LOAD_TIME,
	-- Exec Time:
	ROUND(AVG(EXEC_TIME),3) AS AVG_EXEC_TIME,
	MIN(EXEC_TIME) AS MIN_EXEC_TIME,
	MAX(EXEC_TIME) AS MAX_EXEC_TIME,
	MAX(EXEC_TIME)-MIN(EXEC_TIME) AS DIFF_EXEC_TIME,
	-- Total Time:
	ROUND(AVG(TOTAL_TIME),3) AS AVG_TOTAL_TIME,
	MIN(TOTAL_TIME) AS MIN_TOTAL_TIME,
	MAX(TOTAL_TIME) AS MAX_TOTAL_TIME,
	MAX(TOTAL_TIME)-MIN(TOTAL_TIME) AS DIFF_TOTAL_TIME,
	-- User Mode Time:
	ROUND(AVG(USER_TIME),3) AS AVG_USER_TIME,
	MIN(USER_TIME) AS MIN_USER_TIME,
	MAX(USER_TIME) AS MAX_USER_TIME,
	MAX(USER_TIME)-MIN(USER_TIME) AS DIFF_USER_TIME,
	-- System Mode Time:
	ROUND(AVG(SYS_TIME),3) AS AVG_SYS_TIME,
	MIN(SYS_TIME) AS MIN_SYS_TIME,
	MAX(SYS_TIME) AS MAX_SYS_TIME,
	MAX(SYS_TIME)-MIN(SYS_TIME) AS DIFF_SYS_TIME,
	-- CPU Time:
	ROUND(AVG(SYS_TIME+USER_TIME),3) AS AVG_CPU_TIME,
	MIN(SYS_TIME+USER_TIME) AS MIN_CPU_TIME,
	MAX(SYS_TIME+USER_TIME) AS MAX_CPU_TIME,
	MAX(SYS_TIME+USER_TIME)-MIN(SYS_TIME+USER_TIME) AS DIFF_CPU_TIME,
	-- Real Time:
	ROUND(AVG(REAL_TIME),3) AS AVG_REAL_TIME,
	MIN(REAL_TIME) AS MIN_REAL_TIME,
	MAX(REAL_TIME) AS MAX_REAL_TIME,
	MAX(REAL_TIME)-MIN(REAL_TIME) AS DIFF_REAL_TIME,
	-- Memory:
	ROUND(AVG(MEM),3) AS AVG_MEM,
	MIN(MEM) AS MIN_MEM,
	MAX(MEM) AS MAX_MEM,
	MAX(MEM)-MIN(MEM) AS DIFF_MEM,
	-- Number of Runs:
	COUNT(*) AS NUM_BENCH_RUN
FROM BENCH_RUN
WHERE STATUS <> 'FAILED' AND STATUS <> 'OUTL'
GROUP BY MACHINE, PROG, VER, BENCH, IMPL, FILE_ID, WITH_OUTPUT
ORDER BY MACHINE, PROG, VER, BENCH, IMPL, FILE_ID, WITH_OUTPUT;

-- ============================================================================
-- Benchmark Results for Current Version on the Standard Machine without Output:
-- ============================================================================

CREATE VIEW RESULT AS
SELECT R.BENCH AS BENCH,
	R.IMPL AS IMPL,
	R.FILE_ID AS FILE_ID,
	R.PROG AS PROG,
	R.AVG_LOAD_TIME AS LOAD_T,
	R.AVG_EXEC_TIME AS EXEC_T,
	R.AVG_TOTAL_TIME AS TOTAL_T,
	R.AVG_CPU_TIME AS CPU_T,
	R.AVG_REAL_TIME AS REAL_T,
	R.AVG_MEM AS MEM_MB
FROM   RUN_STAT R, PROG_VER V, MACHINE M, DATA_FILE F
WHERE  R.MACHINE = M.MACHINE AND V.MACHINE = R.MACHINE AND M.STATUS = 'S'
AND    R.PROG = V.PROG AND R.VER = V.VER AND V.STATUS = 'C'
AND    R.WITH_OUTPUT = 'N'
AND    R.FILE_ID = F.FILE_ID
ORDER BY R.BENCH, R.IMPL, F.SEQ_NO, R.PROG;


-- ============================================================================
-- Benchmark Results (Current Ver., Std. Machine, No Output, Best Settings):
-- ============================================================================

CREATE VIEW BEST_RESULT AS
SELECT R.BENCH AS BENCH,
	R.IMPL AS IMPL,
	R.FILE_ID AS FILE_ID,
	R.PROG AS PROG,
	R.AVG_LOAD_TIME AS LOAD_T,
	R.AVG_EXEC_TIME AS EXEC_T,
	R.AVG_TOTAL_TIME AS TOTAL_T,
	R.AVG_CPU_TIME AS CPU_T,
	R.AVG_REAL_TIME AS REAL_T,
	R.AVG_MEM AS MEM_MB
FROM   RUN_STAT R, PROG_VER V, MACHINE M, BENCH_IMPL I, DATA_FILE F
WHERE  R.MACHINE = M.MACHINE AND V.MACHINE = R.MACHINE AND M.STATUS = 'S'
AND    R.PROG = V.PROG AND R.VER = V.VER AND V.STATUS = 'C'
AND    R.WITH_OUTPUT = 'N'
AND    I.PROG = R.PROG AND I.BENCH = R.BENCH AND I.IMPL = R.IMPL
AND    I.STATUS = 'B'
AND    R.FILE_ID = F.FILE_ID
ORDER BY R.BENCH, R.IMPL, F.SEQ_NO, R.PROG;

-- ============================================================================
-- Benchmark Results for BAM (Current Version, Standard Machine, No Output):
-- ============================================================================

CREATE VIEW BAM AS
SELECT BENCH,
	IMPL,
	FILE_ID AS FILE,
	LOAD_T,
	EXEC_T,
	-- TOTAL_T,
	CPU_T,
	REAL_T,
	MEM_MB
FROM RESULT
WHERE PROG = 'bam';

-- ============================================================================
-- Benchmark Results for XSB (Current Version, Standard Machine, No Output):
-- ============================================================================

CREATE VIEW XSB AS
SELECT BENCH,
	IMPL,
	FILE_ID AS FILE,
	LOAD_T,
	EXEC_T,
	-- TOTAL_T,
	CPU_T,
	REAL_T,
	MEM_MB
FROM RESULT
WHERE PROG = 'xsb';

-- ============================================================================
-- Benchmark Results for YAP (Current Version, Standard Machine, No Output):
-- ============================================================================

CREATE VIEW YAP AS
SELECT BENCH,
	IMPL,
	FILE_ID AS FILE,
	LOAD_T,
	EXEC_T,
	-- TOTAL_T,
	CPU_T,
	REAL_T,
	MEM_MB
FROM RESULT
WHERE PROG = 'yap';

-- ============================================================================
-- Benchmark Results for Jena (Current Version, Standard Machine, No Output):
-- ============================================================================

CREATE VIEW JENA AS
SELECT BENCH,
	IMPL,
	FILE_ID,
	-- LOAD_T,
	-- EXEC_T,
	-- TOTAL_T,
	CPU_T,
	REAL_T,
	MEM_MB
FROM RESULT
WHERE PROG = 'jena';

-- ============================================================================
-- Benchmark Results for DataScript (Current Ver., Standard Machine, No Output):
-- ============================================================================

CREATE VIEW DATASCRIPT AS
SELECT BENCH,
	IMPL,
	FILE_ID AS FILE,
	-- LOAD_T,
	-- EXEC_T,
	-- TOTAL_T,
	CPU_T,
	REAL_T,
	MEM_MB
FROM RESULT
WHERE PROG = 'datascript';

-- ============================================================================
-- Benchmark Results for SQLite (Current Ver., Standard Machine, No Output):
-- ============================================================================

CREATE VIEW SQLITE AS
SELECT BENCH,
	IMPL,
	FILE_ID AS FILE,
	-- LOAD_T,
	-- EXEC_T,
	-- TOTAL_T,
	CPU_T,
	REAL_T,
	MEM_MB
FROM RESULT
WHERE PROG = 'sqlite3';

-- ============================================================================
-- Best Option Settings (in Real Time) for a Program for a Benchmark:
-- ============================================================================

-- This looks only at the runtimes of the standard program version
-- on the standard machine.

CREATE VIEW BEST_IMPL_REAL AS
SELECT	R.BENCH,
	R.FILE_ID,
	R.PROG,
	R.IMPL,
	R.REAL_T,
	I.STATUS
FROM	RESULT R, BENCHMARK B, DATA_FILE F, BENCH_IMPL I
WHERE   R.BENCH = B.BENCH AND R.FILE_ID = F.FILE_ID
AND	I.BENCH = R.BENCH AND I.IMPL = R.IMPL AND I.PROG = R.PROG
AND NOT EXISTS(SELECT * FROM RESULT R2
		WHERE R2.BENCH = R.BENCH AND R2.PROG = R.PROG
		AND   R2.FILE_ID = R.FILE_ID
		AND   R2.REAL_T < R.REAL_T)
-- Eliminate trivial tests with run time 0: All options are best.
AND EXISTS(SELECT * FROM RESULT R3
		WHERE R3.BENCH = R.BENCH AND R3.PROG = R.PROG
		AND   R3.FILE_ID = R.FILE_ID
		AND   R3.REAL_T > 0)
ORDER BY B.SEQ_NO, F.SEQ_NO;


-- ============================================================================
-- Best Option Settings (in CPU Time) for a Program for a Benchmark:
-- ============================================================================

-- This looks only at the runtimes of the standard program version
-- on the standard machine.

CREATE VIEW BEST_IMPL_CPU AS
SELECT	R.BENCH,
	R.FILE_ID,
	R.PROG,
	R.IMPL,
	R.CPU_T,
	I.STATUS
FROM	RESULT R, BENCHMARK B, DATA_FILE F, BENCH_IMPL I
WHERE   R.BENCH = B.BENCH AND R.FILE_ID = F.FILE_ID
AND	I.BENCH = R.BENCH AND I.IMPL = R.IMPL AND I.PROG = R.PROG
AND NOT EXISTS(SELECT * FROM RESULT R2
		WHERE R2.BENCH = R.BENCH AND R2.PROG = R.PROG
		AND   R2.FILE_ID = R.FILE_ID
		AND   R2.CPU_T < R.CPU_T)
-- Eliminate trivial tests with run time 0: All options are best.
AND EXISTS(SELECT * FROM RESULT R3
		WHERE R3.BENCH = R.BENCH AND R3.PROG = R.PROG
		AND   R3.FILE_ID = R.FILE_ID
		AND   R3.CPU_T > 0)
ORDER BY B.SEQ_NO, F.SEQ_NO;


-- ============================================================================
-- Plot of Real Time vs. Cost for TCFF Benchmark:
-- ============================================================================

-- This looks only at the runtimes of the standard program version,
-- best implementation, on the standard machine.

CREATE VIEW PLOT_REAL AS
SELECT	R.PROG,
	R.FILE_ID,
	F.SEQ_NO AS NO,
	G.COST,
	R.REAL_T,
	'"' || R.FILE_ID || '","' || G.COST || '","' || R.REAL_T || '"' AS CSV,
	R.FILE_ID || '	' || G.COST || '	' || R.REAL_T AS TSV
FROM	RESULT R, DATA_FILE F, GRAPH_DATA G, BENCH_IMPL I
WHERE   R.BENCH = 'tcff' AND R.FILE_ID = F.FILE_ID AND R.FILE_ID = G.FILE_ID
AND	I.BENCH = R.BENCH AND I.IMPL = R.IMPL AND I.PROG = R.PROG
AND	I.STATUS = 'B'
UNION
SELECT  DISTINCT P.PROG, '', -1, 0 AS COST, 0 AS REAL_T,
	'"df","cost","tavg"' as CSV,
	'df	cost	tavg' as TSV
FROM	PROGRAM P
WHERE	EXISTS(SELECT * FROM RESULT R
		WHERE R.BENCH = 'tcff' AND R.PROG = P.PROG)
ORDER BY PROG, NO;


-- ============================================================================
-- Plot of CPU Time vs. Cost for TCFF Benchmark:
-- ============================================================================

-- This looks only at the runtimes of the standard program version,
-- best implementation, on the standard machine.

CREATE VIEW PLOT_CPU AS
SELECT	R.PROG,
	R.FILE_ID,
	F.SEQ_NO AS NO,
	G.COST,
	R.CPU_T,
	'"' || R.FILE_ID || '","' || G.COST || '","' || R.CPU_T || '"' AS CSV,
	R.FILE_ID || '	' || G.COST || '	' || R.CPU_T AS TSV
FROM	RESULT R, DATA_FILE F, GRAPH_DATA G, BENCH_IMPL I
WHERE   R.BENCH = 'tcff' AND R.FILE_ID = F.FILE_ID AND R.FILE_ID = G.FILE_ID
AND	I.BENCH = R.BENCH AND I.IMPL = R.IMPL AND I.PROG = R.PROG
AND	I.STATUS = 'B'
UNION
SELECT  DISTINCT P.PROG, '', -1, 0 AS COST, 0 AS CPU_T,
	'"df","cost","tavg"' as CSV,
	'df	cost	tavg' as TSV
FROM	PROGRAM P
WHERE	EXISTS(SELECT * FROM RESULT R
		WHERE R.BENCH = 'tcff' AND R.PROG = P.PROG)
ORDER BY PROG, NO;

-- ============================================================================
-- Plot of Memory vs. Number of Tuples for TCFF Benchmark:
-- ============================================================================

-- This looks only at the runtimes of the standard program version,
-- best implementation, on the standard machine.

CREATE VIEW PLOT_MEM AS
SELECT	R.PROG,
	R.FILE_ID,
	F.SEQ_NO AS NO,
	G.NUM_EDGES + G.TC_SIZE AS FACTS,
	R.MEM_MB,
	'"' || R.FILE_ID || '","' || (G.NUM_EDGES+G.TC_SIZE) || '","' ||
		R.MEM_MB || '"' AS CSV,
	R.FILE_ID || '	' || (G.NUM_EDGES+G.TC_SIZE) || '	' ||
		R.MEM_MB AS TSV
FROM	RESULT R, DATA_FILE F, GRAPH_DATA G, BENCH_IMPL I
WHERE   R.BENCH = 'tcff' AND R.FILE_ID = F.FILE_ID AND R.FILE_ID = G.FILE_ID
AND	I.BENCH = R.BENCH AND I.IMPL = R.IMPL AND I.PROG = R.PROG
AND	I.STATUS = 'B'
UNION
SELECT  DISTINCT P.PROG, '', -1, 0 AS FACTS, 0 AS CPU_T,
	'"df","facts","mem"' as CSV,
	'df	facts	mem' as TSV
FROM	PROGRAM P
WHERE	EXISTS(SELECT * FROM RESULT R
		WHERE R.BENCH = 'tcff' AND R.PROG = P.PROG)
ORDER BY PROG, NO;

-- ============================================================================
-- Benchmark Results for TCFF (Comparison of Various Systems on CPU Time):
-- ============================================================================

-- This looks only at the runtimes of the standard program version,
-- best implementation, on the standard machine.

CREATE VIEW TCFF_CPU AS
SELECT	BAM.FILE_ID,
	BAM.CPU_T AS BAM_CPU,
	XSB.CPU_T AS XSB_CPU,
	(CASE WHEN BAM.CPU_T=0 THEN NULL ELSE ROUND(XSB.CPU_T/BAM.CPU_T,1) END)
		AS XSB_F,
	YAP.CPU_T AS YAP_CPU,
	(CASE WHEN BAM.CPU_T=0 THEN NULL ELSE ROUND(YAP.CPU_T/BAM.CPU_T,1) END)
		AS YAP_F,
	JENA.CPU_T AS JENA_CPU,
	(CASE WHEN BAM.CPU_T=0 THEN NULL ELSE ROUND(JENA.CPU_T/BAM.CPU_T,1) END)
		AS JENA_F,
	SQLITE.CPU_T AS SQLITE_CPU,
	(CASE WHEN BAM.CPU_T=0 THEN NULL ELSE ROUND(SQLITE.CPU_T/BAM.CPU_T,1)
		END) AS SQLITE_F
FROM	(SELECT * FROM BEST_RESULT WHERE PROG='bam'AND BENCH='tcff') BAM
	LEFT JOIN
	(SELECT * FROM BEST_RESULT WHERE PROG='xsb' AND BENCH='tcff') XSB
	ON XSB.FILE_ID = BAM.FILE_ID
	LEFT JOIN
	(SELECT * FROM BEST_RESULT WHERE PROG='yap' AND BENCH='tcff') YAP
	ON YAP.FILE_ID = BAM.FILE_ID
	LEFT JOIN
	(SELECT * FROM BEST_RESULT WHERE PROG='jena' AND BENCH='tcff') JENA
	ON JENA.FILE_ID = BAM.FILE_ID
	LEFT JOIN
	(SELECT * FROM BEST_RESULT WHERE PROG='sqlite3' AND BENCH='tcff') SQLITE
	ON SQLITE.FILE_ID = BAM.FILE_ID
	JOIN DATA_FILE F
        ON BAM.FILE_ID = F.FILE_ID
ORDER BY F.SEQ_NO;

-- ============================================================================
-- Benchmark Results for TCFF (Comparison of Various Systems on REAL Time):
-- ============================================================================

-- This looks only at the runtimes of the standard program version,
-- best implementation, on the standard machine.

CREATE VIEW TCFF_REAL AS
SELECT	BAM.FILE_ID,
	BAM.REAL_T AS BAM_REAL,
	XSB.REAL_T AS XSB_REAL,
	(CASE WHEN BAM.REAL_T=0 THEN NULL
		ELSE ROUND(XSB.REAL_T/BAM.REAL_T,1) END)
		AS XSB_F,
	YAP.REAL_T AS YAP_REAL,
	(CASE WHEN BAM.REAL_T=0 THEN NULL
		ELSE ROUND(YAP.REAL_T/BAM.REAL_T,1) END)
		AS YAP_F,
	JENA.REAL_T AS JENA_REAL,
	(CASE WHEN BAM.REAL_T=0 THEN NULL
		ELSE ROUND(JENA.REAL_T/BAM.REAL_T,1) END)
		AS JENA_F,
	SQLITE.REAL_T AS SQLITE_REAL,
	(CASE WHEN BAM.REAL_T=0 THEN NULL
		ELSE ROUND(SQLITE.REAL_T/BAM.REAL_T,1) END)
		AS SQLITE_F
FROM	(SELECT * FROM BEST_RESULT WHERE PROG='bam'AND BENCH='tcff') BAM
	LEFT JOIN
	(SELECT * FROM BEST_RESULT WHERE PROG='xsb' AND BENCH='tcff') XSB
	ON XSB.FILE_ID = BAM.FILE_ID
	LEFT JOIN
	(SELECT * FROM BEST_RESULT WHERE PROG='yap' AND BENCH='tcff') YAP
	ON YAP.FILE_ID = BAM.FILE_ID
	LEFT JOIN
	(SELECT * FROM BEST_RESULT WHERE PROG='jena' AND BENCH='tcff') JENA
	ON JENA.FILE_ID = BAM.FILE_ID
	LEFT JOIN
	(SELECT * FROM BEST_RESULT WHERE PROG='sqlite3' AND BENCH='tcff') SQLITE
	ON SQLITE.FILE_ID = BAM.FILE_ID
	JOIN DATA_FILE F
        ON BAM.FILE_ID = F.FILE_ID
ORDER BY F.SEQ_NO;

-- ============================================================================
-- Benchmark Results for TCFF (Comparison of Various Systems on Memory):
-- ============================================================================

-- This looks only at the runtimes of the standard program version,
-- best implementation, on the standard machine.

CREATE VIEW TCFF_MEM AS
SELECT	BAM.FILE_ID,
	BAM.MEM_MB AS BAM_MEM,
	XSB.MEM_MB AS XSB_MEM,
	(CASE WHEN BAM.MEM_MB=0 THEN NULL
		ELSE ROUND(XSB.MEM_MB/BAM.MEM_MB,1) END)
		AS XSB_F,
	YAP.MEM_MB AS YAP_MEM,
	(CASE WHEN BAM.MEM_MB=0 THEN NULL
		ELSE ROUND(YAP.MEM_MB/BAM.MEM_MB,1) END)
		AS YAP_F,
	JENA.MEM_MB AS JENA_MEM,
	(CASE WHEN BAM.MEM_MB=0 THEN NULL
		ELSE ROUND(JENA.MEM_MB/BAM.MEM_MB,1) END)
		AS JENA_F,
	SQLITE.MEM_MB AS SQLITE_MEM,
	(CASE WHEN BAM.MEM_MB=0 THEN NULL
		ELSE ROUND(SQLITE.MEM_MB/BAM.MEM_MB,1) END)
		AS SQLITE_F
FROM	(SELECT * FROM BEST_RESULT WHERE PROG='bam'AND BENCH='tcff') BAM
	LEFT JOIN
	(SELECT * FROM BEST_RESULT WHERE PROG='xsb' AND BENCH='tcff') XSB
	ON XSB.FILE_ID = BAM.FILE_ID
	LEFT JOIN
	(SELECT * FROM BEST_RESULT WHERE PROG='yap' AND BENCH='tcff') YAP
	ON YAP.FILE_ID = BAM.FILE_ID
	LEFT JOIN
	(SELECT * FROM BEST_RESULT WHERE PROG='jena' AND BENCH='tcff') JENA
	ON JENA.FILE_ID = BAM.FILE_ID
	LEFT JOIN
	(SELECT * FROM BEST_RESULT WHERE PROG='sqlite3' AND BENCH='tcff') SQLITE
	ON SQLITE.FILE_ID = BAM.FILE_ID
	JOIN DATA_FILE F
        ON BAM.FILE_ID = F.FILE_ID
ORDER BY F.SEQ_NO;

-- ============================================================================
-- Benchmark Results for TCFF (Comparison of Various Systems on REAL Time):
-- ============================================================================

-- This looks only at the runtimes of the standard program version,
-- best implementation, on the standard machine.

CREATE VIEW TCFF_REAL_LATEX AS
SELECT	BAM.FILE_ID,
	RPAD(REPLACE(BAM.FILE_ID, '_', '\_'), 12) || '&' ||
	TO_CHAR(BAM.REAL_T, '9990.999') || '&' ||
	(CASE WHEN XSB.REAL_T IS NULL THEN '         &       ' ELSE
		(TO_CHAR(XSB.REAL_T, '9990.999') || '&' ||
		(CASE WHEN BAM.REAL_T=0 THEN '       '
			ELSE TO_CHAR(XSB.REAL_T/BAM.REAL_T, '9990.9') END))
	END) || '&' ||
	(CASE WHEN YAP.REAL_T IS NULL THEN '         &       ' ELSE
		(TO_CHAR(YAP.REAL_T, '9990.999') || '&' ||
		(CASE WHEN BAM.REAL_T=0 THEN '       '
			ELSE TO_CHAR(YAP.REAL_T/BAM.REAL_T, '9990.9') END))
	END) || '&' ||
	(CASE WHEN JENA.REAL_T IS NULL THEN '         &       ' ELSE
		(TO_CHAR(JENA.REAL_T,'9990.999') || '&' ||
		(CASE WHEN BAM.REAL_T=0 THEN '       '
			ELSE TO_CHAR(JENA.REAL_T/BAM.REAL_T, '9990.9') END))
	END) || '&' ||
	(CASE WHEN SQLITE.REAL_T IS NULL THEN '         &       ' ELSE
		(TO_CHAR(SQLITE.REAL_T,'9990.999') || '&' ||
		(CASE WHEN BAM.REAL_T=0 THEN '       '
			ELSE TO_CHAR(SQLITE.REAL_T/BAM.REAL_T, '9990.9') END))
	END) || '\\'
	AS LATEX
FROM	(SELECT * FROM BEST_RESULT WHERE PROG='bam'AND BENCH='tcff') BAM
	LEFT JOIN
	(SELECT * FROM BEST_RESULT WHERE PROG='xsb' AND BENCH='tcff') XSB
	ON XSB.FILE_ID = BAM.FILE_ID
	LEFT JOIN
	(SELECT * FROM BEST_RESULT WHERE PROG='yap' AND BENCH='tcff') YAP
	ON YAP.FILE_ID = BAM.FILE_ID
	LEFT JOIN
	(SELECT * FROM BEST_RESULT WHERE PROG='jena' AND BENCH='tcff') JENA
	ON JENA.FILE_ID = BAM.FILE_ID
	LEFT JOIN
	(SELECT * FROM BEST_RESULT WHERE PROG='sqlite3' AND BENCH='tcff') SQLITE
	ON SQLITE.FILE_ID = BAM.FILE_ID
	JOIN DATA_FILE F
	ON F.FILE_ID = BAM.FILE_ID
ORDER BY F.SEQ_NO;

-- ============================================================================
-- Benchmark Results for TCFF (Comparison of Various Systems on CPU Time):
-- ============================================================================

-- This looks only at the runtimes of the standard program version,
-- best implementation, on the standard machine.

CREATE VIEW TCFF_CPU_LATEX AS
SELECT	BAM.FILE_ID,
	RPAD(REPLACE(BAM.FILE_ID, '_', '\_'), 12) || '&' ||
	TO_CHAR(BAM.CPU_T, '9990.999') || '&' ||
	(CASE WHEN XSB.CPU_T IS NULL THEN '         &       ' ELSE
		(TO_CHAR(XSB.CPU_T, '9990.999') || '&' ||
		(CASE WHEN BAM.CPU_T=0 THEN '       '
			ELSE TO_CHAR(XSB.CPU_T/BAM.CPU_T, '9990.9') END))
	END) || '&' ||
	(CASE WHEN YAP.CPU_T IS NULL THEN '         &       ' ELSE
		(TO_CHAR(YAP.CPU_T, '9990.999') || '&' ||
		(CASE WHEN BAM.CPU_T=0 THEN '       '
			ELSE TO_CHAR(YAP.CPU_T/BAM.CPU_T, '9990.9') END))
	END) || '&' ||
	(CASE WHEN JENA.CPU_T IS NULL THEN '         &       ' ELSE
		(TO_CHAR(JENA.CPU_T,'9990.999') || '&' ||
		(CASE WHEN BAM.CPU_T=0 THEN '       '
			ELSE TO_CHAR(JENA.CPU_T/BAM.CPU_T, '9990.9') END))
	END) || '&' ||
	(CASE WHEN SQLITE.CPU_T IS NULL THEN '         &       ' ELSE
		(TO_CHAR(SQLITE.CPU_T,'9990.999') || '&' ||
		(CASE WHEN BAM.CPU_T=0 THEN '       '
			ELSE TO_CHAR(SQLITE.CPU_T/BAM.CPU_T, '9990.9') END))
	END) || '\\'
	AS LATEX
FROM	(SELECT * FROM BEST_RESULT WHERE PROG='bam'AND BENCH='tcff') BAM
	LEFT JOIN
	(SELECT * FROM BEST_RESULT WHERE PROG='xsb' AND BENCH='tcff') XSB
	ON XSB.FILE_ID = BAM.FILE_ID
	LEFT JOIN
	(SELECT * FROM BEST_RESULT WHERE PROG='yap' AND BENCH='tcff') YAP
	ON YAP.FILE_ID = BAM.FILE_ID
	LEFT JOIN
	(SELECT * FROM BEST_RESULT WHERE PROG='jena' AND BENCH='tcff') JENA
	ON JENA.FILE_ID = BAM.FILE_ID
	LEFT JOIN
	(SELECT * FROM BEST_RESULT WHERE PROG='sqlite3' AND BENCH='tcff') SQLITE
	ON SQLITE.FILE_ID = BAM.FILE_ID
	JOIN DATA_FILE F
	ON F.FILE_ID = BAM.FILE_ID
ORDER BY F.SEQ_NO;

-- ============================================================================
-- Graph Data as LaTeX:
-- ============================================================================

CREATE VIEW GRAPH_DATA_LATEX AS
SELECT	G.FILE_ID,
	REPLACE(G.FILE_ID, '_', '\_') || '&' ||
	G.NUM_NODES || '&' || G.NUM_EDGES || '&' ||
	G.MIN_IN_DEGREE || '&' || G.MAX_IN_DEGREE || '&' ||
	G.MIN_OUT_DEGREE || '&' || G.MAX_OUT_DEGREE || '&' ||
	(CASE WHEN CYCLES='Y' THEN 'yes'
		WHEN CYCLES='N' THEN 'no'
		ELSE '?' END) || '&' ||
	G.TC_SIZE || '&' ||
	(CASE WHEN G.LIMIT_REACHED='Y' THEN '$>$' ELSE '' END) ||
		G.NUM_ITER || '&' ||
		G.COST || '\\'
		AS LATEX
FROM	GRAPH_DATA G, DATA_FILE F
WHERE   G.FILE_ID = F.FILE_ID
ORDER BY F.SEQ_NO;

-- ============================================================================
-- Detect Outliers with regard to Real Time:
-- ============================================================================

CREATE VIEW OUTLIER_REAL AS
SELECT R.REAL_TIME, S.AVG_REAL_TIME, R.PROG, R.VER, R.STATUS,
       R.BENCH, R.IMPL, R.FILE_ID, R.MACHINE, R.WITH_OUTPUT,
       R.RUN_DATE, R.RUN_NO
FROM   BENCH_RUN R, RUN_STAT S
WHERE  R.STATUS <> 'FAILED'
AND    R.BENCH       = S.BENCH
AND    R.IMPL        = S.IMPL
AND    R.FILE_ID     = S.FILE_ID
AND    R.MACHINE     = S.MACHINE
AND    R.PROG        = S.PROG
AND    R.VER         = S.VER
AND    R.WITH_OUTPUT = S.WITH_OUTPUT
AND    R.REAL_TIME   > 2.0 * S.AVG_REAL_TIME
AND    R.REAL_TIME   > S.AVG_REAL_TIME + 1
ORDER BY MACHINE, PROG, VER, BENCH, IMPL, FILE_ID, WITH_OUTPUT;

-- ============================================================================
-- Detect Outliers with regard to CPU Time:
-- ============================================================================

CREATE VIEW OUTLIER_CPU AS
SELECT R.SYS_TIME+R.USER_TIME AS CPU_TIME, S.AVG_CPU_TIME,
       R.PROG, R.VER, R.STATUS,
       R.BENCH, R.IMPL, R.FILE_ID, R.MACHINE, R.WITH_OUTPUT,
       R.RUN_DATE, R.RUN_NO
FROM   BENCH_RUN R, RUN_STAT S
WHERE  R.STATUS <> 'FAILED'
AND    R.BENCH       = S.BENCH
AND    R.IMPL        = S.IMPL
AND    R.FILE_ID     = S.FILE_ID
AND    R.MACHINE     = S.MACHINE
AND    R.PROG        = S.PROG
AND    R.VER         = S.VER
AND    R.WITH_OUTPUT = S.WITH_OUTPUT
AND    R.SYS_TIME+R.USER_TIME > 2.0 * S.AVG_CPU_TIME
AND    R.SYS_TIME+R.USER_TIME > S.AVG_CPU_TIME + 1
ORDER BY MACHINE, PROG, VER, BENCH, IMPL, FILE_ID, WITH_OUTPUT;

-- ============================================================================
-- Detect Outliers with regard to Memory:
-- ============================================================================

CREATE VIEW OUTLIER_MEM AS
SELECT R.MEM, S.AVG_MEM, R.PROG, R.VER, R.STATUS,
       R.BENCH, R.IMPL, R.FILE_ID, R.MACHINE, R.WITH_OUTPUT,
       R.RUN_DATE, R.RUN_NO
FROM   BENCH_RUN R, RUN_STAT S
WHERE  R.STATUS <> 'FAILED'
AND    R.BENCH       = S.BENCH
AND    R.IMPL        = S.IMPL
AND    R.FILE_ID     = S.FILE_ID
AND    R.MACHINE     = S.MACHINE
AND    R.PROG        = S.PROG
AND    R.VER         = S.VER
AND    R.WITH_OUTPUT = S.WITH_OUTPUT
AND    R.MEM         > 2.0 * S.AVG_MEM
AND    R.MEM         > S.AVG_MEM + 5
ORDER BY MACHINE, PROG, VER, BENCH, IMPL, FILE_ID, WITH_OUTPUT;

-- ============================================================================
-- Check where Realtime is much more than CPU time (at least double):
-- ============================================================================

CREATE VIEW REAL_TOO_LARGE AS
SELECT R.REAL_TIME, S.AVG_REAL_TIME,
       R.SYS_TIME+R.USER_TIME AS CPU_TIME, S.AVG_CPU_TIME,
       R.PROG, R.VER, R.STATUS,
       R.BENCH, R.IMPL, R.FILE_ID, R.MACHINE, R.WITH_OUTPUT,
       R.RUN_DATE, R.RUN_NO
FROM   BENCH_RUN R, RUN_STAT S
WHERE  R.STATUS <> 'FAILED'
AND    R.BENCH       = S.BENCH
AND    R.IMPL        = S.IMPL
AND    R.FILE_ID     = S.FILE_ID
AND    R.MACHINE     = S.MACHINE
AND    R.PROG        = S.PROG
AND    R.VER         = S.VER
AND    R.WITH_OUTPUT = S.WITH_OUTPUT
AND    R.REAL_TIME   > 2.0 * (R.SYS_TIME+R.USER_TIME)
AND    R.REAL_TIME   > (R.SYS_TIME+R.USER_TIME) + 1
ORDER BY MACHINE, PROG, VER, BENCH, IMPL, FILE_ID, WITH_OUTPUT;

-- ============================================================================
-- Compare Real Time with CPU Time:
-- ============================================================================

CREATE VIEW PCT_CPU AS
SELECT PROG, RUN_DATE, MACHINE, BENCH,
	ROUND(AVG((SYS_TIME+USER_TIME) * 100 / REAL_TIME)) AS AVG_CPU,
	ROUND(MIN((SYS_TIME+USER_TIME) * 100 / REAL_TIME)) AS MIN_CPU,
	ROUND(MAX((SYS_TIME+USER_TIME) * 100 / REAL_TIME)) AS MAX_CPU
FROM   BENCH_RUN
WHERE  SYS_TIME IS NOT NULL AND USER_TIME IS NOT NULL
AND    REAL_TIME > 0
AND    STATUS <> 'FAILED' AND STATUS <> 'OUTL'
AND    WITH_OUTPUT = 'N'
GROUP  BY PROG, RUN_DATE, MACHINE, BENCH
ORDER  BY PROG, RUN_DATE, MACHINE, BENCH;


-- ============================================================================
-- Compare Real Time with CPU Time:
-- ============================================================================

CREATE VIEW PCT_CPU_INPUT AS
SELECT PROG, MACHINE, BENCH, FILE_ID,
	ROUND(AVG((SYS_TIME+USER_TIME) * 100 / REAL_TIME)) AS AVG_CPU,
	ROUND(MIN((SYS_TIME+USER_TIME) * 100 / REAL_TIME)) AS MIN_CPU,
	ROUND(MAX((SYS_TIME+USER_TIME) * 100 / REAL_TIME)) AS MAX_CPU
FROM   BENCH_RUN
WHERE  SYS_TIME IS NOT NULL AND USER_TIME IS NOT NULL
AND    REAL_TIME > 0
AND    STATUS <> 'FAILED' AND STATUS <> 'OUTL'
AND    WITH_OUTPUT = 'N'
GROUP  BY PROG, MACHINE, BENCH, FILE_ID
ORDER  BY PROG, MACHINE, BENCH, FILE_ID;

-- ============================================================================
-- System Speed in Real Time:
-- ============================================================================

CREATE VIEW SPEED_REAL AS
SELECT PROG, FILE_ID, ROUND((COST/REAL_T)/1000) as SPEED, COST, REAL_T
FROM   PLOT_REAL
WHERE  REAL_T > 0
ORDER  BY PROG, SPEED;

-- ============================================================================
-- System Speed in Real Time With Startup Time:
-- ============================================================================

CREATE VIEW SPEED_REAL_DIFF AS
SELECT P.PROG, P.FILE_ID,
       ROUND(((P.COST-S.COST)/(P.REAL_T-S.REAL_T))/1000) as SPEED,
       P.COST, P.REAL_T,
       (P.COST-S.COST) AS DIFF_COST, (P.REAL_T-S.REAL_T) AS DIFF_REAL_T
FROM   PLOT_REAL P, PLOT_REAL S
WHERE  P.REAL_T-S.REAL_T > 0
AND    P.PROG = S.PROG
AND    S.FILE_ID = 'm_4_2ki'
ORDER  BY PROG, SPEED;

-- ============================================================================
-- System Speed in CPU Time:
-- ============================================================================

CREATE VIEW SPEED_CPU AS
SELECT PROG, FILE_ID, ROUND((COST/CPU_T)/1000) as SPEED, COST, CPU_T
FROM   PLOT_CPU
WHERE  CPU_T > 0
ORDER  BY PROG, SPEED;

-- ============================================================================
-- Memory Efficiency:
-- ============================================================================

CREATE VIEW MEM_EFFICIENCY AS
SELECT PROG, FILE_ID, ROUND(MEM_MB*1024*1024/FACTS) as BYTES_PER_FACT,
       FACTS, MEM_MB
FROM   PLOT_MEM
WHERE  FACTS > 0
ORDER  BY PROG, BYTES_PER_FACT;

-- ============================================================================
-- Check System Speed Estimates (Real Time):
-- ============================================================================

CREATE VIEW ESTIMATE_ERR AS
SELECT R.PROG, E.SPEED AS SPEED, R.FILE_ID,
       R.COST / E.SPEED as ESTIMATE, R.REAL_T,
       R.REAL_T - (R.COST / E.SPEED) AS DIFF,
       ROUND((R.REAL_T - (R.COST / E.SPEED)) * 100 / (R.COST / E.SPEED))
			AS DIFF_PCT
FROM   PLOT_REAL R, SPEED_ESTIMATE E
WHERE  R.PROG = E.PROG AND E.BENCH = 'tcff'
AND    R.REAL_T > 0
ORDER  BY R.PROG, E.SPEED, DIFF;

-- ============================================================================
-- Count the Number of Wrong Estimates (more than 2s wrong and more than 20%):
-- ============================================================================

CREATE VIEW NUM_BAD_ESTIMATES AS
SELECT PROG, SPEED, COUNT(*) AS BAD_ESTIMATES, SUM(ABS(DIFF)) AS ABS_ERR
FROM   ESTIMATE_ERR
WHERE  ABS(REAL_T - ESTIMATE) > 2
-- AND    ABS(DIFF) > 0.2 * REAL_T
AND    ABS(DIFF_PCT) > 20
GROUP  BY PROG, SPEED
ORDER  BY PROG, SPEED;


-- ============================================================================
-- Generate C++ Objects with Graph Data and Runtime (for Optimization):
-- ============================================================================

CREATE VIEW ESTIMATE_DATA AS
SELECT R.PROG, R.BENCH, R.IMPL,
       'runtime_c("' || G.FILE_ID || '", ' ||
		G.NUM_NODES || ', ' ||
		G.NUM_EDGES || ', ' ||
		'INT64_C(' || G.TC_SIZE || '), ' ||
		'INT64_C(' || G.COST    || '), ' ||
		G.NUM_ITER  || ', ' ||
		ROUND(R.REAL_T*1000)|| '),' AS OBJ
FROM	GRAPH_DATA G, DATA_FILE F, BEST_RESULT R
WHERE   G.FILE_ID = F.FILE_ID AND R.FILE_ID = F.FILE_ID
ORDER BY R.PROG, R.BENCH, F.SEQ_NO;

-- ============================================================================
-- Load time per million edges in s:
-- ============================================================================

CREATE VIEW LOAD_EFFICIENCY AS
SELECT R.PROG, R.BENCH, R.IMPL, R.LOAD_T, G.NUM_EDGES,
		ROUND(R.LOAD_T * 1000 * 1000 / G.NUM_EDGES, 3)
			AS LOAD_PER_M_EDGES
FROM    GRAPH_DATA G, DATA_FILE F, BEST_RESULT R
WHERE   G.FILE_ID = F.FILE_ID AND R.FILE_ID = F.FILE_ID
ORDER BY R.PROG, R.BENCH, F.SEQ_NO;

-- ============================================================================
-- Search Range for Parameter Values:
-- ============================================================================

CREATE VIEW PAR_RANGE AS
SELECT R.PROG, G.FILE_ID,
		ROUND(R.REAL_T * 1000 * 1000 / G.NUM_EDGES, 3) AS PAR_LOAD,
		ROUND(R.REAL_T * 1000 * 1000 / G.COST, 3)      AS PAR_RULE,
		ROUND(R.REAL_T * 1000 * 1000 / G.TC_SIZE, 3)   AS PAR_RESULT,
		ROUND(R.REAL_T * 1000 * 1000 / G.NUM_NODES, 3) AS PAR_DATA,
		ROUND(R.REAL_T * 1000 * 1000 / G.NUM_ITER, 3)  AS PAR_ITER
FROM    GRAPH_DATA G, DATA_FILE F, BEST_RESULT R
WHERE   G.FILE_ID = F.FILE_ID AND R.FILE_ID = F.FILE_ID
AND     R.BENCH = 'tcff'
ORDER BY R.PROG, R.BENCH, F.SEQ_NO;


