// ============================================================================
// Project:	rbench - Logic Programming and Database Benchmarks
// Filename:	opt/opt.cpp
// Purpose:	Optimize parameters for runtime estimation formula
// Last Change:	30.04.2019
// Language:	C++
// EMail:	brass@informatik.uni-halle.de
// WWW:		http://www.informatik.uni-halle.de/~brass/
// Address:	Feldschloesschen 15, D-06120 Halle (Saale), GERMANY
// Copyright:	(c) 2018-2019 by Stefan Brass
// License:	See file "LICENSE" for copying conditions.
// Note:	There is no warranty at all - this code may contain bugs.
// ============================================================================

// Please note: This is a quick&dirty solution.
// I am not particularly proud on the programming style.
//
// This must be compiled with the C++11 Standard, because it uses int64_t.
// E.g.:
//	g++ -std=c++11 -Wall -O2 -o opt opt.cpp
// If one uses the parallel version, one needs to add -pthread, e.g.
//	g++ -std=c++11 -pthread -c -Wall -Wextra -Wshadow -Wpointer-arith \ ...
//		-Wcast-qual -Wcast-align -Wunreachable-code -pedantic -O2 \ ...
//		-o opt.o opt.cpp
//	g++ -pthread -o opt opt.o

//-----------------------------------------------------------------------------
// Dimensions:
//-----------------------------------------------------------------------------

// The currently implemented parameters for the estimation formula are:
//
// - DIMEN_STARTUP (Startup, S):
//	Milliseconds for startup (independent of input).
// - DIMEN_LOADX (LoadX, X):
//	Milliseconds for inserting edges in an index (per n*log(n) = 1mio).
// - DIMEN_RULE_APP (Rule App, A):
//	Milliseconds to apply one million rule instances.
// - DIMEN_RESULT (Result, R):
//	Milliseconds to derive one million result tuples.
// - DIMEN_DED_LOGR (Ded*LogR, DL):
//	Milliseconds per one million rule instances * log of result size.
// - DIMEN_ITER (Iter*Tup, IT):
//	Milliseconds per one million scanned tuples (iterations*edges).
// - DIMEN_LOAD (LinLoad, LL):
//	Milliseconds to load one million input tuples.
// - DIMEN_DATA_VAL (DataVal, DV):
//	Milliseconds to work with one million data values (nodes).
// - DIMEN_NEW_FACTS (NewFacts, NF):
//	Milliseconds to process one million new facts (in tc, not in edges).


//=============================================================================
// Do parallel optimization? How many parallel jobs?
//=============================================================================

// If NUM_JOBS is 1, the serial version will be chosen.

//#define NUM_JOBS 1
#define NUM_JOBS 4
//#define NUM_JOBS 30

//=============================================================================
// Do correctness checks in inner loops (hurts performance):
//=============================================================================

// #define DO_CHECKS 0
#define DO_CHECKS 1

//=============================================================================
// How often should output be printed (step size for percent of work):
//=============================================================================

#define OUT_PCT 5

//=============================================================================
// CSV File with Data (Graph Data and System Runtime):
//=============================================================================

#define CSV_FILE "estimate_data.csv"

//=============================================================================
// Maximal Number of Data Points per System:
//=============================================================================

#define MAX_DATAPOINTS	100

//=============================================================================
// Maximal Line Size in CSV File:
//=============================================================================

#define MAX_LINESIZE	128

//=============================================================================
// Buffer Size for Graph ID (limit includes '\0'):
//=============================================================================

#define MAX_GRAPH_CHARS	16

//=============================================================================
// Include Files:
//=============================================================================

#include <iostream>
#include <fstream>
#include <cstdint>

#if NUM_JOBS > 1
#include <thread>
#include <mutex>
#endif

//=============================================================================
// Type for Large Integers (64 Bit):
//=============================================================================

typedef int64_t bigint_t;

//=============================================================================
// String Data Type:
//=============================================================================

typedef const char *str_t;

#define STR_NULL (static_cast<str_t>(0))

//-----------------------------------------------------------------------------
// str_eq: Compare two strings for equality.
//-----------------------------------------------------------------------------

bool	str_eq(str_t str1, str_t str2)
{
	// Avoid crashes:
	if(str1 == nullptr)
		return(str2 == nullptr);
	else if(str2 == nullptr)
		return(false);

	// Compare strings:
	while(*str1 && *str1==*str2) {
		str1++;
		str2++;
	}
	return(*str1==*str2);
}

//-----------------------------------------------------------------------------
// str_cpy: Secure string copy.
//-----------------------------------------------------------------------------

void	str_cpy(char* buf, int len, str_t str)
{
	int i;

	// Avoid crashes:
	if(buf == 0)
		return;
	else if(str == 0) {
		if(len >= 6) {
			*buf++ = '<';
			*buf++ = 'N';
			*buf++ = 'I';
			*buf++ = 'L';
			*buf++ = '>';
			*buf = '\0';
		}
		else if(len >= 1) {
			*buf = '\0';
		}
	}

	// Copy as much as possible:
	i = len;
	while(--i > 0 && *str)
		*buf++ = *str++;
	*buf = '\0';

	// Mark incomplete copy:
	if(*str && len >= 4) {
		buf[-3] = '.';
		buf[-2] = '.';
		buf[-1] = '.';
	}
}


//-----------------------------------------------------------------------------
// is_digit: Classify characters as digits:
//-----------------------------------------------------------------------------

bool is_digit(char c) {
	switch(c) {
		case '0':
		case '1':
		case '2':
		case '3':
		case '4':
		case '5':
		case '6':
		case '7':
		case '8':
		case '9':
			return true;
		default:
			return false;
	}
}

//-----------------------------------------------------------------------------
// digit_val: Return value of a digit.
//-----------------------------------------------------------------------------

int digit_val(char digit) {
	switch(digit) {
		case '0':
			return 0;
		case '1':
			return 1;
		case '2':
			return 2;
		case '3':
			return 3;
		case '4':
			return 4;
		case '5':
			return 5;
		case '6':
			return 6;
		case '7':
			return 7;
		case '8':
			return 8;
		case '9':
			return 9;
		default:
			std::cerr << "digit_val called with non-digit: '" <<
					digit << "'\n";
			exit(10);
			return -1;
	}
}

//-----------------------------------------------------------------------------
// str_long: Translate digit sequence to long value:
//-----------------------------------------------------------------------------

long str_long(str_t digits) {
	long result = 0;
	for(str_t p = digits; *p; p++) {
		long prev_result = result;
		result = result * 10 + digit_val(*p);
		if(result/10 != prev_result) {
			std::cerr << "Overflow!\n";
			exit(10);
		}
	}
	return result;
}

//-----------------------------------------------------------------------------
// str_bigint: Translate digit sequence to bigint_t value:
//-----------------------------------------------------------------------------

bigint_t str_bigint(str_t digits) {
	bigint_t result = 0;
	for(str_t p = digits; *p; p++) {
		bigint_t prev_result = result;
		result = result * 10 + digit_val(*p);
		if(result/10 != prev_result) {
			std::cerr << "Overflow!\n";
			exit(10);
		}
	}
	return result;
}

//=============================================================================
// Minimum Function:
//=============================================================================

template<class T>
inline T MIN(T x, T y) {
        if(x <= y)
                return x;
        else
                return y;
}

//=============================================================================
// Integer Division:
//=============================================================================

//-----------------------------------------------------------------------------
// IDIV_ROUND:
//-----------------------------------------------------------------------------

/**
 * Integer division, round result
 *
 * Note that this works only for non-negative numbers.
 */

#define IDIV_ROUND(X,Y)	(((X)+(Y)/2)/(Y))

//-----------------------------------------------------------------------------
// IDIV_CEILING:
//-----------------------------------------------------------------------------

/**
 * Integer division, round up.
 *
 * Note that this works only for non-negative numbers.
 */

#define IDIV_CEILING(X,Y)	(((X)+(Y)-1)/(Y))

//=============================================================================
// Class for Graph Data:
//=============================================================================

class data_c {
	public:

	data_c(str_t data_graph,
			long data_nodes, long data_edges,
			bigint_t data_tc_size, bigint_t data_cost,
			long data_iter, long data_realtime)
	{
		str_cpy(Graph, MAX_GRAPH_CHARS, data_graph);
		Nodes    = data_nodes;
		Edges    = data_edges;
		TCSize   = data_tc_size;
		Cost     = data_cost;
		Iter     = data_iter;
		RealTime = data_realtime;

		// Compute Product "Iterations * Edges" only once:
		IterEdges= static_cast<bigint_t>(data_iter) * data_edges;

		// Compute "Edges * log(Edges)":
		int log = 0;
		for(long n = Edges >> 1; n > 0; n >>= 1)
			log++;
		EdgesLogEdges = Edges * log;

		// Compute "Cost * log(TCSize)":
		log = 0;
		for(long n = TCSize >> 1; n > 0; n >>= 1)
			log++;
		CostLogTC = Cost * log;

		// Compute number of new facts (in tc result, not given edge):
		NewFacts = data_tc_size - data_edges;
	}

	// Default Constructor:
	data_c() {
		Graph[0] = '\0';
		Nodes = 0;
		Edges = 0;
		TCSize = 0;
		Cost = 0;
		Iter = 0;
		RealTime = 0;
		IterEdges = 0;
		EdgesLogEdges = 0;
		CostLogTC = 0;
		NewFacts = 0;
	}

	// Initialization function (old constructor):
	void init(str_t data_graph,
			long data_nodes, long data_edges,
			bigint_t data_tc_size, bigint_t data_cost,
			long data_iter, long data_realtime)
	{
		str_cpy(Graph, MAX_GRAPH_CHARS, data_graph);
		Nodes    = data_nodes;
		Edges    = data_edges;
		TCSize   = data_tc_size;
		Cost     = data_cost;
		Iter     = data_iter;
		RealTime = data_realtime;

		// Compute Product "Iterations * Edges" only once:
		IterEdges= static_cast<bigint_t>(data_iter) * data_edges;

		// Compute "Edges * log(Edges)":
		int log = 0;
		for(long n = Edges >> 1; n > 0; n >>= 1)
			log++;
		EdgesLogEdges = Edges * log;

		// Compute "Cost * log(TCSize)":
		log = 0;
		for(long n = TCSize >> 1; n > 0; n >>= 1)
			log++;
		CostLogTC = Cost * log;

		// Compute number of new facts (in tc result, not given edge):
		NewFacts = data_tc_size - data_edges;
	}

	// Accessor Functions:
	inline str_t graph() const { return Graph; }
	inline long nodes() const { return Nodes; }
	inline long edges() const { return Edges; }
	inline bigint_t tc_size() const { return TCSize; }
	inline bigint_t cost() const { return Cost; }
	inline long iter() const { return Iter; }
	inline long real_time() const { return RealTime; }
	inline bigint_t iter_edges() const { return IterEdges; }
	inline bigint_t edges_log_edges() const { return EdgesLogEdges; }
	inline bigint_t cost_log_tc() const { return CostLogTC; }
	inline bigint_t new_facts() const { return NewFacts; }
	
	private:
	char Graph[MAX_GRAPH_CHARS];
	long Nodes;
	long Edges;
	bigint_t TCSize;
	bigint_t Cost;
	long Iter;
	long RealTime;
	bigint_t IterEdges;
	bigint_t EdgesLogEdges;
	bigint_t CostLogTC;
	bigint_t NewFacts;
};

//-----------------------------------------------------------------------------
// Pointer Type for Graph Data:
//-----------------------------------------------------------------------------

typedef data_c *data_t;

//=============================================================================
// Simple Reader for Data Values from CSV File:
//=============================================================================


class DataReader {

	// This class has only a static function:
	private:
	DataReader();

	public:
	static int read(str_t filename, str_t sys_name,
				data_c *arr, int arr_length) {

		// Number of data objects read from the CSV file:
		int num_obj = 0;

		// Open input file:
		std::ifstream input;
		input.open(filename);
		if(input.fail()) {
			std::cerr << "Opening input file '" << filename <<
				"' failed.\n";
			exit(8);
		}

		// Read lines:
		int line_no = 1;
		while(!input.eof()) {
			char line[MAX_LINESIZE];
			input.getline(line, MAX_LINESIZE);
			if(input.bad()) {
				std::cerr << "Error while reading from " <<
					"'" << filename << "'.\n";
				exit(8);
			}
			if(input.fail()) {
				line_no--;
				break;
			}

			// The first line with column headers is skipped:
			if(line_no == 1) {
				if(line[0] != '"' || line[1] != 'S' ||
					line[2] != 'Y' || line[3] != 'S' ||
					line[4] != '"' || line[5] != ',') {
					std::cerr << "First line in " <<
					"'" << filename << "' does not " <<
					"contain column headers.\n";
					exit(8);
				}
			}
			else {
				char *p = line;

				// Extract system:
				p = skip_char(p, '"', line_no, line);
				str_t sys = p;
				p = get_sdata(p, '"', line_no, line);
				p = skip_char(p, ',', line_no, line);

				// Extract benchmark:
				p = skip_char(p, '"', line_no, line);
				str_t bench = p;
				p = get_sdata(p, '"', line_no, line);
				p = skip_char(p, ',', line_no, line);

				// Extract implementation:
				p = skip_char(p, '"', line_no, line);
				//str_t impl = p;
				p = get_sdata(p, '"', line_no, line);
				p = skip_char(p, ',', line_no, line);

				// Extract graph:
				p = skip_char(p, '"', line_no, line);
				str_t graph = p;
				p = get_sdata(p, '"', line_no, line);
				p = skip_char(p, ',', line_no, line);

				// Extract nodes:
				p = skip_char(p, '"', line_no, line);
				str_t nodes_digits = p;
				p = get_ndata(p, '"', line_no, line);
				p = skip_char(p, ',', line_no, line);

				// Extract edges:
				p = skip_char(p, '"', line_no, line);
				str_t edges_digits = p;
				p = get_ndata(p, '"', line_no, line);
				p = skip_char(p, ',', line_no, line);

				// Extract size:
				p = skip_char(p, '"', line_no, line);
				str_t size_digits = p;
				p = get_ndata(p, '"', line_no, line);
				p = skip_char(p, ',', line_no, line);

				// Extract cost:
				p = skip_char(p, '"', line_no, line);
				str_t cost_digits = p;
				p = get_ndata(p, '"', line_no, line);
				p = skip_char(p, ',', line_no, line);

				// Extract iter:
				p = skip_char(p, '"', line_no, line);
				str_t iter_digits = p;
				p = get_ndata(p, '"', line_no, line);
				p = skip_char(p, ',', line_no, line);

				// Extract time:
				p = skip_char(p, '"', line_no, line);
				str_t time_digits = p;
				p = get_ndata(p, '"', line_no, line);

				// That was all:
				if(*p != '\0' && *p != '\n')
					csv_err(line_no, line, p);

				// Store data:
				if(str_eq(sys, sys_name) &&
					str_eq(bench, "tcff")) {
					if(num_obj >= arr_length) {
						std::cerr << "Data array " <<
							"too small!\n";
						exit(9);
					}
					arr[num_obj++].init(
							graph,
							str_long(nodes_digits),
							str_long(edges_digits),
							str_bigint(size_digits),
							str_bigint(cost_digits),
							str_long(iter_digits),
							str_long(time_digits));
				}
			}

			// Increment line number:
			line_no++;
		}
		std::cout << "Read " << line_no << " lines from " <<
			"'" << filename << "' " << 
			"(" << num_obj << " data points for " <<
			"'" << sys_name << "')" << "\n";

		// Close input file:
		input.close();

		// Return number of objects read:
		return num_obj;
	}

	// This is called to check that an expected character is there:
	static char *skip_char(char *p, char c, int line_no, char *line) {
		if(*p != c)
			csv_err(line_no, line, p);
		p++;
		return p;
	}

	// This is called to skip the data and change the delimiter to '\0':
	static char *get_sdata(char *p, char c, int line_no, char *line) {
		// this is for String data, c is the delimiter, usually '"'
		while(*p && *p != c)
			p++;
		if(*p != c)
			csv_err(line_no, line, p);
		*p++ = '\0';
		return p;
	}

	// This is called to skip the digits and change the delimiter to '\0':
	static char *get_ndata(char *p, char c, int line_no, char *line) {
		// this is for numeric data, c is the delimiter, usually '"'
		while(*p && *p != c && is_digit(*p))
			p++;
		if(*p != c)
			csv_err(line_no, line, p);
		*p++ = '\0';
		return p;
	}

	// This is called if a format error is found:
	static void csv_err(int line_no, char *line, char *p) {
		std::cerr << "CSV format error in line " << line_no <<
			", column " << (p-line)+1 << "\n";
		exit(8);
	}
};

//=============================================================================
// Class for Systems (with Array of Data Values):
//=============================================================================

class sys_c {
	public:

	// Constructor:
	sys_c(str_t sys_name, int sys_known_opt) {
		Name = sys_name;
		KnownOpt = sys_known_opt;
		DataLoaded = false;
		NumDataPoints = 0;
	}

	// Accessor methods:
	str_t name() const { return Name; }
	int known_opt() const { return KnownOpt; }
	int num_data_points() const { return NumDataPoints; }

	// Return data - ensure that loaded:
	data_c *data() {
		if(!DataLoaded) {
			std::cerr << "Data for system '" << Name << "' " <<
				"not yet loaded.\n";
			exit(12);
		}
		return Data;
	}

	// Load data the this system from the csv file:
	void load_data() {
		NumDataPoints = DataReader::read(
					CSV_FILE, Name, Data, MAX_DATAPOINTS);
		DataLoaded = true;
		if(NumDataPoints == 0) {
			std::cerr << "No data in " <<
					"'" << CSV_FILE << "' " <<
					"for system " <<
					"'" << Name << "'.\n";
			exit(11);
		}
	}

	private:

	// Attributes:
	str_t Name;
	int KnownOpt;
	bool DataLoaded;
	data_c Data[MAX_DATAPOINTS];
	int NumDataPoints;
};

//-----------------------------------------------------------------------------
// Pointer Type for System Data:
//-----------------------------------------------------------------------------

typedef sys_c *sys_t;

//-----------------------------------------------------------------------------
// List of Systems with Data Arrays:
//-----------------------------------------------------------------------------

// The known optimum is for 43 input graphs (old input data).

sys_c systems[] = {
	sys_c("xsb", 41),
	sys_c("bam", 29),
	sys_c("yap", 32),
	sys_c("sqlite", 42),
	sys_c("sqlite3", 42),
	sys_c("psql", 38),
	sys_c("maria_m", 18),
	sys_c("maria_i", 19),
	sys_c("mariadb", 0),
	sys_c("souffle", 24),
	sys_c("jena", 42),
};

const int NUM_SYSTEMS = sizeof(systems)/sizeof(sys_c);

//-----------------------------------------------------------------------------
// Get object for a given system:
//-----------------------------------------------------------------------------

sys_t get_sys(str_t name) {
	for(int i = 0; i < NUM_SYSTEMS; i++) {
		if(str_eq(name, systems[i].name())) {
			systems[i].load_data();
			return &(systems[i]);
		}
	}
	return nullptr;
}

//=============================================================================
// Enumeration Type for Single Parameters of the Formula:
//=============================================================================

// These are dimensions of the parameter space.

typedef enum {
	// Not part of the estimation formula, sometimes needed as NULL value:
	DIMEN_NULL     = 0,

	// Milliseconds for startup (independent of input):
	DIMEN_STARTUP  = 1,

	// Milliseconds for inserting edges in an index (per n*log(n) = 1mio):
	DIMEN_LOADX    = 2,

	// Milliseconds to apply one million rule instances:
	DIMEN_RULE_APP = 3,

	// Milliseconds to derive one million result tuples:
	DIMEN_RESULT   = 4,

	// Milliseconds per one million rule instances * log of result size:
	DIMEN_DED_LOGR = 5,

	// Milliseconds per one million scanned tuples (iterations*edges):
	DIMEN_ITER     = 6,

	// Milliseconds to load one million input tuples:
	DIMEN_LOAD     = 7,

	// Milliseconds to work with one million data values (nodes):
	DIMEN_DATA_VAL = 8,

	// Milliseconds to process one million new facts (in tc, not in edges):
	DIMEN_NEW_FACTS= 9,

	// Invalid value at the end, only temporarily assumed. Must be last:
	DIMEN_INVALID  = 10
} dimen_t;

// We need an invalid dimension at the end,
// because the for-Loops over the dimensions do increment the last dimension
// before stopping the loop.

//-----------------------------------------------------------------------------
// Some important constants:
//-----------------------------------------------------------------------------

static const dimen_t DIMEN_FIRST = DIMEN_STARTUP;
static const dimen_t DIMEN_LAST  = DIMEN_NEW_FACTS;
static const int DIMENSIONS = DIMEN_LAST;
static const int DIMEN_ARRAY_SIZE = DIMEN_INVALID + 1;

//-----------------------------------------------------------------------------
// Increment Operator ++ (Postfix) for Dimension Values:
//-----------------------------------------------------------------------------

// Note that the second "int" argument is never used (dummy).
// C++ uses this trick to mark the function as postfix ++
// in contrast to prefix ++, which does not have this dummy argument.

inline dimen_t& operator++ (dimen_t& d, int) {
#if DO_CHECKS
	// Note that DIMEN_LAST is incremented, but the result is not used.
	if(d == DIMEN_INVALID) {
		std::cerr << "Try to increment maximal dimension!\n";
		exit(1);
	}
#endif
	return d = dimen_t(d + 1);
}

//-----------------------------------------------------------------------------
// Decrement Operator -- (Postfix) for Dimension Values:
//-----------------------------------------------------------------------------

inline dimen_t& operator-- (dimen_t& d, int) {
#if DO_CHECKS
	if(d == DIMEN_NULL) {
		std::cerr << "Try to decrement minimal dimension!\n";
		exit(1);
	}
#endif
	return d = dimen_t(d - 1);
}

//-----------------------------------------------------------------------------
// Parse dimension name:
//-----------------------------------------------------------------------------

dimen_t dimen_parse(str_t name) {
	if(str_eq(name, "S"))
		return DIMEN_STARTUP;
	else if(str_eq(name, "Startup"))
		return DIMEN_STARTUP;
	else if(str_eq(name, "X"))
		return DIMEN_LOADX;
	else if(str_eq(name, "LoadX"))
		return DIMEN_LOADX;
	else if(str_eq(name, "A"))
		return DIMEN_RULE_APP;
	else if(str_eq(name, "RuleApp"))
		return DIMEN_RULE_APP;
	else if(str_eq(name, "R"))
		return DIMEN_RESULT;
	else if(str_eq(name, "Result"))
		return DIMEN_RESULT;
	else if(str_eq(name, "DL"))
		return DIMEN_DED_LOGR;
	else if(str_eq(name, "DedLogR"))
		return DIMEN_DED_LOGR;
	else if(str_eq(name, "IT"))
		return DIMEN_ITER;
	else if(str_eq(name, "IterTup"))
		return DIMEN_ITER;
	else if(str_eq(name, "LL"))
		return DIMEN_LOAD;
	else if(str_eq(name, "LinLoad"))
		return DIMEN_LOAD;
	else if(str_eq(name, "DV"))
		return DIMEN_DATA_VAL;
	else if(str_eq(name, "DataVal"))
		return DIMEN_DATA_VAL;
	else if(str_eq(name, "NF"))
		return DIMEN_NEW_FACTS;
	else if(str_eq(name, "NewFacts"))
		return DIMEN_NEW_FACTS;
	else
		return DIMEN_NULL;
}

//-----------------------------------------------------------------------------
// Return dimension name:
//-----------------------------------------------------------------------------

str_t dimen_name(dimen_t d) {
	switch(d) {
		case DIMEN_STARTUP:
			return "Startup:  ";
		case DIMEN_LOADX:
			return "LoadX:    ";
		case DIMEN_RULE_APP:
			return "RuleApp:  ";
		case DIMEN_RESULT:
			return "Result:   ";
		case DIMEN_DED_LOGR:
			return "DedLogR:  ";
		case DIMEN_ITER:
			return "IterTup:  ";
		case DIMEN_LOAD:
			return "LinLoad:  ";
		case DIMEN_DATA_VAL:
			return "DataVal:  ";
		case DIMEN_NEW_FACTS:
			return "NewFacts: ";
		case DIMEN_NULL:
			return "*NULL*:   ";
		default:
			return "*INVALID*:";
	}
}

//=============================================================================
// Class for Parameters of the Formula:
//=============================================================================

class par_c {

	public:

	//---------------------------------------------------------------------
	// Constructor:
	//---------------------------------------------------------------------

	par_c()
	{
		Par[DIMEN_NULL]    = 0;
		for(dimen_t dimen = DIMEN_FIRST; dimen <= DIMEN_LAST; dimen++)
			Par[dimen] = 0;
		Par[DIMEN_INVALID] = 0;
	}

	//---------------------------------------------------------------------
	// Set parameter value:
	//---------------------------------------------------------------------

	inline void set(dimen_t dimen, long value) {

		// Check dimension:
#if DO_CHECKS
		if(dimen < DIMEN_FIRST || dimen > DIMEN_LAST) {
			std::cerr << "Invalid dimension " << dimen <<
					" in set.\n";
			exit(5);
		}
#endif

		// Set parameter value:
		Par[dimen] = value;
	}

	//---------------------------------------------------------------------
	// Get parameter value:
	//---------------------------------------------------------------------

	inline long get(dimen_t dimen) const {

		// Check dimension:
#if DO_CHECKS
		if(dimen < DIMEN_FIRST || dimen > DIMEN_LAST) {
			std::cerr << "Invalid dimension " << dimen <<
					" in get.\n";
			exit(5);
		}
#endif

		// Return parameter value:
		return Par[dimen];
	}

	//---------------------------------------------------------------------
	// Add step size to parameter value:
	//---------------------------------------------------------------------

	inline void add(dimen_t dimen, long step) {

		// Check dimension:
#if DO_CHECKS
		if(dimen < DIMEN_FIRST || dimen > DIMEN_LAST) {
			std::cerr << "Invalid dimension " << dimen <<
					" in add.\n";
			exit(5);
		}
#endif

		// Do the incrementation:
		Par[dimen] += step;
	}

	//---------------------------------------------------------------------
	// Change parameter in one dimention +1:
	//---------------------------------------------------------------------

	void inc(dimen_t dimen)
	{
		// Check that index is legal:
#if DO_CHECKS
		if(dimen < DIMEN_FIRST || dimen > DIMEN_LAST) {
				std::cerr << "Invalid dimension: " <<
						dimen << " in inc\n";
				exit(5);
		}
#endif

		// Change parameter value:
		Par[dimen]++;
	}

	//---------------------------------------------------------------------
	// Change parameter in one dimention -1:
	//---------------------------------------------------------------------

	void dec(dimen_t dimen)
	{
		// Check that index is legal:
#if DO_CHECKS
		if(dimen < DIMEN_FIRST || dimen > DIMEN_LAST) {
				std::cerr << "Invalid dimension: " <<
						dimen << " in dec\n";
				exit(5);
		}
#endif

		// Change parameter value:
		Par[dimen]--;
	}

	//---------------------------------------------------------------------
	// Milliseconds for startup (independent of input):
	//---------------------------------------------------------------------

	inline long startup_cost() const { return Par[DIMEN_STARTUP]; }

	//---------------------------------------------------------------------
	// Milliseconds for inserting edges in an index (per n*log(n) = 1mio):
	//---------------------------------------------------------------------

	inline long loadx_cost() const { return Par[DIMEN_LOADX]; }

	//---------------------------------------------------------------------
	// Milliseconds to apply one million rule instances:
	//---------------------------------------------------------------------

	inline long rule_app_cost() const { return Par[DIMEN_RULE_APP]; }

	//---------------------------------------------------------------------
	// Number of applied rule instances per millisecond:
	//---------------------------------------------------------------------

	inline long speed() const
	{
		long rule_app = Par[DIMEN_RULE_APP];
		return (rule_app == 0) ? 0 :
					(1000000 + rule_app/2) / rule_app;
	}

	//---------------------------------------------------------------------
	// Milliseconds to derive one million result tuples:
	//---------------------------------------------------------------------

	inline long result_cost() const { return Par[DIMEN_RESULT]; }

	//---------------------------------------------------------------------
	// Milliseconds per one million rule instances * log of result size:
	//---------------------------------------------------------------------

	inline long ded_logr_cost() const { return Par[DIMEN_DED_LOGR]; }

	//---------------------------------------------------------------------
	// Milliseconds per one million scanned tuples (iterations*edges):
	//---------------------------------------------------------------------

	inline long iter_cost() const { return Par[DIMEN_ITER]; }

	//---------------------------------------------------------------------
	// Milliseconds to load one million input tuples:
	//---------------------------------------------------------------------

	inline long load_cost() const { return Par[DIMEN_LOAD]; }

	//---------------------------------------------------------------------
	// Milliseconds to work with one million data values (nodes):
	//---------------------------------------------------------------------

	inline long data_val_cost() const { return Par[DIMEN_DATA_VAL]; }

	//---------------------------------------------------------------------
	// Milliseconds to process one million new facts (in tc, not in edges):
	//---------------------------------------------------------------------

	inline long new_facts_cost() const { return Par[DIMEN_NEW_FACTS]; }

	//---------------------------------------------------------------------
	// Product of all parameter values (to compute total number of steps):
	//---------------------------------------------------------------------

	inline bigint_t prod() const
	{
		bigint_t product = 1;
		for(dimen_t dimen = DIMEN_FIRST; dimen <= DIMEN_LAST; dimen++)
			product *= Par[dimen];
		return product;
	}

	//---------------------------------------------------------------------
	// Print parameters (long version):
	//---------------------------------------------------------------------

	void print(str_t indent)
	{
		std::cout << indent <<	"Startup:  " << Par[DIMEN_STARTUP]
							<< "\n";
		std::cout << indent <<	"LoadX:    " << Par[DIMEN_LOADX]
							<< "\n";
		std::cout << indent <<	"Rule App: " << Par[DIMEN_RULE_APP]
							<< "\n";
		std::cout << indent <<	"Result:   " << Par[DIMEN_RESULT]
							<< "\n";

		// The other parameters are optional:
		if(Par[DIMEN_DED_LOGR] > 0)
			std::cout << indent <<
					"Ded*LogR: " << Par[DIMEN_DED_LOGR]
							<< "\n";
		if(Par[DIMEN_ITER] > 0)
			std::cout << indent <<
					"Iter*Tup: " << Par[DIMEN_ITER]
							<< "\n";

		if(Par[DIMEN_LOAD] > 0)
			std::cout << indent <<
					"LinLoad:  " << Par[DIMEN_LOAD]
							<< "\n";

		if(Par[DIMEN_DATA_VAL] > 0)
			std::cout << indent <<
					"DataVal:  " << Par[DIMEN_DATA_VAL]
							<< "\n";

		if(Par[DIMEN_NEW_FACTS] > 0)
			std::cout << indent <<
					"NewFacts: " << Par[DIMEN_NEW_FACTS]
							<< "\n";
	}

	//---------------------------------------------------------------------
	// Print parameters (short version):
	//---------------------------------------------------------------------

	void print_short()
	{
		// Standard parameters:
		std::cout << "(" <<
			"S: " << Par[DIMEN_STARTUP]  << ", " <<
			"X: " << Par[DIMEN_LOADX]    << ", " <<
			"A: " << Par[DIMEN_RULE_APP] << ", " <<
			"R: " << Par[DIMEN_RESULT];

		// Optional Parameters:
		if(Par[DIMEN_DED_LOGR] > 0)
			std::cout << ", " << "DL: " << Par[DIMEN_DED_LOGR];
		if(Par[DIMEN_ITER] > 0)
			std::cout << ", " << "IT: " << Par[DIMEN_ITER];
		if(Par[DIMEN_LOAD] > 0)
			std::cout << ", " << "LL: " << Par[DIMEN_LOAD];
		if(Par[DIMEN_DATA_VAL] > 0)
			std::cout << ", " << "DV: " << Par[DIMEN_DATA_VAL];
		if(Par[DIMEN_NEW_FACTS] > 0)
			std::cout << ", " << "NF: " << Par[DIMEN_NEW_FACTS];

		// Done:
		std::cout << ")" << std::endl;
	}

	//---------------------------------------------------------------------
	// The Formula to Estimate Runtime from Graph Data:
	//---------------------------------------------------------------------

	inline long formula(data_t data_p)
	{
		bigint_t edges = data_p->edges();
		long load_part = IDIV_ROUND(edges * Par[DIMEN_LOAD], 1000000);

		long rule_part = 0;
		bigint_t cost = data_p->cost();
		rule_part = IDIV_ROUND(cost * Par[DIMEN_RULE_APP], 1000000);
		//bigint_t cost = data_p->cost_log_tc();
		//rule_part = IDIV_ROUND(cost * Par[DIMEN_RULE_APP],
		//		static_cast<bigint_t>(1000000000));

		bigint_t tc_size = data_p->tc_size();
		long result_part = IDIV_ROUND(tc_size * Par[DIMEN_RESULT],
					 1000000);

		bigint_t iter_edges = data_p->iter_edges();
		long iter_part = IDIV_ROUND(iter_edges * Par[DIMEN_ITER],
					 1000000);

		bigint_t nodes = data_p->nodes();
		long data_part = IDIV_ROUND(nodes * Par[DIMEN_DATA_VAL],
					1000000);

		bigint_t edges_log_edges = data_p->edges_log_edges();
		long loadx_part = IDIV_ROUND(edges_log_edges * Par[DIMEN_LOADX],
					1000000);

		bigint_t new_facts = data_p->new_facts();
		long new_facts_part = IDIV_ROUND(new_facts*Par[DIMEN_NEW_FACTS],
					1000000);
		return Par[DIMEN_STARTUP] +
			load_part + rule_part + result_part + iter_part +
			loadx_part + data_part + new_facts_part;
	}

	//---------------------------------------------------------------------
	// Attributes:
	//---------------------------------------------------------------------

	private:
	long Par[DIMEN_ARRAY_SIZE];
};

//-----------------------------------------------------------------------------
// Pointer Type for Formula Parameters:
//-----------------------------------------------------------------------------

typedef par_c *par_t;


//=============================================================================
// Evaluation of the Formula (How good is it?):
//=============================================================================

//-----------------------------------------------------------------------------
// check_formula:
//-----------------------------------------------------------------------------

int check_formula(par_c *par_p, sys_t sys)
{
	int hits = 0;

	data_c *data_p = sys->data();
	for(int i = sys->num_data_points(); i-- > 0; data_p++) {
		long estimate = par_p->formula(data_p);
		long limit = estimate / 5;
		long runtime = data_p->real_time();
		if(runtime >= estimate-limit && runtime <= estimate+limit)
			hits++;
	}

	return hits;
}

//-----------------------------------------------------------------------------
// check_formula_pct: Returns the sum of error percentages.
//-----------------------------------------------------------------------------

long check_formula_pct(par_c *par_p, sys_t sys)
{
	long sum_pct = 0;

	data_c *data_p = sys->data();
	for(int i = sys->num_data_points(); i-- > 0; data_p++) {
		long estimate = par_p->formula(data_p);
		if(estimate > 0) {
			long runtime = data_p->real_time();
			long pct = ((runtime - estimate) * 100 + estimate/2)
				/ estimate;
			if(pct > 0)
				sum_pct += pct;
			else
				sum_pct -= pct;
		}
	}

	return sum_pct;
}

//-----------------------------------------------------------------------------
// Check Formula with Verbose Output:
//-----------------------------------------------------------------------------

int check_formula_with_output(par_c *par_p, sys_t sys)
{
	int hits = 0;
	long sum_pct = 0;
	long num_pct = 0;

	std::cout << "Detailed Formula Evaluation:\n";
	std::cout << "---------------------------\n";
	par_p->print("\t");
	std::cout << "\n";

	data_c *data_p = sys->data();
	for(int i = sys->num_data_points(); i-- > 0; data_p++) {
		long estimate = par_p->formula(data_p);
		long limit = estimate / 5;
		long runtime = data_p->real_time();
		std::cout << "Graph: " << data_p->graph() <<
			", Estimate: " << estimate <<
			", Runtime: " << runtime;
		if(estimate-limit > runtime)
			std::cout << ", miss (+)";
		else if(estimate+limit < runtime)
			std::cout << ", miss (-)";
		else {
			hits++;
			if(estimate > runtime)
				std::cout << ", hit (+)";
			else if(estimate < runtime)
				std::cout << ", hit (-)";
			else
				std::cout << ", hit (!)";
		}
		if(estimate > 0) {
			long pct = ((runtime - estimate) * 100 + estimate/2)
				/ estimate;
			std::cout << " Error: " << pct << "%";
			num_pct++;
			if(pct > 0)
				sum_pct += pct;
			else
				sum_pct -= pct;
		}
		std::cout << "\n";
	}

	std::cout << "Hits: " << hits << "\n";
	if(num_pct > 0) {
		std::cout << "Average Error: " <<
			(sum_pct + num_pct/2)/num_pct << "%\n";
	}
	return hits;
}

//-----------------------------------------------------------------------------
// Distance measure for local optimization:
//-----------------------------------------------------------------------------

bigint_t distance(par_c *par_p, sys_t sys)
{
	bigint_t sum_dist = 0;

	data_c *data_p = sys->data();
	int num_graphs = sys->num_data_points();
	for(int i = num_graphs; i-- > 0; data_p++) {
		long estimate = par_p->formula(data_p);
		long limit = estimate / 5;
		if(estimate <= 0)
			estimate = 1;
		long runtime = data_p->real_time();
		long rel_dist;

		// Compute distance of estimate from real value in promille:
		if(estimate > runtime)
			rel_dist = ((estimate - runtime) * 1000) / estimate;
		else if(estimate == runtime)
			rel_dist = 0;
		else // runtime > estimate
			rel_dist = ((runtime - estimate) * 1000) / estimate;

		// Limit the relative distance to make sure that hits have prio:
		if(rel_dist > 100000/num_graphs)
			rel_dist = 100000/num_graphs;

		// Add relative distance to sum (to optimize within same hits):
		sum_dist += rel_dist;

		// Emphasize distance outside the 20% limit:
		if(rel_dist > 200) {
			//sum_dist += 1000000;
			sum_dist += 100 * (rel_dist - 200);
		}

		// Because of rounding problems, it happened that something
		// was not counted as hit, although the relative distance
		// was 200.
		// To be sure, we use the exact condition for hits (negated):
		if(runtime < estimate-limit || runtime > estimate+limit)
			sum_dist += 10000000;

	}

	return sum_dist;
}

//=============================================================================
// Auxiliary Functions to Determine the Possible Range of Parameters:
//=============================================================================


//-----------------------------------------------------------------------------
// Compute Maximal Value for the Rule Application Cost Parameter:
//-----------------------------------------------------------------------------

// If the runtime has to be explained entirely by the rule application cost
// (i.e. all other parameters are 0), what would be the value?
// This function computes the maximal such value over all graphs:

long compute_max_rule(sys_t sys)
{
	long max_rule = 0;
	long sum_rule = 0;

	data_c *data_p = sys->data();
	int num_graphs = sys->num_data_points();
	for(int i = num_graphs; i-- > 0; data_p++) {
		bigint_t cost = data_p->cost();
		bigint_t real_time = data_p->real_time();
		if(cost > 0) {
			long rule = (real_time*1000000 + (cost-1)) / cost;
			if(rule > max_rule)
				max_rule = rule;
			sum_rule += rule;
		}
	}

	// Return reasonable upper limit:
	long double_avg = (2*sum_rule) / num_graphs;
	return MIN(double_avg, max_rule);
}

//-----------------------------------------------------------------------------
// Compute Maximal Value for the Result Cost Parameter:
//-----------------------------------------------------------------------------

long compute_max_result(sys_t sys)
{
	long max_result = 0;
	long sum_result = 0;

	data_c *data_p = sys->data();
	int num_graphs = sys->num_data_points();
	for(int i = num_graphs; i-- > 0; i++, data_p++) {
		bigint_t tc_size = data_p->tc_size();
		bigint_t real_time = data_p->real_time();
		if(tc_size > 0) {
			long result = (real_time*1000000 + (tc_size-1)) / 
								tc_size;
			if(result > max_result)
				max_result = result;

			sum_result += result;
		}
	}

	// Return reasonable upper limit:
	long double_avg = (2*sum_result) / num_graphs;
	return MIN(double_avg, max_result);
}

//-----------------------------------------------------------------------------
// Compute Maximal Value for the Load Parameter:
//-----------------------------------------------------------------------------

long compute_max_load(sys_t sys)
{
	long max_load = 0;
	long sum_load = 0;

	// Compute maximum value over all graphs:
	data_c *data_p = sys->data();
	int num_graphs = sys->num_data_points();
	for(int i = num_graphs; i-- > 0; data_p++) {
		bigint_t edges = data_p->edges();
		bigint_t real_time = data_p->real_time();
		if(edges > 0) {
			long load = (real_time*1000000 + (edges-1)) / edges;
			if(load > max_load)
				max_load = load;

			sum_load += load;
		}
	}

	// Limit it with a somewhat reasonable value:
	// if(max_load > 10000)
	//	max_load = 10000;

	// Return reasonable upper limit:
	long double_avg = (2*sum_load) / num_graphs;
	return MIN(double_avg, max_load);
}


//-----------------------------------------------------------------------------
// Compute Step Size for a Parameter:
//-----------------------------------------------------------------------------

long step_size(long max_value, long num_values)
{
	// Avoid division by 0:
	if(num_values == 0)
		return 1;

	// One value is always the minimal value:
	if(num_values == 1)
		return max_value + 1;

	// Compute step size (first value is always the minimal value):
	long step = max_value / (num_values-1);

	// Ensure that at least 1:
	if(step < 1)
		step = 1;

	// Return step size:
	return step;
}

//-----------------------------------------------------------------------------
// Compute Maximal Value for the Data Values Cost Parameter:
//-----------------------------------------------------------------------------

long compute_max_dataval(sys_t sys)
{
	long max_dataval = 0;
	long sum_dataval = 0;

	data_c *data_p = sys->data();
	int num_graphs = sys->num_data_points();
	for(int i = num_graphs; i-- > 0; data_p++) {
		bigint_t nodes = data_p->nodes();
		bigint_t real_time = data_p->real_time();
		if(nodes > 0) {
			long dataval = (real_time*1000000 + (nodes-1)) / nodes;
			if(dataval > max_dataval)
				max_dataval = dataval;
			sum_dataval += dataval;
		}
	}

	// Return reasonable upper limit:
	long double_avg = (2*sum_dataval) / num_graphs;
	return MIN(double_avg, max_dataval);
}


//-----------------------------------------------------------------------------
// Compute Maximal Value for the Iter Cost Parameter:
//-----------------------------------------------------------------------------

long compute_max_iter(sys_t sys)
{
	long max_val = 0;
	long sum_val = 0;

	data_c *data_p = sys->data();
	int num_graphs = sys->num_data_points();
	for(int i = num_graphs; i-- > 0; data_p++) {
		bigint_t cost = data_p->iter_edges();
		bigint_t real_time = data_p->real_time();
		if(cost > 0) {
			long val = IDIV_CEILING(real_time*1000000, cost);
			if(val > max_val)
				max_val = val;
			sum_val += val;
		}
	}

	// Return reasonable upper limit:
	long double_avg = (2*sum_val) / num_graphs;
	return MIN(double_avg, max_val);
}

//-----------------------------------------------------------------------------
// Compute Maximal Value for the Index Insertion Cost Parameter (LOADX):
//-----------------------------------------------------------------------------

long compute_max_loadx(sys_t sys)
{
	long max_val = 0;
	long sum_val = 0;

	data_c *data_p = sys->data();
	int num_graphs = sys->num_data_points();
	for(int i = num_graphs; i-- > 0; data_p++) {
		bigint_t cost = data_p->edges_log_edges();
		bigint_t real_time = data_p->real_time();
		if(cost > 0) {
			long val = IDIV_CEILING(real_time*1000000, cost);
			if(val > max_val)
				max_val = val;
			sum_val += val;
		}
	}

	// Return reasonable upper limit:
	long double_avg = (2*sum_val) / num_graphs;
	return MIN(double_avg, max_val);
}

//-----------------------------------------------------------------------------
// Determine Maximal Reasonable Values for All Parameters:
//-----------------------------------------------------------------------------

void compute_max_par(par_t max_par, sys_t sys)
{
	// Determine maximal value for load efficiency:
	long max_load = compute_max_load(sys);
	// The file contains about 8-9 bytes per edge.
	// I.e. it should be possible to read about 10mio edges per second
	// (with a fast disc and a sequentially stored/non-fragmented file).
	// However, the edges must be processed. i.e. entered in an index.
	// For some systems, we measured the load time.
	// xsb uses up to 6s load time per million edges. The average is 2.2s.
	// long max_load = 6000;

	// Limit the load time to a somewhat reasonable value:
	if(max_load > 100000)
		max_load = 100000;

	// Determine maximal value for speed:
	long max_rule = compute_max_rule(sys);

	// Reasonable maximal value for startup time:
	long max_startup = 1000;

	// Compute maximal value for time per one million result tuples:
	long max_result = compute_max_result(sys);

	// Compute maximal value for time per one million scans (iter*edges):
	long max_iter = compute_max_iter(sys);

	// Compute maximal value for time per one million index insertion cost:
	long max_loadx = compute_max_loadx(sys);

	// Compute maximal value for time per one million data values:
	long max_dataval = compute_max_dataval(sys);

	// Set the maximal values:
	max_par->set(DIMEN_STARTUP,  max_startup);
	max_par->set(DIMEN_LOAD,     max_load);
	max_par->set(DIMEN_RULE_APP, max_rule);
	max_par->set(DIMEN_RESULT,   max_result);
	max_par->set(DIMEN_ITER,     max_iter);
	max_par->set(DIMEN_LOADX,    max_loadx);
	max_par->set(DIMEN_DATA_VAL, max_dataval);
}

//=============================================================================
// Function for local optimization:
//=============================================================================

void optimize_local(par_t par, sys_t sys, int max_steps)
{
	dimen_t best_dimen = DIMEN_FIRST;
	bool best_inc = true; // Add +1 to parameter value in best_dimen
	bigint_t best_dist = distance(par, sys);
	while(best_dimen != DIMEN_NULL && max_steps-- > 0) {

		//std::cout << "DIST: " << best_dist <<
		//	", HITS: " << check_formula(par, sys) << ": ";
		//par->print_short();

		// Stop loop if we cannot improve the distance:
		best_dimen = DIMEN_NULL;

		// Try all dimensions, move in direction with smallest distance:
		for(dimen_t dimen = DIMEN_FIRST; dimen < DIMENSIONS; dimen++) {
			par->inc(dimen);
			bigint_t new_dist = distance(par, sys);
			if(new_dist < best_dist) {
				best_dist  = new_dist;
				best_dimen = dimen;
				best_inc   = true;
			}
			// Move back:
			par->dec(dimen);

			// Now do the same for the negative direction:
			par->dec(dimen);
			new_dist = distance(par, sys);
			if(new_dist < best_dist) {
				best_dist  = new_dist;
				best_dimen = dimen;
				best_inc   = false;
			}
			// Move back:
			par->inc(dimen);
		}
		//std::cout << "\tBest Dimen: " << best_dimen << "\n";

		// Now do the best move:
		if(best_dimen != DIMEN_NULL) {
			if(best_inc)
				par->inc(best_dimen);
			else
				par->dec(best_dimen);
		}

	// End of while loop over optimization steps:
	}
}

//=============================================================================
// Function for global optimization with relative optimization at each point:
//=============================================================================

int optimize(par_t best_par, sys_t sys,
		par_t min_par, par_t max_par, par_t inc_par)
{
	int best_hits = 0;

	// Compute total number of steps:
	bigint_t total_steps = 1;
	for(dimen_t d = DIMEN_FIRST; d <= DIMEN_LAST; d++) {
		long diff = max_par->get(d) - min_par->get(d);
		total_steps *= IDIV_CEILING((diff+1), inc_par->get(d));
	}

	// For output of progress:
	bigint_t one_percent = IDIV_CEILING(total_steps, 100);
	bigint_t countdown = 1;
	int pct = 0;

	// Optimization:
	par_c par;
	while(1) {

		// Print control output:
		if(--countdown <= 0) {
			std::cout << pct << "%\n";
			countdown = one_percent;
			pct++;
		}

		par_c opt_par = par;
		optimize_local(&opt_par, sys, 1000);
		//par.print_short();

		int hits = check_formula(&opt_par, sys);
		if(hits > best_hits) {
			*best_par = opt_par;
			best_hits = hits;
		}

		// Determine next parameter value:
		dimen_t dimen = DIMEN_LAST;
		while(dimen >= DIMEN_FIRST) {
			par.add(dimen, inc_par->get(dimen));
			if(par.get(dimen) <= max_par->get(dimen))
				break;
			par.set(dimen, min_par->get(dimen));
			dimen--;
			
		}
		if(dimen < DIMEN_FIRST)
			break;
	}

	return best_hits;
}

//=============================================================================
// Class for Optimization Job (Can be executed in a thread):
//=============================================================================

class job_c {

	public:

	//---------------------------------------------------------------------
	// Default Constructor (needed to declare array of jobs):
	//---------------------------------------------------------------------

	job_c() :
		MinPar(), MaxPar(), IncPar(), BestPar()
	{
		Sys   = nullptr;
		JobNo = 0;
		BestHits = -1;
		SumErrPct = -1;
	}

	//---------------------------------------------------------------------
	// This does the real initialization:
	//---------------------------------------------------------------------

	void init(sys_t job_sys, const par_c& job_min_par,
			const par_c& job_max_par, const par_c& job_inc_par,
			dimen_t split_dimen, int split_jobs, int job_no)
	{
		// Store given data:
		Sys      = job_sys;
		MinPar   = job_min_par;
		MaxPar   = job_max_par;
		IncPar   = job_inc_par;
		JobNo    = job_no;
		BestHits = -1;
		SumErrPct= -1;

		// Check splitting parameters:
#if DO_CHECK
		if(split_jobs < 1) {
			std::cerr << "Invalid number of jobs: " <<
					split_jobs << "\n";
			exit(6);
		}
		if(job_no < 0 || job_no >= split_jobs) {
			std::cerr << "Invalid job no: " << job_no << "\n";
			exit(6);
		}
#endif

		// Stop here if only one job (serial execution):
		if(split_dimen == DIMEN_NULL || split_jobs <= 1)
			return;

		// Split parameter range in given dimension (for parallel exec):
		long min = MinPar.get(split_dimen);
		long max = MaxPar.get(split_dimen);
		long inc = IncPar.get(split_dimen);
		long diff = max - min;
		if(inc == 0) {
			std::cerr << split_jobs << " jobs requested, " <<
				"but increment in dimension " << split_dimen <<
				" is 0.\n";
				exit(7);
		}
		long steps = IDIV_CEILING((diff+1), inc);
		if(steps < split_jobs) {
			std::cerr << split_jobs << " jobs requested, " <<
				"but only " << steps << " steps " <<
				"in dimension " << split_dimen << ".\n";
				exit(7);
		}
		long steps_per_job = steps / split_jobs;
		long remaining_steps = steps % split_jobs;
		long min_in_this_job = min + job_no * steps_per_job * inc;
		long max_in_this_job = min + (job_no+1) * steps_per_job * inc
			- 1;
		// We distribute the remaining_steps over the first jobs
		// (one additional step per job):
		if(job_no < remaining_steps) {
			min_in_this_job += (job_no * inc);
			// The first job has job_no==0, thus the first minimum
			// will not be increased.
			max_in_this_job += ((job_no+1) * inc);
			// This does not change the property that the intervals
			// are consecutive, because the next minimum will be
			// increased by the next job_no, which is job_no+1.
		}
		else {
			min_in_this_job += (remaining_steps * inc);
			max_in_this_job += (remaining_steps * inc);
		}

		// Now set minimum and maximum for the split_dimenion:
		MinPar.set(split_dimen, min_in_this_job);
		MaxPar.set(split_dimen, max_in_this_job);

		// Make sure that the interval of the last job ends at max:
		// (This is probably unnecessary)
		if(job_no == split_jobs - 1)
			MaxPar.set(split_dimen, max);

		// Test Output:
		// std::cout << "Job " << JobNo << ":\n";
		// std::cout << "\tMin:   "; MinPar.print_short();
		// std::cout << "\tMax:   "; MaxPar.print_short();
		// std::cout << "\tInc:   "; IncPar.print_short();
		// std::cout << "\tSteps: " << TotalSteps << std::endl;
	}

	//---------------------------------------------------------------------
	// Do Optimization (without relative optimization):
	//---------------------------------------------------------------------

	int optimize_simple()
	{
		int max_hits = 0;

		// For output of progress:
		bigint_t output_steps = IDIV_CEILING(total_steps() * OUT_PCT,
							100);
		bigint_t countdown = 1;
		int pct = 0;

		// Optimization:
		par_c par = MinPar;
		long min_err_pct = check_formula_pct(&par, Sys);
		while(1) {
			// Print control output:
			if(--countdown <= 0) {
#if NUM_JOBS > 1
				std::lock_guard<std::mutex>
					output(output_mutex);
				std::cout << "[Job " << JobNo << ": " <<
					pct << "%, Best: " << max_hits <<
					"]: ";
#else
				std::cout << pct << "%, Best:" << max_hits;
#endif
				BestPar.print_short();
				std::cout << std::flush;
				countdown = output_steps;
				pct += OUT_PCT;
			}

			//par.print_short();

			int hits = check_formula(&par, Sys);
			if(hits == max_hits) {
				long err_pct = check_formula_pct(&par, Sys);
				if(err_pct < min_err_pct) {
					BestPar = par;
					min_err_pct = err_pct;
				}
			}
			if(hits > max_hits) {
				BestPar = par;
				max_hits = hits;
				min_err_pct = check_formula_pct(&par, Sys);
			}

			// Determine next parameter value:
			dimen_t dimen = DIMEN_LAST;
			while(dimen >= DIMEN_FIRST) {
				long inc = IncPar.get(dimen);
				if(inc > 0) {
					par.add(dimen, inc);
					if(par.get(dimen) <= MaxPar.get(dimen))
						break;
					par.set(dimen, MinPar.get(dimen));
				}
				dimen--;
				
			}
			if(dimen < DIMEN_FIRST)
				break;
		}

		BestHits = max_hits;
		SumErrPct = min_err_pct;
		return max_hits;
	}

	//---------------------------------------------------------------------
	// Attribute Accessor Functions:
	//---------------------------------------------------------------------

	inline const par_c *best_par() const { return &BestPar; }

	inline int best_hits() const { return BestHits; }

	inline int sum_err_pct() const { return SumErrPct; }

	//---------------------------------------------------------------------
	// Compute total number of steps:
	//---------------------------------------------------------------------

	bigint_t total_steps() const
	{
		bigint_t total = 1;
		for(dimen_t d = DIMEN_FIRST; d <= DIMEN_LAST; d++) {
			long diff = MaxPar.get(d) - MinPar.get(d);
			long steps = 1;
			if(IncPar.get(d) > 0)
				steps = IDIV_CEILING((diff+1), IncPar.get(d));
			total *= steps;
		}
		return total;
	}

	//---------------------------------------------------------------------
	// Attributes:
	//---------------------------------------------------------------------

	private:

	sys_t Sys;
	par_c MinPar;
	par_c MaxPar;
	par_c IncPar;
	par_c BestPar;
	int   BestHits;
	int   SumErrPct;
	int   JobNo;

#if NUM_JOBS > 1
	static std::mutex output_mutex;
#endif

};
#if NUM_JOBS > 1
	std::mutex job_c::output_mutex;
#endif


//-----------------------------------------------------------------------------
// Pointer Type for Job:
//-----------------------------------------------------------------------------

typedef job_c *job_t;

//=============================================================================
// Functions For Setting Up and Executing Optimization Jobs:
//=============================================================================

//-----------------------------------------------------------------------------
// Function to be Executed by a Separate Thread:
//-----------------------------------------------------------------------------

void run_job(job_t job) {
	job->optimize_simple();
}

//-----------------------------------------------------------------------------
// Parallel Execution of Simple Optimization:
//-----------------------------------------------------------------------------

#if NUM_JOBS > 1

int optimize_simple(sys_t sys, const par_c& min_par, const par_c&max_par,
			const par_c&inc_par, par_t best_par)
{
	// Find a dimension to split the work between the jobs:
	dimen_t dimen_split = DIMEN_NULL;
	for(dimen_t d = DIMEN_FIRST; d <= DIMEN_LAST; d++) {
		long interval = max_par.get(d) - min_par.get(d);
		long inc = inc_par.get(d);
		if(inc > 0 && interval/inc >= NUM_JOBS) {
			dimen_split = d;
			break; // Take the first possible dimension.
		}
	}
	if(dimen_split == DIMEN_NULL) {
		std::cerr << "Did not find dimension " <<
				"to split work between jobs!\n";
		exit(7);
	}

	// Optimization Jobs:
	job_c jobs[NUM_JOBS];
	for(int i = 0; i < NUM_JOBS; i++) {
		jobs[i].init(sys, min_par, max_par, inc_par,
				dimen_split, NUM_JOBS, i);
	}

	// Compute total number of parameter tuples:
	bigint_t total_steps = 0;
	for(int i = 0; i < NUM_JOBS; i++)
		total_steps += jobs[i].total_steps();
	std::cout << "Number of parameter tuples: " << total_steps << "\n";
	std::cout << std::endl;

	// Start threads for parallel execution of all jobs:
	std::cout << "Starting " << NUM_JOBS << " threads ..." << std::endl;
	std::thread threads[NUM_JOBS];
	for(int i = 0; i < NUM_JOBS; i++) {
		// Move-assign threads:
		threads[i] = std::thread(run_job, &(jobs[i]));
	}

	// Join threads:
	for(int i = 0; i < NUM_JOBS; i++)
		threads[i].join();

	// Output from main thread:
	std::cout << "Threads have finished.\n";

	// Find the best solution among the job results:
	*best_par = *(jobs[0].best_par());
	int best_hits = jobs[0].best_hits();
	long sum_err_pct = jobs[0].sum_err_pct();
	for(int i = 1; i < NUM_JOBS; i++) {
		int job_best_hits = jobs[i].best_hits();
		if(job_best_hits > best_hits) {
			best_hits = job_best_hits;
			sum_err_pct = jobs[i].sum_err_pct();
			*best_par = *(jobs[i].best_par());
		}
		else if(job_best_hits == best_hits &&
				jobs[i].sum_err_pct() < sum_err_pct) {
			sum_err_pct = jobs[i].sum_err_pct();
			*best_par = *(jobs[i].best_par());
		}
	}

	// Return hits of best parameter:
	return best_hits;
}

#endif

//-----------------------------------------------------------------------------
// Serial Version of Simple Optimization (without threads):
//-----------------------------------------------------------------------------

#if NUM_JOBS <= 1

int optimize_simple(sys_t sys, const par_c& min_par, const par_c&max_par,
			const par_c&inc_par, par_t best_par)
{
	// Optimization Job (parameter range is not splitted, only one job):
	job_c job;
	job.init(sys, min_par, max_par, inc_par, DIMEN_NULL, 1, 0);

	// Print total number of parameter tuples:
	std::cout << "Number of parameter tuples: " << job.total_steps() <<
			"\n";
	std::cout << std::endl;

	// Do optimization:
	//best_hits = optimize_simple(&best_par, sys,
	//		&min_par, &max_par, &inc_par);
	job.optimize_simple();

	// Find the best solution among the job results:
	*best_par = *(job.best_par());
	int best_hits = job.best_hits();

	// Return hits of best parameter:
	return best_hits;
}

#endif

//=============================================================================
// Function to Check the Best Known Parameter Values:
//=============================================================================


void best_known()
{
	std::cout << "\n";

	// XSB:
	std::cout << "*** XSB ***\n\n";
	sys_t xsb = get_sys("xsb");
	if(xsb) {
		std::cout << "Number of Graphs: " <<
			xsb->num_data_points() << "\n";
		par_c xsb_par;
		// Old:
		//xsb_par.set(DIMEN_STARTUP,    92);
		//xsb_par.set(DIMEN_LOAD,      999);
		//xsb_par.set(DIMEN_RULE_APP,   97);
		//xsb_par.set(DIMEN_RESULT,    300);
		// This gives 49/50:
		xsb_par.set(DIMEN_STARTUP,   117);
		xsb_par.set(DIMEN_LOADX,     109);
		xsb_par.set(DIMEN_RULE_APP,   84);
		xsb_par.set(DIMEN_RESULT,    332);
		check_formula_with_output(&xsb_par, xsb);
		std::cout << "\n\n";
	}

	//BAM:
	std::cout << "*** BAM ***\n\n";
	sys_t bam = get_sys("bam");
	if(bam) {
		std::cout << "Number of Graphs: " <<
			bam->num_data_points() << "\n";
		par_c bam_par;
		bam_par.set(DIMEN_STARTUP,    53);
		bam_par.set(DIMEN_LOADX,     320);
		bam_par.set(DIMEN_RULE_APP,   45);
		bam_par.set(DIMEN_RESULT,     36);
		bam_par.set(DIMEN_DATA_VAL, 2000);
		check_formula_with_output(&bam_par, bam);
		std::cout << "\n\n";
	}

	//BAM New:
	std::cout << "*** BAM NEW ***\n\n";
	if(bam) {
		std::cout << "Number of Graphs: " <<
			bam->num_data_points() << "\n";
		par_c bam_par2;
		bam_par2.set(DIMEN_STARTUP,     0);
		bam_par2.set(DIMEN_LOADX,       0);
		bam_par2.set(DIMEN_RULE_APP,   50);
		bam_par2.set(DIMEN_RESULT,     48);
		check_formula_with_output(&bam_par2, bam);
		std::cout << "\n\n";
	}

	//YAP:
	std::cout << "*** YAP ***\n\n";
	sys_t yap = get_sys("yap");
	if(yap) {
		std::cout << "Number of Graphs: " <<
			yap->num_data_points() << "\n";
		par_c yap_par;
		// Old:
		//yap_par.set(DIMEN_STARTUP,    33);
		//yap_par.set(DIMEN_LOAD,      670);
		//yap_par.set(DIMEN_RULE_APP,  242);
		//yap_par.set(DIMEN_RESULT,    300);
		yap_par.set(DIMEN_STARTUP,    19);
		yap_par.set(DIMEN_LOADX,       0);
		yap_par.set(DIMEN_RULE_APP,  200);
		yap_par.set(DIMEN_RESULT,    221);
		check_formula_with_output(&yap_par, yap);
		std::cout << "\n\n";
	}

	//souffle:
	std::cout << "*** SOUFFLE ***\n\n";
	sys_t souffle = get_sys("souffle");
	if(souffle) {
		std::cout << "Number of Graphs: " <<
			yap->num_data_points() << "\n";
		par_c souffle_par;
		// Old:
		//souffle_par.set(DIMEN_STARTUP,   100);
		//souffle_par.set(DIMEN_LOADX,      26);
		//souffle_par.set(DIMEN_RULE_APP,    7);
		//souffle_par.set(DIMEN_RESULT,    909);
		// Old2:
		//souffle_par.set(DIMEN_STARTUP,   122);
		//souffle_par.set(DIMEN_LOADX,      63);
		//souffle_par.set(DIMEN_RULE_APP,   42);
		//souffle_par.set(DIMEN_RESULT,    500);
		//
		souffle_par.set(DIMEN_STARTUP,     0);
		souffle_par.set(DIMEN_LOADX,     224);
		souffle_par.set(DIMEN_RULE_APP,   37);
		souffle_par.set(DIMEN_RESULT,    696);
		check_formula_with_output(&souffle_par, souffle);
		std::cout << "\n\n";
	}

	//SQLite:
	std::cout << "*** SQLite ***\n\n";
	sys_t sqlite = get_sys("sqlite3");
	if(sqlite) {
		std::cout << "Number of Graphs: " <<
			sqlite->num_data_points() << "\n";
		par_c sql_par;
		//Old:
		//sql_par.set(DIMEN_STARTUP,    55);
		//sql_par.set(DIMEN_LOAD,     6000);
		//sql_par.set(DIMEN_RULE_APP,  692);
		//sql_par.set(DIMEN_RESULT,   1000);
		//Old:
		//sql_par.set(DIMEN_STARTUP,    50);
		//sql_par.set(DIMEN_LOADX,     380);
		//sql_par.set(DIMEN_RULE_APP,  643);
		//sql_par.set(DIMEN_RESULT,   1524);
		//
		sql_par.set(DIMEN_STARTUP,    46);
		sql_par.set(DIMEN_LOADX,     300);
		sql_par.set(DIMEN_RULE_APP,  600);
		sql_par.set(DIMEN_RESULT,   1213);
		check_formula_with_output(&sql_par, sqlite);
		std::cout << "\n\n";
	}

	//PostgreSQL:
	std::cout << "*** PostgreSQL ***\n\n";
	sys_t psql = get_sys("psql");
	if(psql) {
		std::cout << "Number of Graphs: " <<
			psql->num_data_points() << "\n";
		par_c psql_par;
		// Old:
		//psql_par.set(DIMEN_STARTUP,    75);
		//psql_par.set(DIMEN_LOADX,     570);
		//psql_par.set(DIMEN_RULE_APP,  440);
		//psql_par.set(DIMEN_RESULT,   1020);
		psql_par.set(DIMEN_STARTUP,   174);
		psql_par.set(DIMEN_LOADX,     181);
		psql_par.set(DIMEN_RULE_APP,  422);
		psql_par.set(DIMEN_RESULT,    722);
		check_formula_with_output(&psql_par, psql);
		std::cout << "\n\n";
	}

	//Jena:
	std::cout << "*** Jena ***\n\n";
	sys_t jena = get_sys("jena");
	if(jena) {
		std::cout << "Number of Graphs: " <<
			jena->num_data_points() << "\n";
		par_c jen_par;
		// Old:
		//jen_par.set(DIMEN_STARTUP,  1650);
		//jen_par.set(DIMEN_LOADX,    1710);
		//jen_par.set(DIMEN_RULE_APP,  150);
		//jen_par.set(DIMEN_RESULT,   1500);
		jen_par.set(DIMEN_STARTUP,  1953);
		jen_par.set(DIMEN_LOADX,    1200);
		jen_par.set(DIMEN_RULE_APP,  153);
		jen_par.set(DIMEN_RESULT,   1459);
		check_formula_with_output(&jen_par, jena);
		std::cout << "\n\n";
	}
}

//=============================================================================
// Process command line argument with search specification for one dimension:
//=============================================================================

// Reasonable upper limit for dimension name size or parameter value digits:
#define ARG_PART_SIZE	20

void process_arg(str_t arg, par_t min_par, par_t max_par, par_t inc_par) {
	char arg_part[ARG_PART_SIZE];

	// Copy dimension name:
	str_t p = arg;
	int i;
	for(i = 0; *p && *p != '=' && i < ARG_PART_SIZE - 1; )
		arg_part[i++] = *p++;
	arg_part[i] = '\0';
	if(*p != '=') {
		std::cerr << "Argument '" << arg << "': Did not find '='\n";
		exit(2);
	}
	p++; // Skip =
	dimen_t dimen = dimen_parse(arg_part);
	if(dimen == DIMEN_NULL) {
		std::cerr << "Argument '" << arg << "': Unknown dimension " <<
			"'" << arg_part << "'.\n";
		exit(2);
	}

	// Get first number for parameter range:
	for(i = 0; *p && is_digit(*p) && i < ARG_PART_SIZE - 1; )
		arg_part[i++] = *p++;
	arg_part[i] = '\0';
	if(i == 0) {
		std::cerr << "Argument '" << arg << "': " <<
			"Number missing after '='.\n";
		exit(2);
	}
	long min_val = -1; // Changed later. -1 marks "not specified".
	long max_val = atol(arg_part);
	long inc_val = 1;

	// If there is a second number, that is the upper limit:
	if(*p == '-') {
		p++; // Skip -
		min_val = max_val;
		for(i = 0; *p && is_digit(*p) && i < ARG_PART_SIZE - 1; )
			arg_part[i++] = *p++;
		arg_part[i] = '\0';
		if(i == 0) {
			std::cerr << "Argument '" << arg << "': " <<
				"Number missing after '-'.\n";
			exit(2);
		}
		max_val = atol(arg_part);
	}
	if(max_val < min_val) {
		std::cerr << "Argument '" << arg << "': " <<
			"Minimum " << min_val << " > Maximum " << max_val <<
			".\n";
		exit(2);
	}

	// One can specify the number of steps, increment size, or nothing:
	char spec = *p;
	if(spec == '#' || spec == '/') {
		p++; // Skip separator
		// Implicit minimum is 0:
		if(min_val == -1)
			min_val = 0;
		// Read number:
		for(i = 0; *p && is_digit(*p) && i < ARG_PART_SIZE - 1; )
			arg_part[i++] = *p++;
		arg_part[i] = '\0';
		if(i == 0) {
			std::cerr << "Argument '" << arg << "': " <<
				"Number missing after '" << spec << "'.\n";
			exit(2);
		}
		if(spec == '/')
			inc_val = atol(arg_part);
		else {
			long num_steps = atol(arg_part);
			inc_val = step_size(max_val-min_val+1, num_steps);
		}
	}
	else if(spec == '\0') {
		// Implicit minimum is the maximum (only one value):
		if(min_val == -1) {
			min_val = max_val;
		}
	}
	else {
		std::cerr << "Argument '" << arg << "': " <<
				"Invalid character '" << spec << "'.\n";
		exit(2);
	}

	// Set the values in the parameter vectors:
	min_par->set(dimen, min_val);
	max_par->set(dimen, max_val);
	inc_par->set(dimen, inc_val);
}


//=============================================================================
// The main Function (Does the Optimization):
//=============================================================================


int main(int argc, str_t argv[])
{
	int best_hits = 0;
	par_c best_par;

	// Print welcome:
	std::cout << "Welcome to the TC runtime prediction optimizer.\n";
	std::cout << "\n";
	//std::cout << "Number of defined systems: " << NUM_SYSTEMS << "\n";
	std::cout << "Number of Threads:         " << NUM_JOBS << "\n";

	// List evaluation of best known parameter settings:
	if(argc == 2 and str_eq(argv[1],"best")) {
		best_known();
		return 0;
	}

	// The program should be called with the system name as cmdline arg:
	str_t sys_name = "xsb";
	if(argc >= 2)
		sys_name = argv[1];

	// Get data for system:
	sys_t sys = get_sys(sys_name);
	if(sys == nullptr) {
		std::cerr << "Unkown system: " << sys_name << ".\n";
		exit(2);
	}
	std::cout << "Doing optimization for:    " << sys->name() << "\n";

	// Definition of search space:
	par_c min_par;
	par_c max_par;
	par_c inc_par;

	// Get search space from command line:
	if(argc > 2) {
		for(int i = 2; i < argc; i++) {
			process_arg(argv[i], &min_par, &max_par, &inc_par);
		}
	}

	// If none is specified, try a default search space:
	else {
		// Maximal parameter values (minimum is 0):
		max_par.set(DIMEN_STARTUP,    60);
		max_par.set(DIMEN_LOAD,        0);
		max_par.set(DIMEN_RULE_APP,  100);
		max_par.set(DIMEN_RESULT,   1000);
		max_par.set(DIMEN_ITER,        0);
		max_par.set(DIMEN_LOADX,    2000);
		max_par.set(DIMEN_DATA_VAL,    0);
		max_par.set(DIMEN_NEW_FACTS,  10);

		// Number of steps in each dimension:
		par_c steps;
		steps.set(DIMEN_STARTUP,    60);
		steps.set(DIMEN_LOAD,        1);
		steps.set(DIMEN_RULE_APP,  101);
		steps.set(DIMEN_RESULT,   1001);
		steps.set(DIMEN_ITER,        1);
		steps.set(DIMEN_LOADX,    2000);
		steps.set(DIMEN_DATA_VAL,    1);
		steps.set(DIMEN_NEW_FACTS,  11);

		// Compute step size:
		for(dimen_t d = DIMEN_FIRST; d <= DIMEN_LAST; d++) {
			inc_par.set(d, step_size(max_par.get(d), steps.get(d)));
		}
	}

	// Print search range:
	std::cout << "\n";
	std::cout << "Search Space for Parameter Values:\n";
	std::cout << "---------------------------------\n";
	for(dimen_t d = DIMEN_FIRST; d <= DIMEN_LAST; d++) {
		if(max_par.get(d) != 0 || min_par.get(d) != 0 ||
			inc_par.get(d) != 0) {
			std::cout << "\t" << dimen_name(d) << " " <<
				min_par.get(d) << " - " <<
				max_par.get(d) << " [Step: " <<
				inc_par.get(d) << "]\n";
		}
	}
	std::cout << "\n";

	// Do optimization:
	best_hits = optimize_simple(sys, min_par, max_par, inc_par, &best_par);

	// Show runtimes for best formula:
	std::cout << "\n";
	std::cout << "\n";
	best_hits = check_formula_with_output(&best_par, sys);
	std::cout << "\n";
	std::cout << "\n";
	std::cout << "Best Parameter Values (" << best_hits << "/" <<
					sys->num_data_points() << "):\n";
	best_par.print("\t");
	std::cout << "\n";
}

