// ============================================================================
// Project:	rbench - Logic Programming and Database Benchmarks
// Filename:	opt/opt.cpp
// Purpose:	Optimize parameters for runtime estimation formula
// Last Change:	26.09.2018
// Language:	C++
// EMail:	brass@informatik.uni-halle.de
// WWW:		http://www.informatik.uni-halle.de/~brass/
// Address:	Feldschloesschen 15, D-06120 Halle (Saale), GERMANY
// Copyright:	(c) 2018 by Stefan Brass
// License:	See file "LICENSE" for copying conditions.
// Note:	There is no warranty at all - this code may contain bugs.
// ============================================================================

// This must be compiled with the C++11 Standard, because it uses int64_t.
// E.g.:
//	g++ -std=c++11 -Wall -O2 -o opt opt.cpp

#include <iostream>
#include <cstdint>
#include <thread>

//=============================================================================
// Do correctness checks in inner loops (hurts performance):
//=============================================================================

// #define DO_CHECKS 0
#define DO_CHECKS 1

//=============================================================================
// Do parallel optimization? How many parallel jobs?
//=============================================================================

// If NUM_JOBS is 1, the serial version will be chosen.

//#define NUM_JOBS 1
#define NUM_JOBS 4

//=============================================================================
// How often should output be printed (step size for percent of work):
//=============================================================================

#define OUT_PCT 5

//=============================================================================
// Type for Large Integers (64 Bit):
//=============================================================================

typedef int64_t bigint_t;

//=============================================================================
// String Data Type:
//=============================================================================

typedef const char *str_t;

//-----------------------------------------------------------------------------
// str_eq: Compare two strings for equality.
//-----------------------------------------------------------------------------

bool	str_eq(str_t str1, str_t str2)
{
	// Avoid crashes:
	if(str1 == nullptr)
		return(str2 == nullptr);
	else if(str2 == nullptr)
		return(false);

	// Compare strings:
	while(*str1 && *str1==*str2) {
		str1++;
		str2++;
	}
	return(*str1==*str2);
}

//=============================================================================
// Minimum Function:
//=============================================================================

template<class T>
inline T MIN(T x, T y) {
        if(x <= y)
                return x;
        else
                return y;
}


//=============================================================================
// Integer Division:
//=============================================================================

//-----------------------------------------------------------------------------
// IDIV_ROUND:
//-----------------------------------------------------------------------------

/**
 * Integer division, round result
 *
 * Note that this works only for non-negative numbers.
 */

#define IDIV_ROUND(X,Y)	(((X)+(Y)/2)/(Y))

//-----------------------------------------------------------------------------
// IDIV_CEILING:
//-----------------------------------------------------------------------------

/**
 * Integer division, round up.
 *
 * Note that this works only for non-negative numbers.
 */

#define IDIV_CEILING(X,Y)	(((X)+(Y)-1)/(Y))

//=============================================================================
// Class for Graph Data:
//=============================================================================

class data_c {
	public:

	data_c(str_t rt_graph,
			long rt_nodes, long rt_edges,
			bigint_t rt_tc_size, bigint_t rt_cost,
			long rt_iter, long rt_realtime)
	{
		Graph    = rt_graph;
		Nodes    = rt_nodes;
		Edges    = rt_edges;
		TCSize   = rt_tc_size;
		Cost     = rt_cost;
		Iter     = rt_iter;
		RealTime = rt_realtime;

		// Compute Product "Iterations * Edges" only once:
		IterEdges= static_cast<bigint_t>(rt_iter) * rt_edges;

		// Compute "Edges * log(Edges)":
		int log = 0;
		for(long n = Edges >> 1; n > 0; n >>= 1)
			log++;
		EdgesLogEdges = Edges * log;
	}

	inline str_t graph() const { return Graph; }
	inline long nodes() const { return Nodes; }
	inline long edges() const { return Edges; }
	inline bigint_t tc_size() const { return TCSize; }
	inline bigint_t cost() const { return Cost; }
	inline long iter() const { return Iter; }
	inline long real_time() const { return RealTime; }
	inline bigint_t iter_edges() const { return IterEdges; }
	inline bigint_t edges_log_edges() const { return EdgesLogEdges; }
	
	private:
	str_t Graph;
	long Nodes;
	long Edges;
	bigint_t TCSize;
	bigint_t Cost;
	long Iter;
	long RealTime;
	bigint_t IterEdges;
	bigint_t EdgesLogEdges;
};

//-----------------------------------------------------------------------------
// Pointer Type for Graph Data:
//-----------------------------------------------------------------------------

typedef data_c *data_t;

//=============================================================================
// Data Values (exported form the Database):
//=============================================================================


//-----------------------------------------------------------------------------
// Data for XSB:
//-----------------------------------------------------------------------------

data_c data_xsb[] = {
 data_c("k_1k", 1000, 1000000, INT64_C(1000000), INT64_C(1001000000), 1, 103074),
 data_c("k_2k", 2000, 4000000, INT64_C(4000000), INT64_C(8004000000), 1, 810679),
 data_c("a_1k", 1000, 499500, INT64_C(499500), INT64_C(166666500), 1, 16272),
 data_c("a_2k", 2000, 1999000, INT64_C(1999000), INT64_C(1333333000), 1, 127694),
 data_c("c_1k", 1000, 1000, INT64_C(1000000), INT64_C(1001000), 1000, 574),
 data_c("c_2k", 2000, 2000, INT64_C(4000000), INT64_C(4002000), 2000, 1514),
 data_c("c_4k", 4000, 4000, INT64_C(16000000), INT64_C(16004000), 4000, 5511),
 data_c("s_2k_1", 2000, 4000, INT64_C(4000000), INT64_C(8004000), 10, 1765),
 data_c("s_2k_2", 2000, 6000, INT64_C(4000000), INT64_C(12006000), 10, 2054),
 data_c("s_2k_3", 2000, 8000, INT64_C(4000000), INT64_C(16008000), 10, 2337),
 data_c("s_2k_4", 2000, 10000, INT64_C(4000000), INT64_C(20010000), 10, 2595),
 data_c("p_1k", 1000, 999, INT64_C(499500), INT64_C(499500), 999, 344),
 data_c("p_2k", 2000, 1999, INT64_C(1999000), INT64_C(1999000), 1999, 939),
 data_c("p_4k", 4000, 3999, INT64_C(7998000), INT64_C(7998000), 3999, 3055),
 data_c("m_4ki_2", 8192, 8190, INT64_C(16773120), INT64_C(16773120), 4095, 6167),
 data_c("m_1ki_8", 8192, 8184, INT64_C(4190208), INT64_C(4190208), 1023, 1833),
 data_c("m_256_32", 8192, 8160, INT64_C(1044480), INT64_C(1044480), 255, 726),
 data_c("m_64_128", 8192, 8064, INT64_C(258048), INT64_C(258048), 63, 240),
 data_c("m_16_512", 8192, 7680, INT64_C(61440), INT64_C(61440), 15, 130),
 data_c("m_4_2ki", 8192, 6144, INT64_C(12288), INT64_C(12288), 3, 106),
 data_c("b_17", 131071, 131070, INT64_C(1966082), INT64_C(1966082), 16, 1144),
 data_c("b_18", 262143, 262142, INT64_C(4194306), INT64_C(4194306), 17, 1989),
 data_c("b_19", 524287, 524286, INT64_C(8912898), INT64_C(8912898), 18, 4016),
 data_c("u_1k_50k", 1000, 50000, INT64_C(1000000), INT64_C(50050000), 3, 5230),
 data_c("u_1k_125k", 1000, 125000, INT64_C(1000000), INT64_C(125125000), 2, 12440),
 data_c("u_1k_250k", 1000, 250000, INT64_C(1000000), INT64_C(250250000), 2, 24609),
 data_c("u_2k_250k", 2000, 200000, INT64_C(4000000), INT64_C(400200000), 3, 40852),
 data_c("u_2k_500k", 2000, 500000, INT64_C(4000000), INT64_C(1000500000), 2, 98343),
 data_c("u_2k_1m", 2000, 1000000, INT64_C(4000000), INT64_C(2001000000), 2, 194181),
 data_c("f_1k_50k", 1000, 50000, INT64_C(472863), INT64_C(15421338), 8, 1825),
 data_c("f_1k_125k", 1000, 125000, INT64_C(492170), INT64_C(40705777), 5, 4118),
 data_c("f_1k_250k", 1000, 250000, INT64_C(497810), INT64_C(83073458), 5, 8390),
 data_c("f_2k_250k", 2000, 200000, INT64_C(1946015), INT64_C(128081155), 8, 12854),
 data_c("f_2k_500k", 2000, 500000, INT64_C(1985377), INT64_C(330379789), 6, 32402),
 data_c("f_2k_1m", 2000, 1000000, INT64_C(1995412), INT64_C(665127921), 4, 65082),
};

//-----------------------------------------------------------------------------
// Data for BAM:
//-----------------------------------------------------------------------------

data_c data_bam[] = {
 data_c("k_1k", 1000, 1000000, INT64_C(1000000), INT64_C(1001000000), 1, 32291),
 data_c("k_2k", 2000, 4000000, INT64_C(4000000), INT64_C(8004000000), 1, 363309),
 data_c("a_1k", 1000, 499500, INT64_C(499500), INT64_C(166666500), 1, 10595),
 data_c("a_2k", 2000, 1999000, INT64_C(1999000), INT64_C(1333333000), 1, 130801),
 data_c("c_1k", 1000, 1000, INT64_C(1000000), INT64_C(1001000), 1000, 137),
 data_c("c_2k", 2000, 2000, INT64_C(4000000), INT64_C(4002000), 2000, 449),
 data_c("c_4k", 4000, 4000, INT64_C(16000000), INT64_C(16004000), 4000, 2948),
 data_c("s_2k_1", 2000, 4000, INT64_C(4000000), INT64_C(8004000), 10, 554),
 data_c("s_2k_2", 2000, 6000, INT64_C(4000000), INT64_C(12006000), 10, 660),
 data_c("s_2k_3", 2000, 8000, INT64_C(4000000), INT64_C(16008000), 10, 804),
 data_c("s_2k_4", 2000, 10000, INT64_C(4000000), INT64_C(20010000), 10, 947),
 data_c("p_1k", 1000, 999, INT64_C(499500), INT64_C(499500), 999, 81),
 data_c("p_2k", 2000, 1999, INT64_C(1999000), INT64_C(1999000), 1999, 188),
 data_c("p_4k", 4000, 3999, INT64_C(7998000), INT64_C(7998000), 3999, 758),
 data_c("m_4ki_2", 8192, 8190, INT64_C(16773120), INT64_C(16773120), 4095, 3656),
 data_c("m_1ki_8", 8192, 8184, INT64_C(4190208), INT64_C(4190208), 1023, 331),
 data_c("m_256_32", 8192, 8160, INT64_C(1044480), INT64_C(1044480), 255, 126),
 data_c("m_64_128", 8192, 8064, INT64_C(258048), INT64_C(258048), 63, 110),
 data_c("m_16_512", 8192, 7680, INT64_C(61440), INT64_C(61440), 15, 10),
 data_c("m_4_2ki", 8192, 6144, INT64_C(12288), INT64_C(12288), 3, 0),
 data_c("b_17", 131071, 131070, INT64_C(1966082), INT64_C(1966082), 16, 471),
 data_c("b_18", 262143, 262142, INT64_C(4194306), INT64_C(4194306), 17, 1832),
 data_c("b_19", 524287, 524286, INT64_C(8912898), INT64_C(8912898), 18, 8590),
 data_c("u_1k_50k", 1000, 50000, INT64_C(1000000), INT64_C(50050000), 3, 1914),
 data_c("u_1k_125k", 1000, 125000, INT64_C(1000000), INT64_C(125125000), 2, 5284),
 data_c("u_1k_250k", 1000, 250000, INT64_C(1000000), INT64_C(250250000), 2, 11025),
 data_c("u_2k_250k", 2000, 200000, INT64_C(4000000), INT64_C(400200000), 3, 17155),
 data_c("u_2k_500k", 2000, 500000, INT64_C(4000000), INT64_C(1000500000), 2, 39972),
 data_c("u_2k_1m", 2000, 1000000, INT64_C(4000000), INT64_C(2001000000), 2, 83816),
 data_c("f_1k_50k", 1000, 50000, INT64_C(472863), INT64_C(15421338), 8, 710),
 data_c("f_1k_125k", 1000, 125000, INT64_C(492170), INT64_C(40705777), 5, 1867),
 data_c("f_1k_250k", 1000, 250000, INT64_C(497810), INT64_C(83073458), 5, 3623),
 data_c("f_2k_250k", 2000, 200000, INT64_C(1946015), INT64_C(128081155), 8, 6536),
 data_c("f_2k_500k", 2000, 500000, INT64_C(1985377), INT64_C(330379789), 6, 16727),
 data_c("f_2k_1m", 2000, 1000000, INT64_C(1995412), INT64_C(665127921), 4, 34841),
};

//-----------------------------------------------------------------------------
// Data for YAP:
//-----------------------------------------------------------------------------

data_c data_yap[] = {
 data_c("k_1k", 1000, 1000000, INT64_C(1000000), INT64_C(1001000000), 1, 70120),
 data_c("k_2k", 2000, 4000000, INT64_C(4000000), INT64_C(8004000000), 1, 552711),
 data_c("a_1k", 1000, 499500, INT64_C(499500), INT64_C(166666500), 1, 12660),
 data_c("a_2k", 2000, 1999000, INT64_C(1999000), INT64_C(1333333000), 1, 122537),
 data_c("c_1k", 1000, 1000, INT64_C(1000000), INT64_C(1001000), 1000, 496),
 data_c("c_2k", 2000, 2000, INT64_C(4000000), INT64_C(4002000), 2000, 2110),
 data_c("c_4k", 4000, 4000, INT64_C(16000000), INT64_C(16004000), 4000, 9628),
 data_c("s_2k_1", 2000, 4000, INT64_C(4000000), INT64_C(8004000), 10, 3090),
 data_c("s_2k_2", 2000, 6000, INT64_C(4000000), INT64_C(12006000), 10, 4137),
 data_c("s_2k_3", 2000, 8000, INT64_C(4000000), INT64_C(16008000), 10, 4607),
 data_c("s_2k_4", 2000, 10000, INT64_C(4000000), INT64_C(20010000), 10, 6108),
 data_c("p_1k", 1000, 999, INT64_C(499500), INT64_C(499500), 999, 184),
 data_c("p_2k", 2000, 1999, INT64_C(1999000), INT64_C(1999000), 1999, 1015),
 data_c("p_4k", 4000, 3999, INT64_C(7998000), INT64_C(7998000), 3999, 4341),
 data_c("m_4ki_2", 8192, 8190, INT64_C(16773120), INT64_C(16773120), 4095, 9240),
 data_c("m_1ki_8", 8192, 8184, INT64_C(4190208), INT64_C(4190208), 1023, 2149),
 data_c("m_256_32", 8192, 8160, INT64_C(1044480), INT64_C(1044480), 255, 691),
 data_c("m_64_128", 8192, 8064, INT64_C(258048), INT64_C(258048), 63, 193),
 data_c("m_16_512", 8192, 7680, INT64_C(61440), INT64_C(61440), 15, 57),
 data_c("m_4_2ki", 8192, 6144, INT64_C(12288), INT64_C(12288), 3, 47),
 data_c("b_17", 131071, 131070, INT64_C(1966082), INT64_C(1966082), 16, 1058),
 data_c("b_18", 262143, 262142, INT64_C(4194306), INT64_C(4194306), 17, 2496),
 data_c("b_19", 524287, 524286, INT64_C(8912898), INT64_C(8912898), 18, 4587),
 data_c("u_1k_50k", 1000, 50000, INT64_C(1000000), INT64_C(50050000), 3, 12394),
 data_c("u_1k_125k", 1000, 125000, INT64_C(1000000), INT64_C(125125000), 2, 30228),
 data_c("u_1k_250k", 1000, 250000, INT64_C(1000000), INT64_C(250250000), 2, 53183),
 data_c("u_2k_250k", 2000, 200000, INT64_C(4000000), INT64_C(400200000), 3, 114367),
 data_c("u_2k_500k", 2000, 500000, INT64_C(4000000), INT64_C(1000500000), 2, 291672),
 data_c("u_2k_1m", 2000, 1000000, INT64_C(4000000), INT64_C(2001000000), 2, 515968),
 data_c("f_1k_50k", 1000, 50000, INT64_C(472863), INT64_C(15421338), 8, 2442),
 data_c("f_1k_125k", 1000, 125000, INT64_C(492170), INT64_C(40705777), 5, 7084),
 data_c("f_1k_250k", 1000, 250000, INT64_C(497810), INT64_C(83073458), 5, 17066),
 data_c("f_2k_250k", 2000, 200000, INT64_C(1946015), INT64_C(128081155), 8, 17031),
 data_c("f_2k_500k", 2000, 500000, INT64_C(1985377), INT64_C(330379789), 6, 55599),
 data_c("f_2k_1m", 2000, 1000000, INT64_C(1995412), INT64_C(665127921), 4, 142702),
};

//-----------------------------------------------------------------------------
// Data for SQLite:
//-----------------------------------------------------------------------------


data_c data_sqlite[] = {
 data_c("k_1k", 1000, 1000000, INT64_C(1000000), INT64_C(1001000000), 1, 595849),
 data_c("k_2k", 2000, 4000000, INT64_C(4000000), INT64_C(8004000000), 1, 4983400),
 data_c("a_1k", 1000, 499500, INT64_C(499500), INT64_C(166666500), 1, 95443),
 data_c("a_2k", 2000, 1999000, INT64_C(1999000), INT64_C(1333333000), 1, 811345),
 data_c("c_1k", 1000, 1000, INT64_C(1000000), INT64_C(1001000), 1000, 1999),
 data_c("c_2k", 2000, 2000, INT64_C(4000000), INT64_C(4002000), 2000, 7816),
 data_c("c_4k", 4000, 4000, INT64_C(16000000), INT64_C(16004000), 4000, 31396),
 data_c("s_2k_1", 2000, 4000, INT64_C(4000000), INT64_C(8004000), 10, 9900),
 data_c("s_2k_2", 2000, 6000, INT64_C(4000000), INT64_C(12006000), 10, 12311),
 data_c("s_2k_3", 2000, 8000, INT64_C(4000000), INT64_C(16008000), 10, 14030),
 data_c("s_2k_4", 2000, 10000, INT64_C(4000000), INT64_C(20010000), 10, 15947),
 data_c("p_1k", 1000, 999, INT64_C(499500), INT64_C(499500), 999, 998),
 data_c("p_2k", 2000, 1999, INT64_C(1999000), INT64_C(1999000), 1999, 3740),
 data_c("p_4k", 4000, 3999, INT64_C(7998000), INT64_C(7998000), 3999, 15016),
 data_c("m_4ki_2", 8192, 8190, INT64_C(16773120), INT64_C(16773120), 4095, 32829),
 data_c("m_1ki_8", 8192, 8184, INT64_C(4190208), INT64_C(4190208), 1023, 7688),
 data_c("m_256_32", 8192, 8160, INT64_C(1044480), INT64_C(1044480), 255, 1920),
 data_c("m_64_128", 8192, 8064, INT64_C(258048), INT64_C(258048), 63, 629),
 data_c("m_16_512", 8192, 7680, INT64_C(61440), INT64_C(61440), 15, 206),
 data_c("m_4_2ki", 8192, 6144, INT64_C(12288), INT64_C(12288), 3, 113),
 data_c("b_17", 131071, 131070, INT64_C(1966082), INT64_C(1966082), 16, 3957),
 data_c("b_18", 262143, 262142, INT64_C(4194306), INT64_C(4194306), 17, 8119),
 data_c("b_19", 524287, 524286, INT64_C(8912898), INT64_C(8912898), 18, 17402),
 data_c("u_1k_50k", 1000, 50000, INT64_C(1000000), INT64_C(50050000), 3, 33188),
 data_c("u_1k_125k", 1000, 125000, INT64_C(1000000), INT64_C(125125000), 2, 79150),
 data_c("u_1k_250k", 1000, 250000, INT64_C(1000000), INT64_C(250250000), 2, 155578),
 data_c("u_2k_250k", 2000, 200000, INT64_C(4000000), INT64_C(400200000), 3, 286996),
 data_c("u_2k_500k", 2000, 500000, INT64_C(4000000), INT64_C(1000500000), 2, 676669),
 data_c("u_2k_1m", 2000, 1000000, INT64_C(4000000), INT64_C(2001000000), 2, 1303429),
 data_c("f_1k_50k", 1000, 50000, INT64_C(472863), INT64_C(15421338), 8, 9727),
 data_c("f_1k_125k", 1000, 125000, INT64_C(492170), INT64_C(40705777), 5, 24343),
 data_c("f_1k_250k", 1000, 250000, INT64_C(497810), INT64_C(83073458), 5, 48579),
 data_c("f_2k_250k", 2000, 200000, INT64_C(1946015), INT64_C(128081155), 8, 85702),
 data_c("f_2k_500k", 2000, 500000, INT64_C(1985377), INT64_C(330379789), 6, 213121),
 data_c("f_2k_1m", 2000, 1000000, INT64_C(1995412), INT64_C(665127921), 4, 417273),
};

//-----------------------------------------------------------------------------
// Data for Jena:
//-----------------------------------------------------------------------------

data_c data_jena[] = {
 data_c("k_1k", 1000, 1000000, INT64_C(1000000), INT64_C(1001000000), 1, 180557),
 data_c("k_2k", 2000, 4000000, INT64_C(4000000), INT64_C(8004000000), 1, 1477700),
 data_c("a_1k", 1000, 499500, INT64_C(499500), INT64_C(166666500), 1, 34982),
 data_c("a_2k", 2000, 1999000, INT64_C(1999000), INT64_C(1333333000), 1, 255690),
 data_c("c_1k", 1000, 1000, INT64_C(1000000), INT64_C(1001000), 1000, 3738),
 data_c("c_2k", 2000, 2000, INT64_C(4000000), INT64_C(4002000), 2000, 8098),
 data_c("c_4k", 4000, 4000, INT64_C(16000000), INT64_C(16004000), 4000, 23023),
 data_c("s_2k_1", 2000, 4000, INT64_C(4000000), INT64_C(8004000), 10, 9347),
 data_c("s_2k_2", 2000, 6000, INT64_C(4000000), INT64_C(12006000), 10, 10071),
 data_c("s_2k_3", 2000, 8000, INT64_C(4000000), INT64_C(16008000), 10, 10615),
 data_c("s_2k_4", 2000, 10000, INT64_C(4000000), INT64_C(20010000), 10, 11367),
 data_c("p_1k", 1000, 999, INT64_C(499500), INT64_C(499500), 999, 2885),
 data_c("p_2k", 2000, 1999, INT64_C(1999000), INT64_C(1999000), 1999, 5370),
 data_c("p_4k", 4000, 3999, INT64_C(7998000), INT64_C(7998000), 3999, 13797),
 data_c("m_4ki_2", 8192, 8190, INT64_C(16773120), INT64_C(16773120), 4095, 24628),
 data_c("m_1ki_8", 8192, 8184, INT64_C(4190208), INT64_C(4190208), 1023, 8954),
 data_c("m_256_32", 8192, 8160, INT64_C(1044480), INT64_C(1044480), 255, 3978),
 data_c("m_64_128", 8192, 8064, INT64_C(258048), INT64_C(258048), 63, 2663),
 data_c("m_16_512", 8192, 7680, INT64_C(61440), INT64_C(61440), 15, 2290),
 data_c("m_4_2ki", 8192, 6144, INT64_C(12288), INT64_C(12288), 3, 2065),
 data_c("b_17", 131071, 131070, INT64_C(1966082), INT64_C(1966082), 16, 8082),
 data_c("b_18", 262143, 262142, INT64_C(4194306), INT64_C(4194306), 17, 14817),
 data_c("b_19", 524287, 524286, INT64_C(8912898), INT64_C(8912898), 18, 31808),
 data_c("u_1k_50k", 1000, 50000, INT64_C(1000000), INT64_C(50050000), 3, 13711),
 data_c("u_1k_125k", 1000, 125000, INT64_C(1000000), INT64_C(125125000), 2, 26976),
 data_c("u_1k_250k", 1000, 250000, INT64_C(1000000), INT64_C(250250000), 2, 50019),
 data_c("u_2k_250k", 2000, 200000, INT64_C(4000000), INT64_C(400200000), 3, 85051),
 data_c("u_2k_500k", 2000, 500000, INT64_C(4000000), INT64_C(1000500000), 2, 197237),
 data_c("u_2k_1m", 2000, 1000000, INT64_C(4000000), INT64_C(2001000000), 2, 371938),
 data_c("f_1k_50k", 1000, 50000, INT64_C(472863), INT64_C(15421338), 8, 6631),
 data_c("f_1k_125k", 1000, 125000, INT64_C(492170), INT64_C(40705777), 5, 11791),
 data_c("f_1k_250k", 1000, 250000, INT64_C(497810), INT64_C(83073458), 5, 20468),
 data_c("f_2k_250k", 2000, 200000, INT64_C(1946015), INT64_C(128081155), 8, 31899),
 data_c("f_2k_500k", 2000, 500000, INT64_C(1985377), INT64_C(330379789), 6, 72170),
 data_c("f_2k_1m", 2000, 1000000, INT64_C(1995412), INT64_C(665127921), 4, 138465),
};

//-----------------------------------------------------------------------------
// Normal Array Size:
//-----------------------------------------------------------------------------

const int NUM_GRAPHS = sizeof(data_xsb)/sizeof(data_c);


//=============================================================================
// Class for Systems (with Array of Data Values):
//=============================================================================

class sys_c {
	public:

	// Constructor:
	sys_c(str_t sys_name, int sys_known_opt,
			data_c *sys_data, size_t sys_data_size) {
		Name = sys_name;
		KnownOpt = sys_known_opt;
		Data = sys_data;
		if(sys_data_size != NUM_GRAPHS) {
			std::cerr << "Wrong number of entries " <<
				"in data array for " << sys_name <<
				" (" << sys_data_size << ", correct: " <<
				NUM_GRAPHS << ")!\n";
			exit(4);
		}
	}

	// Accessor methods:
	str_t name() const { return Name; }
	int known_opt() const { return KnownOpt; }
	data_c *data() const { return Data; }


	private:

	// Attributes:
	str_t Name;
	int KnownOpt;
	data_c *Data;
};

//-----------------------------------------------------------------------------
// Pointer Type for System Data:
//-----------------------------------------------------------------------------

typedef sys_c *sys_t;

//-----------------------------------------------------------------------------
// List of Systems with Data Arrays:
//-----------------------------------------------------------------------------

sys_c systems[] = {
	sys_c("xsb", 34, data_xsb, sizeof(data_xsb)/sizeof(data_c)),
	sys_c("bam", 26, data_bam, sizeof(data_bam)/sizeof(data_c)),
	sys_c("yap", 26, data_yap, sizeof(data_yap)/sizeof(data_c)),
	sys_c("sqlite", 35, data_sqlite,
					sizeof(data_sqlite)/sizeof(data_c)),
	sys_c("jena", 29, data_jena, sizeof(data_jena)/sizeof(data_c)),
};

const int NUM_SYSTEMS = sizeof(systems)/sizeof(sys_c);

//-----------------------------------------------------------------------------
// Get object for a given system:
//-----------------------------------------------------------------------------

sys_t get_sys(str_t name) {
	for(int i = 0; i < NUM_SYSTEMS; i++) {
		if(str_eq(name, systems[i].name()))
			return &(systems[i]);
	}
	return nullptr;
}

//=============================================================================
// Class for Parameters of the Formula:
//=============================================================================

class par_c {
	public:

	//---------------------------------------------------------------------
	// Constant for number of single parameter values:
	//---------------------------------------------------------------------

	static const int DIMENSIONS = 7;

	static const int DIMEN_STARTUP  = 1;
	static const int DIMEN_LOADX    = 2;
	static const int DIMEN_RULE_APP = 3;
	static const int DIMEN_RESULT   = 4;
	static const int DIMEN_ITER     = 5;
	static const int DIMEN_LOAD     = 6;
	static const int DIMEN_DATA_VAL = 7;

	static const int DIMEN_FIRST = 1;
	static const int DIMEN_LAST  = 7;

	static const int DIMEN_NULL  = 0;

	//---------------------------------------------------------------------
	// Constructor:
	//---------------------------------------------------------------------

	par_c(long par_startup, long par_load, long par_rule_app,
		long par_result, long par_iter, long par_loadx,
		long par_data_val)
	{
		Par[DIMEN_NULL]     = 0;
		Par[DIMEN_STARTUP]  = par_startup;
		Par[DIMEN_LOAD]     = par_load;
		Par[DIMEN_RULE_APP] = par_rule_app;
		Par[DIMEN_RESULT]   = par_result;
		Par[DIMEN_ITER]     = par_iter;
		Par[DIMEN_LOADX]    = par_loadx;
		Par[DIMEN_DATA_VAL] = par_data_val;
	}

	//---------------------------------------------------------------------
	// Default Constructor:
	//---------------------------------------------------------------------

	par_c()
	{
		Par[DIMEN_NULL]     = 0;
		Par[DIMEN_STARTUP]  = 0;
		Par[DIMEN_LOAD]     = 0;
		Par[DIMEN_RULE_APP] = 0;
		Par[DIMEN_RESULT]   = 0;
		Par[DIMEN_ITER]     = 0;
		Par[DIMEN_LOADX]    = 0;
		Par[DIMEN_DATA_VAL] = 0;
	}

	//---------------------------------------------------------------------
	// Set parameter value:
	//---------------------------------------------------------------------

	inline void set(int dimen, long value) {

		// Check dimension:
#if DO_CHECKS
		if(dimen < DIMEN_FIRST || dimen > DIMEN_LAST) {
			std::cerr << "Invalid dimension " << dimen <<
					" in set.\n";
			exit(5);
		}
#endif

		// Set parameter value:
		Par[dimen] = value;
	}

	//---------------------------------------------------------------------
	// Get parameter value:
	//---------------------------------------------------------------------

	inline long get(int dimen) const {

		// Check dimension:
#if DO_CHECKS
		if(dimen < DIMEN_FIRST || dimen > DIMEN_LAST) {
			std::cerr << "Invalid dimension " << dimen <<
					" in get.\n";
			exit(5);
		}
#endif

		// Return parameter value:
		return Par[dimen];
	}

	//---------------------------------------------------------------------
	// Add step size to parameter value:
	//---------------------------------------------------------------------

	inline void add(int dimen, long step) {

		// Check dimension:
#if DO_CHECKS
		if(dimen < DIMEN_FIRST || dimen > DIMEN_LAST) {
			std::cerr << "Invalid dimension " << dimen <<
					" in add.\n";
			exit(5);
		}
#endif

		// Do the incrementation:
		Par[dimen] += step;
	}

	//---------------------------------------------------------------------
	// Change parameter in one dimention +1/-1:
	//---------------------------------------------------------------------

	void move(int dimen)
	{
		int inc = 1;
		if(dimen < 0) {
			dimen = -dimen;
			inc   = -1;
		}

		// Check that index is legal:
		if(dimen > DIMEN_LAST) {
				std::cerr << "Invalid dimension: " <<
						dimen << " in move\n";
				exit(5);
		}

		// Change parameter value:
		Par[dimen] += inc;
	}

	//---------------------------------------------------------------------
	// Milliseconds for startup (independent of input):
	//---------------------------------------------------------------------

	inline long startup_cost() const { return Par[DIMEN_STARTUP]; }

	//---------------------------------------------------------------------
	// Milliseconds to load one million input tuples:
	//---------------------------------------------------------------------

	inline long load_cost() const { return Par[DIMEN_LOAD]; }

	//---------------------------------------------------------------------
	// Milliseconds to apply one million rule instances:
	//---------------------------------------------------------------------

	inline long rule_app_cost() const { return Par[DIMEN_RULE_APP]; }

	//---------------------------------------------------------------------
	// Number of applied rule instances per millisecond:
	//---------------------------------------------------------------------

	inline long speed() const
	{
		long rule_app = Par[DIMEN_RULE_APP];
		return (rule_app == 0) ? 0 :
					(1000000 + rule_app/2) / rule_app;
	}

	//---------------------------------------------------------------------
	// Milliseconds to derive one million result tuples:
	//---------------------------------------------------------------------

	inline long result_cost() const { return Par[DIMEN_RESULT]; }

	//---------------------------------------------------------------------
	// Milliseconds per one million scanned tuples (iterations*edges):
	//---------------------------------------------------------------------

	inline long iter_cost() const { return Par[DIMEN_ITER]; }

	//---------------------------------------------------------------------
	// Milliseconds for inserting edges in an index (per n*log(n) = 1mio):
	//---------------------------------------------------------------------

	inline long loadx_cost() const { return Par[DIMEN_LOADX]; }

	//---------------------------------------------------------------------
	// Milliseconds to work with one million data values (nodes):
	//---------------------------------------------------------------------

	inline long data_val_cost() const { return Par[DIMEN_DATA_VAL]; }

	//---------------------------------------------------------------------
	// Product of all parameter values (to compute total number of steps):
	//---------------------------------------------------------------------

	inline bigint_t prod() const
	{
		bigint_t product = 1;
		for(int dimen = DIMEN_FIRST; dimen <= DIMEN_LAST; dimen++)
			product *= Par[dimen];
		return product;
	}

	//---------------------------------------------------------------------
	// Print parameters (long version):
	//---------------------------------------------------------------------

	void print(str_t indent)
	{
		std::cout << indent << "Startup:  " << Par[DIMEN_STARTUP]
							<< "\n";
		std::cout << indent << "Load:     " << Par[DIMEN_LOAD]
							<< "\n";
		std::cout << indent << "Rule App: " << Par[DIMEN_RULE_APP]
							<< "\n";
		std::cout << indent << "Speed:    " << speed() << "\n";
		std::cout << indent << "Result:   " << Par[DIMEN_RESULT]
							<< "\n";
		std::cout << indent << "Iter*Tup: " << Par[DIMEN_ITER]
							<< "\n";
		std::cout << indent << "LoadX:    " << Par[DIMEN_LOADX]
							<< "\n";
		std::cout << indent << "DataVal:  " << Par[DIMEN_DATA_VAL]
							<< "\n";
	}

	//---------------------------------------------------------------------
	// Print parameters (short version):
	//---------------------------------------------------------------------

	void print_short()
	{
		std::cout << "(" <<
			"S: " << Par[DIMEN_STARTUP]  << ", " <<
			"L: " << Par[DIMEN_LOAD]     << ", " <<
			"A: " << Par[DIMEN_RULE_APP] << ", " <<
			"R: " << Par[DIMEN_RESULT]   << ", " <<
			"I: " << Par[DIMEN_ITER]     << ", " <<
			"X: " << Par[DIMEN_LOADX]    << ", " <<
			"D: " << Par[DIMEN_DATA_VAL] << ")"  <<
			"\n";
	}

	//---------------------------------------------------------------------
	// The Formula to Estimate Runtime from Graph Data:
	//---------------------------------------------------------------------

	inline long formula(data_t data_p)
	{
		bigint_t edges = data_p->edges();
		long load_part = IDIV_ROUND(edges * Par[DIMEN_LOAD], 1000000);

		long rule_part = 0;
		bigint_t cost = data_p->cost();
		rule_part = IDIV_ROUND(cost * Par[DIMEN_RULE_APP], 1000000);

		bigint_t tc_size = data_p->tc_size();
		long result_part = IDIV_ROUND(tc_size * Par[DIMEN_RESULT],
					 1000000);

		bigint_t iter_edges = data_p->iter_edges();
		long iter_part = IDIV_ROUND(iter_edges * Par[DIMEN_ITER],
					 1000000);

		bigint_t nodes = data_p->nodes();
		long data_part = IDIV_ROUND(nodes * Par[DIMEN_DATA_VAL],
					1000000);

		bigint_t edges_log_edges = data_p->edges_log_edges();
		long loadx_part = IDIV_ROUND(edges_log_edges * Par[DIMEN_LOADX],
					1000000);

		return Par[DIMEN_STARTUP] +
			load_part + rule_part + result_part + iter_part +
			loadx_part + data_part;
	}

	//---------------------------------------------------------------------
	// Attributes:
	//---------------------------------------------------------------------

	private:
	long Par[DIMEN_LAST+1];
};

//-----------------------------------------------------------------------------
// Pointer Type for Formula Parameters:
//-----------------------------------------------------------------------------

typedef par_c *par_t;


//=============================================================================
// Evaluation of the Formula (How good is it?):
//=============================================================================

//-----------------------------------------------------------------------------
// check_formula:
//-----------------------------------------------------------------------------

int check_formula(par_c *par_p, sys_t sys)
{
	int hits = 0;

	data_c *data_p = sys->data();
	for(int i = 0; i < NUM_GRAPHS; i++, data_p++) {
		long estimate = par_p->formula(data_p);
		long limit = estimate / 5;
		long runtime = data_p->real_time();
		if(runtime >= estimate-limit && runtime <= estimate+limit)
			hits++;
	}

	return hits;
}

//-----------------------------------------------------------------------------
// Check Formula with Verbose Output:
//-----------------------------------------------------------------------------

int check_formula_with_output(par_c *par_p, sys_t sys)
{
	int hits = 0;

	std::cout << "Detailed Formula Evaluation:\n";
	std::cout << "---------------------------\n";
	par_p->print("\t");
	std::cout << "\n";

	data_c *data_p = sys->data();
	for(int i = 0; i < NUM_GRAPHS; i++, data_p++) {
		long estimate = par_p->formula(data_p);
		long limit = estimate / 5;
		long runtime = data_p->real_time();
		std::cout << "Graph: " << data_p->graph() <<
			", Estimate: " << estimate <<
			", Runtime: " << runtime;
		if(estimate-limit > runtime)
			std::cout << ", miss (+).\n";
		else if(estimate+limit < runtime)
			std::cout << ", miss (-).\n";
		else {
			hits++;
			if(estimate > runtime)
				std::cout << ", hit (+).\n";
			else if(estimate < runtime)
				std::cout << ", hit (-).\n";
			else
				std::cout << ", hit (!).\n";
		}
	}

	std::cout << "Hits: " << hits << "\n";
	return hits;
}

//-----------------------------------------------------------------------------
// Distance measure for local optimization:
//-----------------------------------------------------------------------------

bigint_t distance(par_c *par_p, sys_t sys)
{
	bigint_t sum_dist = 0;

	data_c *data_p = sys->data();
	for(int i = 0; i < NUM_GRAPHS; i++, data_p++) {
		long estimate = par_p->formula(data_p);
		long limit = estimate / 5;
		if(estimate <= 0)
			estimate = 1;
		long runtime = data_p->real_time();
		long rel_dist;

		// Compute distance of estimate from real value in promille:
		if(estimate > runtime)
			rel_dist = ((estimate - runtime) * 1000) / estimate;
		else if(estimate == runtime)
			rel_dist = 0;
		else // runtime > estimate
			rel_dist = ((runtime - estimate) * 1000) / estimate;

		// Limit the relative distance to make sure that hits have prio:
		if(rel_dist > 100000/NUM_GRAPHS)
			rel_dist = 100000/NUM_GRAPHS;

		// Add relative distance to sum (to optimize within same hits):
		sum_dist += rel_dist;

		// Emphasize distance outside the 20% limit:
		if(rel_dist > 200) {
			//sum_dist += 1000000;
			sum_dist += 100 * (rel_dist - 200);
		}

		// Because of rounding problems, it happened that something
		// was not counted as hit, although the relative distance
		// was 200.
		// To be sure, we use the exact condition for hits (negated):
		if(runtime < estimate-limit || runtime > estimate+limit)
			sum_dist += 10000000;

	}

	return sum_dist;
}

//=============================================================================
// Auxiliary Functions to Determine the Possible Range of Parameters:
//=============================================================================


//-----------------------------------------------------------------------------
// Compute Maximal Value for the Rule Application Cost Parameter:
//-----------------------------------------------------------------------------

// If the runtime has to be explained entirely by the rule application cost
// (i.e. all other parameters are 0), what would be the value?
// This function computes the maximal such value over all graphs:

long compute_max_rule(sys_t sys)
{
	long max_rule = 0;
	long sum_rule = 0;

	data_c *data_p = sys->data();
	for(int i = 0; i < NUM_GRAPHS; i++, data_p++) {
		bigint_t cost = data_p->cost();
		bigint_t real_time = data_p->real_time();
		if(cost > 0) {
			long rule = (real_time*1000000 + (cost-1)) / cost;
			if(rule > max_rule)
				max_rule = rule;
			sum_rule += rule;
		}
	}

	// Return reasonable upper limit:
	long double_avg = (2*sum_rule) / NUM_GRAPHS;
	return MIN(double_avg, max_rule);
}

//-----------------------------------------------------------------------------
// Compute Maximal Value for the Result Cost Parameter:
//-----------------------------------------------------------------------------

long compute_max_result(sys_t sys)
{
	long max_result = 0;
	long sum_result = 0;

	data_c *data_p = sys->data();
	for(int i = 0; i < NUM_GRAPHS; i++, data_p++) {
		bigint_t tc_size = data_p->tc_size();
		bigint_t real_time = data_p->real_time();
		if(tc_size > 0) {
			long result = (real_time*1000000 + (tc_size-1)) / 
								tc_size;
			if(result > max_result)
				max_result = result;

			sum_result += result;
		}
	}

	// Return reasonable upper limit:
	long double_avg = (2*sum_result) / NUM_GRAPHS;
	return MIN(double_avg, max_result);
}

//-----------------------------------------------------------------------------
// Compute Maximal Value for the Load Parameter:
//-----------------------------------------------------------------------------

long compute_max_load(sys_t sys)
{
	long max_load = 0;
	long sum_load = 0;

	// Compute maximum value over all graphs:
	data_c *data_p = sys->data();
	for(int i = 0; i < NUM_GRAPHS; i++, data_p++) {
		bigint_t edges = data_p->edges();
		bigint_t real_time = data_p->real_time();
		if(edges > 0) {
			long load = (real_time*1000000 + (edges-1)) / edges;
			if(load > max_load)
				max_load = load;

			sum_load += load;
		}
	}

	// Limit it with a somewhat reasonable value:
	// if(max_load > 10000)
	//	max_load = 10000;

	// Return reasonable upper limit:
	long double_avg = (2*sum_load) / NUM_GRAPHS;
	return MIN(double_avg, max_load);
}


//-----------------------------------------------------------------------------
// Compute Step Size for a Parameter:
//-----------------------------------------------------------------------------

long step_size(long max_value, long num_values)
{
	// Avoid division by 0:
	if(num_values == 0)
		return 1;

	// One value is always the minimal value:
	if(num_values == 1)
		return max_value + 1;

	// Compute step size (first value is always the minimal value):
	long step = max_value / (num_values-1);

	// Ensure that at least 1:
	if(step < 1)
		step = 1;

	// Return step size:
	return step;
}

//-----------------------------------------------------------------------------
// Compute Maximal Value for the Data Values Cost Parameter:
//-----------------------------------------------------------------------------

long compute_max_dataval(sys_t sys)
{
	long max_dataval = 0;
	long sum_dataval = 0;

	data_c *data_p = sys->data();
	for(int i = 0; i < NUM_GRAPHS; i++, data_p++) {
		bigint_t nodes = data_p->nodes();
		bigint_t real_time = data_p->real_time();
		if(nodes > 0) {
			long dataval = (real_time*1000000 + (nodes-1)) / nodes;
			if(dataval > max_dataval)
				max_dataval = dataval;
			sum_dataval += dataval;
		}
	}

	// Return reasonable upper limit:
	long double_avg = (2*sum_dataval) / NUM_GRAPHS;
	return MIN(double_avg, max_dataval);
}


//-----------------------------------------------------------------------------
// Compute Maximal Value for the Iter Cost Parameter:
//-----------------------------------------------------------------------------

long compute_max_iter(sys_t sys)
{
	long max_val = 0;
	long sum_val = 0;

	data_c *data_p = sys->data();
	for(int i = 0; i < NUM_GRAPHS; i++, data_p++) {
		bigint_t cost = data_p->iter_edges();
		bigint_t real_time = data_p->real_time();
		if(cost > 0) {
			long val = IDIV_CEILING(real_time*1000000, cost);
			if(val > max_val)
				max_val = val;
			sum_val += val;
		}
	}

	// Return reasonable upper limit:
	long double_avg = (2*sum_val) / NUM_GRAPHS;
	return MIN(double_avg, max_val);
}

//-----------------------------------------------------------------------------
// Compute Maximal Value for the Index Insertion Cost Parameter (LOADX):
//-----------------------------------------------------------------------------

long compute_max_loadx(sys_t sys)
{
	long max_val = 0;
	long sum_val = 0;

	data_c *data_p = sys->data();
	for(int i = 0; i < NUM_GRAPHS; i++, data_p++) {
		bigint_t cost = data_p->edges_log_edges();
		bigint_t real_time = data_p->real_time();
		if(cost > 0) {
			long val = IDIV_CEILING(real_time*1000000, cost);
			if(val > max_val)
				max_val = val;
			sum_val += val;
		}
	}

	// Return reasonable upper limit:
	long double_avg = (2*sum_val) / NUM_GRAPHS;
	return MIN(double_avg, max_val);
}

//-----------------------------------------------------------------------------
// Determine Maximal Reasonable Values for All Parameters:
//-----------------------------------------------------------------------------

void compute_max_par(par_t max_par, sys_t sys)
{
	// Determine maximal value for load efficiency:
	long max_load = compute_max_load(sys);
	// The file contains about 8-9 bytes per edge.
	// I.e. it should be possible to read about 10mio edges per second
	// (with a fast disc and a sequentially stored/non-fragmented file).
	// However, the edges must be processed. i.e. entered in an index.
	// For some systems, we measured the load time.
	// xsb uses up to 6s load time per million edges. The average is 2.2s.
	// long max_load = 6000;

	// Limit the load time to a somewhat reasonable value:
	if(max_load > 100000)
		max_load = 100000;

	// Determine maximal value for speed:
	long max_rule = compute_max_rule(sys);

	// Reasonable maximal value for startup time:
	long max_startup = 1000;

	// Compute maximal value for time per one million result tuples:
	long max_result = compute_max_result(sys);

	// Compute maximal value for time per one million scans (iter*edges):
	long max_iter = compute_max_iter(sys);

	// Compute maximal value for time per one million index insertion cost:
	long max_loadx = compute_max_loadx(sys);

	// Compute maximal value for time per one million data values:
	long max_dataval = compute_max_dataval(sys);

	// Set the maximal values:
	max_par->set(par_c::DIMEN_STARTUP,  max_startup);
	max_par->set(par_c::DIMEN_LOAD,     max_load);
	max_par->set(par_c::DIMEN_RULE_APP, max_rule);
	max_par->set(par_c::DIMEN_RESULT,   max_result);
	max_par->set(par_c::DIMEN_ITER,     max_iter);
	max_par->set(par_c::DIMEN_LOADX,    max_loadx);
	max_par->set(par_c::DIMEN_DATA_VAL, max_dataval);
}

//=============================================================================
// Function for local optimization:
//=============================================================================

void optimize_local(par_t par, sys_t sys, int max_steps)
{
	int best_dimen = 1;
	bigint_t best_dist = distance(par, sys);
	while(best_dimen != 0 && max_steps-- > 0) {

		//std::cout << "DIST: " << best_dist <<
		//	", HITS: " << check_formula(par, sys) << ": ";
		//par->print_short();

		// Stop loop if we cannot improve the distance:
		best_dimen = 0;

		// Try all dimensions, move in direction with smallest distance:
		for(int dimen = 1; dimen < par_c::DIMENSIONS; dimen++) {
			par->move(dimen);
			bigint_t new_dist = distance(par, sys);
			if(new_dist < best_dist) {
				best_dist = new_dist;
				best_dimen = dimen;
			}
			// Move back:
			par->move(-dimen);

			// Now do the same for the negative direction:
			par->move(-dimen);
			new_dist = distance(par, sys);
			if(new_dist < best_dist) {
				best_dist = new_dist;
				best_dimen = -dimen;
			}
			// Move back:
			par->move(dimen);
		}
		//std::cout << "\tBest Dimen: " << best_dimen << "\n";

		// Now do the best move:
		if(best_dimen != 0)
			par->move(best_dimen);

	// End of while loop over optimization steps:
	}
}

//=============================================================================
// Function for global optimization with relative optimization at each point:
//=============================================================================

int optimize(par_t best_par, sys_t sys,
		par_t min_par, par_t max_par, par_t inc_par)
{
	int best_hits = 0;

	// Compute total number of steps:
	bigint_t total_steps = 1;
	for(int d = par_c::DIMEN_FIRST; d <= par_c::DIMEN_LAST; d++) {
		long diff = max_par->get(d) - min_par->get(d);
		total_steps *= IDIV_CEILING((diff+1), inc_par->get(d));
	}

	// For output of progress:
	bigint_t one_percent = IDIV_CEILING(total_steps, 100);
	bigint_t countdown = 1;
	int pct = 0;

	// Optimization:
	par_c par;
	while(1) {

		// Print control output:
		if(--countdown <= 0) {
			std::cout << pct << "%\n";
			countdown = one_percent;
			pct++;
		}

		par_c opt_par = par;
		optimize_local(&opt_par, sys, 1000);
		//par.print_short();

		int hits = check_formula(&opt_par, sys);
		if(hits > best_hits) {
			*best_par = opt_par;
			best_hits = hits;
		}

		// Determine next parameter value:
		int dimen = par_c::DIMEN_LAST;
		while(dimen >= par_c::DIMEN_FIRST) {
			par.add(dimen, inc_par->get(dimen));
			if(par.get(dimen) <= max_par->get(dimen))
				break;
			par.set(dimen, min_par->get(dimen));
			dimen--;
			
		}
		if(dimen < par_c::DIMEN_FIRST)
			break;
	}

	return best_hits;
}

//=============================================================================
// Class for Optimization Job (Can be executed in a thread):
//=============================================================================

class job_c {

	public:

	//---------------------------------------------------------------------
	// Default Constructor (needed to declare array of jobs):
	//---------------------------------------------------------------------

	job_c() :
		MinPar(), MaxPar(), IncPar(), BestPar()
	{
		Sys   = nullptr;
		JobNo = 0;
		BestHits = -1;
	}

	//---------------------------------------------------------------------
	// This does the real initialization:
	//---------------------------------------------------------------------

	void init(sys_t job_sys, const par_c& job_min_par,
			const par_c& job_max_par, const par_c& job_inc_par,
			int split_dimen, int split_jobs, int job_no)
	{
		// Store given data:
		Sys      = job_sys;
		MinPar   = job_min_par;
		MaxPar   = job_max_par;
		IncPar   = job_inc_par;
		JobNo    = job_no;
		BestHits = -1;

		// Check splitting parameters:
#if DO_CHECK
		if(split_jobs < 1) {
			std::cerr << "Invalid number of jobs: " <<
					split_jobs << "\n";
			exit(6);
		}
		if(job_no < 0 || job_no >= split_jobs) {
			std::cerr << "Invalid job no: " << job_no << "\n";
			exit(6);
		}
#endif

		// Stop here if only one job (serial execution):
		if(split_dimen == par_c::DIMEN_NULL || split_jobs <= 1)
			return;

		// Split parameter range in given dimension (for parallel exec):
		long min = MinPar.get(split_dimen);
		long max = MaxPar.get(split_dimen);
		long inc = IncPar.get(split_dimen);
		long diff = max - min;
		long steps = IDIV_CEILING((diff+1), inc);
		if(steps < split_jobs) {
			std::cerr << split_jobs << " jobs requested, " <<
				"but only " << steps << " steps " <<
				"in dimension " << split_dimen << ".\n";
				exit(7);
		}
		long steps_per_job = steps / split_jobs;
		long remaining_steps = steps % split_jobs;
		long min_in_this_job = min + job_no * steps_per_job * inc;
		long max_in_this_job = min + (job_no+1) * steps_per_job * inc
			- 1;
		// We distribute the remaining_steps over the first jobs
		// (one additional step per job):
		if(job_no < remaining_steps) {
			min_in_this_job += (job_no * inc);
			// The first job has job_no==0, thus the first minimum
			// will not be increased.
			max_in_this_job += ((job_no+1) * inc);
			// This does not change the property that the intervals
			// are consecutive, because the next minimum will be
			// increased by the next job_no, which is job_no+1.
		}
		else {
			min_in_this_job += (remaining_steps * inc);
			max_in_this_job += (remaining_steps * inc);
		}

		// Now set minimum and maximum for the split_dimenion:
		MinPar.set(split_dimen, min_in_this_job);
		MaxPar.set(split_dimen, max_in_this_job);

		// Make sure that the interval of the last job ends at max:
		// (This is probably unnecessary)
		if(job_no == split_jobs - 1)
			MaxPar.set(split_dimen, max);

		// Test Output:
		// std::cout << "Job " << JobNo << ":\n";
		// std::cout << "\tMin:   "; MinPar.print_short();
		// std::cout << "\tMax:   "; MaxPar.print_short();
		// std::cout << "\tInc:   "; IncPar.print_short();
		// std::cout << "\tSteps: " << TotalSteps << std::endl;
	}

	//---------------------------------------------------------------------
	// Do Optimization (without relative optimization):
	//---------------------------------------------------------------------

	int optimize_simple()
	{
		int max_hits = 0;

		// For output of progress:
		bigint_t output_steps = IDIV_CEILING(total_steps() * OUT_PCT,
							100);
		bigint_t countdown = 1;
		int pct = 0;

		// Optimization:
		par_c par = MinPar;
		while(1) {
			// Print control output:
			if(--countdown <= 0) {
#if NUM_JOBS > 1
				std::cout << "[Job " << JobNo << ": " <<
					pct << "%, Best: " << max_hits <<
					"]: ";
#else
				std::cout << pct << "%, Best:" << max_hits;
#endif
				BestPar.print_short();
				std::cout << std::flush;
				countdown = output_steps;
				pct += OUT_PCT;
			}

			//par.print_short();

			int hits = check_formula(&par, Sys);
			if(hits > max_hits) {
				BestPar = par;
				max_hits = hits;
			}

			// Determine next parameter value:
			int dimen = par_c::DIMEN_LAST;
			while(dimen >= par_c::DIMEN_FIRST) {
				par.add(dimen, IncPar.get(dimen));
				if(par.get(dimen) <= MaxPar.get(dimen))
					break;
				par.set(dimen, MinPar.get(dimen));
				dimen--;
				
			}
			if(dimen < par_c::DIMEN_FIRST)
				break;
		}

		BestHits = max_hits;
		return max_hits;
	}

	//---------------------------------------------------------------------
	// Attribute Accessor Functions:
	//---------------------------------------------------------------------

	inline const par_c *best_par() const { return &BestPar; }

	inline int best_hits() const { return BestHits; }

	//---------------------------------------------------------------------
	// Compute total number of steps:
	//---------------------------------------------------------------------

	bigint_t total_steps() const
	{
		bigint_t total = 1;
		for(int d = par_c::DIMEN_FIRST; d <= par_c::DIMEN_LAST; d++) {
			long diff = MaxPar.get(d) - MinPar.get(d);
			long steps = IDIV_CEILING((diff+1), IncPar.get(d));
			total *= steps;
		}
		return total;
	}

	//---------------------------------------------------------------------
	// Attributes:
	//---------------------------------------------------------------------

	private:

	sys_t Sys;
	par_c MinPar;
	par_c MaxPar;
	par_c IncPar;
	par_c BestPar;
	int   BestHits;
	int   JobNo;
};

//-----------------------------------------------------------------------------
// Pointer Type for Job:
//-----------------------------------------------------------------------------

typedef job_c *job_t;

//=============================================================================
// Functions For Setting Up and Executing Optimization Jobs:
//=============================================================================

//-----------------------------------------------------------------------------
// Function to be Executed by a Separate Thread:
//-----------------------------------------------------------------------------

void run_job(job_t job) {
	job->optimize_simple();
}

//-----------------------------------------------------------------------------
// Parallel Execution of Simple Optimization:
//-----------------------------------------------------------------------------

#if NUM_JOBS > 1

int optimize_simple(sys_t sys, const par_c& min_par, const par_c&max_par,
			const par_c&inc_par, par_t best_par)
{
	// Optimization Jobs:
	job_c jobs[NUM_JOBS];
	for(int i = 0; i < NUM_JOBS; i++) {
		jobs[i].init(sys, min_par, max_par, inc_par,
				par_c::DIMEN_STARTUP, NUM_JOBS, i);
	}

	// Compute total number of parameter tuples:
	bigint_t total_steps = 0;
	for(int i = 0; i < NUM_JOBS; i++)
		total_steps += jobs[i].total_steps();
	std::cout << "Number of parameter tuples: " << total_steps << "\n";
	std::cout << std::endl;

	// Start threads for parallel execution of all jobs:
	std::cout << "Starting " << NUM_JOBS << " threads ..." << std::endl;
	std::thread threads[NUM_JOBS];
	for(int i = 0; i < NUM_JOBS; i++) {
		// Move-assign threads:
		threads[i] = std::thread(run_job, &(jobs[i]));
	}

	// Join threads:
	for(int i = 0; i < NUM_JOBS; i++)
		threads[i].join();

	// Output from main thread:
	std::cout << "Threads have finished.\n";

	// Find the best solution among the job results:
	*best_par = *(jobs[0].best_par());
	int best_hits = jobs[0].best_hits();
	for(int i = 1; i < NUM_JOBS; i++) {
		int job_best_hits = jobs[i].best_hits();
		if(jobs[i].best_hits() > best_hits) {
			best_hits = job_best_hits;
			*best_par = *(jobs[i].best_par());
		}
	}

	// Return hits of best parameter:
	return best_hits;
}

#endif

//-----------------------------------------------------------------------------
// Serial Version of Simple Optimization (without threads):
//-----------------------------------------------------------------------------

#if NUM_JOBS <= 1

int optimize_simple(sys_t sys, const par_c& min_par, const par_c&max_par,
			const par_c&inc_par, par_t best_par)
{
	// Optimization Job (parameter range is not splitted, only one job):
	job_c job;
	job.init(sys, min_par, max_par, inc_par, par_c::DIMEN_NULL, 1, 0);

	// Print total number of parameter tuples:
	std::cout << "Number of parameter tuples: " << job.total_steps() <<
			"\n";
	std::cout << std::endl;

	// Do optimization:
	//best_hits = optimize_simple(&best_par, sys,
	//		&min_par, &max_par, &inc_par);
	job.optimize_simple();

	// Find the best solution among the job results:
	*best_par = *(job.best_par());
	int best_hits = job.best_hits();

	// Return hits of best parameter:
	return best_hits;
}

#endif

//=============================================================================
// The main Function (Does the Optimization):
//=============================================================================


int main(int argc, str_t argv[])
{
	int best_hits = 0;
	par_c best_par;

	// The program should be called with the system name as cmdline arg:
	str_t sys_name = "xsb";
	if(argc == 2)
		sys_name = argv[1];
	else if(argc > 2) {
		std::cerr << "Usage: opt <SystemName>, e.g. \"opt xsb\".\n";
		exit(1);
	}

	// Get data for system:
	sys_t sys = get_sys(sys_name);
	if(sys == nullptr) {
		std::cerr << "Unkown system: " << sys_name << ".\n";
		exit(2);
	}

	// Print welcome:
	std::cout << "Welcome to the TC runtime prediction optimizer.\n";
	std::cout << "\n";
	std::cout << "Number of defined systems: " << NUM_SYSTEMS << "\n";
	std::cout << "Doing optimization for:    " << sys->name() << "\n";
	std::cout << "Number of graphs:          " << NUM_GRAPHS << "\n";
	std::cout << "Number of Threads:         " << NUM_JOBS << "\n";

	// Define number of steps for search:
	//         |Startup|   Load|   Rule| Result|DataVal|
	//par_c steps(     10,     50,    100,     10,     50);
	// This runs 5040.07user 0.08system 1:24:00 on bach
	// The total number of steps is 25mio, i.e. 200s/mio steps
	// Another test used 240s/mio steps

	// Small test search:
	//         |Startup|   Load|   Rule| Result|DataVal|
	//par_c steps(     10,     10,    100,     10,     10);
	//std::cout << "Number of parameter tuples:     " << steps.prod()<<"\n";
	//std::cout << "\n";
	// 240.99user 0.00system 4:00.97elapsed
	// Finds for Jena 19/35
	// Startup:  1346
	// Load:     0
	// Rule App: 224
	// Speed:    4464
	// Result:   30
	// DataVal:  48830

	// Small test search:
	//           |Startup|   Load|   Rule| Result|   Iter|  LoadX|DataVal|
	//par_c steps(     10,     50,    100,     50,      1,      50,     1);
	// This uses about 30s of CPU time

	// Larger Search:
	//           |Startup|   Load|   Rule| Result|   Iter|  LoadX|DataVal|
	//par_c steps(    50,    100,   1000,    1000,    100,      1,      );
	// This took about 32h CPU-time (8h with four threads).

	// Current search:
	//           |Startup|   Load|   Rule| Result|   Iter|  LoadX|DataVal|
	  par_c steps(     50,    100,    100,    100,     10,    100,     5);

	//par_c steps(     60,    100,    100,    100,     10,    100,     5);
	// This needed 50h CPU-time on work128, 2h with 30 threads

	// Compute maximal reasonable parameter values:
	//par_c max_par;
	//compute_max_par(&max_par, sys);

	// Manual setting of reasonable upper limits:
	//           |Startup|   Load|   Rule| Result|   Iter|  LoadX|DataVal|
	  par_c max_par( 3000,   5000,   1000,   3000,   3000,   5000,   3000);

	// Print search range:
	std::cout << "Maximal Parameter Values:\n";
	std::cout << "------------------------\n";
	max_par.print("\t");
	std::cout << "\n";

	// Currently the minimal parameters are 0:
	par_c min_par;

	// Print number of steps:
	std::cout << "Requested Number of Steps " <<
		"(Might Change Due to Rounded Step Size):\n";
	std::cout << "--------------------------" <<
		"---------------------------------------\n";
	steps.print("\t");
	std::cout << "\n";

	// Compute step size:
	par_c inc_par;
	for(int dimen = par_c::DIMEN_FIRST; dimen <= par_c::DIMEN_LAST;
			dimen++) {
		inc_par.set(dimen,
			step_size(max_par.get(dimen), steps.get(dimen)));
	}

	// Print step size:
	std::cout << "Step Size:\n";
	std::cout << "---------\n";
	inc_par.print("\t");

	// Local Optimization form good starting points:
	//          Start   Load  RuleApp Result
	//par_c opt(   90,  1000,      97,   300); // xsb
	//par_c opt(   70,  1130,      50,     0); // bam
	//par_c opt(   30,   670,     253,   300); // yap
	//par_c opt(   40,  6000,     694,  1000); // sqlite
	//par_c opt(  970, 27300,     168,  1558); // jena
	//optimize_local(&opt, sys, 1000);
	//best_par = opt;

	// Do optimization:
	best_hits = optimize_simple(sys, min_par, max_par, inc_par, &best_par);

	// Show runtimes for best formula:
	std::cout << "\n";
	std::cout << "\n";
	best_hits = check_formula_with_output(&best_par, sys);
	std::cout << "\n";
	std::cout << "\n";
	std::cout << "Best Parameter Values (" << best_hits << "/" <<
					NUM_GRAPHS << "):\n";
	best_par.print("\t");
	std::cout << "\n";
}

