-- ============================================================================
-- Project:	Deductive Database
-- Filename:	psql/j1axy.sql
-- Purpose:	Join1 Benchmark, Query a(X,Y), Implementation for PostgreSQL
-- Last Change:	23.03.2019
-- Language:	psql script (SQL for PostgreSQL)
-- Author:	Stefan Brass
-- EMail:	brass@informatik.uni-halle.de
-- WWW:		http://www.informatik.uni-halle.de/~brass/
-- Address:	Feldschloesschen 15, D-06120 Halle (Saale), GERMANY
-- Note:	There is no warranty at all - this code may contain bugs.
-- Copyright:	You can use this as you wish, but don't make me responsible!
--		I would be interested to hear about improved versions.
-- ============================================================================

-- The Benchmark contains the following Joins:
--	a(X, Y)  :- b1(X, Z), b2(Z, Y).
--	b1(X, Y) :- c1(X, Z), c2(Z, Y).
--	b2(X, Y) :- c3(X, Z), c4(Z, Y).
--	c1(X, Y) :- d1(X, Z), d2(Z, Y).
-- The query is a(X, Y). The database predicates are d1, d2, c2, c3, c4.

-- This script must be called with the following variables defined on the
-- command line:
--   - file_id: Input file name without the part for the predicate
--               and without the extenstion '.tsv', e.g. j1k10k
--
-- E.g., the following call would be possible:
--
-- psql -f join1_analysis.sql -v file_id=j1k10k

-- If you do not want to specify the data on the command line,
-- you can set it here (uncomment the next line):
-- \set file_id j1k10k
-- \set file_id j1k50k
-- \set file_id j1k250k

-- The input files must consist of lines that contain two integers
-- separated by a tab (tsv: tab-separated values).
-- There must be files with suffix _c2.tsv, _c3.tsv, _c4.tsv, _d1.tsv, _d2.tsv
-- For other formats, the \copy command below must be adapted.

-- Define here a directory where the data file can be found:
\set input_dir ../data_tsv
-- \set input_dir .

-- Note: The script overwrites the files load_cmd.sql and show_mem.sh!


\echo
\echo '======================================================================='
\echo 'Input File: ' :'file_id' '(with suffix _pred.tsv)'
\echo '======================================================================='

\echo
\echo '-----------------------------------------------------------------------'
\echo 'Print Pure Query Results (without decoration):'
\echo '-----------------------------------------------------------------------'
\echo

-- Tuples only:
\t on
-- Unaligned output (suppresses the mapping of "\n" to "+\n")
\pset format unaligned

\echo
\echo '-----------------------------------------------------------------------'
\echo 'Start Time:'
\echo '-----------------------------------------------------------------------'
\echo

-- Current timestamp (real current time, not start of transaction):
select clock_timestamp();

\echo
\echo '-----------------------------------------------------------------------'
\echo 'Increase work_mem parameter:'
\echo '-----------------------------------------------------------------------'
\echo

show work_mem;
SET work_mem TO '1 GB';
show work_mem;

\echo
\echo '-----------------------------------------------------------------------'
\echo 'Construction of Command to get Server Process Memory:'
\echo '-----------------------------------------------------------------------'
\echo

-- Output file:
\o show_mem.sh
select 'fgrep VmHWM /proc/' || pg_backend_pid() || '/status';
-- Standard output again:
\o

\echo
\echo '-----------------------------------------------------------------------'
\echo 'Create tables:'
\echo '-----------------------------------------------------------------------'
\echo

create temporary table c2(x integer not null, y integer not null);
create temporary table c3(x integer not null, y integer not null);
create temporary table c4(x integer not null, y integer not null);
create temporary table d1(x integer not null, y integer not null);
create temporary table d2(x integer not null, y integer not null);

\echo
\echo '-----------------------------------------------------------------------'
\echo 'Load the data:'
\echo '-----------------------------------------------------------------------'
\echo

\echo 'Write command to load the data to load_cmd.sql:'

-- The problem is that variables are not replaced in the copy command.
-- Therefore, we write a file with the command to load the par table
-- from the input file.

-- Output file:
\o load_cmd.sql
select concat('\copy c2 from ''', :'input_dir', '/', :'file_id', '_c2.tsv'';');
select concat('\copy c3 from ''', :'input_dir', '/', :'file_id', '_c3.tsv'';');
select concat('\copy c4 from ''', :'input_dir', '/', :'file_id', '_c4.tsv'';');
select concat('\copy d1 from ''', :'input_dir', '/', :'file_id', '_d1.tsv'';');
select concat('\copy d2 from ''', :'input_dir', '/', :'file_id', '_d2.tsv'';');
-- Standard output again:
\o

\echo 'LOAD DATA'
\timing on
-- Execute the \copy commands that were generated above:
\i load_cmd.sql
-- create index d2_xy on d2(x,y);
-- create index c4_xy on c4(x,y);
-- create index c2_xy on c2(x,y);
--
-- create index d2_xy on d2 using hash (x);
-- create index c4_xy on c4 using hash (x);
-- create index c2_xy on c2 using hash (x);
analyze d1;
analyze d2;
analyze c2;
analyze c3;
analyze c4;
\timing off

\echo
\echo '-----------------------------------------------------------------------'
\echo 'Create views for derived predicates:'
\echo '-----------------------------------------------------------------------'
\echo

--	a(X, Y)  :- b1(X, Z), b2(Z, Y).
--	b1(X, Y) :- c1(X, Z), c2(Z, Y).
--	b2(X, Y) :- c3(X, Z), c4(Z, Y).
--	c1(X, Y) :- d1(X, Z), d2(Z, Y).

create temporary view c1_nodup as
select distinct d1.x, d2.y
from   d1, d2
where  d1.y = d2.x;

create temporary view b1_nodup as
select distinct c1_nodup.x, c2.y
from   c1_nodup, c2
where  c1_nodup.y = c2.x;

create temporary view b2_nodup as
select distinct c3.x, c4.y
from   c3, c4
where  c3.y = c4.x;

create temporary view a_nodup as
select distinct b1_nodup.x, b2_nodup.y
from   b1_nodup, b2_nodup
where  b1_nodup.y = b2_nodup.x;

\echo
\echo '-----------------------------------------------------------------------'
\echo 'Time Before Benchmark Query:'
\echo '-----------------------------------------------------------------------'
\echo

select clock_timestamp();

\echo
\echo '-----------------------------------------------------------------------'
\echo 'Benchmark Query:'
\echo '-----------------------------------------------------------------------'
\echo

\timing on
-- explain analyze select count(*) from a_nodup;
select count(*) from a_nodup;
\timing off

\echo
\echo '-----------------------------------------------------------------------'
\echo 'Server Process Resident Memory:'
\echo '-----------------------------------------------------------------------'
\echo

\! sh ./show_mem.sh

\echo
\echo '-----------------------------------------------------------------------'
\echo 'Time After Benchmark Query:'
\echo '-----------------------------------------------------------------------'
\echo

select clock_timestamp();

\echo
\echo '-----------------------------------------------------------------------'
\echo 'Drop views and tables:'
\echo '-----------------------------------------------------------------------'
\echo

drop view a_nodup;
drop view b1_nodup;
drop view b2_nodup;
drop view c1_nodup;

drop table c2;
drop table c3;
drop table c4;
drop table d1;
drop table d2;

\echo
\echo '-----------------------------------------------------------------------'
\echo 'End Time:'
\echo '-----------------------------------------------------------------------'
\echo

select clock_timestamp();

\echo
\echo '-----------------------------------------------------------------------'
\echo 'Change back to normal decoration of tables:'
\echo '-----------------------------------------------------------------------'
\echo

-- Normal output again:
\pset format aligned
-- Normal decoration of tables:
\t off

\echo

